systemheader:(AB    )#                                                  AB     1
####################################################################### AB     2
##      Copyright (C) 2000, The Xtal System, The University of        # AB     3
##            Western Australia, xtal@crystal.uwa.edu.au.             # AB     4
## Distributed under the GNU General Public Licence as published by   # AB     5
## the Free Software Foundation, in the hope that it will be useful   # AB     6
## but WITHOUT ANY WARRANTY. See the GNU General Public License       # AB     7
## for more details.                                                  # AB     8
##                                                                    # AB     9
##                       **** ABSORB ****                             # AB    10
##                                                                    # AB    11
## Calculates and applies absorption factors for different types of   # AB    12
## crystal geometries and algorithms. Direct applications for spher-  # AB    13
## ical and cylindrical crystals; and the Gaussian and analytical for # AB    14
## crystals of general shape described by oriented faces. A Tbar      # AB    15
## value may be output for extinction calculations.                   # AB    16
##                                                                    # AB    17
####################################################################### AB    18
#                                                                       AB    19
include:(XMACRO)#                                                       AB    20
#                                                                       AB    21
macro:(setflg:,IF(I==0)$2=$3;ELSEIF(CHRIN(J^J+1)=="$1")$2=ABS($3-1))#   AB    22
macro:(setint:,IF(I==0)$2=$3;ELSEIF(CHRIN(J^J+1)=="$1")#                AB    23
$(ifnm:(BUFIN(I+1))$($2=NINT(BUFIN(I+1));I=I+1$);ELSE $2=$4$))#         AB    24
macro:(setval:,IF(I==0)$2=$3;ELSEIF(CHRIN(J^J+1)=="$1")#                AB    25
$(ifnm:(BUFIN(I+1))$($2=BUFIN(I+1);I=I+1$);ELSE $2=$4$))#               AB    26
macro:(dp:,ifelse:(mxbtwd:,32,DOUBLE PRECISION,REAL))#                  AB    27
macro:(sg:,ifelse:(dp:,REAL,$1,SNGL($1)))#                              AB    28
macro:(db:,ifelse:(dp:,REAL,$1,DBLE($1)))#                              AB    29
macro:(dabs:,ifelse:(dp:,REAL,ABS($1),DABS($1)))#                       AB    30
macro:(dcos:,ifelse:(dp:,REAL,COS($1),DCOS($1)))#                       AB    31
macro:(dsin:,ifelse:(dp:,REAL,SIN($1),DSIN($1)))#                       AB    32
macro:(dasin:,ifelse:(dp:,REAL,ASIN($1),DASIN($1)))#                    AB    33
macro:(datan2:,ifelse:(dp:,REAL,ATAN2($1,$2),DATAN2($1,$2)))#           AB    34
macro:(dsqrt:,ifelse:(dp:,REAL,SQRT($1),DSQRT($1)))#                    AB    35
macro:(tiny:,2.0D-arith:(mxsgfp:,-,2))##                                AB    36
macro:(dsmall:,1.0D-mxsgfp:)#                                           AB    37
#                                                                       AB    38
macro:(postm:,CALL AB04($1,$2,$3))#                                     AB    39
macro:(mmult:,CALL AB05($1,$2,$3))#                                     AB    40
macro:(triple:,CALL AB06($1,$2,$3))#                                    AB    41
#                                                                       AB    42
macro:(ifpos:,IF($1>small:))#                                           AB    43
macro:(incrqq:,[$(IF($2>QQCUR)$(IQXY=MAX($2,QQCUR)+1000#                AB    44
IF(IQXY>QQMAX)iquit:($3);DO IQXZ=QQCUR+1,IQXY;QQ(IQXZ)=0.#              AB    45
QQCUR=IQXY$)$1=$2$)])#                                                  AB    46
macro:(movedoub:,[$(DO IQXY=1,$5;$3($4+IQXY-1)=$1($2+IQXY-1)$)])#       AB    47
#                                                                       AB    48
extrude:(ABCOMN)#                                                       AB    49
dp: ABSF,ABSF2#                        A(mu),a(mu+dmu)                  AB    50
dp: ASTAR(30)#                         Astar tables                     AB    51
dp: TBAR(30)#                          Tbar tables                      AB    52
dp: BX(3,3)#                           Transf hkl to xtal system        AB    53
dp: ROT(5)#                            Rotation flags                   AB    54
dp: DEMU#                              Delta emu (for tbar)             AB    55
dp: DISTSC#                            Convert to mm                    AB    56
dp: DITES#                             Precision limit                  AB    57
dp: DBTES#                             Double precision test            AB    58
dp: EMU#                               Absorption coefficient           AB    59
dp: EMU2#                              Emu plus delta emu (for tbar     AB    60
dp: PHI90#                             Phi at chi 90                    AB    61
INTEGER IAPPLY#                        Flag apply abscor                AB    62
INTEGER ISET#                          Data set being treated           AB    63
INTEGER ITBAR#                         Flag calculate tbar              AB    64
INTEGER ICOS#                          Compute dir. cosines n and d     AB    65
INTEGER STEREO#                        Signal crystal plot type         AB    66
INTEGER LIST#                          Print n reflections              AB    67
INTEGER METHOD#                        Calculation method               AB    68
dp: MT(9)#                             Inverse metric tensor            AB    69
dp: OMIN#                              Min diffractometer omega         AB    70
dp: OMAX#                              Max diffractometer omega         AB    71
dp: RAD#                               Radius of sphere                 AB    72
dp: SCALE#                             Scale*dist, emu/scale            AB    73
dp: TC(3,3)#                           Rotation matrix                  AB    74
dp: UINV(3,3)#                         The inverse of the u matrix      AB    75
dp: UB(3,3)#                           Rotation and translation         AB    76
dp: VOL#                               Volume of the crystal            AB    77
dp: WAV#                               Radiation wavelength             AB    78
dp: XMAX,XMIN#                         Max and min x-coordinate         AB    79
dp: YMAX,YMIN#                         Max and min y-coordinate         AB    80
dp: ZMAX,ZMIN#                         Max and min z-coordinate         AB    81
dp: DVOL#                              Double precision volume          AB    82
dp: DBZ#                               Double precision zero            AB    83
INTEGER MARK2#                         Integration points               AB    84
INTEGER ISTOP2#                        End of storage                   AB    85
INTEGER NSTEP2#                        Words stored per point           AB    86
INTEGER NPOINT#                        Number of points                 AB    87
INTEGER MARKF#                         Faces                            AB    88
INTEGER ISTOPF#                        End of storage                   AB    89
INTEGER NSTEPF#                        Words per face                   AB    90
INTEGER NFACES#                        Number of faces                  AB    91
INTEGER MARKV#                         Vertices                         AB    92
INTEGER ISTOPV#                        End of storage                   AB    93
INTEGER NSTEPV#                        Words stored per vertex          AB    94
INTEGER NVERTS#                        Number of vertices               AB    95
INTEGER NAST#                          Number of astar entries          AB    96
INTEGER LIM#                           Number of reflections treated    AB    97
INTEGER NTBR#                          Number of tbar  entries          AB    98
INTEGER MARKED#                        Edges                            AB    99
INTEGER STOPED#                        End of storage                   AB   100
INTEGER STEPED#                        Words stored per edge            AB   101
INTEGER NEDGES#                        Number of edges                  AB   102
INTEGER MARK1#                                                          AB   103
INTEGER MARKH#                         Howell's points                  AB   104
INTEGER ISTOPH#                        End of storage                   AB   105
INTEGER MARKT#                                                          AB   106
INTEGER ISTOPT#                                                         AB   107
INTEGER MARKP#                         Plot points                      AB   108
INTEGER STOPP#                         End plot points                  AB   109
INTEGER STEPP#                         No. words per edge (7)           AB   110
INTEGER QQCUR,QQMAX#                   Pointers to qq array             AB   111
#                                                                       AB   112
dp: QQ(30000)## <<<<<< note the dp limit - see QQMAX in AB00            AB   113
#                                                                       AB   114
COMMON/COMDAB/$#                                                        AB   115
BX,ROT,DISTSC,EMU,PHI90,TC,UB,UINV,VOL,WAV,TBAR,#                       AB   116
XMAX,XMIN,YMAX,YMIN,ZMAX,ZMIN,ASTAR,SCALE,EMU2,DEMU,DITES,MT,#          AB   117
OMIN,OMAX,ABSF,ABSF2,DVOL,DBZ,RAD,DBTES,QQ#                             AB   118
#                                                                       AB   119
COMMON/COMIAB/$#                                                        AB   120
ISET,ISTOP2,ISTOPF,ISTOPH,ISTOPV,MARK1,MARK2,MARKED,MARKF,MARKH,MARKV,# AB   121
NEDGES,NFACES,NPOINT,NSTEP2,NSTEPF,NSTEPV,NVERTS,STEPED,STOPED,#        AB   122
IAPPLY,ITBAR,METHOD,MARKT,ISTOPT,ICOS,MARKP,STOPP,STEPP,#               AB   123
QQCUR,QQMAX,STEREO,NAST,NTBR,LIM,LIST#                                  AB   124
endext:#                                                                AB   125
#                                                                       AB   126
systemheader:(AB00)#                                                    AB00   1
#################################################################       AB00   2
#                                                               #       AB00   3
#                             AB00                              #       AB00   4
#                                                               #       AB00   5
#################################################################       AB00   6
SUBROUTINE AB00#                                                        AB00   7
IMPLICIT INTEGER (I-N)#                                                 AB00   8
include:(AACOMN)#                                                       AB00   9
include:(ABCOMN)#                                                       AB00  10
#                                                                       AB00  11
## GAUSSIAN PROGRAM:                                                    AB00  12
##   XRAY PROGRAM BY H.L. AMMON                                         AB00  13
##   XTAL PROGRAM BY G. DAVENPORT, J.M. STEWART, J.R. HOLDEN            AB00  14
#                                                                       AB00  15
## ANALYTICAL PROGRAM:    BY G. DAVENPORT                               AB00  16
#                                                                       AB00  17
## STEREO PLOTS AND AUTO MU MODIFICATIONS  BY W. DREISSIG SEPT 1986     AB00  18
## SPHERICAL CORRECTION AND PROJECTION PLOTS  BY  L. M. ENGLEHARDT 1986 AB00  19
## ALLOW FOR PSI SCANS  BY N. SPADACCINI 1988                           AB00  20
#                                                                       AB00  21
FILEXT(IOUNIT(5))='ABS'#               Output plot file of xtal shape   AB00  22
QQMAX=30000;   QQCUR=0#                Start qq array pointers          AB00  23
#                                                                       AB00  24
CALL AB10#                             Read optional data               AB00  25
IF(METHOD<3)#                          If gaussian or analytical        AB00  26
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB00  27
CALL AB11#                             Read the crystal faces           AB00  28
CALL AB13#                             Determine crystal geometry       AB00  29
IF(METHOD==1)#                         Apply gaussian method            AB00  30
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB00  31
CALL AB20#                             Construct gaussian grid          AB00  32
CALL AB31#                             Calculate absorption correctio   AB00  33
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB00  34
ELSE IF(METHOD==2) CALL AB40#          Apply analytical method          AB00  35
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB00  36
#                                                                       AB00  37
RETURN#                                                                 AB00  38
END#                                                                    AB00  39
systemheader:(AB01)#                                                    AB01   1
###################################################################     AB01   2
#                                                                 #     AB01   3
#                             AB01                                #     AB01   4
#                                                                 #     AB01   5
###################################################################     AB01   6
SUBROUTINE AB01(DH1,DH2,DT)#                                            AB01   7
IMPLICIT INTEGER (I-N)#                                                 AB01   8
#                                                                       AB01   9
## COMPUTE NORMALIZED CROSS PRODUCT                                     AB01  10
## XRAY PROGRAM BY H.L. AMMON, SEPTEMBER 1971                           AB01  11
## XTAL PROGRAM BY J.M. STEWART, NOVEMBER 1980                          AB01  12
#                                                                       AB01  13
dp: DH1(3)#                            First vector                     AB01  14
dp: DH2(3)#                            Second vector                    AB01  15
dp: S#                                 Normalization factor             AB01  16
dp: DT(3)#                             Cross product                    AB01  17
dp: DB1#                                                                AB01  18
#                                                                       AB01  19
DB1=db:(1.0)#                                                           AB01  20
DT(1)=DH1(2)*DH2(3)-DH1(3)*DH2(2)#     First element                    AB01  21
DT(2)=DH1(3)*DH2(1)-DH1(1)*DH2(3)#     Second element                   AB01  22
DT(3)=DH1(1)*DH2(2)-DH1(2)*DH2(1)#     Third element                    AB01  23
S=DB1/dsqrt:(DT(1)**2+DT(2)**2+DT(3)**2)# Normalization factor          AB01  24
DT(1)=DT(1)*S ; DT(2)=DT(2)*S ; DT(3)=DT(3)*S# Normalize                AB01  25
#                                                                       AB01  26
RETURN#                                                                 AB01  27
END#                                                                    AB01  28
systemheader:(AB02)#                                                    AB02   1
####################################################################    AB02   2
#                                                                  #    AB02   3
#                             AB02                                 #    AB02   4
#                                                                  #    AB02   5
####################################################################    AB02   6
SUBROUTINE AB02(DF1,DF2,DF3,DXYZ,NEW)#                                  AB02   7
IMPLICIT INTEGER (I-N)#                                                 AB02   8
include:(AACOMN)#                                                       AB02   9
include:(ABCOMN)#                                                       AB02  10
#                                                                       AB02  11
## DETERMINE THE INTERSECTION OF THREE PLANES, TEST TO SEE IF IT        AB02  12
## IS INSIDE THE CRYSTAL                                                AB02  13
## GEORGE DAVENPORT, MARCH 1982                                         AB02  14
#                                                                       AB02  15
dp: DF1(4),DF2(4),DF3(4)#              Double precision of same         AB02  16
dp: BB(3,3)#                           Inverse normal matrix            AB02  17
dp: DET#                               Determinant of normal matrix     AB02  18
dp: DXYZ(3),DR,DQ(4)#                                                   AB02  19
INTEGER II,N#                          Loop index                       AB02  20
INTEGER NEW#                           Flag new vertex                  AB02  21
#                                                                       AB02  22
# SOLVE THREE SIMULTANEOUS EQUATIONS TO FIND THE INTERSECTION           AB02  23
# OF THREE PLANES                                                       AB02  24
# -----------------------------------------------------------           AB02  25
#                                                                       AB02  26
NEW=yes:#                              Assume new vertex                AB02  27
BB(1,1)=DF2(2)*DF3(3)-DF2(3)*DF3(2)#                                    AB02  28
BB(2,1)=DF2(3)*DF3(1)-DF2(1)*DF3(3)#                                    AB02  29
BB(3,1)=DF2(1)*DF3(2)-DF2(2)*DF3(1)#                                    AB02  30
DET=DF1(1)*BB(1,1)+DF1(2)*BB(2,1)+DF1(3)*BB(3,1)#                       AB02  31
IF(dabs:(DET)<dsmall:) NEW=no:#        No intersection                  AB02  32
ELSE#                                  Calculate intersection           AB02  33
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB02  34
BB(1,2)=DF1(3)*DF3(2)-DF1(2)*DF3(3)#                                    AB02  35
BB(2,2)=DF1(1)*DF3(3)-DF1(3)*DF3(1)#                                    AB02  36
BB(3,2)=DF1(2)*DF3(1)-DF1(1)*DF3(2)#                                    AB02  37
BB(1,3)=DF1(2)*DF2(3)-DF1(3)*DF2(2)#                                    AB02  38
BB(2,3)=DF1(3)*DF2(1)-DF1(1)*DF2(3)#                                    AB02  39
BB(3,3)=DF1(1)*DF2(2)-DF1(2)*DF2(1)#                                    AB02  40
DXYZ(1)=(BB(1,1)*DF1(4)+BB(1,2)*DF2(4)+BB(1,3)*DF3(4))/DET#             AB02  41
DXYZ(2)=(BB(2,1)*DF1(4)+BB(2,2)*DF2(4)+BB(2,3)*DF3(4))/DET#             AB02  42
DXYZ(3)=(BB(3,1)*DF1(4)+BB(3,2)*DF2(4)+BB(3,3)*DF3(4))/DET#             AB02  43
#                                                                       AB02  44
FOR(II=MARKF;II<ISTOPF;II=II+NSTEPF)#  Loop over faces                  AB02  45
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB02  46
DO N=1,4; DQ(N)=QQ(II+N)#                                               AB02  47
DR=DXYZ(1)*DQ(1)+DXYZ(2)*DQ(2)+DXYZ(3)*DQ(3)-DQ(4)#                     AB02  48
IF(DR>DBTES)#                          Outside                          AB02  49
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB02  50
NEW=no: ; BREAK#                       Set flag and break               AB02  51
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB02  52
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB02  53
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB02  54
RETURN#                                                                 AB02  55
END#                                                                    AB02  56
systemheader:(AB03)#                                                    AB03   1
####################################################################    AB03   2
#                                                                  #    AB03   3
#                             AB03                                 #    AB03   4
#                                                                  #    AB03   5
####################################################################    AB03   6
SUBROUTINE AB03(IBLOCK,TEMP)#          Modified w. dreissig 1986        AB03   7
IMPLICIT INTEGER (I-N)#                                                 AB03   8
include:(AACOMN)#                                                       AB03   9
include:(ABCOMN)#                                                       AB03  10
#                                                                       AB03  11
###                                                                     AB03  12
## I/O FOR LRREFL:, CALCULATE ANGLES AND BEAM VECTORS                   AB03  13
## GEORGE DAVENPORT, JANUARY 1982                                       AB03  14
## T. MASLEN, JANUARY 1984                                              AB03  15
## N. SPADACCINI, MARCH 1989 - PSI SCANS INCLUDED                       AB03  16
#                                                                       AB03  17
INTEGER BISECT#                        Bisect/psi yes/no                AB03  18
dp: CHI#                               Chi                              AB03  19
dp: COSCHI#                            Cos(chi)                         AB03  20
dp: COSPHI#                            Cos(phi)                         AB03  21
dp: COSPSI#                            Cos(psi)#                        AB03  22
dp: COSOM#                             Cos(om)                          AB03  23
dp: COST#                              Cos(theta)                       AB03  24
dp: DET#                               Determinant of b-matrix          AB03  25
dp: DXYZ(3)#                           Used to get chi, phi             AB03  26
INTEGER I,J,M,II#                                                       AB03  27
INTEGER IBLOCK#                        Block to be executed             AB03  28
INTEGER INP#                           Pointer into input packet        AB03  29
INTEGER IP#                            Miscellaneous pointer            AB03  30
INTEGER IPP#                           Output packet pointer            AB03  31
INTEGER IRP#                           Pointer to psi                   AB03  32
INTEGER IREL(19)#                      Relative pointers                AB03  33
INTEGER IWANT(19)#                     Want list for indexpacket        AB03  34
INTEGER OUTP#                          Output packet pointer            AB03  35
dp: OM#                                Omega - eularian                 AB03  36
dp: OMEGA#                             Omega - diffractometer           AB03  37
dp: PHI#                               Phi                              AB03  38
dp: PSI#                               Psi                              AB03  39
INTEGER PSIZ#                          Input packet size                AB03  40
INTEGER PSIZO#                         Output packet size               AB03  41
INTEGER COUNT,COUNDN#                  Total count of reflections       AB03  42
dp: R(3,3)#                            Normed matrix                    AB03  43
dp: RDST#                              Normalization factor             AB03  44
dp: SINCHI#                            Sin(chi)                         AB03  45
dp: SINPHI#                            Sin(phi)                         AB03  46
dp: SINPSI#                            Sin(psi)                         AB03  47
dp: SINOM#                             Sin(om)                          AB03  48
dp: SINT#                              Sin(theta)                       AB03  49
dp: T(6)#                              Temporary                        AB03  50
dp: T1,T2#                             Temporary                        AB03  51
dp: TEMP(8)#                           Calling arguments for ab03       AB03  52
dp: TH#                                Theta                            AB03  53
dp: TTHETA#                            Twice theta                      AB03  54
dp: X(3)#                              Miller indices                   AB03  55
dp: ABMAX, ABMIN#                      Min astar                        AB03  56
dp: XR13,XR23,XR31,XR32,XR33#                                           AB03  57
dp: T3,T4#                                                              AB03  58
#                                                                       AB03  59
chardata:(AB031, ARCHIV END   )#                                        AB03  60
chardata:(AB032," TbarIrelF2rlFrelsigIsgF2sigFstoreN       DIR. COS$#   AB03  61
      D ")#                                                             AB03  62
chardata:(AB033,"   h   k   l 2theta   omega     chi     phi    A     $#AB03  63
                                                                     ")#AB03  64
chardata:(AB034," Max absorption A*          $#                         AB03  65
Min transmission A       ")#                                            AB03  66
chardata:(AB035," Min absorption A*          $#                         AB03  67
Max transmission A       ")#                                            AB03  68
chardata:(AB036,HKL C  )#                                               AB03  69
realdata:(FM,[40413.,80413.,120413.,190622.,350722.,430722.,270722.,#   AB03  70
560642.,640831.,730831.,800721.,870721.,940742.,1010742.,1080742.,#     AB03  71
1150742.,1220742.,1290742.])#                                           AB03  72
realdata:(FM1,[150213.,190413.,230413.,270413.,370922.,470922.,#        AB03  73
800862.,350742.,430742.,510742.,590742.,670742.,750742.,#               AB03  74
370902.])#                                                              AB03  75
intdata:(KEY,[2,8,0,0])#                                                AB03  76
SAVE ABMAX,ABMIN,COUNDN,COUNT,IPP,IREL#                                 AB03  77
datastuff:#                                                             AB03  78
#                                                                       AB03  79
BISECT=yes: ; OM=0.0 ; PSI=0.0#        Default - bisecting mode         AB03  80
#                                                                       AB03  81
# INITIALIZE LRREFL: I/O                                                AB03  82
# ----------------------                                                AB03  83
#                                                                       AB03  84
IF(IBLOCK==1)#                         Set up lrrefl: i/o               AB03  85
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB03  86
ABMAX=0. ; ABMIN=100.#                 Initialize astar max and min     AB03  87
COUNT=0 ; COUNDN=0#                    Zero counts                      AB03  88
DO I=1,8; TEMP(I)=0.#                  Zero temp array                  AB03  89
IWANT(1)=1#                            Request hkl                      AB03  90
IWANT(2)=2#                            Request stol                     AB03  91
KEY(1)=1; KEY(2)=2; KEY(4)=0#          Hkl required; update mode        AB03  92
IF(IAPPLY==no:)#                       Store correction                 AB03  93
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB03  94
IWANT(4)=ISET+201#                     Request absf                     AB03  95
IWANT(3)=ISET+11#                      Request psi                      AB03  96
KEY(2)=3#                              Optional                         AB03  97
M=4#                                   No. of input items               AB03  98
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB03  99
ELSE#                                                                   AB03 100
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB03 101
IWANT(3)=ISET+298+2*IAPPLY#                                             AB03 102
IWANT(4)=IWANT(3)+1#                                                    AB03 103
IWANT(5)=ISET+308#                     Rcode                            AB03 104
IWANT(6)=ISET+11#                      Request psi                      AB03 105
KEY(2)=6#                                                               AB03 106
M=6#                                   No. of input items               AB03 107
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB03 108
IF(ICOS>0) KEY(3)=M+ITBAR+6#           Set no. of output item           AB03 109
ELSE       KEY(3)=M+ITBAR#             Set no. of output item           AB03 110
IF(ITBAR==yes:) IWANT(M+1)=ISET+200#                                    AB03 111
IF(ICOS>0)#                                                             AB03 112
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB03 113
M=M+1#                                                                  AB03 114
DO I=1,6; IWANT(M+I)=ISET+205+I#       Add dir.cosines                  AB03 115
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB03 116
indexpkt:(1,lrrefl:,PSIZ,IP,2,KEY,IWANT,IREL)# Search directory         AB03 117
IF(IP<=0) iquit:(AB0301)#              Missing lrrefl:                  AB03 118
IF(KEY(4)<0) iquit:(AB0302)#           Missing hkl from bdf             AB03 119
IF(IAPPLY>0&(IREL(3)==0\IREL(4)==0))iquit:(AB0310)# Missing coeff       AB03 120
PSIZO=KEY(4)#                          Output packet size               AB03 121
#                                                                       AB03 122
#                                                                       AB03 123
LINRM=3#                                                                AB03 124
IF(ITBAR==yes:) movechar:(AB032,2,AB033,53,4,0)# Heading                AB03 125
J=0#                                   Header pointer increment         AB03 126
IF(IAPPLY!=no:)#                       Headings                         AB03 127
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB03 128
II=4*IAPPLY+2#                         Point to ab032                   AB03 129
movechar:(AB032,II,AB033,61,4,0)#      Heading                          AB03 130
movechar:(AB032,30,AB033,69,5,0)#      Corrected                        AB03 131
movechar:(AB032,II+12,AB033,77,4,0)#   Heading for sigma                AB03 132
movechar:(AB032,30,AB033,83,5,0)#      Corrected                        AB03 133
J=36#                                  Incr header pointer              AB03 134
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB03 135
IF(ICOS>0)#                                                             AB03 136
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB03 137
movechar:(AB032,35,AB033,63+J,23,0)#   Heading for dir.cosine           AB03 138
#         PREPARE FOR LATER COMPUTING OF N,D-VECTORS                    AB03 139
DO I=1,3;DO J=1,3; R(I,J)=BX(I,J)#     Store transformation matrix      AB03 140
DO I=1,3; CALL AB07(R(1,I),DET)#       Normalize components o           AB03 141
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB03 142
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB03 143
#                                                                       AB03 144
# CALCULATE ANGLES AND BEAM VECTORS                                     AB03 145
# ---------------------------------                                     AB03 146
#                                                                       AB03 147
ELSE IF(IBLOCK==2)#                    Angles and beam vectors          AB03 148
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB03 149
readwpkt:(1,lrrefl:,PSIZ,IP,0)#        Read a packet                    AB03 150
IF(IP<=0) TEMP(1)=1.0D00#              Reset flag for break             AB03 151
ELSE#                                  No break                         AB03 152
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB03 153
writepkt:(2,lrrefl:,PSIZO,IPP)#        **** writepkt ****               AB03 154
movereal:(QX,IP+1,QX,IPP+1,PSIZ,1)#    Copy old pkt across              AB03 155
TEMP(1)=0.0D00#                        Set flag for no break            AB03 156
TEMP(2)=0.0D00#                        Set flag to process reflecti     AB03 157
COUNT=COUNT+1#                         Count reflections                AB03 158
IF(COUNT>LIM) TEMP(2)=1.0D00#          Set limit flag                   AB03 159
ELSE#                                  Process this reflection          AB03 160
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB03 161
COUNDN=COUNDN+1#                       Count processed reflections      AB03 162
INP=IP+IREL(1)#                        Pointer to hkl                   AB03 163
gethkl:(INP,X)#                        Get hkl                          AB03 164
II=IP+IREL(2)#                         Point to stol                    AB03 165
IF(II>IP) SINT=db:(QX(II))*WAV#        Calculate sin(theta)             AB03 166
ELSE#                                  If stol not on bdf               AB03 167
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB03 168
SINT=X(1)*(X(1)*MT(1)+X(2)*MT(2)+X(3)*MT(3))$# Calc 4*stol**2           AB03 169
+    X(2)*(X(1)*MT(4)+X(2)*MT(5)+X(3)*MT(6))$# Calc 4*stol**2           AB03 170
+    X(3)*(X(1)*MT(7)+X(2)*MT(8)+X(3)*MT(9))# Calc 4*stol**2            AB03 171
SINT=WAV*dsqrt:(SINT)/2.D0#            Calc sine theta                  AB03 172
IF(SINT>1.) iquit:(AB0303)#            Nonsense sine theta              AB03 173
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB03 174
IRP=IREL(6)#                           Pointer to psi in degrees        AB03 175
IF(IAPPLY==no:) IRP=IREL(3)#           Pointer to psi if no apply       AB03 176
IF(IRP>0)#                                                              AB03 177
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB03 178
BISECT=no:#                            Not bisecting mode               AB03 179
PSI=db:(QX(IP+IRP))/57.29577951D00#    Psi in radians                   AB03 180
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB03 181
TH=dasin:(SINT)#                       Theta in radians                 AB03 182
TEMP(4)=SINT/WAV#                      Sin(theta)/wavelength            AB03 183
TTHETA=114.591559D00*dasin:(SINT)#     Two theta                        AB03 184
IF(METHOD<3)#                          Do if non-spherical corr         AB03 185
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AB03 186
COST=dsqrt:(1.0D0-SINT**2)#            Calculate cos(theta)             AB03 187
RDST=0.5*ROT(1)*WAV/SINT#              Multiplier                       AB03 188
DXYZ(1)=UB(1,1)*X(1)+UB(1,2)*X(2)+UB(1,3)*X(3)#                         AB03 189
DXYZ(2)=UB(2,1)*X(1)+UB(2,2)*X(2)+UB(2,3)*X(3)#                         AB03 190
DXYZ(3)=UB(3,1)*X(1)+UB(3,2)*X(2)+UB(3,3)*X(3)#                         AB03 191
DXYZ(1)=DXYZ(1)*RDST#                                                   AB03 192
DXYZ(2)=DXYZ(2)*RDST#                                                   AB03 193
DXYZ(3)=DXYZ(3)*RDST#                                                   AB03 194
#                                                                       AB03 195
# CALCULATE CHI, PHI, COSINES AND SINES - BISECTING MODE                AB03 196
# ------------------------------------------------------                AB03 197
#                                                                       AB03 198
CHI=datan2:((ROT(2)*DXYZ(3)),dsqrt:(DXYZ(1)**2+DXYZ(2)**2))#            AB03 199
IF(NINT(dabs:(57.29577951D00*CHI))==90) PHI=PHI90#                      AB03 200
ELSE PHI=datan2:((ROT(3)*DXYZ(2)),DXYZ(1))#                             AB03 201
COSPHI=dcos:(PHI) ; SINPHI=dsin:(PHI)#                                  AB03 202
COSCHI=dcos:(CHI) ; SINCHI=dsin:(CHI)#                                  AB03 203
#                                                                       AB03 204
# CALCULATE SETTING ANGLES - GENERAL MODE                               AB03 205
# ---------------------------------------                               AB03 206
#                                                                       AB03 207
IF(BISECT==no:)#                       Non-bisecting mode               AB03 208
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AB03 209
SINPSI=dsin:(PSI) ; COSPSI=dcos:(PSI)# Sin and cos of psi               AB03 210
XR13=ROT(2)*SINCHI#                                                     AB03 211
XR23=ROT(5)*SINPSI*COSCHI#                                              AB03 212
XR31=ROT(3)*ROT(5)*SINPSI*SINPHI-ROT(2)*COSPSI*SINCHI*COSPHI#           AB03 213
XR32=-ROT(5)*SINPSI*COSPHI-ROT(2)*ROT(3)*COSPSI*SINCHI*SINPHI#          AB03 214
XR33=COSPSI*COSCHI#                                                     AB03 215
CHI=datan2:(dsqrt:(XR31**2+XR32**2),XR33)#                              AB03 216
PHI=datan2:(-ROT(2)*ROT(3)*XR32,-ROT(2)*XR31)#                          AB03 217
OM=datan2:(-ROT(4)*ROT(2)*XR23,ROT(2)*XR13)#                            AB03 218
#                                                                       AB03 219
# IS OMEGA OUT OF RANGE (-48 < OMEGA < 58) - USE EQUIVALENT SETTING     AB03 220
# -----------------------------------------------------------------     AB03 221
#                                                                       AB03 222
OMEGA=TH+OM#                           Diffractometer angle omega       AB03 223
IF((OMEGA<OMIN)\(OMEGA>OMAX))#         Outside diffractomter limits     AB03 224
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AB03 225
CHI=-CHI#                              Chi to minus chi                 AB03 226
PHI=3.141592654D00+PHI#                Rotate phi through pi            AB03 227
OM =3.141592654D00+OM#                 Rotate om through pi             AB03 228
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AB03 229
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AB03 230
COSPHI=dcos:(PHI) ; SINPHI=dsin:(PHI)# Store sin and cos of angles      AB03 231
COSCHI=dcos:(CHI) ; SINCHI=dsin:(CHI)#                                  AB03 232
COSOM=dcos:(OM) ; SINOM=dsin:(OM)#                                      AB03 233
COST=dcos:(TH) ; SINT=dsin:(TH)#                                        AB03 234
#                                                                       AB03 235
OMEGA=(TH+OM)*57.29577951D00#          Omega in degrees                 AB03 236
IF(OMEGA-360.>=0.) OMEGA=OMEGA-360.#   Reset to between 0 and 360       AB03 237
TH= TH *57.29577951D00#                Theta in degrees                 AB03 238
PHI=PHI*57.29577951D00#                Phi in degrees                   AB03 239
CHI=CHI*57.29577951D00#                Chi in degrees                   AB03 240
#                                                                       AB03 241
# CALCULATE BEAM VECTORS                                                AB03 242
# ----------------------                                                AB03 243
#                                                                       AB03 244
T1=ROT(1)*SINT*COSOM*COSPHI*COSCHI#    Elements of the primary and      AB03 245
T2=ROT(4)*COST*SINOM*COSCHI*COSPHI#    Diffracted beam vectors          AB03 246
T3=ROT(3)*COST*COSOM*SINPHI#                                            AB03 247
T4=ROT(1)*ROT(4)*ROT(3)*SINT*SINOM*SINPHI#                              AB03 248
T(1)=T1+T2+T3-T4 ; T(4)=T1-T2-T3-T4#                                    AB03 249
#                                                                       AB03 250
T1=ROT(1)*ROT(3)*SINT*COSOM*SINPHI*COSCHI#                              AB03 251
T2=ROT(4)*ROT(3)*COST*SINOM*COSCHI*SINPHI#                              AB03 252
T3=COST*COSOM*COSPHI#                                                   AB03 253
T4=ROT(1)*ROT(4)*SINT*SINOM*COSPHI#                                     AB03 254
T(2)=T1+T2-T3+T4 ; T(5)=T1-T2+T3+T4#                                    AB03 255
#                                                                       AB03 256
T1=ROT(1)*ROT(2)*SINT*COSOM*SINCHI#                                     AB03 257
T2=ROT(4)*ROT(2)*COST*SINOM*SINCHI#                                     AB03 258
T(3)=T1+T2 ; T(6)=T1-T2#                                                AB03 259
#                                                                       AB03 260
postm:(UINV,T,TEMP(3))#                Post multiply                    AB03 261
postm:(UINV,T(4),TEMP(6))#             Post multiply                    AB03 262
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AB03 263
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB03 264
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB03 265
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB03 266
#                                                                       AB03 267
# STORE ITEMS AND APPLY CORRECTION                                      AB03 268
# --------------------------------                                      AB03 269
#                                                                       AB03 270
ELSE IF(IBLOCK==3)#                    Store items etc.                 AB03 271
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AB03 272
TEMP(1)=MAX(0.0D00,TEMP(1))#           Protect square root              AB03 273
ABMAX=MAX(ABMAX,TEMP(1))#              Keep track of max astar          AB03 274
ABMIN=MIN(ABMIN,TEMP(1))#              Keep track of min astar          AB03 275
IF(IAPPLY==no:)#                                                        AB03 276
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AB03 277
OUTP=IPP+IREL(4)#                                                       AB03 278
QX(OUTP)=sg:(1.0/TEMP(1))#                                              AB03 279
J=5#                                                                    AB03 280
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AB03 281
ELSE#                                                                   AB03 282
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AB03 283
OUTP=IPP+IREL(3)#                                                       AB03 284
BUFOT(9)=QX(OUTP)#                                                      AB03 285
IF(IAPPLY==3) QX(OUTP)=QX(OUTP)*sg:(dsqrt:(TEMP(1)))#                   AB03 286
ELSE          QX(OUTP)=QX(OUTP)*sg:(TEMP(1))#                           AB03 287
BUFOT(10)=QX(OUTP)#                                                     AB03 288
OUTP=IPP+IREL(4)#                                                       AB03 289
BUFOT(11)=QX(OUTP)#                                                     AB03 290
IF(IAPPLY==3) QX(OUTP)=QX(OUTP)*sg:(dsqrt:(TEMP(1)))#                   AB03 291
ELSE          QX(OUTP)=QX(OUTP)*sg:(TEMP(1))#                           AB03 292
BUFOT(12)=QX(OUTP)#                                                     AB03 293
J=7#                                                                    AB03 294
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AB03 295
IF(ITBAR==yes:)#                                                        AB03 296
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AB03 297
OUTP=IPP+IREL(J)#                                                       AB03 298
IF(METHOD>2) QX(OUTP)=10.*TEMP(2)#     Store tbar in mm                 AB03 299
ELSE         QX(OUTP)=10.*sg:((1.-TEMP(1)/TEMP(2))/DEMU)# Tbar in mm    AB03 300
BUFOT(8)=QX(OUTP)#                     Prepare to write                 AB03 301
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AB03 302
IF(ICOS>0)#                                                             AB03 303
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AB03 304
OUTP=IPP+IREL(J+1)-1#                  Output address                   AB03 305
CALL AB01(TEMP(3),TEMP(6),T)#          Vector d=[p,s] in xtal-space     AB03 306
CALL AB01(T,TEMP(3),T(4))#             Vector n=[[p,s],p] in xtal-spa   AB03 307
#                    (SEE HAMILTON, ACTA CRYST.A26,P.71)                AB03 308
DO I=1,3#                              Loop over components             AB03 309
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AB03 310
CALL AB09(R(1,I),T(4),db:(QX(OUTP+I)))# Dot product - vector n          AB03 311
CALL AB09(R(1,I),T,db:(QX(OUTP+3+I)))# Vector d in crystal sy           AB03 312
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AB03 313
movereal:(QX,OUTP+1,BUFOT,13,6,0)#     Store for printing               AB03 314
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AB03 315
IF(COUNDN<=LIST)#                                                       AB03 316
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AB03 317
DO I=1,3; BUFOT(I)=sg:(X(I))#          Hkl                              AB03 318
BUFOT(4)=sg:(TTHETA)#                                                   AB03 319
ncodefld:(BUFOT,1,CHROT,FM,4)#                                          AB03 320
IF(METHOD<3)#                          If not sphere or cyl             AB03 321
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AB03 322
BUFOT(5)=sg:(CHI)#                                                      AB03 323
BUFOT(6)=sg:(PHI)#                                                      AB03 324
BUFOT(7)=sg:(OMEGA)#                                                    AB03 325
ncodefld:(BUFOT,5,CHROT,FM(5),7)#                                       AB03 326
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AB03 327
BUFOT(7)=sg:(1.0/TEMP(1))#                                              AB03 328
ncodefld:(BUFOT,7,CHROT,490542.,1)#    Encode a*                        AB03 329
IF(ITBAR==yes:) ncodefld:(BUFOT,8,CHROT,FM(8),1)# Encode tbar           AB03 330
IF(IAPPLY!=no:) ncodefld:(BUFOT,9,CHROT,FM(9),4)# Encode item           AB03 331
IF(ICOS>0) ncodefld:(BUFOT,13,CHROT,FM(13),6)# Encode dir.co            AB03 332
writeline:(0,AB033,NAB033,1,3)#                                         AB03 333
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AB03 334
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AB03 335
#                                                                       AB03 336
# COPY REST OF FILE                                                     AB03 337
# -----------------                                                     AB03 338
#                                                                       AB03 339
ELSE IF(IBLOCK==4)#                    Finish run                       AB03 340
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AB03 341
copyfile:(1,2,incr:(lrrefl:),endrecord:)# Copy rest of file             AB03 342
LINRM=6#                               Make room                        AB03 343
BUFOT(1)=sg:(ABMAX); BUFOT(2)=sg:(ABMIN)# Store as single precision     AB03 344
ncodefld:(BUFOT,1,AB034,250532.,1)#    Load max astar for output        AB03 345
ncodefld:(1./BUFOT(1),1,AB034,530542.,1)# Load min a for output         AB03 346
ncodefld:(BUFOT,2,AB035,250532.,1)#    Load min astar for output        AB03 347
ncodefld:(1./BUFOT(2),1,AB035,530542.,1)# Load max a for output         AB03 348
writeline:(1,AB034,NAB034,3,3)#                                         AB03 349
writeline:(0,AB035,NAB035,3,3)#                                         AB03 350
IF(METHOD==1 & LINID!=2)#              If gauss, read end line          AB03 351
readline:(AB031,NAB031)#               Read =end=                       AB03 352
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AB03 353
RETURN#                                                                 AB03 354
END#                                                                    AB03 355
systemheader:(AB04)#                                                    AB04   1
####################################################################    AB04   2
#                                                                  #    AB04   3
#                              AB04                                #    AB04   4
#                                                                  #    AB04   5
####################################################################    AB04   6
SUBROUTINE AB04(A,B,C)#                                                 AB04   7
IMPLICIT INTEGER (I-N)#                                                 AB04   8
#                                                                       AB04   9
dp: A(3,3)#                            Matrix                           AB04  10
dp: B(3)#                              Vector                           AB04  11
dp: C(3)#                              New vector c=a*b                 AB04  12
#                                                                       AB04  13
C(1)=A(1,1)*B(1)+A(1,2)*B(2)+A(1,3)*B(3)# First element                 AB04  14
C(2)=A(2,1)*B(1)+A(2,2)*B(2)+A(2,3)*B(3)# Second element                AB04  15
C(3)=A(3,1)*B(1)+A(3,2)*B(2)+A(3,3)*B(3)# Third element                 AB04  16
RETURN#                                                                 AB04  17
END#                                                                    AB04  18
systemheader:(AB05)#                                                    AB05   1
#################################################################       AB05   2
#                                                               #       AB05   3
#                            AB05                               #       AB05   4
#                                                               #       AB05   5
#################################################################       AB05   6
SUBROUTINE AB05(A,B,C)#                                                 AB05   7
IMPLICIT INTEGER (I-N)#                                                 AB05   8
#                                                                       AB05   9
## 3X3 MATRIX MULTIPLY                                                  AB05  10
## H.L. AMMON, 1971                                                     AB05  11
## FORSYTH, 1973                                                        AB05  12
## J.M. STEWART, 1980                                                   AB05  13
#                                                                       AB05  14
dp: A(3,3)#                            First matrix                     AB05  15
dp: B(3,3)#                            Second matrix                    AB05  16
dp: C(3,3)#                            Product matrix                   AB05  17
INTEGER I#                             Loop index                       AB05  18
INTEGER J#                             Loop index                       AB05  19
INTEGER K#                             Loop index                       AB05  20
#                                                                       AB05  21
DO I=1,3#                              Loop over 3 elements             AB05  22
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB05  23
DO J=1,3#                              Loop over 3 elements             AB05  24
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB05  25
C(I,J)=0.0#                            Initialize output element        AB05  26
DO K=1,3#                              Sum of element products          AB05  27
C(I,J)=C(I,J)+A(I,K)*B(K,J)#           Product element                  AB05  28
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB05  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB05  30
#                                                                       AB05  31
RETURN#                                                                 AB05  32
END#                                                                    AB05  33
systemheader:(AB06)#                                                    AB06   1
##################################################################      AB06   2
#                                                                #      AB06   3
#                            AB06                                #      AB06   4
#                                                                #      AB06   5
##################################################################      AB06   6
SUBROUTINE AB06(H1,H2,T)#                                               AB06   7
IMPLICIT INTEGER (I-N)#                                                 AB06   8
#                                                                       AB06   9
## FORM A RIGHT-HANDED UNIT VECTOR TRIPLE                               AB06  10
## H.L. AMMON, SEPTEMBER 1971                                           AB06  11
## J.M. STEWART, NOVEMBER 1980                                          AB06  12
#                                                                       AB06  13
dp: H1(3)#                             First input vector               AB06  14
dp: H2(3)#                             Second input vector              AB06  15
dp: S#                                 Normalization factor             AB06  16
dp: T(3,3)#                            Vector triple (columns)          AB06  17
dp: TEMP(3)#                           Temporary storage                AB06  18
dp: TEMP1(3)#                          Temporary storage                AB06  19
#                                                                       AB06  20
S=1.0/dsqrt:(H1(1)**2+H1(2)**2+H1(3)**2)# Normalization factor          AB06  21
T(1,1)=H1(1)*S#                        First element vector 1           AB06  22
T(2,1)=H1(2)*S#                        Second                           AB06  23
T(3,1)=H1(3)*S#                        Third                            AB06  24
#                                                                       AB06  25
CALL AB01(H1,H2,TEMP)#                 Vector 3                         AB06  26
T(1,3)=TEMP(1)#                        First element                    AB06  27
T(2,3)=TEMP(2)#                        Second                           AB06  28
T(3,3)=TEMP(3)#                        Third                            AB06  29
#                                                                       AB06  30
CALL AB01(TEMP,H1,TEMP1)#              Vector 2                         AB06  31
T(1,2)=TEMP1(1)#                       First element                    AB06  32
T(2,2)=TEMP1(2)#                       Second                           AB06  33
T(3,2)=TEMP1(3)#                       Third                            AB06  34
#                                                                       AB06  35
RETURN#                                                                 AB06  36
END#                                                                    AB06  37
systemheader:(AB07)#                   Vector normalization             AB07   1
###############################################################         AB07   2
#                                                                       AB07   3
#                             AB07                                      AB07   4
#                                                                       AB07   5
###############################################################         AB07   6
SUBROUTINE AB07(V,B)#                  W. dreissig 10/85                AB07   7
IMPLICIT INTEGER (I-N)#                                                 AB07   8
#                                                                       AB07   9
# VECTOR NORMALIZATION V=V/SQRT(VX**2+VY**2+VZ**2)                      AB07  10
#                                                                       AB07  11
dp:     V(3)#                          Vector (unnormalized,n           AB07  12
dp:     B#                             Value of vector                  AB07  13
INTEGER  I#                            Index                            AB07  14
#                                                                       AB07  15
B=0.#                                                                   AB07  16
DO I=1,3; B=B+V(I)*V(I)#               Add squares                      AB07  17
B=dsqrt:(B)#                           Value                            AB07  18
DO I=1,3; V(I)=V(I)/B#                 Normalization                    AB07  19
#                                                                       AB07  20
RETURN#                                                                 AB07  21
END#                                                                    AB07  22
systemheader:(AB09)#                   Vector dot product               AB09   1
###############################################################         AB09   2
#                                                                       AB09   3
#                             AB09                                      AB09   4
#                                                                       AB09   5
###############################################################         AB09   6
SUBROUTINE AB09(V1,V2,P)#              W. dreissig 10/85                AB09   7
#                                                                       AB09   8
dp:     V1(3),V2(3),P#                 Vectors                          AB09   9
INTEGER  I#                            Index                            AB09  10
#                                                                       AB09  11
P=0.0#                                 Initialize                       AB09  12
DO I=1,3; P=P+V1(I)*V2(I)#             Add component                    AB09  13
#                                                                       AB09  14
RETURN#                                                                 AB09  15
END#                                                                    AB09  16
systemheader:(AB10)#                                                    AB10   1
################################################################        AB10   2
#                                                              #        AB10   3
#                            AB10                              #        AB10   4
#                                                              #        AB10   5
################################################################        AB10   6
SUBROUTINE AB10#                                                        AB10   7
IMPLICIT INTEGER (I-N)#                                                 AB10   8
include:(AACOMN)#                                                       AB10   9
include:(ABCOMN)#                                                       AB10  10
#                                                                       AB10  11
## J.M. STEWART, NOVEMBER 1980                                          AB10  12
## G. DAVENPORT, JANUARY 1982                                           AB10  13
## T. MASLEN, JANUARY 1984                                              AB10  14
## N. SPADACCINI, MARCH 1989 - PSI SCANS INCLUDED                       AB10  15
#                                                                       AB10  16
INTEGER I#                             Loop index                       AB10  17
INTEGER IP#                            Packet pointer                   AB10  18
INTEGER IPP#                           Pointer                          AB10  19
INTEGER IREL(4)#                       Relative pointers                AB10  20
INTEGER IWANT(3)#                      Want list for indexpkt           AB10  21
INTEGER J,K,L,MARK#                    Loop index                       AB10  22
INTEGER JP#                            Pointer                          AB10  23
INTEGER KEY(4)#                        Keys for indexpkt                AB10  24
INTEGER M#                             Start of string                  AB10  25
INTEGER N#                             End of string                    AB10  26
INTEGER PSIZE#                         Packet size                      AB10  27
INTEGER IWAV#                          Wave length index                AB10  28
INTEGER INDX#                          Table index for mua-ta           AB10  29
INTEGER IDXMU#                         Key for emu-source               AB10  30
INTEGER NSF#                           (number of atom types)           AB10  31
#                                                                       AB10  32
dp: CHI#                               Chi                              AB10  33
dp: PHI#                               Phi                              AB10  34
dp: TCTRAN(3,3)#                       Tc transpose                     AB10  35
dp: TPHI(3,3)#                         Vector triple                    AB10  36
dp: U(3,3)#                            Rotation matrix                  AB10  37
dp: VEC(10)#                           Orref vectors                    AB10  38
dp: ORT(10)#                           Orient input                     AB10  39
dp: CVOL#                                                               AB10  40
#                                                                       AB10  41
string:(ACH,108)#                      Char store for atom types        AB10  42
#                                                                       AB10  43
chardata:(AB101,"   gaussian analytical  sphericalcylindrical$#         AB10  44
     tables ")#                                                         AB10  45
chardata:(AB102, DIFF ORIENT ASTAR TBAR END )#                          AB10  46
chardata:(AB103,"AC noyes")#                                            AB10  47
chardata:(AB104," Calculation method                     ")#            AB10  48
chardata:(AB105," Apply correction to                    ")#            AB10  49
chardata:(AB106," Store Tbar on the bdf                  ")#            AB10  50
chardata:(AB107," Limit reflections to                   ")#            AB10  51
chardata:(AB108," List reflections                       ")#            AB10  52
chardata:(AB109," Dataset number                         ")#            AB10  53
chardata:(AB10A," Absorption coef /mm                    ")#            AB10  54
chardata:(AB10C," Internal precision factor              ")#            AB10  55
chardata:(AB10D," Calculate direction cosines         yes")#            AB10  56
chardata:(AB10E," Radiation wavelength                   ")#            AB10  57
chardata:(AB10F,"(input)     (calculated)(default)          ")#         AB10  58
chardata:(AB10G," Atom type   Atomic number   Atoms Cell        MUA")#  AB10  59
chardata:(AB10H," Plot crystal views in                      ")#        AB10  60
chardata:(AB10I,"projection    stereo ")#                               AB10  61
chardata:(AB10J," Rotation senses  theta/chi/phi                   ")#  AB10  62
chardata:(AB10K,"/omega/psi  ")#                                        AB10  63
chardata:(AB10L,"store irelf2rel frel ")#                               AB10  64
chardata:(AB10M," Crystal radius in mm                   ")#            AB10  65
#                                                                       AB10  66
#  MU(A) FOR CU (CM**2)  INT TAB VOL C P193-198.                        AB10  67
#                                                                       AB10  68
realdata:(TCU,#                                                         AB10  69
[.0655,.194,.576,1.66,4.15,8.99,17.3,30.4,49.8,76.8,#                   AB10  70
114.,161.,222.,297.,388.,497.,624.,772.,940.,1130.,#                    AB10  71
1350.,1590.,1850.,2130.,2460.,2800.,3140.,476.,547.,629.,#              AB10  72
719.,819.,929.,1050.,1180.,1320.,1480.,1650.,1830.,2030.,#              AB10  73
2230.,2460.,2700.,2950.,3230.,3520.,3820.,4150.,4500.,4860.,#           AB10  74
5250.,5650.,6070.,6520.,7000.,7500.,8030.,8570.,9120.,9680.,#           AB10  75
10200.,10800.,11000.,10500.,8470.,9770.,3470.,3670.,3930.,4100.,#       AB10  76
4500.,4600.,4850.,5130.,5720.,5800.,6240.,6340.,6690.,6680.,#           AB10  77
7540.,7980.,8430.,8810.,8650.,9720.,10200.,10200.,14300.,11800.,#       AB10  78
10600.,11200.])#                                                        AB10  79
#                                                                       AB10  80
#  MU(A) FOR MO (CM**2)  INT TAB VOL C P193-198.                        AB10  81
#                                                                       AB10  82
realdata:(TMO,#                                                         AB10  83
[.0624,.134,.228,.383,.661,1.15,1.96,3.25,5.15,7.86,#                   AB10  84
11.6,16.5,22.9,31.,41.,53.2,67.8,85.1,105.,129.,#                       AB10  85
156.,186.,220.,258.,302.,349.,401.,457.,518.,586.,#                     AB10  86
660.,738.,822.,911.,1000.,1100.,1210.,1320.,1430.,247.,#                AB10  87
273.,300.,332.,364.,399.,436.,476.,518.,563.,611.,#                     AB10  88
662.,716.,773.,834.,898.,965.,1040.,1110.,1190.,1270.,#                 AB10  89
1350.,1440.,1540.,1630.,1740.,1840.,1950.,2070.,2190.,2310.,#           AB10  90
2440.,2580.,2720.,2860.,3010.,3160.,3310.,3480.,3650.,3820.,#           AB10  91
4010.,4190.,4380.,4580.,4070.,3980.,3220.,3300.,5400.,3700.,#           AB10  92
3870.,4030.])#                                                          AB10  93
#                                                                       AB10  94
#  MU(A) FOR AG (CM**2)  INT TAB VOL C P193-198.                        AB10  95
#                                                                       AB10  96
realdata:(TAG,#                                                         AB10  97
[.0614,.128,.206,.313,.479,.745,1.17,1.82,2.77,4.12,#                   AB10  98
5.96,8.42,11.6,15.6,20.6,26.7,34.1,42.9,53.2,65.2,#                     AB10  99
78.9,94.7,112.,133.,155.,180.,207.,238.,271.,307.,#                     AB10 100
346.,387.,433.,482.,535.,592.,652.,715.,780.,847.,#                     AB10 101
922.,1150.,1070.,192.,210.,230.,251.,273.,297.,323.,#                   AB10 102
350.,378.,409.,441.,475.,511.,549.,588.,630.,674.,#                     AB10 103
720.,768.,819.,872.,927.,985.,1040.,1110.,1170.,1240.,#                 AB10 104
1310.,1390.,1460.,1540.,1620.,1710.,1800.,1890.,1990.,2090.,#           AB10 105
2190.,2290.,2400.,2510.,2620.,2730.,2850.,2980.,3110.,3230.,#           AB10 106
3420.,3500.])#                                                          AB10 107
realdata:(FM,[210303.,350403.,521042.])#                                AB10 108
datastuff:#                                                             AB10 109
#                                                                       AB10 110
# SET UP INPUT AND OUTPUT FILE BUFFERS                                  AB10 111
# ------------------------------------                                  AB10 112
#                                                                       AB10 113
IOMARK(1)=QXSTR#                       Start of input buffer            AB10 114
IOMARK(2)=IOMARK(1)+bdfbuf:#           Start of output buffer           AB10 115
IOMARK(5)=IOMARK(2)+bdfbuf:#           Start plot buffer                AB10 116
incrqx:(MARK,IOMARK(5)+bdfbuf:,AB1001)# Get mem for buffers             AB10 117
MARKF=0#                               Start of face array              AB10 118
#                                                                       AB10 119
# INTERPRET THE ABSORB LINE                                             AB10 120
# -------------------------                                             AB10 121
#                                                                       AB10 122
FOR(I=0;I>=0;I=I+1)#                   Loop over input options          AB10 123
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB10 124
stringlim:(I,J,K)#                     Get string limits                AB10 125
#                                                                       AB10 126
setint:(GA,METHOD,    2,    1)#        Method is gaussian general       AB10 127
setint:(AN,METHOD,    2,    2)#        Method is analytical general     AB10 128
setint:(SP,METHOD,    2,    3)#        Method is spherical              AB10 129
setint:(CY,METHOD,    2,    4)#        Method is cylindrical            AB10 130
setint:(TA,METHOD,    2,    5)#        Method is user tables            AB10 131
setint:(NA,IAPPLY,    1,    0)#        Do not apply absorb factors      AB10 132
setint:(IR,IAPPLY,    1,    1)#        Apply to absorb to irel          AB10 133
setint:(F2,IAPPLY,    1,    2)#        Apply to absorb to f2rl          AB10 134
setint:(FR,IAPPLY,    1,    3)#        Apply to absorb to frel          AB10 135
setint:(PR,LIST,      0,99999)#        Print reflections                AB10 136
setint:(LI,LIM,   99999,   10)#        Limit reflections                AB10 137
setint:(DS,ISET,      1,    1)#        Data set number                  AB10 138
#                                                                       AB10 139
setflg:(NT, ITBAR, yes:)#              Do not store tbar                AB10 140
setflg:(ST,STEREO, no:)#               Set stereo plot                  AB10 141
setflg:(DC,  ICOS, no:)#               Output d.c.'s                    AB10 142
#                                                                       AB10 143
setval:(PS, SCALE,   1.0,  1.0)#       Set precision scale              AB10 144
setval:(RA,   RAD,   0.0,  0.0)#       Set crystal radius in mm         AB10 145
setval:(MU,   EMU,   0.0,  0.0)#       Absorbtion coefficient in mm     AB10 146
#                                                                       AB10 147
ifvo:(BUFIN(I+1)) BREAK#               Exit when field void             AB10 148
ifnm:(BUFIN(I+1)) iquit:(AB1010,I+1)#  Must be char string              AB10 149
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB10 150
IF(ICOS>0) ITBAR=yes:#                 Set tbar if dir.cosines          AB10 151
IF(METHOD==3&RAD<small:) iquit:(AB1009)# Radius must be set             AB10 152
IF(METHOD==4&RAD<small:) iquit:(AB1009)# Radius must be set             AB10 153
copyfile:(1,2,1,decr:(lrcell:))#       Copy up to lrcell                AB10 154
#                                                                       AB10 155
# OUTPUT FOR USER                                                       AB10 156
# ---------------                                                       AB10 157
#                                                                       AB10 158
movechar:(AB101,METHOD*11-10,AB104,30,11,0)# Store method               AB10 159
writeline:(1,AB104,NAB104,3,3)#        Print method                     AB10 160
ncodefld:(sg:(RAD),1,AB10M,400952.,1)# Format crystal radius            AB10 161
IF(METHOD==3\METHOD==4)#               If sphere or cylinder            AB10 162
writeline:(0,AB10M,NAB10M,3,3)#        Print radius in mm               AB10 163
movechar:(AB10L,IAPPLY*5+1,AB105,36,5,0)# Store coeff                   AB10 164
writeline:(0,AB105,NAB105,3,3)#        Print coeff                      AB10 165
movechar:(AB103,ITBAR*3+3,AB106,38,3,0)# Store tbar on bdf              AB10 166
writeline:(0,AB106,NAB106,3,3)#        Print tbar on bdf                AB10 167
movechar:(AB10I,STEREO*10+1,AB10H,31,10,0)# Plot method                 AB10 168
writeline:(0,AB10H,NAB10H,3,3)#        Print plot method                AB10 169
IF(ICOS>0) writeline:(0,AB10D,NAB10D,3,3)# Store d.c.'s                 AB10 170
ncodefld:(FLOAT(LIST),1,AB108,400613.,1)# Format list reflns            AB10 171
writeline:(0,AB108,NAB108,3,3)#        Print list limit                 AB10 172
ncodefld:(FLOAT(LIM),1,AB107,400613.,1)# Format process limit           AB10 173
writeline:(0,AB107,NAB107,3,3)#        Print process limit              AB10 174
ncodefld:(FLOAT(ISET),1,AB109,400413.,1)# Format data set number        AB10 175
writeline:(0,AB109,NAB109,3,3)#        Print data set number            AB10 176
ncodefld:(sg:(SCALE),1,AB10C,400822.,1)# Format internal scale          AB10 177
writeline:(0,AB10C,NAB10C,3,3)#        Print internal scale             AB10 178
EMU=EMU*10.; DISTSC=0.1; RAD=RAD*0.1#  Convert to cm                    AB10 179
#                                                                       AB10 180
# GET THE B MATRIX FROM LRCELL:                                         AB10 181
# -----------------------------                                         AB10 182
#                                                                       AB10 183
DO K=1,10#                             Loop over lrcell packets         AB10 184
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB10 185
readwpkt:(1,lrcell:,PSIZE,IP,0)#       Point to next lrcell pkt         AB10 186
IF(IP<=0) BREAK#                       Exit after last packet           AB10 187
writepkt:(2,lrcell:,PSIZE,IPP)#                                         AB10 188
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                    AB10 189
IF(K==5)#                              Skip unless trans matrix         AB10 190
DO I=1,3;  DO J=1,3#                   Loop over columns & rows         AB10 191
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB10 192
JP=IP+3*I-3+J#                         Point into qx                    AB10 193
BX(J,I)=db:(QX(JP))#                   Store element                    AB10 194
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB10 195
IF(K==6) CVOL=db:(QX(IP+1))#           Get cell volume                  AB10 196
IF(K==8)#                              Test if inv metric tensor        AB10 197
DO I=1,9; MT(I)=db:(QX(IP+I))#         Store metric tensor              AB10 198
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB10 199
#                                                                       AB10 200
# GET ATOM TYPES FROM LRSCAT: IF NO MU GIVEN                            AB10 201
# ------------------------------------------                            AB10 202
#                                                                       AB10 203
IDXMU=0#                               Assume mu given by use           AB10 204
ifeq:(EMU,0.)#                         Is mu given by user              AB10 205
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB10 206
IDXMU=1#                               Set mu calculated                AB10 207
copyfile:(1,2,incr:(lrcell:),decr:(lrscat:))# Copy to atom type         AB10 208
IWANT(1)=1; IWANT(2)=3; IWANT(3)=11#   Select atoms/cell, at.           AB10 209
KEY(1)=1; KEY(2)=3; KEY(3)=0; KEY(4)=0# Input three items               AB10 210
indexpkt:(1,lrscat:,PSIZE,IP,2,KEY,IWANT,IREL)# Read directory          AB10 211
NSF=1; L=1#                            Set indices for atom t           AB10 212
REPEAT#                                Loop over atom types             AB10 213
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB10 214
readwpkt:(1,lrscat:,PSIZE,IP,0)#       Read packet                      AB10 215
IF(IP<=0) BREAK#                       No more packets                  AB10 216
writepkt:(2,lrscat:,PSIZE,IPP)#        Copy to output                   AB10 217
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#   Transfer to op packet            AB10 218
BUFOT(NSF+1)=QX(IP+IREL(1))#           Store no.atoms/cell              AB10 219
BUFOT(NSF  )=QX(IP+IREL(2))#           Store atomic number              AB10 220
NSF=NSF+3#                             Update output pointer            AB10 221
movertoc:(QX(IP+IREL(3)),1,ACH,L,6,0)# Store atom name                  AB10 222
L=L+6#                                 Update name pointer              AB10 223
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB10 224
IF(NSF==1) IDXMU=2#                    Set default mu                   AB10 225
copyfile:(1,2,incr:(lrscat:),decr:(lrexpl:))# Copy up to lrexpl         AB10 226
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB10 227
ELSE copyfile:(1,2,incr:(lrcell:),decr:(lrexpl:))# Copy up to l         AB10 228
#                                                                       AB10 229
# GET LAMBDA FROM LREXPL:                                               AB10 230
# -----------------------                                               AB10 231
#                                                                       AB10 232
WAV=0.71073#                           Default molybednum               AB10 233
IWANT(1)=1; IWANT(2)=2#                Request lambda                   AB10 234
KEY(1)=1; KEY(3)=0; KEY(2)=2; KEY(4)=0# Set search keys                 AB10 235
indexpkt:(1,lrexpl:,PSIZE,IP,2,KEY,IWANT,IREL)#                         AB10 236
REPEAT#                                Loop over lrexpl: packets        AB10 237
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB10 238
readwpkt:(1,lrexpl:,PSIZE,IP,0)#       Point to next pkt                AB10 239
IF(IP<=0) BREAK#                       Exit after last pkt              AB10 240
writepkt:(2,lrexpl:,PSIZE,IPP)#                                         AB10 241
movereal:(QX,IP+1,QX,IPP+1,PSIZE,1)#                                    AB10 242
IPP=IP+IREL(1)#                        Point to data set number         AB10 243
IF(ISET!=NINT(QX(IPP))) NEXT#          Skip if wrong dset               AB10 244
IF(IREL(2)==0) NEXT#                   Skip if lambda not present       AB10 245
WAV=db:(QX(IP+IREL(2)))#               Lambda                           AB10 246
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB10 247
ncodefld:(sg:(WAV),1,AB10E,400952.,1)# Format wavelength                AB10 248
writeline:(0,AB10E,NAB10E,3,3)#        Print radiation wavelength       AB10 249
ISET=ISET*1000#                                                         AB10 250
#                                                                       AB10 251
#                                                                       AB10 252
IF(IDXMU==1)#                          Is mu to be calculated           AB10 253
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB10 254
IWAV=0#                                Default no common wave           AB10 255
IF(dabs:(WAV-1.5418D0)<.01) IWAV=1#    Is it near cu                    AB10 256
IF(dabs:(WAV-0.7106D0)<.01) IWAV=2#    Is it near mo                    AB10 257
IF(IWAV==0) IDXMU=2#                   Set default mu                   AB10 258
ELSE#                                                                   AB10 259
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB10 260
DO I=1,NSF-3,3#                        Loop over atom types             AB10 261
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB10 262
INDX=NINT(BUFOT(I))#                   Index for mua table=at           AB10 263
IF(IWAV==1) BUFOT(I+2)=TCU(INDX)#      Fetch mua from table             AB10 264
IF(IWAV==2) BUFOT(I+2)=TMO(INDX)#      Fetch mua from table             AB10 265
IF(BUFOT(I+2)<=small:) IDXMU=2#        If mua not given, set            AB10 266
EMU=EMU+db:(BUFOT(I+1)*BUFOT(I+2))#    Add up absorption coef           AB10 267
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB10 268
EMU=EMU*10.D00/CVOL#                   Adjust mu                        AB10 269
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB10 270
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB10 271
IF(IDXMU==2) EMU=10.#                  Set default mu                   AB10 272
movechar:(AB10F,IDXMU*12+1,AB10A,22,12,0)# Update abs.coeff.header      AB10 273
ncodefld:(sg:(EMU)/10.,1,AB10A,400622.,1)# Insert mu in /mm             AB10 274
writeline:(0,AB10A,NAB10A,3,3)#                                         AB10 275
DEMU=0.001*EMU#                        Arbitrary value for delta        AB10 276
EMU2=EMU+DEMU#                         Second emu value                 AB10 277
EMU=EMU/SCALE#                         Scale down emu                   AB10 278
EMU2=EMU2/SCALE#                       And emu2                         AB10 279
#                                                                       AB10 280
IF(IDXMU==1)#                                                           AB10 281
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB10 282
writeline:(1,AB10G,NAB10G,3,3)#        Output header                    AB10 283
L=1#                                   Atom type index                  AB10 284
DO I=1,NSF-3,3#                        Loop over atom types             AB10 285
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB10 286
movechar:(ACH,L,CHROT,5,6,0)#          Atom type to output li           AB10 287
L=L+6#                                 Update atom type index           AB10 288
ncodefld:(BUFOT,I,CHROT,FM,3,0)#       Encode atomic number,            AB10 289
writeline:(0,0,0,1,3)#                 Output atom type line            AB10 290
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB10 291
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB10 292
#                                                                       AB10 293
copyfile:(1,2,incr:(lrexpl:),decr:(lrrefl:))# Copy up to lrrefl:        AB10 294
#                                                                       AB10 295
# READ DIFF AND ORIENT LINES                                            AB10 296
# -------------------------                                             AB10 297
#                                                                       AB10 298
OMIN=-.837758 ; OMAX=1.01229#          Default omega limits - syntex    AB10 299
IF(METHOD<3)#                          For gaussian and analytical      AB10 300
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AB10 301
PHI90=0.0#                             Default 0.0                      AB10 302
DO J=1,5; ROT(J)=1.0#                  Default clockwise                AB10 303
DO K=1,2#                              Loop over input lines            AB10 304
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AB10 305
readline:(AB102,NAB102)#               Read control line                AB10 306
#                                                                       AB10 307
# DIFF LINE                                                             AB10 308
# ---------                                                             AB10 309
#                                                                       AB10 310
IF(LINID==1)#                          Diff line                        AB10 311
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AB10 312
DO J=1,3#                              Loop over compulsory fields 1-   AB10 313
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AB10 314
ifnm:(BUFIN(J)) iquit:(AB1002)#        Character field                  AB10 315
stringlim:(J,M,N)#                     Get string limits                AB10 316
compchar:(CHRIN,M,AB103,1,1,1,2,N)#    Identify string                  AB10 317
IF(N==0) iquit:(AB1002)#               Default clockwise                AB10 318
IF(N==1) ROT(J)=-1.0#                  Store rotation element           AB10 319
movechar:(CHRIN,M,AB10J,40+(J-1)*2,1,0)# Store in op header             AB10 320
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AB10 321
DO J=4,5#                                                               AB10 322
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AB10 323
ifvo:(BUFIN(J)) NEXT#                  If not there - next              AB10 324
movechar:(AB10K,1+(J-4)*6,AB10J,32+(J-4)*6,6,0)# Title                  AB10 325
stringlim:(J,M,N)#                     Get string limits                AB10 326
compchar:(CHRIN,M,AB103,1,1,1,2,N)#    Identify string                  AB10 327
IF(N==0) iquit:(AB1002)#               Default clockwise                AB10 328
IF(N==1) ROT(J)=-1#                                                     AB10 329
movechar:(AB103,N,AB10J,40+(J-1)*2,1,0)#                                AB10 330
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AB10 331
writeline:(0,AB10J,NAB10J,3,3)#                                         AB10 332
ifnm:(BUFIN(6)) PHI90=db:(BUFIN(6)*.017453)# Phi at chi=90              AB10 333
ELSE            PHI90=0.0#             Default 0.0                      AB10 334
ifnm:(BUFIN(7)) OMIN=db:(BUFIN(7)/57.29578)# Omega min in radians       AB10 335
ifnm:(BUFIN(8)) OMAX=db:(BUFIN(8)/57.29578)# Omega max in radians       AB10 336
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AB10 337
#                                                                       AB10 338
# ORIENT LINE                                                           AB10 339
# -----------                                                           AB10 340
#                                                                       AB10 341
ELSE IF(LINID==2)#                     Orient line                      AB10 342
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AB10 343
DO I=1,10#                             Loop over fields                 AB10 344
IF(BUFIN(I)<=voidflg:) iquit:(AB1003)# Non-numeric, so quit             AB10 345
DO I=1,10; ORT(I)=db:(BUFIN(I))#       Store as dp                      AB10 346
postm:(BX,ORT,VEC)#                    B x hkl                          AB10 347
postm:(BX,ORT(6),VEC(4))#              B x hkl                          AB10 348
triple:(VEC,VEC(4),TC)#                Form vector triple               AB10 349
#                                                                       AB10 350
CHI=0.01745329D00*ORT(4)#              Chi                              AB10 351
PHI=0.01745329D00*ORT(5)#              Phi                              AB10 352
VEC(1)=ROT(1)*dcos:(CHI)*dcos:(PHI)#   Element 1                        AB10 353
VEC(2)=ROT(1)*ROT(3)*dcos:(CHI)*dsin:(PHI)# Element 2                   AB10 354
VEC(3)=ROT(1)*ROT(2)*dsin:(CHI)#       Element 3                        AB10 355
CHI=0.01745329D00*ORT(9)#              Chi                              AB10 356
PHI=0.01745329D00*ORT(10)#             Phi                              AB10 357
VEC(4)=ROT(1)*dcos:(CHI)*dcos:(PHI)#   Element 1                        AB10 358
VEC(5)=ROT(1)*ROT(3)*dcos:(CHI)*dsin:(PHI)# Element 2                   AB10 359
VEC(6)=ROT(1)*ROT(2)*dsin:(CHI)#       Element 3                        AB10 360
#                                                                       AB10 361
triple:(VEC,VEC(4),TPHI)#              Form vector triple               AB10 362
#                                                                       AB10 363
DO I=1,3; DO J=1,3; TCTRAN(I,J)=TC(J,I)#                                AB10 364
mmult:(TPHI,TCTRAN,U)#                 Matrix multiply                  AB10 365
#                                                                       AB10 366
DO I=1,3; DO J=1,3; UINV(I,J)=U(J,I)#                                   AB10 367
mmult:(U,BX,UB)#                       Matrix multiply                  AB10 368
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AB10 369
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AB10 370
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AB10 371
ELSE IF(METHOD==3) CALL AB24#          Spherical correction             AB10 372
ELSE IF(METHOD==4) CALL AB25#          Cylindrical correction           AB10 373
ELSE#                                  User supplied tables             AB10 374
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AB10 375
NAST=0;  NTBR=0#                       Initialize count                 AB10 376
DO I=1,30; ASTAR(I)=0.#                Zero astar table                 AB10 377
DO I=1,30;  TBAR(I)=0.#                Zero tbar table                  AB10 378
REPEAT#                                Loop over astar/tbar lines       AB10 379
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AB10 380
readline:(AB102,NAB102)#               Read the next line               AB10 381
IF(LINID==5) BREAK#                    Exit when end                    AB10 382
IF(LINID==3)#                          Tables of astar                  AB10 383
DO J=1,30#                             Loop over entries                AB10 384
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AB10 385
ifvo:(BUFIN(J))BREAK#                  Exit after last item read        AB10 386
NAST=NAST+1#                           Count entries (astar)            AB10 387
ASTAR(NAST)=db:(BUFIN(J))#             Store entries in astar           AB10 388
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AB10 389
IF(LINID==4)#                          Tables of tbar                   AB10 390
DO J=1,30#                             Loop over entries                AB10 391
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AB10 392
ifvo:(BUFIN(J))BREAK#                  Exit after last item read        AB10 393
NTBR=NTBR+1#                           Count entries (tbar)             AB10 394
TBAR(NTBR)=db:(BUFIN(J))#              Store entries in tbar            AB10 395
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AB10 396
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AB10 397
CALL AB18#                             Apply spherical correction       AB10 398
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AB10 399
#                                                                       AB10 400
RETURN#                                                                 AB10 401
END#                                                                    AB10 402
systemheader:(AB11)#                   Read the crystal face lines      AB11   1
####################################################################    AB11   2
#                                                                  #    AB11   3
#                             AB11                                 #    AB11   4
#                                                                  #    AB11   5
####################################################################    AB11   6
SUBROUTINE AB11#                                                        AB11   7
IMPLICIT INTEGER (I-N)#                                                 AB11   8
include:(AACOMN)#                                                       AB11   9
include:(ABCOMN)#                                                       AB11  10
#                                                                       AB11  11
## GEORGE DAVENPORT, JANUARY 1982                                       AB11  12
## T. MASLEN, JANUARY 1984                                              AB11  13
#                                                                       AB11  14
dp: CHI#                               Chi                              AB11  15
INTEGER LINID1#                        Method of specifying faces       AB11  16
INTEGER I,II,JJ,KK#                    Pointer into face storage        AB11  17
dp: OME#                               Omega                            AB11  18
dp: PHI#                               Phi                              AB11  19
dp: SU,SCL#                            Distance                         AB11  20
dp: ORT(4)#                            Local buffer                     AB11  21
dp: UPRIME(3,3)#                       Rotation matrix                  AB11  22
dp: VEC(6)#                            Face normals                     AB11  23
dp: VFACE(3,3)#                        Vector triple                    AB11  24
dp: VT(3,3)#                           Transpose of vface               AB11  25
dp: DIMAX#                                                              AB11  26
#                                                                       AB11  27
chardata:(AB111, ORFACE FACEPT FACEML FACE FACEOM GRID END )#           AB11  28
chardata:(AB112," Equations of planes as Ax+By+Cz=D in crystal system")#AB11  29
chardata:(AB113,               $#                                       AB11  30
A           B           C           D     )#                            AB11  31
chardata:(AB114," Recommended precision scale               ")#         AB11  32
realdata:(FM,[161042.,281042.,401042.,521042.])#                        AB11  33
datastuff:#                                                             AB11  34
#                                                                       AB11  35
DIMAX=0.0#                             Initialize crystal length        AB11  36
readline:(AB111,NAB111)#               Call for face data               AB11  37
LINID1=LINID#                          Memorize the option used         AB11  38
#                                                                       AB11  39
# ORFACE LINE.  SET LINID1 TO RECOGNIZE FACEPT LINES.                   AB11  40
# -------------------------------------------------                     AB11  41
#                                                                       AB11  42
IF(LINID==1)#                          Orface line                      AB11  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB11  44
CALL AB12(VEC,SU)#                     Determine face normal            AB11  45
readline:(AB111,NAB111)#               Call for second orface line      AB11  46
CALL AB12(VEC(4),SU)#                  Determine face normal            AB11  47
triple:(VEC,VEC(4),VFACE)#             Form a vector triple             AB11  48
DO II=1,3; DO JJ=1,3; VT(II,JJ)=VFACE(JJ,II)#                           AB11  49
mmult:(TC,VT,UPRIME)#                  Uprime by matrix multiply        AB11  50
LINID1=LINID1+1#                       Flag for facept lines            AB11  51
readline:(AB111,NAB111)#               Get first line of face data      AB11  52
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB11  53
#                                                                       AB11  54
ISTOPF=MARKF#                          End of face data storage         AB11  55
KK=MARKF#                              Set pointer                      AB11  56
NSTEPF=9#                              Store nine items per face        AB11  57
NFACES=0#                              Initialize count of faces        AB11  58
writeline:(1,AB112,NAB112,3,3)#        Write a heading                  AB11  59
writeline:(0,AB113,NAB113,3,3)#        Write heading                    AB11  60
#                                                                       AB11  61
WHILE(LINID==LINID1)#                  Loop over face descriptions      AB11  62
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB11  63
NFACES=NFACES+1#                       Increment counter                AB11  64
incrqq:(ISTOPF,ISTOPF+NSTEPF,AB1101)#  Get memory                       AB11  65
#                                                                       AB11  66
# FACEPT LINE                                                           AB11  67
# -----------                                                           AB11  68
#                                                                       AB11  69
IF(LINID==2)#                          Using facept lines               AB11  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB11  71
CALL AB12(VEC,SU)#                     Determine face normal            AB11  72
QQ(KK+1)=UPRIME(1,1)*VEC(1)+UPRIME(1,2)*VEC(2)+UPRIME(1,3)*VEC(3)#      AB11  73
QQ(KK+2)=UPRIME(2,1)*VEC(1)+UPRIME(2,2)*VEC(2)+UPRIME(2,3)*VEC(3)#      AB11  74
QQ(KK+3)=UPRIME(3,1)*VEC(1)+UPRIME(3,2)*VEC(2)+UPRIME(3,3)*VEC(3)#      AB11  75
QQ(KK+4)=SU*dsqrt:(QQ(KK+1)**2+QQ(KK+2)**2+QQ(KK+3)**2)#                AB11  76
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB11  77
#                                                                       AB11  78
# FACEML LINE                                                           AB11  79
# -----------                                                           AB11  80
#                                                                       AB11  81
ELSE IF(LINID==3)#                     Using faceml lines               AB11  82
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB11  83
DO I=1,4; ORT(I)=db:(BUFIN(I))#        Store as dp                      AB11  84
QQ(KK+1)=BX(1,1)*ORT(1)+BX(1,2)*ORT(2)+BX(1,3)*ORT(3)#                  AB11  85
QQ(KK+2)=BX(2,1)*ORT(1)+BX(2,2)*ORT(2)+BX(2,3)*ORT(3)#                  AB11  86
QQ(KK+3)=BX(3,1)*ORT(1)+BX(3,2)*ORT(2)+BX(3,3)*ORT(3)#                  AB11  87
QQ(KK+4)=dsqrt:(QQ(KK+1)**2+QQ(KK+2)**2+QQ(KK+3)**2)$#                  AB11  88
        *SCALE*DISTSC*ORT(4)#                                           AB11  89
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB11  90
#                                                                       AB11  91
# FACE LINE                                                             AB11  92
# ---------                                                             AB11  93
#                                                                       AB11  94
ELSE IF(LINID==4)#                     Using face lines                 AB11  95
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB11  96
DO I=1,3; ORT(I)=db:(BUFIN(I))#        Store as dp                      AB11  97
CHI=0.01745329D00*ORT(1)#              Get chi                          AB11  98
PHI=0.01745329D00*ORT(2)#              Get phi                          AB11  99
VEC(1)=ROT(1)*dcos:(CHI)*dcos:(PHI)#   Determine face normal            AB11 100
VEC(2)=ROT(1)*ROT(3)*dcos:(CHI)*dsin:(PHI)#                             AB11 101
VEC(3)=ROT(1)*ROT(2)*dsin:(CHI)##                                       AB11 102
QQ(KK+1)=UINV(1,1)*VEC(1)+UINV(1,2)*VEC(2)+UINV(1,3)*VEC(3)#            AB11 103
QQ(KK+2)=UINV(2,1)*VEC(1)+UINV(2,2)*VEC(2)+UINV(2,3)*VEC(3)#            AB11 104
QQ(KK+3)=UINV(3,1)*VEC(1)+UINV(3,2)*VEC(2)+UINV(3,3)*VEC(3)#            AB11 105
QQ(KK+4)=dsqrt:(QQ(KK+1)**2+QQ(KK+2)**2+QQ(KK+3)**2)$#                  AB11 106
        *SCALE*DISTSC*ORT(3)#                                           AB11 107
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB11 108
#                                                                       AB11 109
# FACEOM LINE                                                           AB11 110
# -----------                                                           AB11 111
#                                                                       AB11 112
ELSE IF(LINID==5)#                     Use omega angle as well          AB11 113
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB11 114
DO I=1,4; ORT(I)=db:(BUFIN(I))#        Store as dp                      AB11 115
CHI=0.01745329D00*ORT(1)#              Get chi                          AB11 116
PHI=0.01745329D00*ORT(2)#              Get phi                          AB11 117
OME=0.01745329D00*ORT(3)#              Get omega                        AB11 118
VEC(1)=-dcos:(CHI)*dcos:(PHI)*dcos:(OME)+$#                             AB11 119
        ROT(3)*ROT(4)*dsin:(PHI)*dsin:(OME)# Face normal                AB11 120
VEC(2)=-ROT(3)*dcos:(CHI)*dsin:(PHI)*dcos:(OME)-$#                      AB11 121
        ROT(4)*dsin:(OME)*dcos:(PHI)#                                   AB11 122
VEC(3)=-ROT(2)*dsin:(CHI)*dcos:(OME)##                                  AB11 123
QQ(KK+1)=UINV(1,1)*VEC(1)+UINV(1,2)*VEC(2)+UINV(1,3)*VEC(3)#            AB11 124
QQ(KK+2)=UINV(2,1)*VEC(1)+UINV(2,2)*VEC(2)+UINV(2,3)*VEC(3)#            AB11 125
QQ(KK+3)=UINV(3,1)*VEC(1)+UINV(3,2)*VEC(2)+UINV(3,3)*VEC(3)#            AB11 126
QQ(KK+4)=dsqrt:(QQ(KK+1)**2+QQ(KK+2)**2+QQ(KK+3)**2)$#                  AB11 127
        *SCALE*DISTSC*ORT(4)#                                           AB11 128
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB11 129
#                                                                       AB11 130
ncodefld:(FLOAT(NFACES),1,CHROT,40313.,1)#                              AB11 131
DO I=1,4; BUFOT(I)=sg:(QQ(KK+I))#                                       AB11 132
ncodefld:(BUFOT,1,CHROT,FM,4)#                                          AB11 133
writeline:(0,0,0,1,3)#                 Write line                       AB11 134
#                                                                       AB11 135
DIMAX=MAX(DIMAX,QQ(KK+4))#             Adjust crystal length            AB11 136
KK=ISTOPF#                             Increment pointer                AB11 137
readline:(AB111,NAB111)#               Call for next line               AB11 138
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB11 139
IF(NFACES<4) iquit:(AB1102)#           Illegal crystal shape            AB11 140
ISTOPF=KK#                             Save end of faces                AB11 141
#                                                                       AB11 142
#    ESTIMATE PRECISION SCALE                                           AB11 143
#    ------------------------                                           AB11 144
#                                                                       AB11 145
IF(METHOD==1&DIMAX>0.2)#               Gaussian scale too large         AB11 146
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB11 147
SCL=SCALE*0.1/DIMAX#                   Improved scale                   AB11 148
ncodefld:(sg:(SCL),1,AB114,420822.,1)#                                  AB11 149
writeline:(1,AB114,NAB114,3,3)#                                         AB11 150
iquit:(AB1103)#                        A                                AB11 151
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB11 152
ELSE IF(METHOD==2&DIMAX<0.005)#        Analytical scale too small       AB11 153
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB11 154
SCL=SCALE*10./DIMAX#                   Recommended scale                AB11 155
ncodefld:(sg:(SCL),1,AB114,420822.,1)#                                  AB11 156
writeline:(0,AB114,NAB114,3,3)#                                         AB11 157
iquit:(AB1104)#                                                         AB11 158
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB11 159
DITES=MAX(DIMAX*tiny:,dsmall:)#        Precision limit                  AB11 160
DBTES=DITES*DITES#                     Double precision limit           AB11 161
#                                                                       AB11 162
LINID=LINID-5#                                                          AB11 163
#                                                                       AB11 164
RETURN#                                                                 AB11 165
END#                                                                    AB11 166
systemheader:(AB12)## determine face normal for facept line             AB12   1
####################################################################    AB12   2
#                                                                  #    AB12   3
#                             AB12                                 #    AB12   4
#                                                                  #    AB12   5
####################################################################    AB12   6
SUBROUTINE AB12(FACEN,SU)#             G davenport 1982                 AB12   7
IMPLICIT INTEGER (I-N)#                                                 AB12   8
include:(AACOMN)#                                                       AB12   9
include:(ABCOMN)#                                                       AB12  10
#                                                                       AB12  11
INTEGER I#                             Loop index                       AB12  12
dp: FACEN(3)#                          Normal to face                   AB12  13
dp: FACEOR(9)#                         Three points on face             AB12  14
dp: SU#                                Distance                         AB12  15
dp: V1(3)#                             Vector on the face               AB12  16
dp: V2(3)#                             Vector on the face               AB12  17
#                                                                       AB12  18
# GET THREE POINTS ON FACE                                              AB12  19
# ------------------------                                              AB12  20
#                                                                       AB12  21
DO I=1,9#                              Read three points                AB12  22
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB12  23
ifnm:(BUFIN(I))#                       Numeric field                    AB12  24
FACEOR(I)=db:(BUFIN(I))*DISTSC*SCALE#  Get coordinate                   AB12  25
ELSE iquit:(AB1201)#                   Quit if not numeric              AB12  26
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB12  27
#                                                                       AB12  28
# FORM TWO VECTORS ON FACE                                              AB12  29
# ------------------------                                              AB12  30
#                                                                       AB12  31
DO I=1,3#                              Loop over coordinates            AB12  32
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB12  33
V1(I)=FACEOR(I+3)-FACEOR(I)#           First vector                     AB12  34
V2(I)=FACEOR(I+6)-FACEOR(I)#           Second vector                    AB12  35
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB12  36
CALL AB01(V1,V2,FACEN)#                Form face normal                 AB12  37
#                                                                       AB12  38
# FIND DISTANCE FROM ORIGIN TO FACE                                     AB12  39
# ---------------------------------                                     AB12  40
#                                                                       AB12  41
SU=FACEOR(1)*FACEN(1)+FACEOR(2)*FACEN(2)+FACEOR(3)*FACEN(3)#            AB12  42
IF(SU<0.0)#                            If negative distance             AB12  43
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB12  44
SU=-SU#                                Make positive                    AB12  45
FACEN(1)=-FACEN(1)#                    Switch sign of x                 AB12  46
FACEN(2)=-FACEN(2)#                    Switch sign of y                 AB12  47
FACEN(3)=-FACEN(3)#                    Switch sign of z                 AB12  48
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB12  49
RETURN#                                                                 AB12  50
END#                                                                    AB12  51
systemheader:(AB13)#                   Determine the crystal geometry   AB13   1
####################################################################    AB13   2
#                                                                  #    AB13   3
#                              AB13                                #    AB13   4
#                                                                  #    AB13   5
####################################################################    AB13   6
SUBROUTINE AB13#                       G. davenport 1882                AB13   7
IMPLICIT INTEGER (I-N)#                                                 AB13   8
include:(AACOMN)#                                                       AB13   9
include:(ABCOMN)#                                                       AB13  10
#                                                                       AB13  11
dp: FACE(2)#                           Face shared by 2 vertices        AB13  12
REAL FMT#                              Output format                    AB13  13
INTEGER I#                             Loop index                       AB13  14
INTEGER ICOUNT#                        Common face between vertices     AB13  15
INTEGER IDECR#                         Redundant vertices*nstepv        AB13  16
INTEGER IEULER#                        Test of euler's rule             AB13  17
INTEGER III#                           Loop index                       AB13  18
INTEGER IP,KP#                         Pointer                          AB13  19
INTEGER IP1#                           Pointer                          AB13  20
INTEGER IP2#                           Pointer                          AB13  21
INTEGER ISON#                          Flag point on crystal            AB13  22
INTEGER J#                             Loop index                       AB13  23
INTEGER JP#                            Pointer                          AB13  24
INTEGER JP1#                           Pointer                          AB13  25
INTEGER JP2#                           Pointer                          AB13  26
INTEGER K#                             Loop index                       AB13  27
INTEGER KK#                            Pointer                          AB13  28
INTEGER L#                             Loop index                       AB13  29
INTEGER M#                             Loop index                       AB13  30
INTEGER MARKFL#                        Start of face lists              AB13  31
INTEGER NUFACE#                        Flag face for list               AB13  32
INTEGER NWORDS#                        Length of face lists             AB13  33
INTEGER STOPFL#                        End of face lists                AB13  34
INTEGER I1,I2#                         Index to faces 1,2               AB13  35
dp: T1#                                Temporary                        AB13  36
dp: T2#                                Temporary                        AB13  37
dp: T3#                                Temporary                        AB13  38
dp: T4,T5,T6,XMA,YMA,ZMA,XMI,YMI,ZMI#  Projection plot junk             AB13  39
dp: PROJSC#                            Projection plot scale            AB13  40
dp: XYZ(3)#                            Intersection of three faces      AB13  41
#                                                                       AB13  42
chardata:(AB131," Coordinates of vertices in crystal system")#          AB13  43
chardata:(AB132,"               $#                                      AB13  44
x           y           z             faces   ")#                       AB13  45
chardata:(AB133," $#                                                    AB13  46
edge          between faces...        between vertices...      ")#      AB13  47
chardata:(AB134," Face     is improperly described")#                   AB13  48
realdata:(FM,[40313.,161052.,281052.,401052.])#                         AB13  49
realdata:(FM1,[40413.,150813.,270813.,390813.,510813.])#                AB13  50
datastuff:#                                                             AB13  51
#                                                                       AB13  52
XMAX=-9999.D00#                        Initial max x                    AB13  53
XMIN= 9999.D00#                        Initial min x                    AB13  54
YMAX=-9999.D00#                        Initial max y                    AB13  55
YMIN= 9999.D00#                        Initial min y                    AB13  56
ZMAX=-9999.D00#                        Initial max z                    AB13  57
ZMIN= 9999.D00#                        Initial min z                    AB13  58
FOR(I=MARKF;I<ISTOPF;I=I+NSTEPF)#      Loop over faces                  AB13  59
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB13  60
QQ(I+6)=0.0#                           Vertices around face             AB13  61
QQ(I+7)=0.0#                           Mean x of those                  AB13  62
QQ(I+8)=0.0#                           Mean y of those                  AB13  63
QQ(I+9)=0.0#                           Mean z of those                  AB13  64
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB13  65
NVERTS=0#                              Count vertices                   AB13  66
NSTEPV=8#                              8 items per vertex               AB13  67
MARKV=ISTOPF#                          Start of vertex storage          AB13  68
KK=MARKV#                              Pointer                          AB13  69
ISTOPV=MARKV#                          End of vertex storage            AB13  70
#                                                                       AB13  71
# FIND INTERSECTIONS OF THREE FACES, TEST FOR VERTICES                  AB13  72
# ----------------------------------------------------                  AB13  73
#                                                                       AB13  74
FOR(K=MARKF;K<(ISTOPF-2*NSTEPF);K=K+NSTEPF)# Pick the first face        AB13  75
FOR(L=(K+NSTEPF);L<(ISTOPF-NSTEPF);L=L+NSTEPF)# Second face             AB13  76
FOR(M=(L+NSTEPF);M<ISTOPF;M=M+NSTEPF)# Find third face                  AB13  77
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB13  78
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),XYZ,ISON)# Find point and test        AB13  79
IF(ISON==yes:)#                        If so                            AB13  80
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB13  81
incrqq:(ISTOPV,ISTOPV+NSTEPV,AB1301)#  Get more memory                  AB13  82
QQ(KK+1)=XYZ(1)#                       Store x coordinate               AB13  83
QQ(KK+2)=XYZ(2)#                       Store y coordinate               AB13  84
QQ(KK+3)=XYZ(3)#                       Store z coordinate               AB13  85
QQ(KK+4)=db:(FLOAT(K))#                Store first face                 AB13  86
QQ(KK+5)=db:(FLOAT(L))#                Store second face                AB13  87
QQ(KK+6)=db:(FLOAT(M))#                Store third face                 AB13  88
NVERTS=NVERTS+1#                       Increment vertex count           AB13  89
KK=ISTOPV#                             Increment pointer                AB13  90
IF(XYZ(1)<XMIN) XMIN=XYZ(1)#           Min x                            AB13  91
IF(XYZ(2)<YMIN) YMIN=XYZ(2)#           Min y                            AB13  92
IF(XYZ(3)<ZMIN) ZMIN=XYZ(3)#           Min z                            AB13  93
IF(XYZ(1)>XMAX) XMAX=XYZ(1)#           Max x                            AB13  94
IF(XYZ(2)>YMAX) YMAX=XYZ(2)#           Max y                            AB13  95
IF(XYZ(3)>ZMAX) ZMAX=XYZ(3)#           Max z                            AB13  96
QQ(K+6)=QQ(K+6)+1.0#                   Count vertices around face       AB13  97
QQ(K+7)=QQ(K+7)+XYZ(1)#                Average x                        AB13  98
QQ(K+8)=QQ(K+8)+XYZ(2)#                Average y                        AB13  99
QQ(K+9)=QQ(K+9)+XYZ(3)#                Average z                        AB13 100
#                                                                       AB13 101
QQ(L+6)=QQ(L+6)+1.0#                   Count vertices around face       AB13 102
QQ(L+7)=QQ(L+7)+XYZ(1)#                Average x                        AB13 103
QQ(L+8)=QQ(L+8)+XYZ(2)#                Average y                        AB13 104
QQ(L+9)=QQ(L+9)+XYZ(3)#                Average z                        AB13 105
#                                                                       AB13 106
QQ(M+6)=QQ(M+6)+1.0#                   Count vertices around face       AB13 107
QQ(M+7)=QQ(M+7)+XYZ(1)#                Average x                        AB13 108
QQ(M+8)=QQ(M+8)+XYZ(2)#                Average y                        AB13 109
QQ(M+9)=QQ(M+9)+XYZ(3)#                Average z                        AB13 110
#                                                                       AB13 111
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB13 112
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB13 113
FOR(I=MARKF;I<ISTOPF;I=I+NSTEPF)#      Loop over faces                  AB13 114
IF(QQ(I+6)>2.0)#                       Enough vertices around face      AB13 115
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB13 116
QQ(I+7)=QQ(I+7)/QQ(I+6)#               Average x                        AB13 117
QQ(I+8)=QQ(I+8)/QQ(I+6)#               Average y                        AB13 118
QQ(I+9)=QQ(I+9)/QQ(I+6)#               Average z                        AB13 119
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB13 120
ELSE#                                  Bad description                  AB13 121
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB13 122
BUFOT(1)=FLOAT((I-MARKF)/NSTEPF)+1.0#  Face number                      AB13 123
ncodefld:(BUFOT,1,AB134,90313.,1)#     Encode it                        AB13 124
writeline:(1,AB134,NAB134,3,3)#        Write message                    AB13 125
iquit:(AB1304)#                        And quit                         AB13 126
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB13 127
#                                                                       AB13 128
# DELETE REDUNDANT VERTICES                                             AB13 129
# -------------------------                                             AB13 130
#                                                                       AB13 131
MARKFL=ISTOPV#                         Set start of face lists          AB13 132
I=MARKV#                               Set pointer                      AB13 133
incrqq:(STOPFL,MARKFL+NFACES*NVERTS,AB1302)# Get memory                 AB13 134
WHILE(I<ISTOPV)#                       Loop over all vertices           AB13 135
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB13 136
IF(I==MARKV) QQ(I+7)=db:(FLOAT(MARKFL+1))# First vertex                 AB13 137
ELSE         QQ(I+7)=QQ(I)+1.0#        Other vertex                     AB13 138
IP=NINT(sg:(QQ(I+7)))#                 Next space in face list          AB13 139
QQ(IP)=QQ(I+4)#                        First face of vertex             AB13 140
QQ(IP+1)=QQ(I+5)#                      Second face of vertex            AB13 141
QQ(IP+2)=QQ(I+6)#                      Third face of vertex             AB13 142
JP=IP+3#                               Add to pointer                   AB13 143
IF(I<(ISTOPV-NSTEPV))#                 If not on last vertex            AB13 144
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB13 145
J=I+NSTEPV#                            Start of rest of vertices        AB13 146
WHILE(J<ISTOPV)#                       Loop over rest of vertices       AB13 147
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB13 148
T1=dabs:(QQ(I+1)-QQ(J+1))#             Difference in x coords           AB13 149
T2=dabs:(QQ(I+2)-QQ(J+2))#             Difference in y coords           AB13 150
T3=dabs:(QQ(I+3)-QQ(J+3))#             Difference in z coords           AB13 151
IF((T1+T2+T3)<DITES)#                  A redundant vertex               AB13 152
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB13 153
FOR(L=J+4;L<=(J+6);L=L+1)#             Loop over its faces              AB13 154
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB13 155
NUFACE=yes:#                           Assume new face                  AB13 156
FOR(K=IP;K<JP;K=K+1)#                  Loop over face list              AB13 157
IF(NINT(sg:(QQ(K)))==NINT(sg:(QQ(L)))) NUFACE=no:# Check if already t   AB13 158
IF(NUFACE==yes:)#                      Add new face to list             AB13 159
$(#                                    5>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB13 160
QQ(JP)=QQ(L)#                          Store address                    AB13 161
JP=JP+1#                               Increment pointer                AB13 162
$)#                                    5<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB13 163
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB13 164
movedoub:(QQ,J+9,QQ,J+1,(ISTOPV-J-8),1)# Delete redundant vertex        AB13 165
ISTOPV=ISTOPV-NSTEPV#                  Change end of markv array        AB13 166
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB13 167
ELSE J=J+NSTEPV#                       Next vertex                      AB13 168
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB13 169
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB13 170
QQ(I+8)=db:(FLOAT(JP-1))#              End of current face list         AB13 171
I=I+NSTEPV#                            Next vertex                      AB13 172
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB13 173
NWORDS=JP-MARKFL-1#                    Total size of face lists         AB13 174
IDECR=MARKFL-ISTOPV#                   Number of words deleted          AB13 175
movedoub:(QQ,MARKFL+1,QQ,ISTOPV+1,NWORDS,1)# Move face lists            AB13 176
writeline:(1,AB131,NAB131,3,3)#        Write heading                    AB13 177
writeline:(0,AB132,NAB132,3,3)#        Write heading                    AB13 178
XMA=0.;   YMA=0.;   ZMA=0.#            Start max coords                 AB13 179
XMI=999.D00; YMI=999.D00; ZMI=999.D00# Start min coords                 AB13 180
#                                                                       AB13 181
FOR(I=MARKV;I<ISTOPV;I=I+NSTEPV)#      Loop over vertices               AB13 182
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AB13 183
XMA=MAX(XMA,QQ(I+1))#                  Get max coordinate               AB13 184
YMA=MAX(YMA,QQ(I+2))#                  Get max coordinate               AB13 185
ZMA=MAX(ZMA,QQ(I+3))#                  Get max coordinate               AB13 186
XMI=MIN(XMI,QQ(I+1))#                  Get min coordinate               AB13 187
YMI=MIN(YMI,QQ(I+2))#                  Get min coordinate               AB13 188
ZMI=MIN(ZMI,QQ(I+3))#                  Get min coordinate               AB13 189
QQ(I+7)=QQ(I+7)-db:(FLOAT(IDECR))#     Adjust face list pointer         AB13 190
QQ(I+8)=QQ(I+8)-db:(FLOAT(IDECR))#     Adjust face list pointer         AB13 191
BUFOT(1)=FLOAT((I-MARKV)/NSTEPV+1)#    Vertex number                    AB13 192
DO III=1,3; BUFOT(III+1)=sg:(QQ(I+III))# Move coords                    AB13 193
ncodefld:(BUFOT,1,CHROT,FM,4)#         Encode items                     AB13 194
FMT=520613.#                           Starting format                  AB13 195
IP=NINT(sg:(QQ(I+7)))#                 Start of face list               AB13 196
JP=NINT(sg:(QQ(I+8)))#                 End of face list                 AB13 197
DO J=IP,JP#                            Loop over face list              AB13 198
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AB13 199
KP=NINT(sg:(QQ(J)))#                   Address of a face                AB13 200
BUFOT(5)=FLOAT((KP-MARKF)/NSTEPF+1)## Face number                       AB13 201
ncodefld:(BUFOT,5,CHROT,FMT,1)#        Encode faces in face list        AB13 202
FMT=FMT+80000.#                        Change format                    AB13 203
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AB13 204
writeline:(0,0,0,1,3)#                 Write line                       AB13 205
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AB13 206
STOPFL=MARKFL+NWORDS-IDECR#            End of face list                 AB13 207
NVERTS=(ISTOPV-MARKV)/NSTEPV#          Number of vertices               AB13 208
#                                                                       AB13 209
# GET PLOT SCALE AND INITIALIZE PLOT                                    AB13 210
# ----------------------------------                                    AB13 211
#                                                                       AB13 212
T1=XMA-XMI+ZMA-ZMI#                    Calc xspan + z span              AB13 213
T2=YMA-YMI+ZMA-ZMI#                    Calc yspan + z span              AB13 214
T3=MAX(T1,T2)#                         Calc max. span                   AB13 215
PROJSC=7./T3#                          Calc plot scale                  AB13 216
IF(T3==T1)#                            If max.span is x                 AB13 217
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AB13 218
T3=1.-XMI*PROJSC#                      Offsets for plot                 AB13 219
T4=4.5-T2*PROJSC/2.-YMI*PROJSC#        Offsets for plot                 AB13 220
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AB13 221
ELSE#                                  If max.span is y                 AB13 222
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AB13 223
T3=4.5-T1*PROJSC/2.-XMI*PROJSC#        Offsets for plot                 AB13 224
T4=1.-YMI*PROJSC#                      Offsets for plot                 AB13 225
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AB13 226
T5=T3+XMA*PROJSC-ZMI*PROJSC+1.#        Offsets for plot                 AB13 227
T6=T4+YMA*PROJSC-ZMI*PROJSC+1.#        Offsets for plot                 AB13 228
IF(STEREO==no:)#                       Projection plot?                 AB13 229
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AB13 230
CALL AB17(0,0.D0,0.D0,1,0)#            Initialize plot                  AB13 231
CALL AB17(0,0.D0,0.D0,6,0)#            Initialize plot                  AB13 232
CALL AB17(3,0.D0,0.D0,3,0)#            Force line color change          AB13 233
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AB13 234
#                                                                       AB13 235
# DETERMINE THE EDGES OF THE CRYSTAL                                    AB13 236
# ----------------------------------                                    AB13 237
#                                                                       AB13 238
writeline:(1,AB133,NAB133,3,3)#        Write heading                    AB13 239
MARKED=STOPFL#                         Start of edge storage            AB13 240
STEPED=10#                             Words stored per edge            AB13 241
STOPED=MARKED#                         Preliminary end                  AB13 242
KK=MARKED#                             Pointer                          AB13 243
NEDGES=0#                              Initial count of edges           AB13 244
FOR(I=MARKV;I<(ISTOPV-NSTEPV);I=I+NSTEPV)# Loop over vertices           AB13 245
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AB13 246
IP1=NINT(sg:(QQ(I+7)))#                Start of face list 1             AB13 247
IP2=NINT(sg:(QQ(I+8)))#                End of face list 1               AB13 248
FOR(J=I+NSTEPV;J<ISTOPV;J=J+NSTEPV)#   Loop over vertices               AB13 249
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AB13 250
JP1=NINT(sg:(QQ(J+7)))#                Start of face list 2             AB13 251
JP2=NINT(sg:(QQ(J+8)))#                End of face list 2               AB13 252
ICOUNT=0#                              Count of common faces            AB13 253
DO K=IP1,IP2#                          Loop over first face list        AB13 254
DO L=JP1,JP2#                          Loop over second face list       AB13 255
IF(NINT(sg:(QQ(K)))==NINT(sg:(QQ(L))))# A face in common                AB13 256
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AB13 257
ICOUNT=ICOUNT+1#                       Increment counter                AB13 258
FACE(ICOUNT)=QQ(K)#                    Store face address               AB13 259
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AB13 260
IF(ICOUNT==2)#                         Found an edge                    AB13 261
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AB13 262
IF(STEREO==no:)#                       Projection plot?                 AB13 263
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 21 AB13 264
I1=(NINT(sg:(FACE(1)))-MARKF)/NSTEPF+1# Index to face 1                 AB13 265
I2=(NINT(sg:(FACE(2)))-MARKF)/NSTEPF+1# Index to face 2                 AB13 266
IF(QQ(MARKF+NSTEPF*(I1-1)+2)<0.0\QQ(MARKF+NSTEPF*(I2-1)+2)<0.0)#        AB13 267
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 22 AB13 268
CALL AB17(3,QQ(I+1)*PROJSC+T3,QQ(I+3)*PROJSC+T6,3,1)# Move vertex 1 x   AB13 269
CALL AB17(2,QQ(J+1)*PROJSC+T3,QQ(J+3)*PROJSC+T6,3,1)# Draw vertex 2 x   AB13 270
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 22 AB13 271
ELSE CALL AB19(QQ(I+1)*PROJSC+T3,QQ(I+3)*PROJSC+T6$#                    AB13 272
,QQ(J+1)*PROJSC+T3,QQ(J+3)*PROJSC+T6)# Draw dashed line                 AB13 273
IF(QQ(MARKF+NSTEPF*(I1-1)+3)>0.0\QQ(MARKF+NSTEPF*(I2-1)+3)>0.0)#        AB13 274
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 23 AB13 275
CALL AB17(3,QQ(I+1)*PROJSC+T3,QQ(I+2)*PROJSC+T4,3,1)# Move vertex 1 x   AB13 276
CALL AB17(2,QQ(J+1)*PROJSC+T3,QQ(J+2)*PROJSC+T4,3,1)# Draw vertex 2 x   AB13 277
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 23 AB13 278
ELSE CALL AB19(QQ(I+1)*PROJSC+T3,QQ(I+2)*PROJSC+T4$#                    AB13 279
,QQ(J+1)*PROJSC+T3,QQ(J+2)*PROJSC+T4)# Draw dashed line                 AB13 280
IF(QQ(MARKF+NSTEPF*(I1-1)+1)<0.0\QQ(MARKF+NSTEPF*(I2-1)+1)<0.0)#        AB13 281
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 24 AB13 282
CALL AB17(3,QQ(I+3)*PROJSC+T5,QQ(I+2)*PROJSC+T4,3,1)# Move vertex 1 z   AB13 283
CALL AB17(2,QQ(J+3)*PROJSC+T5,QQ(J+2)*PROJSC+T4,3,1)# Draw vertex 2 z   AB13 284
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 24 AB13 285
ELSE CALL AB19(QQ(I+3)*PROJSC+T5,QQ(I+2)*PROJSC+T4$#                    AB13 286
,QQ(J+3)*PROJSC+T5,QQ(J+2)*PROJSC+T4)# Draw dashed line                 AB13 287
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 21 AB13 288
incrqq:(STOPED,STOPED+STEPED,AB1303)#  Get more memory                  AB13 289
NEDGES=NEDGES+1#                       Increment count of edges         AB13 290
QQ(KK+1)=FACE(1)#                      Address of first face            AB13 291
QQ(KK+2)=FACE(2)#                      Address of second face           AB13 292
QQ(KK+5)=QQ(J+1)-QQ(I+1)#              X elt of edge vector             AB13 293
QQ(KK+6)=QQ(J+2)-QQ(I+2)#              Y elt of edge vector             AB13 294
QQ(KK+7)=QQ(J+3)-QQ(I+3)#              Z elt of edge vector             AB13 295
QQ(KK+8)=QQ(I+1)#                      X of first vertex                AB13 296
QQ(KK+9)=QQ(I+2)#                      Y of first vertex                AB13 297
QQ(KK+10)=QQ(I+3)#                     Z of first vertex                AB13 298
BUFOT(1)=FLOAT(NEDGES)#                Edge number                      AB13 299
BUFOT(2)=FLOAT((NINT(sg:(QQ(KK+1)))-MARKF)/NSTEPF+1)# Face 1            AB13 300
BUFOT(3)=FLOAT((NINT(sg:(QQ(KK+2)))-MARKF)/NSTEPF+1)# Face 2            AB13 301
BUFOT(4)=FLOAT((I-MARKV)/NSTEPV+1)#    Vertex 1                         AB13 302
BUFOT(5)=FLOAT((J-MARKV)/NSTEPV+1)#    Vertex 2                         AB13 303
ncodefld:(BUFOT,1,CHROT,FM1,5)#        Encode items                     AB13 304
writeline:(0,0,0,1,3)#                 Write line                       AB13 305
KK=STOPED#                             Increment pointer                AB13 306
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AB13 307
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AB13 308
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AB13 309
IEULER=NFACES+NVERTS-NEDGES#           Test compliance with eulers      AB13 310
IF(IEULER!=2) iquit:(AB1305)#          A                                AB13 311
#                                                                       AB13 312
# PLOT HEADINGS AND END PLOT                                            AB13 313
#                                                                       AB13 314
T1=T3+PROJSC*(XMA+XMI)/2.-1.0#         Heading position                 AB13 315
T2=T5+PROJSC*(ZMA+ZMI)/2.-1.0#         Heading position                 AB13 316
T3=PROJSC*SCALE/100.#                  Scale factor (0.1 mm)            AB13 317
T4=T4+YMI*PROJSC-0.6#                  Heading position                 AB13 318
T6=T6+ZMI*PROJSC-0.6#                  Heading position                 AB13 319
IF(STEREO==no:)#                       Projection plot?                 AB13 320
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 25 AB13 321
CALL AB17(3,0.D0,0.D0,3,0)#            Force line color change          AB13 322
CALL AB17(3,0.5D0,9.6D0,3,3)#          Put scale on plot                AB13 323
CALL AB17(2,0.5D0+T3,9.6D0,3,3)#       Put scale on plot                AB13 324
CALL AB17(3,0.5D0,9.7D0,3,3)#          Put scale on plot                AB13 325
CALL AB17(2,0.5D0,9.5D0,3,3)#          Put scale on plot                AB13 326
CALL AB17(3,0.5D0+T3,9.7D0,3,3)#       Put scale on plot                AB13 327
CALL AB17(2,0.5D0+T3,9.5D0,3,3)#       Put scale on plot                AB13 328
CALL AB17(3,0.D0,0.D0,3,0)#            Force line color change          AB13 329
CALL AB17(0,T1,T6,5,0)#                Plot heading xz                  AB13 330
CALL AB17(1,T1,T4,5,0)#                Plot heading xy                  AB13 331
CALL AB17(2,T2,T4,5,0)#                Plot heading zy                  AB13 332
CALL AB17(3,T3+0.5D0,9.5D0,5,0)#       Plot heading scale               AB13 333
CALL AB17(0,0.0D0,0.0D0,10,0)#         End of plot                      AB13 334
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 25 AB13 335
ELSE CALL AB14#                        It must be a stereo view         AB13 336
#                                                                       AB13 337
RETURN#                                                                 AB13 338
END#                                                                    AB13 339
systemheader:(AB14)#                   Stereo of crystal shape          AB14   1
###############################################################         AB14   2
#                                                                       AB14   3
#                            AB14                                       AB14   4
#                                                                       AB14   5
###############################################################         AB14   6
SUBROUTINE AB14#                       W. dreissig 10/85                AB14   7
IMPLICIT INTEGER (I-N)#                                                 AB14   8
include:(AACOMN)#                      S.r. hall oct 91                 AB14   9
include:(ABCOMN)#                                                       AB14  10
#                                                                       AB14  11
dp: R(3,3)#       ## Rotation matrix                                    AB14  12
dp: R1(3,3)#      ## Rotation matrix                                    AB14  13
dp: R2(3,3)#      ## Rotation matrix                                    AB14  14
dp: V1(3)#        ## Vector                                             AB14  15
dp: VA(3),VP(3)#  ## Vectors for dash line calculation                  AB14  16
dp: XP1,XP2#      ## Two-dimensional plot coordinates                   AB14  17
dp: RMAX(3)#      ## Maxima of rotated coordinates                      AB14  18
dp: RMIN(3)#      ## Minima of rotated coordinates                      AB14  19
dp: MAXMAX#       ## Maximum of all three coordinates                   AB14  20
dp: MINMIN#       ## Minimum of all coordinates                         AB14  21
dp: SCAL#         ## Scale factor to plot coordinates                   AB14  22
dp: OR(3)#        ## Plot origin                                        AB14  23
dp: XNP#          ## Stereo translation                                 AB14  24
dp: T3#           ## Temporary store                                    AB14  25
dp: VIEW,ROT1,ROT2,ROT3,YNP#                                            AB14  26
INTEGER I,J,K,L#   ## Indices                                           AB14  27
INTEGER KK#        ## Index in edge list                                AB14  28
INTEGER IDX1,IDX2# ## Pointer to face list                              AB14  29
INTEGER NE#        ## Line counter in plot list                         AB14  30
INTEGER IL,NL#     ## Dash line indices                                 AB14  31
INTEGER IAX#       ## Rec. axis index                                   AB14  32
INTEGER ISTOP,NP#                                                       AB14  33
chardata:(AB142,'      mm')#                                            AB14  34
chardata:(AB143,'a* b* c* ')#                                           AB14  35
datastuff:#                                                             AB14  36
#                                                                       AB14  37
#     PREPARE ROTATION MATRIX FROM XTAL-SYSTEM TO A MORE COMFOR         AB14  38
#                                                                       AB14  39
VIEW=25.#                              View distance 15 units           AB14  40
ROT1=15.; ROT2=-10.; ROT3=1.#          Rotations (degrees)              AB14  41
CALL AB15(ROT1,3,R1)#                  Rotation about z-axis            AB14  42
CALL AB15(ROT2,2,R2)#                  Rotation about y-axis            AB14  43
mmult:(R1,R2,R)#                       Multiply both                    AB14  44
#                                                                       AB14  45
DO J=1,3; RMAX(J)=-999.#               Initialize maximum coord         AB14  46
DO J=1,3; RMIN(J)=999.#                Initialize minimum coord         AB14  47
MARKP=STOPED+1#                        Prepare plot list                AB14  48
STEPP=7#                               Seven words per edge             AB14  49
incrqq:(STOPP,MARKP+STEPP*(NEDGES+3),AB1401)# Fetch memory              AB14  50
KK=MARKP#                              Initializ pointer                AB14  51
#                                                                       AB14  52
FOR(I=MARKED;I<STOPED;I=I+STEPED)#     Loop over edges                  AB14  53
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB14  54
IDX1=NINT(sg:(QQ(I+1)))#               First face of this edge          AB14  55
IDX2=NINT(sg:(QQ(I+2)))#               Second face of this edge         AB14  56
postm:(R,QQ(IDX1+1),V1)#               Rotate vector first              AB14  57
QQ(KK+7)=1.#                           Set edge visible                 AB14  58
IF(V1(1)<0.)#                                                           AB14  59
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB14  60
postm:(R,QQ(IDX2+1),V1)#               Rotate vector second             AB14  61
IF(V1(1)<0.) QQ(KK+7)=2.#              Set edge invisible               AB14  62
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB14  63
DO J=1,3; V1(J)=QQ(I+4+J)+QQ(I+7+J)#   Compute first point              AB14  64
postm:(R,V1,QQ(KK+1))#                 Rotate and store                 AB14  65
postm:(R,QQ(I+8),QQ(KK+4))#            Rotate and store sec             AB14  66
DO J=1,3#                                                               AB14  67
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB14  68
RMAX(J)=MAX(RMAX(J),QQ(KK+3+J),QQ(KK+J))# Determine maximum             AB14  69
RMIN(J)=MIN(RMIN(J),QQ(KK+3+J),QQ(KK+J))# Determine minimum             AB14  70
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB14  71
KK=KK+STEPP#                           Update pointer                   AB14  72
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB14  73
ISTOP=KK#                              Save end of crystal data         AB14  74
#                                                                       AB14  75
DO I=1,3#                              Loop over cell axes              AB14  76
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB14  77
DO J=1,3#                              Loop over coords                 AB14  78
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB14  79
QQ(KK+J)=0.#                           First point is zero              AB14  80
V1(J)=0.2*BX(J,I)#                     Second point is axes             AB14  81
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB14  82
QQ(KK+7)=3.#                           Set visible, differe             AB14  83
postm:(R,V1,QQ(KK+4))#                 Rotate end points                AB14  84
KK=KK+STEPP#                           Update pointer                   AB14  85
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB14  86
#                                                                       AB14  87
#   CALCULATE PERSPECTIVE SCALE                                         AB14  88
#   ---------------------------                                         AB14  89
#                                                                       AB14  90
MAXMAX=MAX(RMAX(1),RMAX(2),RMAX(3))#   Maximum of all coords            AB14  91
MINMIN=MIN(RMIN(1),RMIN(2),RMIN(3))#   Minimum                          AB14  92
SCAL=4.50/(MAXMAX-MINMIN)#             Scale to approx. 5.0 u           AB14  93
DO J=1,3; OR(J)=(RMAX(J)+RMIN(J))*SCAL/2.# Origin                       AB14  94
#                                                                       AB14  95
CALL AB16(0.D0,0.D0,0.D0,0.D0,0.D0,0,1,AB143)# Init plot-file           AB14  96
CALL AB16(10.D0,6.D0,0.D0,0.D0,0.D0,2,6,AB143)# Dimensions of plot      AB14  97
#                                                                       AB14  98
#  NOW GENERATE PLOT COMMANDS                                           AB14  99
#  --------------------------                                           AB14 100
#                                                                       AB14 101
DO L=1,2#                              Loop over stereo pair            AB14 102
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB14 103
XNP=FLOAT(L-1)*5.+2.5#                 Set stereo-correctio             AB14 104
YNP=3.5#                               Y-np                             AB14 105
CALL AB15(ROT3*db:(FLOAT(2*L-3)),3,R)# Rot.matrix for stere             AB14 106
NE=0#                                  Init number of lines             AB14 107
#                                                                       AB14 108
FOR(KK=MARKP;KK<STOPP;KK=KK+STEPP)#    Loop over plot lines             AB14 109
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB14 110
NE=NE+1#                               Update line count                AB14 111
DO K=1,2#                              Loop over two points             AB14 112
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB14 113
IF((QQ(KK+7)!=2.)\(K!=2))#             Test if invisible edge           AB14 114
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB14 115
DO J=1,3; VA(J)=QQ(KK+(K-1)*3+J)#      Restore vector                   AB14 116
NP=MOD(K,2)+2#                         Set pen index                    AB14 117
postm:(R,VA,V1)#                       Stereo rotation this             AB14 118
IF(KK<ISTOP) DO J=1,3; V1(J)=V1(J)*SCAL-OR(J)# Scale crystal            AB14 119
ELSE         DO J=1,3; V1(J)=V1(J)*50. -OR(J)# Scale axes               AB14 120
T3=VIEW/(VIEW-V1(1))#                  Perspective factor               AB14 121
XP1=V1(2)*T3+XNP#                      X plot coordinate                AB14 122
XP2=V1(3)*T3+YNP#                      Y plot coordinate                AB14 123
IF(NP!=2)#                             Test for pen up move             AB14 124
CALL AB16(XP1,XP2,0.D0,0.D0,0.D0,3,3,AB142)# Write to plot file         AB14 125
ELSE#                                  Else a pen down move             AB14 126
CALL AB16(XP1,XP2,0.D0,0.D0,QQ(KK+7),2,3,AB142)# Write to plot file     AB14 127
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB14 128
ELSE#                                  Else invisible edge              AB14 129
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB14 130
T3=0.#                                 Init vector length               AB14 131
DO J=1,3#                              Loop three axes                  AB14 132
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB14 133
VP(J)=QQ(KK+(K-1)*3+J)-VA(J)#          Difference end point             AB14 134
T3=T3+VP(J)*VP(J)#                     Sum squared differen             AB14 135
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB14 136
T3=dsqrt:(T3)#                         Vector length                    AB14 137
NL=MAX(2*(NINT(T3*SCAL/0.08)/2),2)#    Number of dashes                 AB14 138
DO J=1,3; VP(J)=VP(J)/db:(FLOAT(NL))#  Compute increment ve             AB14 139
FOR(IL=2;IL<=NL;IL=IL+1)#              Loop over dashes                 AB14 140
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AB14 141
DO J=1,3; VA(J)=VA(J)+VP(J)#           Next dash point                  AB14 142
NP=MOD(IL,2)+2#                        Pen index up/down                AB14 143
postm:(R,VA,V1)#                       Stereo rotation this             AB14 144
DO J=1,3; V1(J)=V1(J)*SCAL-OR(J)#      Scale and translate              AB14 145
T3=VIEW/(VIEW-V1(1))#                  Perspective factor               AB14 146
XP1=V1(2)*T3+XNP#                      X plot coordinate                AB14 147
XP2=V1(3)*T3+YNP#                      Y plot coordinate                AB14 148
IF(NP!=2)#                             Test for pen up move             AB14 149
CALL AB16(XP1,XP2,0.D0,0.D0,0.D0,3,3,AB142)# Write to plot file         AB14 150
ELSE#                                  Else a pen down move             AB14 151
CALL AB16(XP1,XP2,0.D0,0.D0,QQ(KK+7),2,3,AB142)# Write to plot file     AB14 152
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AB14 153
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB14 154
IF((NE>NEDGES)&(K==2))#                Plot axes symbols                AB14 155
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AB14 156
IAX=NE-NEDGES#                         Axes index                       AB14 157
IF(IAX==1) XP1=XP1-.2#                 Shift x for x-axes               AB14 158
CALL AB16(XP1,XP2,.15D0,0.D0,2.D0,0,5,AB143(IAX*3-2^))# Store a* b* c   AB14 159
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AB14 160
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB14 161
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB14 162
ncodefld:(sg:(18.73/(SCAL*SCALE)),1,AB142,50532.,1)# Format mm scale    AB14 163
CALL AB16(XNP-0.5D0,.30D0, 0.D0,0.D0,0.D0,3,3,' ')# Move to scale lin   AB14 164
CALL AB16(XNP+1.5D0,.30D0, 0.D0,0.D0,1.D0,2,3,' ')# Plot scale line     AB14 165
CALL AB16(XNP-0.5D0,.33D0, 0.D0,0.D0,0.D0,3,3,' ')# Move to scale lin   AB14 166
CALL AB16(XNP-0.5D0,.27D0, 0.D0,0.D0,1.D0,2,3,' ')# Plot scale line     AB14 167
CALL AB16(XNP+1.5D0,.33D0, 0.D0,0.D0,0.D0,3,3,' ')# Move to scale lin   AB14 168
CALL AB16(XNP+1.5D0,.27D0, 0.D0,0.D0,1.D0,2,3,' ')# Plot scale line     AB14 169
CALL AB16(XNP+0.1D0,.40D0,.15D0,0.D0,8.D0,0,5,AB142)# Plot scale valu   AB14 170
CALL AB16(XNP-2.0D0,.30D0,.25D0,0.D0,6.D0,0,5,COMPID)# Plot compid ch   AB14 171
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB14 172
CALL AB16(0.D0,0.D0,0.D0,0.D0,0.D0,0,10,' ')# Finish off plot           AB14 173
RETURN#                                                                 AB14 174
END#                                                                    AB14 175
systemheader:(AB15)#                   Set up rotation matr             AB15   1
###############################################################         AB15   2
#                                                                       AB15   3
#                            AB15                                       AB15   4
#                                                                       AB15   5
###############################################################         AB15   6
SUBROUTINE AB15(ANG,IAX,R)#            W. dreissig 10/85                AB15   7
IMPLICIT INTEGER (I-N)#                                                 AB15   8
#                                                                       AB15   9
#  CONSTRUCT A ROTATION MATRIX FROM ROTATIONAL                          AB15  10
#   ANGLE (DEGREES) AND AXIS DEFINITION                                 AB15  11
#                                                                       AB15  12
dp: R(3,3)#      ## Rotation matrix                                     AB15  13
dp: ANG#         ## Rotation angle (degrees)                            AB15  14
dp: T1,T2#       ## Cos, sin                                            AB15  15
dp: T0#          ## Angle in radians                                    AB15  16
INTEGER IAX#      ## Rotational axis                                    AB15  17
INTEGER I1,I2,I3# ## Indices                                            AB15  18
#                                                                       AB15  19
T0=ANG/57.29578#                       Change angle to radians          AB15  20
T1=dcos:(T0)#                          Cos                              AB15  21
T2=dsin:(T0)#                          Sin                              AB15  22
I3=MOD(IAX+2,3)+1#                     Indices for rotation ma          AB15  23
I1=MOD(I3,3)+1#                                                         AB15  24
I2=MOD(I1,3)+1#                                                         AB15  25
R(I1,I1)=T1#                           Matrix elements                  AB15  26
R(I1,I2)=T2#                                                            AB15  27
R(I1,I3)=0.#                                                            AB15  28
R(I2,I1)=-T2#                                                           AB15  29
R(I2,I2)=T1#                                                            AB15  30
R(I2,I3)=0.#                                                            AB15  31
R(I3,I1)=0.#                                                            AB15  32
R(I3,I2)=0.#                                                            AB15  33
R(I3,I3)=1.#                           Rotation axis                    AB15  34
RETURN#                                                                 AB15  35
END#                                                                    AB15  36
systemheader:(AB16)#                   Plot stereo view of xtal         AB16   1
###############################################################         AB16   2
#                                                                       AB16   3
#                            AB16                                       AB16   4
#                                                                       AB16   5
###############################################################         AB16   6
SUBROUTINE AB16(X,Y,X0,Y0,SUBJ,NPEN,IPL,CHAR)# W. dreissig 10/85        AB16   7
IMPLICIT INTEGER (I-N)#                                                 AB16   8
#                                                                       AB16   9
include:(AACOMN)#                                                       AB16  10
#                                                                       AB16  11
dp: X,Y#         ## Plot coordinates (relative to x0,y0)                AB16  12
dp: X0,Y0#       ## Zero correction (ipl=3,6), height (ipl=4,5          AB16  13
dp: SUBJ#        ## Subject (colour) index (ipl=3), theta (ipl          AB16  14
#                 ##  XMAX-XMIN (6)                                     AB16  15
INTEGER NPEN#     ## Pen up/down (#3/3) (ipl=3), ymax-ymin (ipl         AB16  16
INTEGER IPL#      ## 1 = initialize plotter (no other parameter         AB16  17
#                 ## 2 = FRAME ADVANCE                                  AB16  18
#                 ## 3 = PLOT COMMAND (MOVE OR DRAW, DEPENDENT          AB16  19
#                 ## 4 = NUMBER COMMAND                                 AB16  20
#                 ## 5 = CHARACTER STRING COMMAND (FOLLOWED BY          AB16  21
#                 ##       STRING >CHAR<, PACKED INTO SEVEN WOR         AB16  22
#                 ## 6 = PLOT DIMENSIONS (INFORMATION FOR PLOT-         AB16  23
#                 ## 10= FINISH PLOT                                    AB16  24
string:(CHAR,*)#  ## Character array (ipl=5)                            AB16  25
INTEGER IP#       ## Packet index                                       AB16  26
INTEGER NWORD#    ## No. of words needed to hold ipl=5 characters       AB16  27
INTEGER CHRPPK#   ## No. of characers per 7 word packet                 AB16  28
INTEGER NCHAR#    ## No. of characters for current ipl=5 output         AB16  29
INTEGER CHROUT#   ## No. of characters in each packet write             AB16  30
INTEGER I,J#      ## Index for packet writes and for character counts   AB16  31
DATA CHRPPK/arith:(mxchwd:,*,7)/## Total characters in one 7 word pkt   AB16  32
datastuff:#                                                             AB16  33
#                                                                       AB16  34
#   PLOT COMMANDS ARE WRITTEN ON FILE E (RELATIVE UNIT 5), RECO         AB16  35
#    PACKET SIZE 7 WORDS. PACKET BUFFER BEGINS AT IOMARK(5)             AB16  36
#                                                                       AB16  37
writepkt:(5,23,7,IP)#                  Prepare to write packet          AB16  38
QX(IP+1)=sg:(X)#                       First word of packet             AB16  39
QX(IP+2)=sg:(Y)#                       Second word                      AB16  40
QX(IP+3)=sg:(X0)#                      Third word                       AB16  41
QX(IP+4)=sg:(Y0)#                      Fourth word                      AB16  42
QX(IP+5)=sg:(SUBJ)#                    Fifth word                       AB16  43
QX(IP+6)=FLOAT(NPEN)#                  Sixth word                       AB16  44
QX(IP+7)=FLOAT(IPL)#                   Last word, command key           AB16  45
#                                                                       AB16  46
IF(IPL==10) writepkt:(5,endrecord:,7,IP)# Write last packet             AB16  47
ELSE IF(IPL==5)#                       Is this a character stri         AB16  48
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB16  49
NCHAR=INT(SUBJ)#                       Integerize no. of characters     AB16  50
NWORD=(NCHAR-1)/mxchwd:+1#             Find total words to hold 'em     AB16  51
J=1#                                   Point to first character to go   AB16  52
FOR (I=1;I<=NWORD;I=I+7)#              Loop over packets 7 words per    AB16  53
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB16  54
writepkt:(5,23,7,IP)#                  Get a packet for characters      AB16  55
movector:(BLANK,1,QX(IP+1),1,CHRPPK,1)# Blank out the whole packet      AB16  56
CHROUT=MIN0(CHRPPK,NCHAR)#             No trash in last pkt, please     AB16  57
movector:(CHAR,J,QX(IP+1),1,CHROUT,0)# Move segment of characters       AB16  58
J=J+CHRPPK ; NCHAR=NCHAR-CHRPPK#       Keep track of position           AB16  59
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB16  60
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB16  61
ELSE IF(IPL==6)#                       Is this a scale packet           AB16  62
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB16  63
QX(IP+3)=FLOAT(NPEN)#                  Stereo flag (2=yes 1=no)         AB16  64
QX(IP+4)=sg:(SUBJ)#                    Internal scale                   AB16  65
QX(IP+5)=sg:(X0)#                      X offset                         AB16  66
QX(IP+6)=sg:(Y0)#                      Y offset                         AB16  67
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB16  68
RETURN#                                                                 AB16  69
END#                                                                    AB16  70
systemheader:(AB17)#                   Write plot packets               AB17   1
#####################################################################   AB17   2
#                                                                   #   AB17   3
#                           AB17                                    #   AB17   4
#                                                                   #   AB17   5
#####################################################################   AB17   6
SUBROUTINE AB17(IPEN,X,Y,K,COL)#       L. m. engelhardt 1986            AB17   7
IMPLICIT INTEGER (I-N)#                                                 AB17   8
include:(AACOMN)#                                                       AB17   9
include:(ABCOMN)#                                                       AB17  10
#                                                                       AB17  11
# WRITE ALL COMMANDS TO TO PLOT INSTRUCTION FILE                        AB17  12
# PLOT FUNCTIONS ARE GIVEN BY K:                                        AB17  13
#   1 .. START OF PLOT                                                  AB17  14
#   2 .. PLOT FRAME ADVANCE SIGNAL                                      AB17  15
#   3 .. PEN MOVEMENT UP/DOWN GIVEN BY IPEN                             AB17  16
#   5 .. PLOT A SPECIAL SYMBOL OR CHAR STRING                           AB17  17
#   6 .. AUTO CENTRE AND SCALE                                          AB17  18
#  10 .. END OF PLOT SIGNAL                                             AB17  19
#                                                                       AB17  20
INTEGER IPT,K,J,IPEN,COL#                                               AB17  21
dp:    X,Y#                                                             AB17  22
chardata:(HEAD,"x across  z up-pagex across  y up-page$#                AB17  23
z across  y up-page 0.1 mm            ")#                               AB17  24
datastuff:#                                                             AB17  25
#                                                                       AB17  26
writepkt:(5,23,7,IPT)#                 Write the packet                 AB17  27
DO J=1,6 ; QX(IPT+J)=0.0#              Zero the packet                  AB17  28
QX(IPT+7)=FLOAT(K)#                    Plot function signal             AB17  29
IF(K>=3&K<=5)#                         Fill x, y coords here            AB17  30
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB17  31
QX(IPT+1)=sg:(X)#                      X-coordinate                     AB17  32
QX(IPT+2)=sg:(Y)#                      Y-coordinate                     AB17  33
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB17  34
#                                                                       AB17  35
IF(K==3)#                              Move pen to (x,y)                AB17  36
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB17  37
QX(IPT+5)=FLOAT(COL)#                  Colour index                     AB17  38
QX(IPT+6)=FLOAT(IPEN)#                 Pen up/down signal               AB17  39
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB17  40
ELSE IF(K==5)#                         Plot symbol or char string       AB17  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB17  42
QX(IPT+3)=0.15#                        Height of symbol/chars           AB17  43
QX(IPT+5)=19.#                         Spec char no/string length       AB17  44
writepkt:(5,23,7,IPT)#                 Write the packet                 AB17  45
movector:(HEAD,IPEN*19+1,QX(IPT+1),1,19,0)# Transfer characters         AB17  46
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB17  47
ELSE IF(K==6)#                         Auto centre and scale            AB17  48
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB17  49
QX(IPT+1)=12.#                         Plot width                       AB17  50
QX(IPT+2)=10.#                         Plot height                      AB17  51
QX(IPT+3)=1.#                          X-coord of centre                AB17  52
QX(IPT+4)=0.#                          Y-coord of centre                AB17  53
QX(IPT+5)=0.#                          Plot lower left x offset         AB17  54
QX(IPT+6)=0.#                          Plot lower left y offset         AB17  55
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB17  56
ELSE IF(K==10) writepkt:(5,endrecord:,4,J)# Write end of plot record    AB17  57
#                                                                       AB17  58
RETURN#                                                                 AB17  59
END#                                                                    AB17  60
systemheader:(AB18)#                   Apply tabular corrections        AB18   1
####################################################################    AB18   2
#                                                                  #    AB18   3
#                              AB18                                #    AB18   4
#                                                                  #    AB18   5
####################################################################    AB18   6
SUBROUTINE AB18#                       L.m. engelhardt 1986             AB18   7
include:(AACOMN)#                                                       AB18   8
include:(ABCOMN)#                                                       AB18   9
#                                                                       AB18  10
INTEGER I#                             Loop index                       AB18  11
INTEGER J#                             Loop index                       AB18  12
INTEGER K#                             Loop index                       AB18  13
INTEGER L#                             Loop index                       AB18  14
INTEGER M#                             Index                            AB18  15
dp: T1,SMAX,T(8)#                      Temporary                        AB18  16
dp: STHOL(30)#                         Sinth/wav array                  AB18  17
datastuff:#                                                             AB18  18
#                                                                       AB18  19
# FORM SIN(TH)/WAV ARRAY                                                AB18  20
# ----------------------                                                AB18  21
#                                                                       AB18  22
DO I=1,8; T(I)=0.D00#                  Zero temp array                  AB18  23
STHOL(1)=0.;   T1=0.#                  First element                    AB18  24
DO M=2,NAST+4#                         Loop over abs entries            AB18  25
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB18  26
T1=T1+0.0872664#                       Increment 5 deg.(pi/36)          AB18  27
STHOL(M)=dsin:(T1)/WAV#                Store sin(th)/wav table          AB18  28
ifeq:(ASTAR(M),0.) ASTAR(M)=ASTAR(M-1)# Ensure astar non-zero           AB18  29
IF(ITBAR==yes:) ifeq:(TBAR(M),0.) TBAR(M)=TBAR(M-1)# Tbar non-zero      AB18  30
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB18  31
SMAX=STHOL(NAST)#                      Set table maximum                AB18  32
CALL AB03(1,T)#                        Initialize llrefl i/o            AB18  33
#                                                                       AB18  34
# CORRECTIONS INTERPOLATED FROM TABLES                                  AB18  35
# ------------------------------------                                  AB18  36
#                                                                       AB18  37
REPEAT#                                Loop over reflections            AB18  38
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB18  39
CALL AB03(2,T)#                        Read reflection                  AB18  40
ifeq:(T(1),1.) BREAK#                  Break at end of lr               AB18  41
ifeq:(T(2),1.) NEXT#                   Treat this reflection            AB18  42
IF(T(4)>SMAX) iquit:(AB1801)#          Quit if angle>table limit        AB18  43
I=3;         K=NAST+1#                 Set limits                       AB18  44
WHILE((T(4)>STHOL(I))&(I<K)) I=I+1#    Find nearest                     AB18  45
I=I-2; L=I+3#                          Reset limits                     AB18  46
#                                                                       AB18  47
T(1)=0.;   T(2)=0.#                    Set intial value                 AB18  48
DO K=I,L#                              Loop over table                  AB18  49
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB18  50
T1=1.#                                 Set scaling term                 AB18  51
DO J=I,L#                              Loop over table                  AB18  52
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB18  53
IF(K==J)NEXT#                          Skip identical terms             AB18  54
T1=T1*(T(4)-STHOL(J))/(STHOL(K)-STHOL(J))# Calc scale                   AB18  55
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB18  56
T(1)=T(1)+T1*ASTAR(K)#                 Incr interpolated value          AB18  57
IF(ITBAR==yes:) T(2)=T(2)+T1*TBAR(K)#  Interpolated tbar                AB18  58
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB18  59
T(1)=MIN(T(1),ASTAR(1))#               Protect zero angle absorption    AB18  60
IF(ITBAR==yes:) T(2)=MIN(T(2),TBAR(1))# Protect zero angle absorpt.     AB18  61
CALL AB03(3,T)#                        Apply correction                 AB18  62
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB18  63
CALL AB03(4,T)#                        Copy rest of file                AB18  64
#                                                                       AB18  65
RETURN#                                                                 AB18  66
END#                                                                    AB18  67
systemheader:(AB19)#                   Draw dashed line                 AB19   1
####################################################################    AB19   2
#                                                                  #    AB19   3
#                              AB19                                #    AB19   4
#                                                                  #    AB19   5
####################################################################    AB19   6
SUBROUTINE AB19(X1,Y1,X2,Y2)#          L.m. engelhardt 1986             AB19   7
IMPLICIT INTEGER (I-N)#                                                 AB19   8
#                                                                       AB19   9
include:(AACOMN)#                                                       AB19  10
include:(ABCOMN)#                                                       AB19  11
INTEGER I#                             Loop index                       AB19  12
INTEGER IPEN#                          Pen signal                       AB19  13
dp: D#                                 Length of line                   AB19  14
dp: XINC,YINC#                         X,y increments                   AB19  15
dp: X1,Y1#                             Starting point of line           AB19  16
dp: X2,Y2#                             End point of line                AB19  17
#                                                                       AB19  18
D=dsqrt:((X2-X1)**2+(Y2-Y1)**2)#       Calc. length                     AB19  19
IF(D>0.2)#                             Return if d too short            AB19  20
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB19  21
XINC=0.1*(X2-X1)/D#                    Calc. x increment                AB19  22
YINC=0.1*(Y2-Y1)/D#                    Calc. y increment                AB19  23
CALL AB17(3,X1,Y1,3,1)#                Move to start of line            AB19  24
IPEN=2#                                Set pen to draw                  AB19  25
DO I=1,100#                            Loop along line                  AB19  26
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB19  27
X1=X1+XINC#                            Increment x                      AB19  28
Y1=Y1+YINC#                            Increment y                      AB19  29
D=(X2-X1)**2+(Y2-Y1)**2#               Length left to draw              AB19  30
IF(D<0.01)BREAK#                       Finished                         AB19  31
IF(IPEN==2)#                           If pen down                      AB19  32
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB19  33
CALL AB17(IPEN,X1,Y1,3,1)#             Draw dash                        AB19  34
IPEN=3#                                Pen up                           AB19  35
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB19  36
ELSE#                                  If pen up                        AB19  37
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB19  38
CALL AB17(IPEN,X1,Y1,3,1)#             Move                             AB19  39
IPEN=2#                                Pen down                         AB19  40
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB19  41
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB19  42
CALL AB17(3,X2,Y2,3,1)#                Move to end of line              AB19  43
CALL AB17(2,X2-XINC,Y2-YINC,3,1)#      Draw dash at end                 AB19  44
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB19  45
RETURN#                                                                 AB19  46
END#                                                                    AB19  47
systemheader:(AB20)#                   Construct gaussian grid          AB20   1
####################################################################    AB20   2
#                                                                  #    AB20   3
#                              AB20                                #    AB20   4
#                                                                  #    AB20   5
####################################################################    AB20   6
SUBROUTINE AB20#                                                        AB20   7
IMPLICIT INTEGER (I-N)#                                                 AB20   8
include:(AACOMN)#                                                       AB20   9
include:(ABCOMN)#                                                       AB20  10
#                                                                       AB20  11
## J.M. STEWART, NOVEMBER 1980                                          AB20  12
## G. DAVENPORT, FEBRUARY 1982                                          AB20  13
#                                                                       AB20  14
dp: BB(2,2)#                           2 simultaneous equations         AB20  15
dp: DET#                               Determinant of bb matrix         AB20  16
dp: DI(2)#                             2 simultaneous equations         AB20  17
INTEGER I#                             Loop index                       AB20  18
INTEGER IPT#                           Pointer into gauss array         AB20  19
INTEGER ISON#                          Flag point inside crystal        AB20  20
INTEGER J#                             Loop index                       AB20  21
INTEGER K#                             Loop index                       AB20  22
INTEGER M2#                            Pointer into mark2               AB20  23
INTEGER NGRID(3)#                      Grid points x y z                AB20  24
dp: GAUSS(122)#                        Single coord weights&pivots      AB20  25
dp: PIVOT#                             Pivot for a coordinate           AB20  26
dp: VOLUME#                            Sum of volume elements           AB20  27
dp: XELT#                              X length element                 AB20  28
dp: XYZ(3)#                            Integration point                AB20  29
dp: YELT#                              Y length element                 AB20  30
dp: ZELT#                              Z length element                 AB20  31
#                                                                       AB20  32
chardata:(AB221," Grid points- A   B   C  Total  Volume mm**3")#        AB20  33
realdata:(FM1,[150413.,190413.,230413.,300613.,411151.])#               AB20  34
dbledata:(GAUS1,[$#                    8 points in the integeration     AB20  35
.019855072,.050614268,#                                                 AB20  36
.101666761,.111190517,#                                                 AB20  37
.237233795,.156853323,#                                                 AB20  38
.408282679,.181341892,#                                                 AB20  39
.013046736,.033335672,#                10 points in the integration     AB20  40
.067468317,.074725675,#                                                 AB20  41
.160295216,.109543181,#                                                 AB20  42
.283302303,.134633360,#                                                 AB20  43
.425562831,.147762112,#                                                 AB20  44
.009219683,.023587668,#                12 points in the integration     AB20  45
.047941372,.053469663,#                                                 AB20  46
.115048663,.080039164,#                                                 AB20  47
.206341023,.101583713,#                                                 AB20  48
.316084251,.116746268,#                                                 AB20  49
.437333296,.124573523])#                                                AB20  50
dbledata:(GAUS2,[$#                    14 points in the integration     AB20  51
.006858096,.017559730,#                                                 AB20  52
.035782558,.040079044,#                                                 AB20  53
.086399342,.060759285,#                                                 AB20  54
.156353548,.078601584,#                                                 AB20  55
.242375682,.092769199,#                                                 AB20  56
.340443816,.102599232,#                                                 AB20  57
.445972526,.107631927,#                                                 AB20  58
.005299533,.013576230,#                16 points in the integration     AB20  59
.027712488,.031126762,#                                                 AB20  60
.067184399,.047579256,#                                                 AB20  61
.122297796,.062314486,#                                                 AB20  62
.191061878,.074797994,#                                                 AB20  63
.270991611,.084578260,#                                                 AB20  64
.359198225,.091301708,#                                                 AB20  65
.452493745,.094725305])#                                                AB20  66
dbledata:(GAUS3,[$#                    6 points in the integration      AB20  67
.033765243,.085662246,#                                                 AB20  68
.169395307,.180380787,#                                                 AB20  69
.380690407,.233956967,#                                                 AB20  70
.002406390,.006170615,#                16 points in the integration     AB20  71
.012635722,.014265694,#                                                 AB20  72
.030862724,.022138719,#                                                 AB20  73
.056792236,.029649292,#                                                 AB20  74
.089999007,.036673241,#                                                 AB20  75
.129937904,.043095081,#                                                 AB20  76
.175953174,.048809326,#                                                 AB20  77
.227289264,.053722135,#                                                 AB20  78
.283103246,.057752834,#                                                 AB20  79
.342478660,.060835236,#                                                 AB20  80
.404440566,.062918728,#                                                 AB20  81
.467971553,.063969098])#                                                AB20  82
dbledata:(GAUS4,[$#                    32 points int the integratio     AB20  83
.001368069,.003509305,#                                                 AB20  84
.007194244,.008137197,#                                                 AB20  85
.017618872,.012696033,#                                                 AB20  86
.032546962,.017136932,#                                                 AB20  87
.051839422,.021417949,#                                                 AB20  88
.075316193,.025499030,#                                                 AB20  89
.102758102,.029342047,#                                                 AB20  90
.133908940,.032911111,#                                                 AB20  91
.168477867,.036172897,#                                                 AB20  92
.206142121,.039096948,#                                                 AB20  93
.246550045,.041655962,#                                                 AB20  94
.289324362,.043826047,#                                                 AB20  95
.334065699,.045586939,#                                                 AB20  96
.380356319,.046922200,#                                                 AB20  97
.427764019,.047819360,#                                                 AB20  98
.475846167,.048270044])#                                                AB20  99
intdata:(NADRX,0);  intdata:(NADRY,0);  intdata:(NADRZ,0)#              AB20 100
intdata:(NPTX,0);   intdata:(NPTY,0);   intdata:(NPTZ,0)#               AB20 101
intdata:(NUMBER,[6,8,10,12,14,16,24,32,60,0,8,18,30,44,66,90])#         AB20 102
datastuff:#                                                             AB20 103
#                                                                       AB20 104
DO I=1,30;GAUSS(I)=GAUS1(I)#           Merge gaussian points            AB20 105
DO I=1,30;GAUSS(I+30)=GAUS2(I)#        Merge gaussian points            AB20 106
DO I=1,30;GAUSS(I+60)=GAUS3(I)#        Merge gaussian points            AB20 107
DO I=1,32;GAUSS(I+90)=GAUS4(I)#        Merge gaussian points            AB20 108
DO I=1,3; NGRID(I)=8#                  Default grid points              AB20 109
#                                                                       AB20 110
# READ THE INFORMATION FROM THE GRID LINE                               AB20 111
# ---------------------------------------                               AB20 112
#                                                                       AB20 113
IF(LINID==1)#                          If grid line entered             AB20 114
DO I=1,3; ifnm:(BUFIN(I)) NGRID(I)=NINT(BUFIN(I))# Enter grid values    AB20 115
#                                                                       AB20 116
DO I=1,8#                              Find gauss array pointer         AB20 117
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB20 118
IF(NGRID(1)==NUMBER(I))#               Test for match                   AB20 119
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB20 120
NPTX=NGRID(1)#                         Number of x intervals            AB20 121
NADRX=NUMBER(I+8)#                     Gauss pointer                    AB20 122
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB20 123
IF(NGRID(2)==NUMBER(I))#               Test for match                   AB20 124
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB20 125
NPTY=NGRID(2)#                         Number of y intervals            AB20 126
NADRY=NUMBER(I+8)#                     Gauss pointer                    AB20 127
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB20 128
IF(NGRID(3)==NUMBER(I))#               Test for match                   AB20 129
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB20 130
NPTZ=NGRID(3)#                         Number of z intervals            AB20 131
NADRZ=NUMBER(I+8)#                     Gauss pointer                    AB20 132
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB20 133
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB20 134
#                                                                       AB20 135
# DETERMINE INTEGRATION POINTS                                          AB20 136
# ----------------------------                                          AB20 137
#                                                                       AB20 138
MARK2=ISTOPF#                          Start point storage              AB20 139
NSTEP2=4#                              Four items per point             AB20 140
incrqq:(ISTOP2,MARK2+NPTX*NPTY*NPTZ*NSTEP2,AB2001)# Get memory          AB20 141
VOLUME=0.0#                            Initial sum volume elements      AB20 142
M2=MARK2#                              Pointer into mark2               AB20 143
#                                                                       AB20 144
# LOOP OVER X DIVISIONS                                                 AB20 145
# ---------------------                                                 AB20 146
#                                                                       AB20 147
DO I=1,NPTX#                           Loop over x divisions            AB20 148
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB20 149
IF(NPTX<(2*I))#                        Use second half of table         AB20 150
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB20 151
IPT=NADRX+2*(NPTX-I)#                  Pointer                          AB20 152
PIVOT=1.0-GAUSS(IPT+1)#                Pivot                            AB20 153
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB20 154
ELSE#                                  Use first half of table          AB20 155
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB20 156
IPT=NADRX+2*(I-1)#                     Pointer                          AB20 157
PIVOT=GAUSS(IPT+1)#                    Pivot                            AB20 158
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB20 159
XYZ(1)=XMIN+(XMAX-XMIN)*PIVOT#         X-coordinate                     AB20 160
XELT=GAUSS(IPT+2)*(XMAX-XMIN)#         X element of volum               AB20 161
#                                                                       AB20 162
# FIND MAX AND MIN Y FOR GIVEN X                                        AB20 163
# ------------------------------                                        AB20 164
#                                                                       AB20 165
YMAX=-9999.#                           Initial ymax                     AB20 166
YMIN= 9999.#                           Initial ymin                     AB20 167
FOR(J=MARKF;J<(ISTOPF-NSTEPF);J=J+NSTEPF)# Loop over faces              AB20 168
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB20 169
BB(1,1)=QQ(J+2)#                       Y coeff, equation 1              AB20 170
BB(1,2)=QQ(J+3)#                       Z coeff, equation 1              AB20 171
DI(1)=QQ(J+4)-QQ(J+1)*XYZ(1)#          Constant coeff, equation 1       AB20 172
FOR(K=MARKF+NSTEPF;K<ISTOPF;K=K+NSTEPF)# Loop over faces                AB20 173
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB20 174
BB(2,1)=QQ(K+2)#                       Y coeff, equation 2              AB20 175
BB(2,2)=QQ(K+3)#                       Z coeff, equation 2              AB20 176
DET=BB(1,1)*BB(2,2)-BB(1,2)*BB(2,1)#   Determinant of bb matrix         AB20 177
ifne:(DET,0.0)#                        Continue if nonsingular          AB20 178
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB20 179
DI(2)=QQ(K+4)-QQ(K+1)*XYZ(1)#          Constant coeff, equation 2       AB20 180
XYZ(2)=(DI(1)*BB(2,2)-DI(2)*BB(1,2))/DET# Solve for y                   AB20 181
XYZ(3)=(DI(2)*BB(1,1)-DI(1)*BB(2,1))/DET# Solve for z                   AB20 182
CALL AB22(XYZ,ISON)#                   Test the point                   AB20 183
IF(ISON==yes:)#                        Continue if in or on crystal     AB20 184
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB20 185
IF(XYZ(2)<YMIN) YMIN=XYZ(2)#           Test for new ymin                AB20 186
IF(XYZ(2)>YMAX) YMAX=XYZ(2)#           Test for new ymax                AB20 187
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB20 188
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB20 189
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB20 190
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB20 191
#                                                                       AB20 192
# LOOP OVER Y INTERVALS                                                 AB20 193
# ---------------------                                                 AB20 194
#                                                                       AB20 195
DO J=1,NPTY#                           Loop over y intervals            AB20 196
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AB20 197
IF(NPTY<(2*J))#                        Second half of table             AB20 198
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AB20 199
IPT=NADRY+2*(NPTY-J)#                  Pointer                          AB20 200
PIVOT=1.0-GAUSS(IPT+1)#                Pivot                            AB20 201
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AB20 202
ELSE#                                  First half of table              AB20 203
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 14 AB20 204
IPT=NADRY+2*(J-1)#                     Pointer                          AB20 205
PIVOT=GAUSS(IPT+1)#                    Pivot                            AB20 206
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 14 AB20 207
XYZ(2)=YMIN+(YMAX-YMIN)*PIVOT#         Y coordinate                     AB20 208
YELT=GAUSS(IPT+2)*(YMAX-YMIN)#         Y element of volum               AB20 209
#                                                                       AB20 210
# DETERMINE MAX AND MIN Z FOR GIVEN X,Y                                 AB20 211
# -------------------------------------                                 AB20 212
#                                                                       AB20 213
ZMAX=-9999.#                           Initial max z                    AB20 214
ZMIN= 9999.#                           Initial min z                    AB20 215
FOR(K=MARKF;K<ISTOPF;K=K+NSTEPF)#      Loop over faces                  AB20 216
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 15 AB20 217
IF(dabs:(QQ(K+3))>1.0E-8)#             Nonzero z coeff                  AB20 218
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 16 AB20 219
XYZ(3)=(QQ(K+4)-QQ(K+1)*XYZ(1)-QQ(K+2)*XYZ(2))/QQ(K+3)# Solve for z     AB20 220
CALL AB22(XYZ,ISON)#                   Test the point                   AB20 221
IF(ISON==yes:)#                        Point is on or in crystal        AB20 222
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 17 AB20 223
IF(XYZ(3)<ZMIN) ZMIN=XYZ(3)#           Test for new zmin                AB20 224
IF(XYZ(3)>ZMAX) ZMAX=XYZ(3)#           Test for new zmax                AB20 225
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 17 AB20 226
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 16 AB20 227
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 15 AB20 228
#                                                                       AB20 229
# LOOP OVER Z INTERVALS                                                 AB20 230
# ---------------------                                                 AB20 231
#                                                                       AB20 232
DO K=1,NPTZ#                           Loop over z intervals            AB20 233
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 18 AB20 234
IF(NPTZ<(2*K))#                        Second half of table             AB20 235
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 19 AB20 236
IPT=NADRZ+2*(NPTZ-K)#                  Pointer                          AB20 237
PIVOT=1.0-GAUSS(IPT+1)#                Pivot                            AB20 238
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 19 AB20 239
ELSE#                                  First half of table              AB20 240
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 20 AB20 241
IPT=NADRZ+2*(K-1)#                     Pointer                          AB20 242
PIVOT=GAUSS(IPT+1)#                    Pivot                            AB20 243
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 20 AB20 244
XYZ(3)=ZMIN+(ZMAX-ZMIN)*PIVOT#         Z-coordinate                     AB20 245
ZELT=GAUSS(IPT+2)*(ZMAX-ZMIN)#         Z element of volum               AB20 246
VOLUME=VOLUME+XELT*YELT*ZELT#          Accumulate volume                AB20 247
QQ(M2+1)=XYZ(1)#                       Store x coordinate               AB20 248
QQ(M2+2)=XYZ(2)#                       Store y coordinate               AB20 249
QQ(M2+3)=XYZ(3)#                       Store z coordinate               AB20 250
QQ(M2+4)=XELT*YELT*ZELT#               Store volume elemen              AB20 251
M2=M2+NSTEP2#                          Add to pointer                   AB20 252
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 18 AB20 253
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AB20 254
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB20 255
#                                                                       AB20 256
# WRITE A SUMMARY FOR USER                                              AB20 257
# ------------------------                                              AB20 258
#                                                                       AB20 259
BUFOT(1)=FLOAT(NPTX)#                  X intervals                      AB20 260
BUFOT(2)=FLOAT(NPTY)#                  Y intervals                      AB20 261
BUFOT(3)=FLOAT(NPTZ)#                  Z intervals                      AB20 262
BUFOT(4)=FLOAT(NPTX*NPTY*NPTZ)#        Total points                     AB20 263
BUFOT(5)=1000.*VOLUME/SCALE**3#        Volume                           AB20 264
ncodefld:(BUFOT,1,CHROT,FM1,5)#        Encode items                     AB20 265
writeline:(1,AB221,NAB221,2,3)#        Write out items                  AB20 266
#                                                                       AB20 267
# STORE TWO ITEMS IN COMMON                                             AB20 268
# -------------------------                                             AB20 269
#                                                                       AB20 270
VOL=VOLUME#                            Volume of crystal in cm**3       AB20 271
NPOINT=NPTX*NPTY*NPTZ#                 Number of points                 AB20 272
RETURN#                                                                 AB20 273
END#                                                                    AB20 274
systemheader:(AB22)#                                                    AB22   1
####################################################################    AB22   2
#                                                                  #    AB22   3
#                               AB22                               #    AB22   4
#                                                                  #    AB22   5
####################################################################    AB22   6
SUBROUTINE AB22(XYZ,ISON)#                                              AB22   7
IMPLICIT INTEGER (I-N)#                                                 AB22   8
include:(AACOMN)#                                                       AB22   9
include:(ABCOMN)#                                                       AB22  10
#                                                                       AB22  11
## TEST FOR POINT INSIDE OR ON CRYSTAL                                  AB22  12
## J.M. STEWART, NOVEMBER 1980                                          AB22  13
#                                                                       AB22  14
INTEGER I#                             Loop index                       AB22  15
INTEGER ISON#                          Flag point in or on crystal      AB22  16
dp: R#                                 Temporary                        AB22  17
dp: XYZ(3)#                            The point                        AB22  18
#                                                                       AB22  19
ISON=yes:#                             Assume in or on crystal          AB22  20
FOR(I=MARKF;I<ISTOPF;I=I+NSTEPF)#      Loop over faces                  AB22  21
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB22  22
R=QQ(I+1)*XYZ(1)+QQ(I+2)*XYZ(2)+QQ(I+3)*XYZ(3)-QQ(I+4)#                 AB22  23
ifpos:(R)#                             Positive r                       AB22  24
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB22  25
ISON=no:#                              Outside crystal                  AB22  26
BREAK#                                 Quit checking                    AB22  27
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB22  28
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB22  29
RETURN#                                                                 AB22  30
END#                                                                    AB22  31
systemheader:(AB24)#                   Apply spherical corrections      AB24   1
####################################################################    AB24   2
#                                                                  #    AB24   3
#                              AB24                                #    AB24   4
#                                                                  #    AB24   5
####################################################################    AB24   6
SUBROUTINE AB24#                       N. spadaccini oct 1990           AB24   7
include:(AACOMN)#                                                       AB24   8
include:(ABCOMN)#                                                       AB24   9
#                                                                       AB24  10
dp: MUR#                               Mu * r                           AB24  11
dp: SINT#                              Sin(theta)                       AB24  12
dp: AJ(7)#                             Tibball's a(mur,theta(j))        AB24  13
dp: TJ(7)#                             Tibball's tbar(mur,theta(j))     AB24  14
dp: LM(7)#                             Interpolating in theta (astar)   AB24  15
dp: PM(7)#                             Interpolating in theta (tbar)    AB24  16
dp: AST#                               Calculated astar                 AB24  17
dp: ASTTB#                             Calculated astar*tbar            AB24  18
dp: T(8)#                              Temporary values                 AB24  19
INTEGER J,M,N#                                                          AB24  20
#                                                                       AB24  21
chardata:(AB241, END   )#                                               AB24  22
dbledata:(KT,[$#                       Tibball's k coefficients         AB24  23
.15E1,-.752339D-1,-.709348D-2,-.230962D-2,.183229D-2,$#                 AB24  24
-.512588D-3,.602654D-4,$#              Theta = 0                        AB24  25
.15E1,-.943198D-1,-.107369D-1,-.213317D-2,.117113D-2,$#                 AB24  26
-.126523D-3,.79317D-5,$#               Theta = 15                       AB24  27
.15E1,-.151088,-.180267D-1,-.146926D-2,.467837D-2,$#                    AB24  28
-.144905D-2,.164889D-3,$#              Theta = 30                       AB24  29
.15E1,-.243819,-.110884D-1,.742054D-2,.309696D-2,$#                     AB24  30
-.167405D-2,.217745D-3,$#              Theta = 45                       AB24  31
.15E1,-.352186,.142654D-1,.248318D-1,-.102843D-1,$#                     AB24  32
.219098D-2,-.223911D-3,$#              Theta = 60                       AB24  33
.15E1,-.440424,.400214D-1,.443076D-1,-.279866D-1,$#                     AB24  34
.770067D-2,-.8557E-3,$#                Theta = 75                       AB24  35
.15E1,-.477449,.610838D-1,.373941D-1,-.258788D-1,$#                     AB24  36
.714579D-2,-.788119D-3])#              Theta = 90                       AB24  37
dbledata:(CT,[$#                       Tibball's (c**-1) coefficients   AB24  38
1.0,-24.333333333333,172.66666666667,-533.33333333333,$#                AB24  39
810.66666666667,-597.33333333333,170.66666666667,$# Theta = 0           AB24  40
0.0,29.856406460551,-280.80338717126,963.31615074619,$#                 AB24  41
-1542.8375056147,1171.8016689125,-341.33333333333,$# Theta = 15         AB24  42
0.0,-8.0,162.66666666667,-730.66666666667,1344.0,$#                     AB24  43
-1109.3333333333,341.33333333333,$#    Theta = 30                       AB24  44
0.0,4.0,-89.333333333333,512.0,-1109.3333333333,1024.0,$#               AB24  45
-341.33333333333,$#                    Theta = 45                       AB24  46
0.0,-2.6666666666667,61.333333333333,-378.66666666667,$#                AB24  47
917.33333333333,-938.66666666667,341.33333333333,$# Theta = 60          AB24  48
0.0,2.143593539449,-49.863279495408,316.68384925381,$#                  AB24  49
-803.82916105197,876.19833108746,-341.33333333333,$# Theta = 75         AB24  50
0.0,-1.0,23.333333333333,-149.33333333333,384.0,$#                      AB24  51
-426.66666666667,170.66666666667])#    Theta = 90                       AB24  52
datastuff:#                                                             AB24  53
#                                                                       AB24  54
MUR=db:(EMU)*db:(RAD)*SCALE#           Mu*r for this sphere             AB24  55
#                                                                       AB24  56
DO J=1,7#                              Zero summation arrays            AB24  57
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB24  58
T(J)=0.; AJ(J)=0.; TJ(J)=0.; LM(J)=0.; PM(J)=0.#                        AB24  59
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB24  60
#                                                                       AB24  61
#  INTERPOLATE IN MUR FOR FIXED ANGLES OF THETA.                        AB24  62
#                                                                       AB24  63
DO J=1,7#                                                               AB24  64
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB24  65
DO N=1,7#                                                               AB24  66
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB24  67
AJ(J)=AJ(J)+KT((J-1)*7+N)*(MUR)**N#                                     AB24  68
TJ(J)=TJ(J)+N*KT((J-1)*7+N)*(MUR)**N#                                   AB24  69
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB24  70
AJ(J)=EXP(AJ(J))#                                                       AB24  71
TJ(J)=TJ(J)/EMU#                                                        AB24  72
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB24  73
#                                                                       AB24  74
#  GENERATE COEFFICIENTS FOR INTERPOLATION IN THETA                     AB24  75
#                                                                       AB24  76
DO M=1,7#                                                               AB24  77
DO J=1,7#                                                               AB24  78
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB24  79
LM(M)=LM(M)+CT((J-1)*7+M)*AJ(J)#                                        AB24  80
PM(M)=PM(M)+CT((J-1)*7+M)*AJ(J)*TJ(J)#                                  AB24  81
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB24  82
CALL AB03(1,T)#                                                         AB24  83
#                                                                       AB24  84
REPEAT#                                                                 AB24  85
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB24  86
CALL AB03(2,T)#                        Read reflection                  AB24  87
ifeq:(T(1),1.) BREAK#                  Stop in last reflection          AB24  88
ifeq:(T(2),1.) NEXT#                   Skip if outside limits           AB24  89
AST=0.0 ; ASTTB=0.0#                   Reset values to zero             AB24  90
SINT=T(4)*WAV#                         Sin(theta)                       AB24  91
DO M=1,7#                              Loop over coefficients           AB24  92
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB24  93
AST=AST+LM(M)*SINT**(2*M-2)#           Calculate astar                  AB24  94
ASTTB=ASTTB+PM(M)*SINT**(2*M-2)#       Calculate astar*tbar             AB24  95
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB24  96
T(1)=AST#                              Value of astar                   AB24  97
T(2)=ASTTB/AST#                        Value of tbar                    AB24  98
CALL AB03(3,T)#                        Pass to ab03                     AB24  99
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB24 100
CALL AB03(4,T)#                        Copy rest of file                AB24 101
readline:(AB241,NAB241)#               Read =end= line                  AB24 102
#                                                                       AB24 103
RETURN#                                                                 AB24 104
END#                                                                    AB24 105
systemheader:(AB25)#                   Apply cylindrical corrections    AB25   1
####################################################################    AB25   2
#                                                                  #    AB25   3
#                              AB25                                #    AB25   4
#                                                                  #    AB25   5
####################################################################    AB25   6
SUBROUTINE AB25#                       N. spadaccini oct 1990           AB25   7
include:(AACOMN)#                                                       AB25   8
include:(ABCOMN)#                                                       AB25   9
#                                                                       AB25  10
dp: MUR#                               Mu * r for this sphere           AB25  11
dp: SINT#                              Reflection sin(theta)            AB25  12
dp: AJ(7)#                             Tibball's a(mur,theta(j))        AB25  13
dp: TJ(7)#                             Tibball's t(mur,theta(j))        AB25  14
dp: LM(7)#                             Theta interpolation - astar      AB25  15
dp: PM(7)#                             Theta interpolation - astar*tb   AB25  16
dp: AST#                               Astar                            AB25  17
dp: ASTTB#                             Astar*tbar                       AB25  18
dp: T(8)#                              Temporary array                  AB25  19
INTEGER J,M,N#                                                          AB25  20
#                                                                       AB25  21
chardata:(AB251, END   )#                                               AB25  22
dbledata:(KT,[$#                       Tibball's k coefficients         AB25  23
.169765E1,-.578322D-1,-.147368D-1,.523993D-2,$#                         AB25  24
-.409582D-2,.131784D-2,-.145D-3,$#     Theta = 0                        AB25  25
.169765E1,-.819004D-1,-.195508D-1,.129345D-2,$#                         AB25  26
-.283494D-2,.127313D-2,-.148465D-3,$#  Theta = 15                       AB25  27
.169765E1,-.156508,-.228828D-1,-.123010D-1,$#                           AB25  28
.962488D-2,-.198809D-2,.144144D-3,$#   Theta = 30                       AB25  29
.169765E1,-.270478,-.273454D-1,.684421D-2,$#                            AB25  30
.750339D-2,-.302110D-2,.342218D-3,$#   Theta = 45                       AB25  31
.169765E1,-.403166,-.880718D-2,.406889D-1,$#                            AB25  32
-.112950D-1,.944685D-3,.314921D-4,$#   Theta = 60                       AB25  33
.169765E1,-.514973,.266370D-1,.613715D-1,$#                             AB25  34
-.293972D-1,.603560D-2,-.492057D-3,$#  Theta = 75                       AB25  35
.169765E1,-.558374,.414199D-1,.689630D-1,$#                             AB25  36
-.365556D-1,.809646D-2,-.705729D-3])#  Theta = 90                       AB25  37
dbledata:(CT,[$#                       Tibball's (c**-1) coefficients   AB25  38
1.0,-24.333333333333,172.66666666667,-533.33333333333,$#                AB25  39
810.66666666667,-597.33333333333,170.66666666667,$# Theta = 0           AB25  40
0.0,29.856406460551,-280.80338717126,963.31615074619,$#                 AB25  41
-1542.8375056147,1171.8016689125,-341.33333333333,$# Theta = 15         AB25  42
0.0,-8.0,162.66666666667,-730.66666666667,1344.0,$#                     AB25  43
-1109.3333333333,341.33333333333,$#    Theta = 30                       AB25  44
0.0,4.0,-89.333333333333,512.0,-1109.3333333333,1024.0,$#               AB25  45
-341.33333333333,$#                    Theta = 45                       AB25  46
0.0,-2.6666666666667,61.333333333333,-378.66666666667,$#                AB25  47
917.33333333333,-938.66666666667,341.33333333333,$# Theta = 60          AB25  48
0.0,2.143593539449,-49.863279495408,316.68384925381,$#                  AB25  49
-803.82916105197,876.19833108746,-341.33333333333,$# Theta = 75         AB25  50
0.0,-1.0,23.333333333333,-149.33333333333,384.0,$#                      AB25  51
-426.66666666667,170.66666666667])#    Theta = 90                       AB25  52
datastuff:#                                                             AB25  53
#                                                                       AB25  54
MUR=db:(EMU)*db:(RAD)*SCALE#           Mu*r for this sphere             AB25  55
DO J=1,7#                              Zero summation arrays            AB25  56
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB25  57
T(J)=0.; AJ(J)=0.; TJ(J)=0.; LM(J)=0.; PM(J)=0.#                        AB25  58
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB25  59
#                                                                       AB25  60
# CALCULATE VALUES OF ASTAR AND TBAR FOR FIXED THETA                    AB25  61
#                                                                       AB25  62
DO J=1,7#                                                               AB25  63
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB25  64
DO N=1,7#                                                               AB25  65
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB25  66
AJ(J)=AJ(J)+KT((J-1)*7+N)*(MUR)**N#                                     AB25  67
TJ(J)=TJ(J)+N*KT((J-1)*7+N)*(MUR)**N#                                   AB25  68
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB25  69
AJ(J)=EXP(AJ(J))#                                                       AB25  70
TJ(J)=TJ(J)/EMU#                                                        AB25  71
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB25  72
#                                                                       AB25  73
# CALCULATE COEFFICIENTS FOR INTERPOLATION IN THETA                     AB25  74
#                                                                       AB25  75
DO M=1,7#                                                               AB25  76
DO J=1,7#                                                               AB25  77
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB25  78
LM(M)=LM(M)+CT((J-1)*7+M)*AJ(J)#                                        AB25  79
PM(M)=PM(M)+CT((J-1)*7+M)*AJ(J)*TJ(J)#                                  AB25  80
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB25  81
CALL AB03(1,T)#                                                         AB25  82
#                                                                       AB25  83
REPEAT#                                                                 AB25  84
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB25  85
CALL AB03(2,T)#                        Read reflection                  AB25  86
ifeq:(T(1),1.) BREAK#                  Break if last reflection         AB25  87
ifeq:(T(2),1.) NEXT#                   Skip if outside limits           AB25  88
AST=0.0 ; ASTTB=0.0#                   Zero values                      AB25  89
SINT=T(4)*WAV#                         Reflection sin(theta)            AB25  90
DO M=1,7#                              Loop over interpolation coeffs   AB25  91
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB25  92
AST=AST+LM(M)*SINT**(2*M-2)#           Sum astar                        AB25  93
ASTTB=ASTTB+PM(M)*SINT**(2*M-2)#       Sum astar*tbar                   AB25  94
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB25  95
T(1)=AST#                              Astar                            AB25  96
T(2)=ASTTB/AST#                        Tbar                             AB25  97
CALL AB03(3,T)#                        Pass to ab03                     AB25  98
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB25  99
CALL AB03(4,T)#                        Copy rest of file                AB25 100
readline:(AB251,NAB251)#               Read =end= line                  AB25 101
#                                                                       AB25 102
RETURN#                                                                 AB25 103
END#                                                                    AB25 104
systemheader:(AB31)#                   Calculate absorption correctio   AB31   1
####################################################################    AB31   2
#                                                                  #    AB31   3
#                              AB31                                #    AB31   4
#                                                                  #    AB31   5
####################################################################    AB31   6
SUBROUTINE AB31#                                                        AB31   7
IMPLICIT INTEGER (I-N)#                                                 AB31   8
include:(AACOMN)#                                                       AB31   9
include:(ABCOMN)#                                                       AB31  10
#                                                                       AB31  11
## J.M. STEWART, J.R. HOLDEN, NOVEMBER 1980                             AB31  12
## GEORGE DAVENPORT, JANUARY 1982                                       AB31  13
## T. MASLEN, JANUARY 1984                                              AB31  14
#                                                                       AB31  15
dp: COR,COR2#                          The absorption correction        AB31  16
dp: EUR#                               Coarse table offset              AB31  17
dp: EXPFAC#                            4*mu                             AB31  18
dp: EXPFA2#                            4*mu2 (for tbar)                 AB31  19
dp: GMIN#                              Incoming path length             AB31  20
dp: HMIN#                              Outgoing path length             AB31  21
INTEGER I#                             Loop index                       AB31  22
INTEGER ICOUNT#                        Count beam1*normal>0             AB31  23
INTEGER IMARK#                         Pointer into qx                  AB31  24
INTEGER IP#                            Pointer                          AB31  25
INTEGER ISTOP#                                                          AB31  26
INTEGER JCOUNT#                        Count beam2*normal>0             AB31  27
INTEGER JMARK#                         Pointer into qx                  AB31  28
INTEGER JP#                            Pointer                          AB31  29
INTEGER K#                             Loop index                       AB31  30
INTEGER KK#                            Pointer to face                  AB31  31
INTEGER M2#                            Loop index                       AB31  32
INTEGER LUR#                           Fine table pointer               AB31  33
INTEGER MARKE1#                        Beginning of coarse table        AB31  34
INTEGER MARKE2#                        Beginning of fine table          AB31  35
INTEGER METHEX#                        Method of exponentiation         AB31  36
INTEGER MUR#                           Coarse table pointer             AB31  37
dp: TEMP,T(8)#                         Temporary variable               AB31  38
#                                                                       AB31  39
chardata:(AB211," Exponentials by Fortran function")#                   AB31  40
chardata:(AB212," Exponentials by table lookup")#                       AB31  41
datastuff:#                                                             AB31  42
#                                                                       AB31  43
# SET UP EXPONENTIAL TABLES IF NECESSARY                                AB31  44
# --------------------------------------                                AB31  45
#                                                                       AB31  46
METHEX=no:#                            Default is no tables             AB31  47
IF(LINID==1)#                          If grid line entered             AB31  48
ifpos:(BUFIN(4)) METHEX=yes:#          Table lookup                     AB31  49
IF(METHEX==yes:)#                      Tables needed                    AB31  50
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB31  51
writeline:(3,AB212,NAB212,3,3)#        Write message                    AB31  52
MARKE1=ISTOP2#                         Start of coarse table            AB31  53
MARKE2=MARKE1+64#                      Start of fine table              AB31  54
incrqq:(IMARK,MARKE2+512,AB3101)#      Get memory                       AB31  55
FOR(I=MARKE1;I<MARKE2;I=I+1)#          Loop over coarse table           AB31  56
QQ(I+1)=EXP(-0.25*db:(FLOAT(I-MARKE1)))# Table element                  AB31  57
FOR(I=MARKE2;I<IMARK;I=I+1)#           Loop over fine table             AB31  58
QQ(I+1)=EXP(-0.25*db:(FLOAT(I-MARKE2))/512.)# Table element             AB31  59
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB31  60
ELSE#                                  Library routines used            AB31  61
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB31  62
IMARK=ISTOP2#                          If no tables                     AB31  63
writeline:(3,AB211,NAB211,3,3)#        Write message                    AB31  64
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB31  65
JMARK=IMARK+2*NFACES#                                                   AB31  66
incrqq:(ISTOP,JMARK+2*NFACES,AB3101)#  Get memory                       AB31  67
#                                                                       AB31  68
# CALCULATE ABSORPTION CORRECTION                                       AB31  69
# -------------------------------                                       AB31  70
#                                                                       AB31  71
EXPFAC=4.0*EMU#                                                         AB31  72
EXPFA2=4.0*EMU2#                                                        AB31  73
CALL AB03(1,T)#                        Set up lrrefl: i/o               AB31  74
REPEAT#                                Loop over reflections            AB31  75
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB31  76
CALL AB03(2,T)#                        Read reflection                  AB31  77
IF(NINT(T(1))==1) BREAK#               Break at end of lr               AB31  78
IF(NINT(T(2))!=1)#                     Treat this reflection            AB31  79
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB31  80
#                                                                       AB31  81
# STORE AND COUNT BEAM VECTORS*FACE NORMALS IF POSITIVE                 AB31  82
# -----------------------------------------------------                 AB31  83
#                                                                       AB31  84
ICOUNT=0#                              Count beam1*normal>0             AB31  85
JCOUNT=0#                              Count beam2*normal>0             AB31  86
FOR(KK=MARKF;KK<ISTOPF;KK=KK+NSTEPF)#  Loop over faces                  AB31  87
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB31  88
IP=IMARK+ICOUNT#                       Pointer into imark               AB31  89
QQ(IP+1)=T(3)*QQ(KK+1)+T(4)*QQ(KK+2)+T(5)*QQ(KK+3)# Dot product         AB31  90
IF(QQ(IP+1)>0.0)#                      Positive                         AB31  91
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB31  92
JP=IP+NFACES#                          Pointer                          AB31  93
QQ(JP+1)=db:(FLOAT(KK))#                                                AB31  94
ICOUNT=ICOUNT+1#                                                        AB31  95
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB31  96
IP=JMARK+JCOUNT#                                                        AB31  97
QQ(IP+1)=T(6)*QQ(KK+1)+T(7)*QQ(KK+2)+T(8)*QQ(KK+3)#                     AB31  98
IF(QQ(IP+1)>0.0)#                                                       AB31  99
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB31 100
JP=IP+NFACES#                                                           AB31 101
QQ(JP+1)=db:(FLOAT(KK))#                                                AB31 102
JCOUNT=JCOUNT+1#                                                        AB31 103
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB31 104
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB31 105
#                                                                       AB31 106
# ABSORPTION CORRECTION AND TBAR                                        AB31 107
# ------------------------------                                        AB31 108
#                                                                       AB31 109
COR=0.0#                                                                AB31 110
COR2=0.0#                              Summation (for tbar)             AB31 111
FOR(M2=MARK2;M2<ISTOP2;M2=M2+NSTEP2)#  Loop over gaussian grid          AB31 112
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB31 113
GMIN=9999.#                            Primary path length              AB31 114
HMIN=9999.#                            Diffracted path length           AB31 115
DO K=1,ICOUNT#                         Loop over dot products           AB31 116
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB31 117
IP=IMARK+K#                            Pointer                          AB31 118
JP=IP+NFACES#                          Pointer                          AB31 119
KK=NINT(QQ(JP))#                                                        AB31 120
TEMP=(QQ(KK+4)-QQ(KK+1)*QQ(M2+1)-QQ(KK+2)*QQ(M2+2)$#                    AB31 121
-QQ(KK+3)*QQ(M2+3))/QQ(IP)#                                             AB31 122
IF(TEMP<GMIN) GMIN=TEMP#                                                AB31 123
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB31 124
DO K=1,JCOUNT#                                                          AB31 125
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB31 126
IP=JMARK+K#                                                             AB31 127
JP=IP+NFACES#                                                           AB31 128
KK=NINT(QQ(JP))#                                                        AB31 129
TEMP=(QQ(KK+4)-QQ(KK+1)*QQ(M2+1)-QQ(KK+2)*QQ(M2+2)$#                    AB31 130
-QQ(KK+3)*QQ(M2+3))/QQ(IP)#                                             AB31 131
IF(TEMP<HMIN) HMIN=TEMP#                                                AB31 132
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB31 133
#                                                                       AB31 134
IF(METHEX==no:)#                                                        AB31 135
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB31 136
COR=COR + QQ(M2+4)*EXP(-(GMIN+HMIN)*EMU)#                               AB31 137
IF(ITBAR==yes:)#                       If tbar is to be calculated      AB31 138
COR2=COR2 + QQ(M2+4)*EXP(-(GMIN+HMIN)*EMU2)# Accumulate second sum      AB31 139
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB31 140
ELSE#                                                                   AB31 141
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 12 AB31 142
EUR=MIN(64.D0,EXPFAC*(GMIN+HMIN))#                                      AB31 143
MUR=INT(EUR) + MARKE1#                 Make the coarse table pointe     AB31 144
LUR=INT(MOD(EUR,1.0D0)*512.0)+MARKE2#  Find the fine table entry        AB31 145
COR=COR + QQ(M2+4)*QQ(LUR+1)*QQ(MUR+1)# See long comment in ab31        AB31 146
IF(ITBAR==yes:)#                       Do second summation for tbar     AB31 147
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 13 AB31 148
EUR=MIN(64.D0,EXPFA2*(GMIN+HMIN))#                                      AB31 149
MUR=INT(EUR) + MARKE1#                 Make the coarse table pointe     AB31 150
LUR=INT(MOD(EUR,1.0D0)*512.0)+MARKE2#  Find the fine table entry        AB31 151
COR2=COR2 + QQ(M2+4)*QQ(LUR+1)*QQ(MUR+1)#                               AB31 152
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 13 AB31 153
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 12 AB31 154
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB31 155
#                                                                       AB31 156
T(1)=VOL/COR#                                                           AB31 157
IF(ITBAR==yes:) T(2)=VOL/COR2#                                          AB31 158
CALL AB03(3,T)#                                                         AB31 159
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB31 160
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB31 161
CALL AB03(4,T)#                                                         AB31 162
RETURN#                                                                 AB31 163
END#                                                                    AB31 164
systemheader:(AB40)#                   Analytical method                AB40   1
####################################################################    AB40   2
#                                                                  #    AB40   3
#                              AB40                                #    AB40   4
#                                                                  #    AB40   5
####################################################################    AB40   6
SUBROUTINE AB40#                                                        AB40   7
IMPLICIT INTEGER (I-N)#                                                 AB40   8
include:(AACOMN)#                                                       AB40   9
include:(ABCOMN)#                                                       AB40  10
#                                                                       AB40  11
## PROGRAM FOR ANALYTICAL ABSORPTION CORRECTION                         AB40  12
## GEORGE DAVENPORT, MARCH 1982                                         AB40  13
## T. MASLEN, JANUARY 1984                                              AB40  14
#                                                                       AB40  15
dp: DOT#                               Beam vector*face normal          AB40  16
INTEGER I,J,K#                         Loop indices                     AB40  17
INTEGER II,JJ#                         Qx pointers                      AB40  18
INTEGER FIRST#                                                          AB40  19
dp: TEMP(10)#                                                           AB40  20
chardata:(CVOL,' Crystal volume                 ')#                     AB40  21
datastuff:#                                                             AB40  22
#                                                                       AB40  23
DBZ=db:(0.0)#                                                           AB40  24
#                                                                       AB40  25
# SET UP LRREFL: I/O                                                    AB40  26
# ------------------                                                    AB40  27
#                                                                       AB40  28
FIRST=yes:#                            Set for first reflection         AB40  29
CALL AB03(1,TEMP)#                     Set up for lrrefl i/o            AB40  30
#                                                                       AB40  31
# LOOP OVER REFLECTIONS                                                 AB40  32
# ---------------------                                                 AB40  33
#                                                                       AB40  34
REPEAT#                                Loop over reflections            AB40  35
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB40  36
VOL=0.0#                               Crystal volume                   AB40  37
DVOL=DBZ#                                                               AB40  38
ABSF=DBZ#                              Absorption factor                AB40  39
ABSF2=DBZ#                             Second summation (for tbar)      AB40  40
CALL AB03(2,TEMP)#                     Angles and beam vectors          AB40  41
ifeq:(TEMP(1),1.) BREAK#               Test end of lrrefl:              AB40  42
ifeq:(TEMP(2),1.) NEXT#                Skip this reflection             AB40  43
#                                                                       AB40  44
# DETERMINE WHETHER FACES ARE LIT OR SHADED, FOR EACH BEAM              AB40  45
# --------------------------------------------------------              AB40  46
#                                                                       AB40  47
FOR(I=MARKF;I<ISTOPF;I=I+NSTEPF)#      Loop over faces                  AB40  48
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB40  49
DOT=TEMP(3)*QQ(I+1)+TEMP(4)*QQ(I+2)+TEMP(5)*QQ(I+3)#                    AB40  50
IF(DOT>dsmall:) QQ(I+5)= 1.0#          Positive -- lit face             AB40  51
ELSE            QQ(I+5)=-1.0#          Otherwise -- shaded face         AB40  52
DOT=TEMP(6)*QQ(I+1)+TEMP(7)*QQ(I+2)+TEMP(8)*QQ(I+3)#                    AB40  53
IF(DOT>dsmall:) QQ(I+6)= 1.0#          Positive -- lit face             AB40  54
ELSE            QQ(I+6)=-1.0#          Otherwise -- shaded face         AB40  55
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB40  56
#                                                                       AB40  57
# DETERMINE WHETHER EDGES ARE LIT OR SHADED, FOR EACH BEAM              AB40  58
# --------------------------------------------------------              AB40  59
#                                                                       AB40  60
FOR(I=MARKED;I<STOPED;I=I+STEPED)#     Loop over edges                  AB40  61
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB40  62
II=NINT(QQ(I+1))#                      Address of first face            AB40  63
JJ=NINT(QQ(I+2))#                      Address of second face           AB40  64
QQ(I+3)=0.5*(QQ(II+5)+QQ(JJ+5))#       Lit or shaded for beam1          AB40  65
QQ(I+4)=0.5*(QQ(II+6)+QQ(JJ+6))#       Lit or shaded for beam2          AB40  66
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB40  67
#                                                                       AB40  68
# LOOP OVER FACES LIT BY BEAM1                                          AB40  69
# ----------------------------                                          AB40  70
#                                                                       AB40  71
MARK1=STOPED#                          Start of projected planes        AB40  72
FOR(I=MARKF;I<ISTOPF;I=I+NSTEPF)#      Loop over faces                  AB40  73
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB40  74
IF(NINT(QQ(I+5))!=1) NEXT#             Skip face not lit by beam1       AB40  75
#                                                                       AB40  76
# PROJECT LIT EDGES ALONG BEAM1                                         AB40  77
# -----------------------------                                         AB40  78
#                                                                       AB40  79
II=MARK1#                              Running pointer                  AB40  80
MARK2=MARK1#                           End of mark1 array               AB40  81
FOR(J=MARKED;J<STOPED;J=J+STEPED)#     Loop over edges                  AB40  82
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB40  83
IF(NINT(QQ(J+3))!=1) NEXT#             Edge not lit by beam1            AB40  84
IF((I!=NINT(QQ(J+1)))&(I!=NINT(QQ(J+2)))) NEXT# Edge not on face        AB40  85
incrqq:(MARK2,MARK2+4,AB4001)#         Get more mem                     AB40  86
CALL AB01(TEMP(3),QQ(J+5),QQ(II+1))#   Find plane normal                AB40  87
QQ(II+4)=QQ(J+8)*QQ(II+1)+QQ(J+9)*QQ(II+2)+QQ(J+10)*QQ(II+3)# Dist      AB40  88
IF(QQ(II+4)<0.0)#                      Reverse negative distance        AB40  89
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB40  90
QQ(II+1)=-QQ(II+1)#                    Reverse normal                   AB40  91
QQ(II+2)=-QQ(II+2)#                    Reverse normal                   AB40  92
QQ(II+3)=-QQ(II+3)#                    Reverse normal                   AB40  93
QQ(II+4)=-QQ(II+4)#                    Make distance positive           AB40  94
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB40  95
II=MARK2#                              Add to pointer                   AB40  96
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB40  97
#                                                                       AB40  98
# LOOP OVER FACES LIT BY BEAM2                                          AB40  99
# ----------------------------                                          AB40 100
#                                                                       AB40 101
FOR(J=MARKF;J<ISTOPF;J=J+NSTEPF)#      Loop over faces                  AB40 102
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB40 103
IF(NINT(QQ(J+6))!=1) NEXT#             Skip face not lit by beam2       AB40 104
JJ=MARK2#                              Running pointer                  AB40 105
MARKH=MARK2#                           End of mark2 array               AB40 106
#                                                                       AB40 107
# PROJECT LIT EDGES ALONG BEAM2                                         AB40 108
# -----------------------------                                         AB40 109
#                                                                       AB40 110
FOR(K=MARKED;K<STOPED;K=K+STEPED)#     Loop over edges                  AB40 111
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB40 112
IF(NINT(QQ(K+4))!=1) NEXT#             Edge not lit by beam2            AB40 113
IF((J!=NINT(QQ(K+1)))&(J!=NINT(QQ(K+2)))) NEXT# Edge not on face        AB40 114
incrqq:(MARKH,MARKH+4,AB4001)#         Get more mem                     AB40 115
CALL AB01(TEMP(6),QQ(K+5),QQ(JJ+1))#   Find plane normal                AB40 116
QQ(JJ+4)=QQ(K+8)*QQ(JJ+1)+QQ(K+9)*QQ(JJ+2)+QQ(K+10)*QQ(JJ+3)# Dist      AB40 117
IF(QQ(JJ+4)<0.0)#                      Reverse negative distance        AB40 118
$(#                                    4>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB40 119
QQ(JJ+1)=-QQ(JJ+1)#                    Reverse normal                   AB40 120
QQ(JJ+2)=-QQ(JJ+2)#                    Reverse normal                   AB40 121
QQ(JJ+3)=-QQ(JJ+3)#                    Reverse normal                   AB40 122
QQ(JJ+4)=-QQ(JJ+4)#                    Make distance positive           AB40 123
$)#                                    4<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB40 124
JJ=MARKH#                              Add to pointer                   AB40 125
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB40 126
#                                                                       AB40 127
# DETERMINE THE VERTICES OF THE HOWELL'S POLYHEDRON                     AB40 128
# -------------------------------------------------                     AB40 129
#                                                                       AB40 130
CALL AB41(I,J,TEMP(3),TEMP(6))#        Next stage                       AB40 131
#                                                                       AB40 132
# IF POLYHEDRON IS NOT DEGENERATE, CALCULATE TRANSMITTANCE              AB40 133
# --------------------------------------------------------              AB40 134
#                                                                       AB40 135
NPOINT=(ISTOPH-MARKH)/4#               Number of hp vertices            AB40 136
IF(NPOINT>=4) CALL AB43#               Calculate transmittance          AB40 137
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB40 138
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB40 139
IF(FIRST==yes:)#                       For the first reflection         AB40 140
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB40 141
FIRST=no:#                             Print only once                  AB40 142
BUFOT(1)=1000.*DVOL/SCALE**3#          Volume                           AB40 143
ncodefld:(BUFOT,1,CVOL,321551.,1)#     Encode items                     AB40 144
writeline:(1,CVOL,NCVOL,3,3)#          Write out items                  AB40 145
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB40 146
TEMP(1)=DVOL/ABSF#                                                      AB40 147
IF(ITBAR==yes:) TEMP(2)=DVOL/ABSF2#                                     AB40 148
CALL AB03(3,TEMP)#                     Send it to lrrefl:               AB40 149
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB40 150
CALL AB03(4,TEMP)#                     Finish off the bdf               AB40 151
RETURN#                                                                 AB40 152
END#                                                                    AB40 153
systemheader:(AB41)#                   Determine hp vertices            AB41   1
####################################################################    AB41   2
#                                                                  #    AB41   3
#                              AB41                                #    AB41   4
#                                                                  #    AB41   5
####################################################################    AB41   6
SUBROUTINE AB41(I,J,DOD,DD)#                                            AB41   7
IMPLICIT INTEGER (I-N)#                                                 AB41   8
include:(AACOMN)#                                                       AB41   9
include:(ABCOMN)#                                                       AB41  10
#                                                                       AB41  11
## DETERMINE VERTICES OF A HOWELL'S POLYHEDRON                          AB41  12
## GEORGE DAVENPORT, JANUARY 1982                                       AB41  13
## T. MASLEN, JANUARY 1984                                              AB41  14
#                                                                       AB41  15
dp: DOT1,DOT2#                         Beam*normal                      AB41  16
INTEGER I,J#                           Addresses of lit faces           AB41  17
INTEGER II#                            Running pointer into markh       AB41  18
INTEGER ISON#                          Flag polyhedron vertex           AB41  19
INTEGER K,L,M#                         Loop indices                     AB41  20
dp: R1,R2#                             D-ax-by-cz                       AB41  21
dp: DOD(3),DD(3)#                                                       AB41  22
#                                                                       AB41  23
# ------------------------------------------------------------------    AB41  24
# THE VERTICES OF THE HOWELL'S POLYHEDRON ARE OBTAINED BY TAKING        AB41  25
# THE INTERSECTIONS OF COMBINATIONS OF 3 FACES FROM THE ARRAYS          AB41  26
# MARKF, MARK1, AND MARK2.  CORRELATION TO ALCOCK'S METHOD:             AB41  27
#                                                                       AB41  28
# FROM MARKF / FROM MARK1 / FROM MARK2 / ALCOCK'S DESIGNATION           AB41  29
# -----------------------------------------------------------           AB41  30
#      3            0            0       S (CRYSTAL VERTEX)             AB41  31
#      2            1            0       S01                            AB41  32
#      2            0            1       S02                            AB41  33
#      1            2            0       S10                            AB41  34
#      1            1            1       S00                            AB41  35
#      1            0            2       S20                            AB41  36
#      0            3            0       IMPOSSIBLE                     AB41  37
#      0            2            1       S12                            AB41  38
#      0            1            2       S21                            AB41  39
#      0            0            3       IMPOSSIBLE                     AB41  40
#-------------------------------------------------------------------    AB41  41
#                                                                       AB41  42
# FIND CRYSTAL VERTICES THAT ARE ALSO POLYHEDRON VERTICES               AB41  43
# -------------------------------------------------------               AB41  44
#                                                                       AB41  45
II=MARKH#                              Running pointer                  AB41  46
ISTOPH=MARKH#                          End of markh array               AB41  47
FOR(K=MARKV;K<ISTOPV;K=K+NSTEPV)#      Loop over vertices               AB41  48
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB41  49
CALL AB42(I,J,K,ISON)#                 Test if inside                   AB41  50
IF(ISON==no:) NEXT#                    Not polyhedron vertex            AB41  51
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more mem                     AB41  52
movedoub:(QQ,K+1,QQ,II+1,3,1)#         Store vertex in markh            AB41  53
II=ISTOPH#                             Add to pointer                   AB41  54
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB41  55
#                                                                       AB41  56
# 2 F AND 1 P1, 2 F AND 1 P2 (ALCOCK'S S01 AND S02)                     AB41  57
# -------------------------------------------------                     AB41  58
#                                                                       AB41  59
IF(MARKH>MARK1)#                       At least 1 projected plane       AB41  60
FOR(K=MARKF;K<(ISTOPF-NSTEPF);K=K+NSTEPF)# Loop over faces              AB41  61
FOR(L=K+NSTEPF;L<ISTOPF;L=L+NSTEPF)#   Loop over faces                  AB41  62
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB41  63
#                                                                       AB41  64
# 2 F AND 1 P1 (S01)                                                    AB41  65
# ------------------                                                    AB41  66
#                                                                       AB41  67
IF((I!=K)&(I!=L))#                                                      AB41  68
IF((NINT(QQ(K+5))!=1)&(NINT(QQ(L+5))!=1))# Neither face lit 1           AB41  69
FOR(M=MARK1;M<MARK2;M=M+4)#            Loop over projected planes       AB41  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB41  71
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more memory                  AB41  72
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41  73
IF(ISON==no:) NEXT#                    Point outside crystal            AB41  74
CALL AB42(I,J,II,ISON)#                Check if polyhedron vertex       AB41  75
IF(ISON==yes:) II=II+4#                Save this point                  AB41  76
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB41  77
#                                                                       AB41  78
# 2 F AND 1 P2 (S02)                                                    AB41  79
# ------------------                                                    AB41  80
#                                                                       AB41  81
IF((J!=K)&(J!=L))#                                                      AB41  82
IF((NINT(QQ(K+6))!=1)&(NINT(QQ(L+6))!=1))# Neither face lit 2           AB41  83
FOR(M=MARK2;M<MARKH;M=M+4)#            Loop over projected planes       AB41  84
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB41  85
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more memory                  AB41  86
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41  87
IF(ISON==no:) NEXT#                    Point outside crystal            AB41  88
CALL AB42(I,J,II,ISON)#                Check if polyhedron vertex       AB41  89
IF(ISON==yes:) II=II+4#                Save this point                  AB41  90
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB41  91
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB41  92
#                                                                       AB41  93
# 1 F AND 2 P1, 1 F AND 1 P1 AND 1 P2, 1 F AND 2 P2 -- S10,S00,S20      AB41  94
# ----------------------------------------------------------------      AB41  95
#                                                                       AB41  96
IF(MARKH>(MARK1+4))#                                                    AB41  97
FOR(K=MARKF;K<ISTOPF;K=K+NSTEPF)#      Loop over faces                  AB41  98
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB41  99
#                                                                       AB41 100
# 1 F AND 2 P1 (S10)                                                    AB41 101
# ------------------                                                    AB41 102
#                                                                       AB41 103
IF(I!=K)#                                                               AB41 104
IF(MARK2>(MARK1+4))#                                                    AB41 105
FOR(L=MARK1;L<(MARK2-4);L=L+4)#        Loop over projected 1 planes     AB41 106
FOR(M=L+4;M<MARK2;M=M+4)#              Loop over projected 1 planes     AB41 107
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB41 108
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more memory                  AB41 109
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41 110
IF(ISON==no:) NEXT#                    If not inside crystal            AB41 111
CALL AB42(I,J,II,ISON)#                Check if inside polyhedron       AB41 112
IF(ISON==yes:) II=II+4#                If inside polyhedron             AB41 113
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB41 114
#                                                                       AB41 115
# 1 F AND 1 P1 AND 1 P2 (S00)                                           AB41 116
# ---------------------------                                           AB41 117
#                                                                       AB41 118
IF((I!=K)&(J!=K))#                                                      AB41 119
IF((MARK2>MARK1)&(MARKH>MARK2))#                                        AB41 120
FOR(L=MARK1;L<MARK2;L=L+4)#            Loop over projected 1 planes     AB41 121
FOR(M=MARK2;M<MARKH;M=M+4)#            Loop over projected 2 planes     AB41 122
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB41 123
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more memory                  AB41 124
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41 125
IF(ISON==no:) NEXT#                    If not inside crystal            AB41 126
CALL AB42(I,J,II,ISON)#                Check if inside polyhedron       AB41 127
IF(ISON==yes:) II=II+4#                If inside polyhedron             AB41 128
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB41 129
#                                                                       AB41 130
# 1 F AND 2 P2 (S20)                                                    AB41 131
# ------------------                                                    AB41 132
#                                                                       AB41 133
IF(J!=K)#                                                               AB41 134
IF(MARKH>(MARK2+4))#                                                    AB41 135
FOR(L=MARK2;L<(MARKH-4);L=L+4)#        Loop over projected 2 planes     AB41 136
FOR(M=L+4;M<MARKH;M=M+4)#              Loop over projected 2 planes     AB41 137
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB41 138
incrqq:(ISTOPH,ISTOPH+40,AB4101)#      Get more memory                  AB41 139
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41 140
IF(ISON==no:) NEXT#                    If not inside crystal            AB41 141
CALL AB42(I,J,II,ISON)#                Check if inside polyhedron       AB41 142
IF(ISON==yes:) II=II+4#                If inside polyhedron             AB41 143
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB41 144
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB41 145
#                                                                       AB41 146
# 2 P1 AND 1 P2 (S12)                                                   AB41 147
# -------------------                                                   AB41 148
#                                                                       AB41 149
IF((MARK2>(MARK1+4))&(MARKH>MARK2))#                                    AB41 150
FOR(K=MARK1;K<(MARK2-4);K=K+4)#        Loop over projected 1 planes     AB41 151
FOR(L=K+4;L<MARK2;L=L+4)#              Loop over projected 1 planes     AB41 152
FOR(M=MARK2;M<MARKH;M=M+4)#            Loop over projected 2 planes     AB41 153
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 9  AB41 154
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more memory                  AB41 155
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41 156
IF(ISON==no:) NEXT#                    If not inside crystal            AB41 157
CALL AB42(I,J,II,ISON)#                Check if inside polyhedron       AB41 158
IF(ISON==yes:) II=II+4#                If inside polyhedron             AB41 159
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 9  AB41 160
#                                                                       AB41 161
# 1 P1 AND 2 P2 (S21)                                                   AB41 162
# -------------------                                                   AB41 163
#                                                                       AB41 164
IF((MARK2>MARK1)&(MARKH>(MARK2+4)))#                                    AB41 165
FOR(K=MARK1;K<MARK2;K=K+4)#            Loop over projected 1 planes     AB41 166
FOR(L=MARK2;L<(MARKH-4);L=L+4)#        Loop over projected 2 planes     AB41 167
FOR(M=L+4;M<MARKH;M=M+4)#              Loop over projected 2 planes     AB41 168
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 10 AB41 169
incrqq:(ISTOPH,ISTOPH+4,AB4101)#       Get more memory                  AB41 170
CALL AB02(QQ(K+1),QQ(L+1),QQ(M+1),QQ(II+1),ISON)# Find intersection     AB41 171
IF(ISON==no:) NEXT#                    If not inside crystal            AB41 172
CALL AB42(I,J,II,ISON)#                Check if inside polyhedron       AB41 173
IF(ISON==yes:) II=II+4#                If inside polyhedron             AB41 174
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 10 AB41 175
ISTOPH=II#                             End of markh array               AB41 176
#                                                                       AB41 177
# CALCULATE OPTICAL PATH LENGTHS                                        AB41 178
# ------------------------------                                        AB41 179
#                                                                       AB41 180
FOR(K=MARKH;K<ISTOPH;K=K+4)#                                            AB41 181
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 11 AB41 182
R1=QQ(I+4)-QQ(I+1)*QQ(K+1)-QQ(I+2)*QQ(K+2)-QQ(I+3)*QQ(K+3)#             AB41 183
DOT1=DOD(1)*QQ(I+1)+DOD(2)*QQ(I+2)+DOD(3)*QQ(I+3)#                      AB41 184
R2=QQ(J+4)-QQ(J+1)*QQ(K+1)-QQ(J+2)*QQ(K+2)-QQ(J+3)*QQ(K+3)#             AB41 185
DOT2=DD(1)*QQ(J+1)+DD(2)*QQ(J+2)+DD(3)*QQ(J+3)#                         AB41 186
QQ(K+4)=EMU*(dabs:(R1/DOT1)+dabs:(R2/DOT2))#                            AB41 187
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 11 AB41 188
RETURN#                                                                 AB41 189
END#                                                                    AB41 190
systemheader:(AB42)#                   Verify hp vertex                 AB42   1
####################################################################    AB42   2
#                                                                  #    AB42   3
#                               AB42                               #    AB42   4
#                                                                  #    AB42   5
####################################################################    AB42   6
SUBROUTINE AB42(II,JJ,KK,ISON)#                                         AB42   7
IMPLICIT INTEGER (I-N)#                                                 AB42   8
include:(AACOMN)#                                                       AB42   9
include:(ABCOMN)#                                                       AB42  10
#                                                                       AB42  11
## CHECK FOR HOWELL'S POLYHEDRON VERTEX                                 AB42  12
## GEORGE DAVENPORT, MARCH 1982                                         AB42  13
## T. MASLEN, JANUARY 1984                                              AB42  14
#                                                                       AB42  15
dp: DT1,DT2,DT3#                       Coordinate differences           AB42  16
INTEGER I,N#                           Loop index                       AB42  17
INTEGER II,JJ#                         Addresses of lit faces           AB42  18
INTEGER ISON#                          Flag point in polyhedron         AB42  19
INTEGER KK#                            Address of possible vertex       AB42  20
dp: DBT1,DBT2,TEST#                                                     AB42  21
dp: DQX(4),DQP(4)#                                                      AB42  22
#                                                                       AB42  23
# CHECK POSSIBLE NEW HP VERTEX FOR DUPLICATION                          AB42  24
# --------------------------------------------                          AB42  25
#                                                                       AB42  26
ISON=yes:#                             Assume no duplication            AB42  27
FOR(I=MARKH;I<KK;I=I+4)#               Loop over hp vertices            AB42  28
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB42  29
DT1=dabs:(QQ(KK+1)-QQ(I+1))#           X difference                     AB42  30
DT2=dabs:(QQ(KK+2)-QQ(I+2))#           Y difference                     AB42  31
DT3=dabs:(QQ(KK+3)-QQ(I+3))#           Z difference                     AB42  32
IF((DT1+DT2+DT3)>DITES) NEXT#          Not duplicate vertex             AB42  33
ISON=no:#                              Flag point is no good            AB42  34
BREAK#                                 Stop testing                     AB42  35
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB42  36
IF(ISON==no:) GOTO 999#                Avoid unnecessary tests          AB42  37
#                                                                       AB42  38
# CHECK FOR POINT OUTSIDE HOWELLS POLYHEDRON                            AB42  39
# ------------------------------------------                            AB42  40
#                                                                       AB42  41
FOR(I=MARK1;I<MARK2;I=I+4)#            Loop over projected planes       AB42  42
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB42  43
DO N=1,4; DQX(N)=QQ(I+N)#                                               AB42  44
DO N=1,4; DQP(N)=QQ(KK+N)#                                              AB42  45
DBT1=DQX(4)-DQX(1)*DQP(1)-DQX(2)*DQP(2)-DQX(3)*DQP(3)#                  AB42  46
IF(dabs:(DBT1)<DITES) NEXT#                                             AB42  47
DO N=1,3; DQP(N)=QQ(II+6+N)#                                            AB42  48
DBT2=DQX(4)-DQX(1)*DQP(1)-DQX(2)*DQP(2)-DQX(3)*DQP(3)#                  AB42  49
TEST=DBT1*DBT2#                                                         AB42  50
IF(TEST>-DBTES) NEXT#                  Same side of plane               AB42  51
ISON=no:#                              Flag point no good               AB42  52
BREAK#                                 Quit testing                     AB42  53
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB42  54
IF(ISON==no:) GOTO 999#                                                 AB42  55
FOR(I=MARK2;I<MARKH;I=I+4)#            Loop over projected planes       AB42  56
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB42  57
DO N=1,4; DQX(N)=QQ(I+N)#                                               AB42  58
DO N=1,4; DQP(N)=QQ(KK+N)#                                              AB42  59
DBT1=DQX(4)-DQX(1)*DQP(1)-DQX(2)*DQP(2)-DQX(3)*DQP(3)#                  AB42  60
IF(dabs:(DBT1)<DITES) NEXT#                                             AB42  61
DO N=1,4; DQP(N)=QQ(JJ+6+N)#                                            AB42  62
DBT2=DQX(4)-DQX(1)*DQP(1)-DQX(2)*DQP(2)-DQX(3)*DQP(3)#                  AB42  63
TEST=DBT1*DBT2#                                                         AB42  64
IF(TEST>-DBTES) NEXT#                  Same side of plane               AB42  65
ISON=no:#                              Flag point no good               AB42  66
BREAK#                                 Quit testing                     AB42  67
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB42  68
999 CONTINUE#                                                           AB42  69
RETURN#                                                                 AB42  70
END#                                                                    AB42  71
systemheader:(AB43)#                   Divide into tetrahedra           AB43   1
####################################################################    AB43   2
#                                                                  #    AB43   3
#                               AB43                               #    AB43   4
#                                                                  #    AB43   5
####################################################################    AB43   6
SUBROUTINE AB43#                                                        AB43   7
IMPLICIT INTEGER (I-N)#                                                 AB43   8
include:(AACOMN)#                                                       AB43   9
include:(ABCOMN)#                                                       AB43  10
#                                                                       AB43  11
## DIVIDE A POLYHEDRON INTO TETRAHEDRA                                  AB43  12
## GEORGE DAVENPORT, APRIL 1982                                         AB43  13
#                                                                       AB43  14
INTEGER I,J,K#                         Loop indices                     AB43  15
INTEGER IC#                            Count of switches                AB43  16
INTEGER II,JJ#                         Pointers into markt              AB43  17
INTEGER IP1,IP2,IP3,IP4#               Pointers to vertices             AB43  18
INTEGER M1,M2,M3,M4#                   Match flags                      AB43  19
dp: TEM(4)#                            Temporary for switch             AB43  20
dp: VTETRA#                            Tetrahedron volume               AB43  21
#                                                                       AB43  22
# SET UP ARRAY FOR TETRAHEDRON FACES                                    AB43  23
# ----------------------------------                                    AB43  24
#                                                                       AB43  25
MARKT=ISTOPH#                          Start of array                   AB43  26
ISTOPT=MARKT#                          End of array                     AB43  27
II=MARKT#                              Pointer into markt array         AB43  28
#                                                                       AB43  29
# DETERMINE THE FIRST TETRAHEDRON                                       AB43  30
# -------------------------------                                       AB43  31
#                                                                       AB43  32
IC=NPOINT-4#                           Vertices besides 1st 4           AB43  33
IP1=MARKH#                             Point to first vertex            AB43  34
IP2=MARKH+4#                           Point to second vertex           AB43  35
IP3=MARKH+8#                           Point to third vertex            AB43  36
IP4=MARKH+12#                          Point to fourth vertex           AB43  37
REPEAT#                                Repeat until nonzero volume      AB43  38
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB43  39
CALL AB45(IP1,IP2,IP3,IP4,VTETRA)#     Determine volume                 AB43  40
IF(dabs:(VTETRA)>=dsmall:) BREAK#      Nonzero volume                   AB43  41
IF(IC==0) GOTO 999#                    No more switches to do           AB43  42
movedoub:(QQ,MARKH+13,TEM,1,4,0)#      Switch                           AB43  43
movedoub:(QQ,MARKH+17,QQ,MARKH+13,ISTOPH-MARKH-16,1)# Switch            AB43  44
movedoub:(TEM,1,QQ,ISTOPH-3,4,0)#      Switch                           AB43  45
IC=IC-1#                               Number of switches left          AB43  46
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB43  47
#                                                                       AB43  48
# VOLUME AND TRANSMITTANCE OF FIRST TETRAHEDRON                         AB43  49
# ---------------------------------------------                         AB43  50
#                                                                       AB43  51
incrqq:(ISTOPT,ISTOPT+12,AB4301)#      Get mem for 1st tetra            AB43  52
IF(VTETRA<DBZ) CALL AB46(IP1,IP2,IP3,IP4,II)# Neg vol, store faces      AB43  53
ELSE           CALL AB46(IP1,IP2,IP4,IP3,II)# Pos vol, store faces      AB43  54
CALL AB44(IP1,IP2,IP3,IP4,dabs:(VTETRA))# Determine transmittance       AB43  55
#                                                                       AB43  56
# FIND REST OF TETRAHEDRA                                               AB43  57
# -----------------------                                               AB43  58
#                                                                       AB43  59
FOR(I=MARKH+16;I<ISTOPH;I=I+4)#        Loop over rest of vertices       AB43  60
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB43  61
JJ=II#                                                                  AB43  62
FOR(J=MARKT;J<JJ;J=J+3)#               Loop over faces                  AB43  63
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB43  64
IP1=NINT(QQ(J+1))#                     Pointer to vertex 1 of face      AB43  65
IP2=NINT(QQ(J+2))#                     Pointer to vertex 2 of face      AB43  66
IP3=NINT(QQ(J+3))#                     Pointer to vertex 3 of face      AB43  67
CALL AB45(IP1,IP2,IP3,I,VTETRA)#       Determine volume                 AB43  68
IF(VTETRA<dsmall:) NEXT#               Skip to next face                AB43  69
CALL AB46(IP1,IP2,I,IP3,II)#           Put 3 faces on stack             AB43  70
#                                                                       AB43  71
# CHECK FOR AND DELETE COMMON FACES                                     AB43  72
# ---------------------------------                                     AB43  73
#                                                                       AB43  74
FOR(K=MARKT;K<JJ;K=K+3)#               Loop over tetrahedron faces      AB43  75
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB43  76
M1=no: ; M2=no:#                       Match flags                      AB43  77
M3=no: ; M4=no:#                       Match flags                      AB43  78
IF( (IP1==NINT(QQ(K+1)))\$#                                             AB43  79
    (IP1==NINT(QQ(K+2)))\$#                                             AB43  80
    (IP1==NINT(QQ(K+3))) ) M1=yes:#    First vertex matches             AB43  81
IF( (IP2==NINT(QQ(K+1)))\$#                                             AB43  82
    (IP2==NINT(QQ(K+2)))\$#                                             AB43  83
    (IP2==NINT(QQ(K+3))) ) M2=yes:#    Second vertex matches            AB43  84
IF( (IP3==NINT(QQ(K+1)))\$#                                             AB43  85
    (IP3==NINT(QQ(K+2)))\$#                                             AB43  86
    (IP3==NINT(QQ(K+3))) ) M4=yes:#    Third vertex matches             AB43  87
IF( (I==NINT(QQ(K+1)))\$#                                               AB43  88
    (I==NINT(QQ(K+2)))\$#                                               AB43  89
    (I==NINT(QQ(K+3))) ) M3=yes:#      Fourth vertex matches            AB43  90
IF((M1==yes:)&(M2==yes:)&(M3==yes:))#                                   AB43  91
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 5  AB43  92
QQ(K+1)=0. ; QQ(K+2)=0. ; QQ(K+3)=0.0#                                  AB43  93
QQ(II-8)=0. ; QQ(II-7)=0. ; QQ(II-6)=0.#                                AB43  94
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 5  AB43  95
ELSE IF((M1==yes:)&(M3==yes:)&(M4==yes:))#                              AB43  96
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 6  AB43  97
QQ(K+1)=0. ; QQ(K+2)=0. ; QQ(K+3)=0.0#                                  AB43  98
QQ(II-5)=0. ; QQ(II-4)=0. ; QQ(II-3)=0.#                                AB43  99
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 6  AB43 100
ELSE IF((M3==yes:)&(M2==yes:)&(M4==yes:))#                              AB43 101
$(#                                    3>>>>>>>>>>>>>>>>>>>>>>>>>>>> 7  AB43 102
QQ(K+1)=0. ; QQ(K+2)=0. ; QQ(K+3)=0.0#                                  AB43 103
QQ(II-2)=0. ; QQ(II-1)=0. ; QQ(II)=0.#                                  AB43 104
$)#                                    3<<<<<<<<<<<<<<<<<<<<<<<<<<<< 7  AB43 105
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB43 106
QQ(J+1)=0. ; QQ(J+2)=0. ; QQ(J+3)=0.#                                   AB43 107
FOR(K=MARKT;K<II;K=K+3)#                                                AB43 108
$(#                                    2>>>>>>>>>>>>>>>>>>>>>>>>>>>> 8  AB43 109
IF(NINT(QQ(K+1))!=0) NEXT#                                              AB43 110
IF(II>K+3) movedoub:(QQ,K+4,QQ,K+1,II-K-3,1)#                           AB43 111
II=II-3#                                                                AB43 112
IF(K<=J) J=J-3#                                                         AB43 113
K=K-3#                                                                  AB43 114
$)#                                    2<<<<<<<<<<<<<<<<<<<<<<<<<<<< 8  AB43 115
JJ=II#                                                                  AB43 116
CALL AB44(IP1,IP2,IP3,I,dabs:(VTETRA))# Determine transmittance         AB43 117
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB43 118
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB43 119
999 CONTINUE#                                                           AB43 120
RETURN#                                                                 AB43 121
END#                                                                    AB43 122
systemheader:(AB44)#                   Transmittance                    AB44   1
####################################################################    AB44   2
#                                                                  #    AB44   3
#                               AB44                               #    AB44   4
#                                                                  #    AB44   5
####################################################################    AB44   6
SUBROUTINE AB44(I,J,K,L,DVELT)#                                         AB44   7
IMPLICIT INTEGER (I-N)#                                                 AB44   8
include:(AACOMN)#                                                       AB44   9
include:(ABCOMN)#                                                       AB44  10
#                                                                       AB44  11
## CALCULATE THE TRANSMITTANCE OF A TETRAHEDRON                         AB44  12
## GEORGE DAVENPORT, APRIL 1982                                         AB44  13
## T. MASLEN, JANUARY 1984                                              AB44  14
#                                                                       AB44  15
dp: D1,D2,D3,D4#                       Path length differences          AB44  16
dp: H2,H3,H4#                          H(d2),h(d3),h(d4)                AB44  17
INTEGER I,J,K,L#                       Pointers to vertices             AB44  18
INTEGER II,JJ#                         Loop indices                     AB44  19
dp: PATH(4)#                           Ordered path lengths             AB44  20
dp: TELT#                              Transmittance element            AB44  21
dp: TEMP#                              Temporary                        AB44  22
dp: DFAC#                              Temporary                        AB44  23
dp: DVELT#                             Volume element                   AB44  24
dp: DBSM1,DBO6,DBH,DB1#                                                 AB44  25
dp: DTEST#                                                              AB44  26
dp: SIXTH#                                                              AB44  27
#                                                                       AB44  28
DBSM1=db:(0.001)#                                                       AB44  29
SIXTH=1.0/6.0#                                                          AB44  30
DBO6=db:(SIXTH)#                                                        AB44  31
DBH=db:(0.5)#                                                           AB44  32
DB1=db:(1.0)#                                                           AB44  33
#                                                                       AB44  34
# ARRANGE THE PATH LENGTHS IN ORDER FROM LEAST TO GREATEST              AB44  35
# --------------------------------------------------------              AB44  36
#                                                                       AB44  37
PATH(1)=QQ(I+4) ; PATH(2)=QQ(J+4)#     1st & 2nd path lengths           AB44  38
PATH(3)=QQ(K+4) ; PATH(4)=QQ(L+4)#     3rd & 4th path lengths           AB44  39
FOR(II=1;II<4;II=II+1)#                Loop over path lengths           AB44  40
FOR(JJ=II+1;JJ<5;JJ=JJ+1)#             Loop over path lengths           AB44  41
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB44  42
IF(PATH(JJ)>=PATH(II)) NEXT#           If in order                      AB44  43
TEMP=PATH(II)#                         Else switch                      AB44  44
PATH(II)=PATH(JJ)#                     Switch                           AB44  45
PATH(JJ)=TEMP#                         Switch                           AB44  46
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB44  47
#                                                                       AB44  48
# CALCULATE THE DIFFERENCES AND VALUES OF H(X)=(1-EXP(-X))/X            AB44  49
# ----------------------------------------------------------            AB44  50
#                                                                       AB44  51
D1=PATH(1)#                            G1                               AB44  52
D2=PATH(2)-PATH(1)#                    G2-g1                            AB44  53
D3=PATH(3)-PATH(1)#                    G3-g1                            AB44  54
D4=PATH(4)-PATH(1)#                    G4-g1                            AB44  55
#                                                                       AB44  56
DTEST=DBSM1#                           Set initial test limit           AB44  57
DO II=1,2#                             Second time is for tbar          AB44  58
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 2  AB44  59
IF(D2<DTEST) H2=DB1-DBH*D2#            H(d2)                            AB44  60
ELSE         H2=(DB1-EXP(-D2))/D2#     H(d2)                            AB44  61
IF(D3<DTEST) H3=DB1-DBH*D3#            H(d3)                            AB44  62
ELSE         H3=(DB1-EXP(-D3))/D3#     H(d3)                            AB44  63
IF(D4<DTEST) H4=DB1-DBH*D4#            H(d4)                            AB44  64
ELSE         H4=(DB1-EXP(-D4))/D4#     H(d4)                            AB44  65
#                                                                       AB44  66
# CALCULATE TRANSMITTANCE                                               AB44  67
# -----------------------                                               AB44  68
#                                                                       AB44  69
IF(D4-D2<DTEST)#                       1<2=3=4                          AB44  70
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 3  AB44  71
IF(D2>DTEST) TELT=DBH*H2+(H2-DBH)/D2+(H2-DB1)/D2**2# ***** ok *****     AB44  72
ELSE         TELT=DBO6#                Transmittance                    AB44  73
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 3  AB44  74
#                                                                       AB44  75
ELSE IF(D3-D2<DTEST)#                  1<2=3<4                          AB44  76
$(#                                    1>>>>>>>>>>>>>>>>>>>>>>>>>>>> 4  AB44  77
IF(D2>DTEST) TELT=(H2+(H2-DB1)/D2-(H2-H4)/(D4-D2))/(D4-D2)#             AB44  78
ELSE         TELT=(DBH-(DB1-H4)/D4)/D4#                                 AB44  79
$)#                                    1<<<<<<<<<<<<<<<<<<<<<<<<<<<< 4  AB44  80
#                                                                       AB44  81
ELSE IF(D4-D3<DTEST) TELT=((H2-H3)/(D3-D2)+(DB1-H3)/D3-H3)/(D3-D2)#     AB44  82
ELSE                 TELT=((H2-H3)/(D3-D2)-(H3-H4)/(D4-D3))/(D4-D2)#    AB44  83
#                                                                       AB44  84
TELT=TELT*dabs:(DVELT)*EXP(-D1)#                                        AB44  85
#                                                                       AB44  86
IF(II==1) ABSF=ABSF+TELT#              Main summation                   AB44  87
ELSE      ABSF2=ABSF2+TELT#            2nd summation (for tbar)         AB44  88
IF(ITBAR==no:) BREAK#                  Skip 2nd if no tbar needed       AB44  89
DFAC=EMU2/EMU#                         Ratio                            AB44  90
D1=D1*DFAC ; D2=D2*DFAC#               Op paths using 2nd mu            AB44  91
D3=D3*DFAC ; D4=D4*DFAC#                                                AB44  92
DTEST=DBSM1*DFAC#                      Scale test limit                 AB44  93
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 2  AB44  94
#                                                                       AB44  95
TEMP=DVELT*DBO6#                                                        AB44  96
DVOL=DVOL+dabs:(TEMP)#                                                  AB44  97
VOL=DVOL#                                                               AB44  98
RETURN#                                                                 AB44  99
END#                                                                    AB44 100
systemheader:(AB45)#                   Tetrahedron volume               AB45   1
####################################################################    AB45   2
#                                                                  #    AB45   3
#                               AB45                               #    AB45   4
#                                                                  #    AB45   5
####################################################################    AB45   6
SUBROUTINE AB45(I,J,K,L,DVELT)#                                         AB45   7
IMPLICIT INTEGER (I-N)#                                                 AB45   8
include:(AACOMN)#                                                       AB45   9
include:(ABCOMN)#                                                       AB45  10
#                                                                       AB45  11
## COMPUTE THE VOLUME OF A TETRAHEDRON                                  AB45  12
## GEORGE DAVENPORT, AUGUST 1982                                        AB45  13
#                                                                       AB45  14
dp: DET1,DET2,DET3#                    Codeterminants                   AB45  15
dp: DIJ1,DIJ2,DIJ3#                    Differences                      AB45  16
dp: DIK1,DIK2,DIK3#                    Differences                      AB45  17
dp: DIL1,DIL2,DIL3#                    Differences                      AB45  18
dp: DVELT#                             Volume element                   AB45  19
INTEGER I,J,K,L#                       Pointers to vertices             AB45  20
#                                                                       AB45  21
DIJ1=QQ(I+1)-QQ(J+1)#                                                   AB45  22
DIJ2=QQ(I+2)-QQ(J+2)#                                                   AB45  23
DIJ3=QQ(I+3)-QQ(J+3)#                                                   AB45  24
DIK1=QQ(I+1)-QQ(K+1)#                                                   AB45  25
DIK2=QQ(I+2)-QQ(K+2)#                                                   AB45  26
DIK3=QQ(I+3)-QQ(K+3)#                                                   AB45  27
DIL1=QQ(I+1)-QQ(L+1)#                                                   AB45  28
DIL2=QQ(I+2)-QQ(L+2)#                                                   AB45  29
DIL3=QQ(I+3)-QQ(L+3)#                                                   AB45  30
#                                                                       AB45  31
DET1=DIK2*DIL3-DIL2*DIK3#              Determinant 1                    AB45  32
DET2=DIK1*DIL3-DIL1*DIK3#              Determinant 2                    AB45  33
DET3=DIK1*DIL2-DIL1*DIK2#              Determinant 3                    AB45  34
DVELT=DET1*DIJ1-DET2*DIJ2+DET3*DIJ3#   Dp volume                        AB45  35
#                                                                       AB45  36
RETURN#                                                                 AB45  37
END#                                                                    AB45  38
systemheader:(AB46)#                   Store tetrahedron faces          AB46   1
####################################################################    AB46   2
#                                                                  #    AB46   3
#                              AB46                                #    AB46   4
#                                                                  #    AB46   5
####################################################################    AB46   6
SUBROUTINE AB46(I,J,K,L,II)#                                            AB46   7
IMPLICIT INTEGER (I-N)#                                                 AB46   8
include:(AACOMN)#                                                       AB46   9
include:(ABCOMN)#                                                       AB46  10
#                                                                       AB46  11
## STORE FACES OF TETRAHEDRA                                            AB46  12
## GEORGE DAVENPORT, AUGUST 1982                                        AB46  13
#                                                                       AB46  14
INTEGER I,J,K,L#                       Pointers to vertices             AB46  15
INTEGER II#                            Running pointer into markt       AB46  16
#                                                                       AB46  17
IF((II+9)>=ISTOPT)#                    More memory needed               AB46  18
incrqq:(ISTOPT,ISTOPT+30,AB4601)#      Get more memory                  AB46  19
#                                                                       AB46  20
# 1ST TETRAHEDRON: 4 FACES, OTHERS: 3 FACES                             AB46  21
# -----------------------------------------                             AB46  22
#                                                                       AB46  23
IF(II==MARKT)#                         First tetrahedron                AB46  24
$(#                                    0>>>>>>>>>>>>>>>>>>>>>>>>>>>> 1  AB46  25
QQ(II+1)=FLOAT(I)#                     1st vertex of face               AB46  26
QQ(II+2)=FLOAT(L)#                     2nd vertex of face               AB46  27
QQ(II+3)=FLOAT(J)#                     3rd vertex of face               AB46  28
II=II+3#                               Add to pointer                   AB46  29
$)#                                    0<<<<<<<<<<<<<<<<<<<<<<<<<<<< 1  AB46  30
#                                                                       AB46  31
QQ(II+1)=FLOAT(I)#                     1st vertex of face               AB46  32
QQ(II+2)=FLOAT(J)#                     2nd vertex of face               AB46  33
QQ(II+3)=FLOAT(K)#                     3rd vertex of face               AB46  34
QQ(II+4)=FLOAT(I)#                     1st vertex of face               AB46  35
QQ(II+5)=FLOAT(K)#                     2nd vertex of face               AB46  36
QQ(II+6)=FLOAT(L)#                     3rd vertex of face               AB46  37
QQ(II+7)=FLOAT(K)#                     1st vertex of face               AB46  38
QQ(II+8)=FLOAT(J)#                     2nd vertex of face               AB46  39
QQ(II+9)=FLOAT(L)#                     3rd vertex of face               AB46  40
II=II+9#                                                                AB46  41
#                                                                       AB46  42
RETURN#                                                                 AB46  43
END#                                                                    AB46  44
pmacro:([no:])#                        Purge xtal system macros         AB46  45
