/* 
 *      Copyright (C) 2000, The Xtal System, The University of          
 *            Western Australia, xtal@crystal.uwa.edu.au.               
 *                                                                      
 * Distributed under the GNU General Public Licence as published by     
 * the Free Software Foundation, in the hope that it will be useful     
 * but WITHOUT ANY WARRANTY. See the GNU General Public License         
 * for more details.                                                    
 *                                                                      
 *
 * tklib.c: This file is a complete hack upon tkCanvas.c, tkBind.c and 
 *          tkEvent.c  supplied as part of the tcl/tk8.2.1 distribution.
 *	    It implements an alternative canvas widgets for the Tk 
 *          toolkit which displays simple 2D line and area and text 
 *          rendering, complete with a rudimentary fortran binding 
 *          interface for accessing various tcl/tk attributes.
 *
 * tkCanvas.c, tkBind.c and tkEvent.c were  ...
 * Copyright (c) 1991-1994 The Regents of the University of California.
 * Copyright (c) 1994-1997 Sun Microsystems, Inc.
 * Copyright (c) 1998-1999 by Scriptics Corporation.
 *
 */

#ifdef __cplusplus
extern "C" {
#endif

#include "tklib.h" 

static int             ownColMap        =False; 
static Tk_Window       toplevel         =NULL ; 
static GC              gcCol[NUM_COLORS] ; 
static char            *cstring[NUM_COLORS];   /* array of pointers to colours*/
static int             colGC            =(int) NULL; 
static XtalCanvas      *currentGraphic   =NULL;
static XtalCanvas      *xtalWindowList   =NULL;   /* current window list */
static XtalCanvas      *lastWindowInList =NULL;  /* current window list */
static Display         *display;
static int             screen_num;
static int             maxColUsed=0;
static unsigned long   gcFunc = GXcopy;
static xCommandRec     tmpCmd;                 /* temporary storage space */
static Bool            colorDisplay;
static timerRec        *timerList = NULL;       /* list of timer callbacks */
static Tcl_HashTable   eventTable;
static Tcl_HashTable   mainWinTable;

#ifdef __WIN32__
static XIC             inputContext;
#endif

static Tk_Cursor       arrowCursor=(Tk_Cursor)NULL;/*tells user to pick seg */
static Tk_Cursor       crossCursor=(Tk_Cursor)NULL;/*tells user to pick point */
static Tk_Cursor       textCursor =(Tk_Cursor)NULL;/*tells user to type text */
static Tk_Cursor       watchCursor=(Tk_Cursor)NULL;/*tells user to wait */

#define XTAL_NOMODE      0
#define XTAL_FRAME       1
#define XTAL_GETSTRING   2
#define XTAL_GETXY       3
#define XTAL_GETSEG      4
static int             xtalEventMode=XTAL_NOMODE;
static int             xtalXCoord =-1;
static int             xtalYCoord =-1;
static int             xtalSegment=0;
static int             xtalStringNOut=0;
static char            xtalStringBuf[STR_BUF_SIZE];/* temporary char strorage buffer*/
static char *          xtalWidget =NULL;
static int             xtalWidgetLevel = 0;


char * GetField(char *p,char *copy,int  size);

void  vectorChar(XtalCanvas *graphic,int key,char ch, float x, float y,
                               float delx, float dely, int italic);
xSegment  xFindSegment(xSegment *source, int id);
void     xFreeSegments(xSegment *segptr);
void     xExecuteCommands(XtalCanvas *graphic,xSegment *segptr);
static void     xSetFunction(int mode);
void xResize(XtalCanvas *graphic, int width, int height);
void  xLines(XtalCanvas *graphic, xLinesCommand linesCmd);
void   xMark(XtalCanvas *graphic, xMarkCommand  markCmd);
void   xFill(XtalCanvas *graphic, xFillCommand  fillCmd);
void   xText(XtalCanvas *graphic, xTextCommand  textCmd);
static void xQueryGC();

static void parseEvent(XEvent *eventPtr, unsigned long *state,
      unsigned long *buttonNumber, unsigned long *time, 
      int *XCoord, int *YCoord, unsigned long *type);

#define  XTALTCLERROR   if (error ) { \
           printf("\n--- Tk interface: %s \n",interp->result); \
            fflush(stdout);  \
            return; }
   
#define UCHAR(c) ((unsigned char) (c))




/*
 * Information used for argv parsing.
 */

static Tk_ConfigSpec configSpecs[] = {
    {TK_CONFIG_BORDER, "-background", "background", "Background",
	DEF_CANVAS_BG_COLOR, Tk_Offset(XtalCanvas, bgBorder),
	TK_CONFIG_COLOR_ONLY},
    {TK_CONFIG_BORDER, "-background", "background", "Background",
	DEF_CANVAS_BG_MONO, Tk_Offset(XtalCanvas, bgBorder),
	TK_CONFIG_MONO_ONLY},
    {TK_CONFIG_SYNONYM, "-bd", "borderWidth", (char *) NULL,
	(char *) NULL, 0, 0},
    {TK_CONFIG_SYNONYM, "-bg", "background", (char *) NULL,
	(char *) NULL, 0, 0},
    {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
	DEF_CANVAS_BORDER_WIDTH, Tk_Offset(XtalCanvas, borderWidth), 0},
    {TK_CONFIG_BOOLEAN, "-confine", "confine", "Confine",
	DEF_CANVAS_CONFINE, Tk_Offset(XtalCanvas, confine), 0},
    {TK_CONFIG_PIXELS, "-height", "height", "Height",
	DEF_CANVAS_HEIGHT, Tk_Offset(XtalCanvas, height), 0},
    {TK_CONFIG_COLOR, "-highlightbackground", "highlightBackground",
	"HighlightBackground", DEF_CANVAS_HIGHLIGHT_BG,
	Tk_Offset(XtalCanvas, highlightBgColorPtr), 0},
    {TK_CONFIG_COLOR, "-highlightcolor", "highlightColor", "HighlightColor",
	DEF_CANVAS_HIGHLIGHT, Tk_Offset(XtalCanvas, highlightColorPtr), 0},
    {TK_CONFIG_PIXELS, "-highlightthickness", "highlightThickness",
	"HighlightThickness",
	DEF_CANVAS_HIGHLIGHT_WIDTH, Tk_Offset(XtalCanvas, highlightWidth), 0},
    {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
	DEF_CANVAS_RELIEF, Tk_Offset(XtalCanvas, relief), 0},
    {TK_CONFIG_STRING, "-scrollregion", "scrollRegion", "ScrollRegion",
	DEF_CANVAS_SCROLL_REGION, Tk_Offset(XtalCanvas, regionString),
	TK_CONFIG_NULL_OK},
    {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
	DEF_CANVAS_TAKE_FOCUS, Tk_Offset(XtalCanvas, takeFocus),
	TK_CONFIG_NULL_OK},
    {TK_CONFIG_PIXELS, "-width", "width", "Width",
	DEF_CANVAS_WIDTH, Tk_Offset(XtalCanvas, width), 0},
    {TK_CONFIG_STRING, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
	DEF_CANVAS_X_SCROLL_CMD, Tk_Offset(XtalCanvas, xScrollCmd),
	TK_CONFIG_NULL_OK},
    {TK_CONFIG_PIXELS, "-xscrollincrement", "xScrollIncrement",
	"ScrollIncrement",
	DEF_CANVAS_X_SCROLL_INCREMENT,Tk_Offset(XtalCanvas, xScrollIncrement),
	0},
    {TK_CONFIG_STRING, "-yscrollcommand", "yScrollCommand", "ScrollCommand",
	DEF_CANVAS_Y_SCROLL_CMD, Tk_Offset(XtalCanvas, yScrollCmd),
	TK_CONFIG_NULL_OK},
    {TK_CONFIG_PIXELS, "-yscrollincrement", "yScrollIncrement",
	"ScrollIncrement",
	DEF_CANVAS_Y_SCROLL_INCREMENT,Tk_Offset(XtalCanvas, yScrollIncrement),
	0},
    {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
	(char *) NULL, 0, 0}
} ;


/*
 * Prototypes for procedures defined later in this file:
 */
static int              xtalBinding _ANSI_ARGS_((ClientData clientData,
			   Tcl_Interp *interp, int objc,Tcl_Obj *CONST objv[]));
static int              getIndexedColour _ANSI_ARGS_((ClientData clientData,
			   Tcl_Interp *interp, int objc,Tcl_Obj *CONST objv[]));

static void             parseEvent(XEvent *eventPtr, unsigned long *state, 
                            unsigned long *button, unsigned long *time, 
                            int *XCoord, int *YCoord, unsigned long *Type);
static int		XtalCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
static void		CanvasBindProc _ANSI_ARGS_((ClientData clientData,
			    XEvent *eventPtr));
static void		CanvasCmdDeletedProc _ANSI_ARGS_((
			    ClientData clientData));
static void		CanvasEventProc _ANSI_ARGS_((ClientData clientData,
			    XEvent *eventPtr));
static void		CanvasFocusProc _ANSI_ARGS_((XtalCanvas *canvasPtr,
			    int gotFocus));
static void		CanvasLostSelection _ANSI_ARGS_((
			    ClientData clientData));
static void		CanvasSetOrigin _ANSI_ARGS_((XtalCanvas *canvasPtr,
			    int xOrigin, int yOrigin));
static void		CanvasUpdateScrollbars _ANSI_ARGS_((
			    XtalCanvas *canvasPtr));
static int		CanvasWidgetCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
static void		CanvasWorldChanged _ANSI_ARGS_((
			    ClientData instanceData));
static int		ConfigureCanvas _ANSI_ARGS_((Tcl_Interp *interp,
			    XtalCanvas *canvasPtr, int argc, char **argv,
			    int flags));
static void		DestroyCanvas _ANSI_ARGS_((char *memPtr));
static void		DisplayCanvas _ANSI_ARGS_((ClientData clientData));
static int		FindArea _ANSI_ARGS_((Tcl_Interp *interp,
			    XtalCanvas *canvasPtr, char **argv, Tk_Uid uid,
			    int enclosed));
static double		GridAlign _ANSI_ARGS_((double coord, double spacing));
static void		PrintScrollFractions _ANSI_ARGS_((int screen1,
			    int screen2, int object1, int object2,
			    char *string));
static void		RelinkItems _ANSI_ARGS_((XtalCanvas *canvasPtr,
			    char *tag, Tk_Item *prevPtr));

static void     TkCanvasEventuallyRedraw _ANSI_ARGS_((XtalCanvas *canvas, 
                int x1,int y1, int x2, int y2));
static int      XtalCanvasGetCoord _ANSI_ARGS_((Tcl_Interp *interp,
                     XtalCanvas *canvas, char *string, double *doublePtr));
static int      ParseEventDescription _ANSI_ARGS_((Tcl_Interp *interp,
                     char **eventStringPtr,unsigned long *eventMask));

				
/*
 *--------------------------------------------------------------
 *
 * xcont_ --
 *
 *	This procedure is the first routine an xtal program
 *	should invoke to fire up the Tcl/Tk interpreters
 *	
 * Results:
 *	returns with reply=0 on success
 *
 * Side effects:
 *	Starts the tcl interpreter
 *      sources the supplied filename to create the required
 *      window.
 *
 *--------------------------------------------------------------
 */

void xcont_(int *reply, char *filename,int len)
{
   char command[256];  /* empty command string */
   char  *str1 = "lappend argv -colormap new" ;
   char  *str2 = "set XTALHOME $env(XTALHOME)" ;
   char  *str3 = "source [file join $XTALHOME xtalpref.tcl]" ;
   char  *str4 = "catch {rename XtalBinding XtalBinding_disabled}" ;
   Tcl_HashEntry *hPtr;
   ModInfo       *modPtr;
   EventInfo     *eiPtr;
   int           dummy;
   int           error;


   if (!interp) {
      interp = Tcl_CreateInterp() ;  
      Tcl_FindExecutable(NULL);       /* initialize built in search path */
      if (Tcl_Init(interp) != TCL_OK) { 
         *reply = TCL_ERROR ;
	 printf("\n--- Tk interface: %s \n",interp->result);
         fflush(stdout);
         return ;
      }

      /* ensure that toplevel "." window uses a private colormap */
      if (ownColMap ) { 
         (void) strcpy(command,str1);
         (void) Tcl_Eval(interp, command);
      }

      if (Tk_Init(interp)!= TCL_OK) {  /* initiates a main window */
         *reply = TCL_ERROR ;
         return ;
      }

      (void) strcpy(command,str2);
      (void) Tcl_Eval(interp, command);
      /* load default settings from xtalrc preferred resources */
      (void) strcpy(command,str3);
      error = Tcl_Eval(interp, command);
      if (error == TCL_ERROR ) { 
          printf("\n--- Tk interface: Error sourcing resource file $XTALHOME/xtalpref.tcl");
          printf("                  %s.\n\n",interp->result); 
          exit (1) ;        /* fatal error */
      }
      toplevel  = Tk_NameToWindow(interp,".", Tk_MainWindow(interp));
      display   = Tk_Display(toplevel);
      screen_num= Tk_ScreenNumber(toplevel);


      /* The following is essential in order to use an xtal canvas 
           in any of the program initialization scripts */

      Tcl_CreateCommand(interp, "xtal", XtalCmd,
                (ClientData)Tk_MainWindow(interp),
                (Tcl_CmdDeleteProc *)NULL);

      Tcl_InitHashTable(&modTable, TCL_STRING_KEYS);
      for (modPtr = modArray; modPtr->name != NULL; modPtr++) {
          hPtr = Tcl_CreateHashEntry(&modTable, modPtr->name, &dummy);
          Tcl_SetHashValue(hPtr, modPtr);
      }

      Tcl_InitHashTable(&eventTable, TCL_STRING_KEYS);
      for (eiPtr = eventArray; eiPtr->name != NULL; eiPtr++) {
          hPtr = Tcl_CreateHashEntry(&eventTable, eiPtr->name, &dummy);
          Tcl_SetHashValue(hPtr, eiPtr);
      }

      /* make an assumption */
      colorDisplay=True;
      arrowCursor = Tk_GetCursor(interp,toplevel,"top_left_arrow");
      crossCursor = Tk_GetCursor(interp,toplevel,"crosshair");
      textCursor  = Tk_GetCursor(interp,toplevel,"xterm");
      watchCursor = Tk_GetCursor(interp,toplevel,"watch");

   }

   *reply=2;
   for( ;len>0 && filename[len-1] == ' '; len--) ;
   sprintf(command,"source [file join $XTALHOME %.*s]\n",len,filename); 
   /* read and execute the tcl setup script */
   printf("\n %s\n",command);fflush(stdout);
   error = Tcl_Eval(interp, command); 
   if (error != TCL_OK) { 
       printf("\n--- Tk interface: Error parsing startup script:\n{%s}\n",command);
       printf("                  %s.\n\n",interp->result); 
       fflush(stdout);  
       exit (1) ;        /* fatal error */
   }

   (void) strcpy(command,str4);
   error=Tcl_GlobalEval(interp,command);
   /* ignore any errors on that one */

   /* now redefine the actual command */
   Tcl_CreateObjCommand(interp, "XtalBinding", xtalBinding,
                (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

   /* add a command to return colour map colours */
   Tcl_CreateObjCommand(interp, "GetXtalColorIndex",getIndexedColour ,
                (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

   *reply=0 ;       /* success */
}


extern void __stdcall XCONT(int *reply, char *filename,int len)
{ xcont_(reply, filename,len) ; }
#ifndef VMS
void xcont(int *reply, char *filename,int len)
{ xcont_(reply, filename,len) ; }
#endif


/*
 *--------------------------------------------------------------
 *
 * XtalCmd --
 *
 *	This procedure is invoked to process the "xtal" Tcl
 *	command when invoked in any xtal scripts 
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *      interprets the passed in string arguments
 *	and initializes an xtalcanvas structure
 *
 *--------------------------------------------------------------
 */

int
XtalCmd(clientData, interp, argc, argv)
    ClientData clientData;	/* Main window associated with
				 * interpreter. */
    Tcl_Interp *interp;		/* Current interpreter. */
    int argc;			/* Number of arguments. */
    char **argv;		/* Argument strings. */
{
    Tk_Window tkwin = (Tk_Window) clientData;
    XtalCanvas *canvasPtr;
    Tk_Window new;


    if (argc < 2) {
	Tcl_AppendResult(interp, "wrong # args: should be \"",
		argv[0], " pathName ?options?\"", (char *) NULL);
	return TCL_ERROR;
    }

    new = Tk_CreateWindowFromPath(interp, tkwin, argv[1], (char *) NULL);
    if (new == NULL) {
	return TCL_ERROR;
    }

    /*
     * Initialize fields that won't be initialized by ConfigureCanvas,
     * or which ConfigureCanvas expects to have reasonable values
     * (e.g. resource pointers).
     */

    canvasPtr = (XtalCanvas *) ckalloc(sizeof(XtalCanvas));
    canvasPtr->tkwin = new;
    canvasPtr->display = Tk_Display(new);
    canvasPtr->interp = interp;
    canvasPtr->widgetCmd = Tcl_CreateCommand(interp,
	    Tk_PathName(canvasPtr->tkwin), CanvasWidgetCmd,
	    (ClientData) canvasPtr, CanvasCmdDeletedProc);
    canvasPtr->borderWidth = 0;
    canvasPtr->bgBorder = NULL;
    canvasPtr->relief = TK_RELIEF_FLAT;
    canvasPtr->highlightWidth = 0;
    canvasPtr->highlightBgColorPtr = NULL;
    canvasPtr->highlightColorPtr = NULL;
    canvasPtr->inset = 0;
    canvasPtr->pixmapGC = None;
    canvasPtr->width = None;
    canvasPtr->height = None;
    canvasPtr->confine = 0;
    canvasPtr->xOrigin = canvasPtr->yOrigin = 0;
    canvasPtr->drawableXOrigin = canvasPtr->drawableYOrigin = 0;
    canvasPtr->bindingTable = NULL;
    canvasPtr->xScrollCmd = NULL;
    canvasPtr->yScrollCmd = NULL;
    canvasPtr->scrollX1 = 0;
    canvasPtr->scrollY1 = 0;
    canvasPtr->scrollX2 = 0;
    canvasPtr->scrollY2 = 0;
    canvasPtr->regionString = NULL;
    canvasPtr->xScrollIncrement = 0;
    canvasPtr->yScrollIncrement = 0;
    canvasPtr->scanX = 0;
    canvasPtr->scanXOrigin = 0;
    canvasPtr->scanY = 0;
    canvasPtr->scanYOrigin = 0;
    canvasPtr->cursor = None;
    canvasPtr->takeFocus = NULL;
    canvasPtr->pixelsPerMM = WidthOfScreen(Tk_Screen(new));
    canvasPtr->pixelsPerMM /= WidthMMOfScreen(Tk_Screen(new));
    canvasPtr->flags = 0;

    canvasPtr->pixmap      =  (Pixmap) NULL;
    canvasPtr->doubleBuffered  = 0 ;      /* double buffering requested */
    canvasPtr->segmentList = canvasPtr->openSegment = canvasPtr->lastSegment = NULL;
    canvasPtr->auxrefcom   = NULL;
    canvasPtr->auxwinconf  = NULL;
    canvasPtr->auxwinxscroll = NULL ;
    canvasPtr->auxwinyscroll = NULL ;
    canvasPtr->depth       = 0;
    canvasPtr->aspectRatio = 0.0;
    canvasPtr->xScale      = canvasPtr->yScale =  1.0 ;
    canvasPtr->xCentre     = canvasPtr->yCentre = 0.5 ;
    canvasPtr->resizeFactor.w = 0;
    canvasPtr->resizeFactor.h = 0;
    canvasPtr->xOffset = 0;
    canvasPtr->yOffset = 0;
    canvasPtr->saspect = NULL ;  /* set pointer to shadow variable width*/
    canvasPtr->sxmag   = NULL ;  /* set pointer to shadow variable x mag */
    canvasPtr->symag   = NULL ;  /* set pointer to shadow variable Y magnif */
    canvasPtr->sxcent  = NULL ; /* set pointer to shadow variable  X Centre */
    canvasPtr->sycent  = NULL ; /* set pointer to shadow variable  Y Centre */

        /* set graphics settings to default values */
    canvasPtr->penUp       = True;
    canvasPtr->pen.x       = canvasPtr->pen.y   = 0 ;
        /* set graphics settings to default values */
    canvasPtr->fillStyle = 2;                  /* solid */
    canvasPtr->lineStyle = 1;                  /* solid */
    canvasPtr->lineWidth = 0;                  /* thin */
    canvasPtr->lineIndex = 1;                   /* solid */
    canvasPtr->fillColor = NUM_COLORS - 1;      /* white */
    canvasPtr->lineColor = NUM_COLORS - 1;      /* white */
    canvasPtr->markerColor = NUM_COLORS - 1;    /* white */

    /* add new window to end of list  */
    if (xtalWindowList) {
       lastWindowInList->next = canvasPtr;
       lastWindowInList       = canvasPtr;
    }
    else  {
       xtalWindowList   = canvasPtr; /* first in list */
       lastWindowInList = canvasPtr; /* first is also last */
    }
    canvasPtr->next = NULL;  

    Tk_SetClass(canvasPtr->tkwin, "Xtal");
/* TkSetClassProcs(canvasPtr->tkwin, &canvasClass, (ClientData) canvasPtr); */

    Tk_CreateEventHandler(canvasPtr->tkwin,
	    ExposureMask|StructureNotifyMask|FocusChangeMask,
	    CanvasEventProc, (ClientData) canvasPtr);
    Tk_CreateEventHandler(canvasPtr->tkwin, KeyPressMask | KeyReleaseMask
	    |ButtonPressMask|ButtonReleaseMask|EnterWindowMask
	    |LeaveWindowMask|PointerMotionMask|VirtualEventMask,
	    CanvasBindProc, (ClientData) canvasPtr);

                 /* each canvas gets one */
    Tcl_InitHashTable(&(canvasPtr->eventtable),TCL_ONE_WORD_KEYS);



    /* essentially calls and passes the configuration options */
    if (ConfigureCanvas(interp, canvasPtr, argc-2, argv+2, 0) != TCL_OK) {
	goto error;
    }

    Tcl_SetResult(interp, Tk_PathName(canvasPtr->tkwin), TCL_STATIC);
    return TCL_OK;

    error:
    Tk_DestroyWindow(canvasPtr->tkwin);
    return TCL_ERROR;
}


/*
 *----------------------------------------------------------------------
 *
 * ConfigureCanvas --
 *
 *	This procedure is called to process an argv/argc list, plus
 *	the Tk option database, in order to configure (or
 *	reconfigure) a canvas widget.
 *
 * Results:
 *	The return value is a standard Tcl result.  If TCL_ERROR is
 *	returned, then the interp's result contains an error message.
 *
 * Side effects:
 *	Configuration information, such as colors, border width,
 *	etc. get set for canvasPtr;  old resources get freed,
 *	if there were any.
 *
 *----------------------------------------------------------------------
 */

static int
ConfigureCanvas(interp, canvasPtr, argc, argv, flags)
    Tcl_Interp *interp;		/* Used for error reporting. */
    XtalCanvas *canvasPtr;	/* Information about widget;  may or may
				 * not already have values for some fields. */
    int argc;			/* Number of valid entries in argv. */
    char **argv;		/* Arguments. */
    int flags;			/* Flags to pass to Tk_ConfigureWidget. */
{
    XGCValues gcValues;
    GC new;


    /*parse the command line args */
    if ( Tk_ConfigureWidget(interp, canvasPtr->tkwin, configSpecs,
	    argc, argv, (char *) canvasPtr, flags ) != TCL_OK) {
	return TCL_ERROR;
    }


    /*
     * A few options need special processing, such as setting the
     * background from a 3-D border and creating a GC for copying
     * bits to the screen.
     */

    Tk_SetBackgroundFromBorder(canvasPtr->tkwin, canvasPtr->bgBorder);

    if (canvasPtr->highlightWidth < 0) {
	canvasPtr->highlightWidth = 0;
    }
    canvasPtr->inset = canvasPtr->borderWidth + canvasPtr->highlightWidth;

    gcValues.function = GXcopy;
    gcValues.foreground = Tk_3DBorderColor(canvasPtr->bgBorder)->pixel;
    gcValues.graphics_exposures = False;

    new = Tk_GetGC(canvasPtr->tkwin,
	    GCFunction|GCForeground|GCGraphicsExposures, &gcValues);
    if (canvasPtr->pixmapGC != None) {
	Tk_FreeGC(canvasPtr->display, canvasPtr->pixmapGC);
    }
    canvasPtr->pixmapGC = new;

    /*
     * Reset the desired dimensions for the window.
     */

    Tk_GeometryRequest(canvasPtr->tkwin, canvasPtr->width + 2*canvasPtr->inset,
	    canvasPtr->height + 2*canvasPtr->inset);

    if (canvasPtr->aspectRatio <= 0.000005) {
        if (Tk_ReqHeight(canvasPtr->tkwin) > 0) 
            canvasPtr->aspectRatio = (float)Tk_ReqWidth(canvasPtr->tkwin)/
                                     (float)Tk_ReqHeight(canvasPtr->tkwin);
    }
    /*
     * Restart the cursor timing sequence in case the on-time or off-time
     * just changed.
     */


    /*
     * Recompute the scroll region.
     */

    canvasPtr->scrollX1 = 0;
    canvasPtr->scrollY1 = 0;
    canvasPtr->scrollX2 = 0;
    canvasPtr->scrollY2 = 0;
    if (canvasPtr->regionString != NULL) {
	int argc2;
	char **argv2;

	if (Tcl_SplitList(canvasPtr->interp, canvasPtr->regionString,
		&argc2, &argv2) != TCL_OK) {
	    return TCL_ERROR;
	}
	if (argc2 != 4) {
	    Tcl_AppendResult(interp, "bad scrollRegion \"",
		    canvasPtr->regionString, "\"", (char *) NULL);
	    badRegion:
	    ckfree(canvasPtr->regionString);
	    ckfree((char *) argv2);
	    canvasPtr->regionString = NULL;
	    return TCL_ERROR;
	}
	if ((Tk_GetPixels(canvasPtr->interp, canvasPtr->tkwin,
		    argv2[0], &canvasPtr->scrollX1) != TCL_OK)
		|| (Tk_GetPixels(canvasPtr->interp, canvasPtr->tkwin,
		    argv2[1], &canvasPtr->scrollY1) != TCL_OK)
		|| (Tk_GetPixels(canvasPtr->interp, canvasPtr->tkwin,
		    argv2[2], &canvasPtr->scrollX2) != TCL_OK)
		|| (Tk_GetPixels(canvasPtr->interp, canvasPtr->tkwin,
		    argv2[3], &canvasPtr->scrollY2) != TCL_OK)) {
	    goto badRegion;
	}
	ckfree((char *) argv2);
    }

    /*
     * Reset the canvas's origin (this is a no-op unless confine
     * mode has just been turned on or the scroll region has changed).
     */

    CanvasSetOrigin(canvasPtr, canvasPtr->xOrigin, canvasPtr->yOrigin);
    canvasPtr->flags |= UPDATE_SCROLLBARS|REDRAW_BORDERS;
    TkCanvasEventuallyRedraw(canvasPtr,
	    canvasPtr->xOrigin, canvasPtr->yOrigin,
	    canvasPtr->xOrigin + Tk_Width(canvasPtr->tkwin),
	    canvasPtr->yOrigin + Tk_Height(canvasPtr->tkwin));
    if(! canvasPtr->cursor) {
        Tk_DefineCursor(canvasPtr->tkwin,watchCursor);
        canvasPtr->cursor = watchCursor;
    }
    return TCL_OK;
}

/*
 *--------------------------------------------------------------
 *
 * CanvasWidgetCmd --
 *
 *	This procedure is invoked to process the Tcl command
 *	that corresponds to a widget managed by this module.
 *	See the user documentation for details on what it does.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	See the user documentation.
 *
 *--------------------------------------------------------------
 */

static int
CanvasWidgetCmd(clientData, interp, argc, argv)
    ClientData clientData;		/* Information about canvas
					 * widget. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{

    XtalCanvas *canvasPtr = (XtalCanvas *) clientData;
    size_t length;
    int c, result;
    Tk_Item *itemPtr = NULL;		/* Initialization needed only to
					 * prevent compiler warning. */

    if (argc < 2) {
	Tcl_AppendResult(interp, "wrong # args: should be \"",
		argv[0], " option ?arg arg ...?\"", (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_Preserve((ClientData) canvasPtr);
    result = TCL_OK;
    c = argv[1][0];
    length = strlen(argv[1]);

    if ((c == 'c') && (strcmp(argv[1], "canvasx") == 0)) {
	int x;
	double grid;
	char buf[TCL_DOUBLE_SPACE];

	if ((argc < 3) || (argc > 4)) {
	    Tcl_AppendResult(interp, "wrong # args: should be \"",
		    argv[0], " canvasx screenx ?gridspacing?\"",
		    (char *) NULL);
	    goto error;
	}
	if (Tk_GetPixels(interp, canvasPtr->tkwin, argv[2], &x) != TCL_OK) {
	    goto error;
	}
	if (argc == 4) {
	    if (XtalCanvasGetCoord(interp, canvasPtr, argv[3],
		    &grid) != TCL_OK) {
		goto error;
	    }
	} else {
	    grid = 0.0;
	}
	x += canvasPtr->xOrigin;
	Tcl_PrintDouble(interp, GridAlign((double) x, grid), buf);
	Tcl_SetResult(interp, buf, TCL_VOLATILE);
    } else if ((c == 'c') && (strcmp(argv[1], "canvasy") == 0)) {
	int y;
	double grid;
	char buf[TCL_DOUBLE_SPACE];

	if ((argc < 3) || (argc > 4)) {
	    Tcl_AppendResult(interp, "wrong # args: should be \"",
		    argv[0], " canvasy screeny ?gridspacing?\"",
		    (char *) NULL);
	    goto error;
	}
	if (Tk_GetPixels(interp, canvasPtr->tkwin, argv[2], &y) != TCL_OK) {
	    goto error;
	}
	if (argc == 4) {
	    if (XtalCanvasGetCoord(interp, canvasPtr,
		    argv[3], &grid) != TCL_OK) {
		goto error;
	    }
	} else {
	    grid = 0.0;
	}
	y += canvasPtr->yOrigin;
	Tcl_PrintDouble(interp, GridAlign((double) y, grid), buf);
	Tcl_SetResult(interp, buf, TCL_VOLATILE);

    } else if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
	    && (length >= 2)) {
	if (argc != 3) {
	    Tcl_AppendResult(interp, "wrong # args: should be \"",
		    argv[0], " cget option\"",
		    (char *) NULL);
	    goto error;
	}
	result = Tk_ConfigureValue(interp, canvasPtr->tkwin, configSpecs,
		(char *) canvasPtr, argv[2], 0);
    } else if ((c == 'c') && (strncmp(argv[1], "configure", length) == 0)
	    && (length >= 3)) {
	if (argc == 2) {
	    result = Tk_ConfigureInfo(interp, canvasPtr->tkwin, configSpecs,
		    (char *) canvasPtr, (char *) NULL, 0);
	} else if (argc == 3) {
	    result = Tk_ConfigureInfo(interp, canvasPtr->tkwin, configSpecs,
		    (char *) canvasPtr, argv[2], 0);
	} else {
	    result = ConfigureCanvas(interp, canvasPtr, argc-2, argv+2,
		    TK_CONFIG_ARGV_ONLY);
	}


    } else if ((c == 's') && (strncmp(argv[1], "scan", length) == 0)
	    && (length >= 3)) {
	int x, y;

	if (argc != 5) {
	    Tcl_AppendResult(interp, "wrong # args: should be \"",
		    argv[0], " scan mark|dragto x y\"", (char *) NULL);
	    goto error;
	}
	if ((Tcl_GetInt(interp, argv[3], &x) != TCL_OK)
		|| (Tcl_GetInt(interp, argv[4], &y) != TCL_OK)){
	    goto error;
	}
	if ((argv[2][0] == 'm')
		&& (strncmp(argv[2], "mark", strlen(argv[2])) == 0)) {
	    canvasPtr->scanX = x;
	    canvasPtr->scanXOrigin = canvasPtr->xOrigin;
	    canvasPtr->scanY = y;
	    canvasPtr->scanYOrigin = canvasPtr->yOrigin;
	} else if ((argv[2][0] == 'd')
		&& (strncmp(argv[2], "dragto", strlen(argv[2])) == 0)) {
	    int newXOrigin, newYOrigin, tmp;

	    /*
	     * Compute a new view origin for the canvas, amplifying the
	     * mouse motion.
	     */

	    tmp = canvasPtr->scanXOrigin - 10*(x - canvasPtr->scanX)
		    - canvasPtr->scrollX1;
	    newXOrigin = canvasPtr->scrollX1 + tmp;
	    tmp = canvasPtr->scanYOrigin - 10*(y - canvasPtr->scanY)
		    - canvasPtr->scrollY1;
	    newYOrigin = canvasPtr->scrollY1 + tmp;
	    CanvasSetOrigin(canvasPtr, newXOrigin, newYOrigin);
	} else {
	    Tcl_AppendResult(interp, "bad scan option \"", argv[2],
		    "\": must be mark or dragto", (char *) NULL);
	    goto error;
	}

    } else if ((c == 'x') && (strncmp(argv[1], "xview", length) == 0)) {
	int count, type;
	int newX = 0;		/* Initialization needed only to prevent
				 * gcc warnings. */
	double fraction;

	if (argc == 2) {
	    PrintScrollFractions(canvasPtr->xOrigin + canvasPtr->inset,
		    canvasPtr->xOrigin + Tk_Width(canvasPtr->tkwin)
		    - canvasPtr->inset, canvasPtr->scrollX1,
		    canvasPtr->scrollX2, Tcl_GetStringResult(interp));
	} else {
	    type = Tk_GetScrollInfo(interp, argc, argv, &fraction, &count);
	    switch (type) {
		case TK_SCROLL_ERROR:
		    goto error;
		case TK_SCROLL_MOVETO:
		    newX = canvasPtr->scrollX1 - canvasPtr->inset
			    + (int) (fraction * (canvasPtr->scrollX2
			    - canvasPtr->scrollX1) + 0.5);
		    break;
		case TK_SCROLL_PAGES:
		    newX = (int) (canvasPtr->xOrigin + count * .9
			 * (Tk_Width(canvasPtr->tkwin) - 2*canvasPtr->inset));
		    break;
		case TK_SCROLL_UNITS:
		    if (canvasPtr->xScrollIncrement > 0) {
			newX = canvasPtr->xOrigin
				+ count*canvasPtr->xScrollIncrement;
		    } else {
			newX = (int) (canvasPtr->xOrigin + count * .1
				* (Tk_Width(canvasPtr->tkwin)
				- 2*canvasPtr->inset));
		    }
		    break;
	    }
	    CanvasSetOrigin(canvasPtr, newX, canvasPtr->yOrigin);
	}
    } else if ((c == 'y') && (strncmp(argv[1], "yview", length) == 0)) {
	int count, type;
	int newY = 0;		/* Initialization needed only to prevent
				 * gcc warnings. */
	double fraction;

	if (argc == 2) {
	    PrintScrollFractions(canvasPtr->yOrigin + canvasPtr->inset,
		    canvasPtr->yOrigin + Tk_Height(canvasPtr->tkwin)
		    - canvasPtr->inset, canvasPtr->scrollY1,
		    canvasPtr->scrollY2, Tcl_GetStringResult(interp));
	} else {
	    type = Tk_GetScrollInfo(interp, argc, argv, &fraction, &count);
	    switch (type) {
		case TK_SCROLL_ERROR:
		    goto error;
		case TK_SCROLL_MOVETO:
		    newY = canvasPtr->scrollY1 - canvasPtr->inset
			    + (int) (fraction*(canvasPtr->scrollY2
			    - canvasPtr->scrollY1) + 0.5);
		    break;
		case TK_SCROLL_PAGES:
		    newY = (int) (canvasPtr->yOrigin + count * .9
			    * (Tk_Height(canvasPtr->tkwin)
			    - 2*canvasPtr->inset));
		    break;
		case TK_SCROLL_UNITS:
		    if (canvasPtr->yScrollIncrement > 0) {
			newY = canvasPtr->yOrigin
				+ count*canvasPtr->yScrollIncrement;
		    } else {
			newY = (int) (canvasPtr->yOrigin + count * .1
				* (Tk_Height(canvasPtr->tkwin)
				- 2*canvasPtr->inset));
		    }
		    break;
	    }
	    CanvasSetOrigin(canvasPtr, canvasPtr->xOrigin, newY);
	}
    } else {
	Tcl_AppendResult(interp, "bad option \"", argv[1],
		"\": must be canvasx, canvasy, cget, configure, coords, ",
		"scale, scan, xview, or yview",
		(char *) NULL);  
	goto error;
    }
 
    Tcl_Release((ClientData) canvasPtr);
    return result;

    error:
    Tcl_Release((ClientData) canvasPtr);
    return TCL_ERROR;
}

/*
 *----------------------------------------------------------------------
 *
 * DestroyCanvas --
 *
 *	This procedure is invoked by Tcl_EventuallyFree or Tcl_Release
 *	to clean up the internal structure of a canvas at a safe time
 *	(when no-one is using it anymore).
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Everything associated with the canvas is freed up.
 *
 *----------------------------------------------------------------------
 */

static void
DestroyCanvas(memPtr)
    char *memPtr;		/* Info about canvas widget. */
{
    XtalCanvas *canvasPtr = (XtalCanvas *) memPtr;
    XtalCanvas *next;

  /* unlink xtal windows from the list */
   if(xtalWindowList == canvasPtr) {         /* canvas is first in list */
      xtalWindowList = canvasPtr->next;      /* unlink */
      if (lastWindowInList == canvasPtr)     /* canvasPtr is the last in the list */
          lastWindowInList = NULL;           /* reset last window in list */
   } 
   else {

     for(next=xtalWindowList; next->next && next->next!=canvasPtr; next=next->next) ;

     if (next->next == canvasPtr ) {
         next->next=canvasPtr->next ;           /* unlink the canvasPtr */
         if (lastWindowInList == canvasPtr)     /* canvasPtr is the last in the list */
             lastWindowInList = next;           /* reset last previous in list */
     }
     /* else it wasn't even in the list */
   }
   
    if (xtalWindowList == lastWindowInList &&
        xtalWindowList == NULL ) {
         if(Tk_GetNumMainWindows() <= 0 ) {
             xtalWidget    = NULL;
             xtalSegment   = -1;
             xtalEventMode = XTAL_NOMODE;
         }
         if(arrowCursor) Tk_FreeCursor(display, arrowCursor );
         if(crossCursor) Tk_FreeCursor(display, crossCursor );
         if(textCursor) Tk_FreeCursor(display, textCursor );
         if(watchCursor) Tk_FreeCursor(display, watchCursor );
         arrowCursor = (Tk_Cursor)NULL;
         crossCursor = (Tk_Cursor)NULL;
         textCursor  = (Tk_Cursor)NULL;
         watchCursor = (Tk_Cursor)NULL;
    }


    /*
     * Free up all the stuff that requires special handling,
     * then let Tk_FreeOptions handle all the standard option-related
     * stuff.
     */
    if (canvasPtr->pixmapGC != None) {
	Tk_FreeGC(canvasPtr->display, canvasPtr->pixmapGC);
    }
    canvasPtr->pixmapGC=None;

    if (canvasPtr->pixmap)
        Tk_FreePixmap(display, canvasPtr->pixmap);
    canvasPtr->pixmap = (Pixmap) 0;

    if (canvasPtr->bindingTable != NULL) {
	Tk_DeleteBindingTable(canvasPtr->bindingTable);
    }
    canvasPtr->bindingTable=NULL;

    xFreeSegments(&(canvasPtr->segmentList));
    canvasPtr->openSegment = canvasPtr->lastSegment = canvasPtr->segmentList = NULL;

    Tk_FreeOptions(configSpecs, (char *) canvasPtr, canvasPtr->display, 0);
 
    ckfree((char *) canvasPtr);
}


/*
 *---------------------------------------------------------------------------
 *
 * CanvasWorldChanged --
 *
 *      This procedure is called when the world has changed in some
 *      way and the widget needs to recompute all its graphics contexts
 *	and determine its new geometry.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *	Configures all items in the canvas with a empty argc/argv, for
 *	the side effect of causing all the items to recompute their
 *	geometry and to be redisplayed.
 *
 *---------------------------------------------------------------------------
 */
 
static void
CanvasWorldChanged(instanceData)
    ClientData instanceData;	/* Information about widget. */
{
    XtalCanvas *canvasPtr;

    canvasPtr = (XtalCanvas *) instanceData;
    canvasPtr->flags |= REPICK_NEEDED;

    TkCanvasEventuallyRedraw(canvasPtr,
	    canvasPtr->xOrigin, canvasPtr->yOrigin,
	    canvasPtr->xOrigin + Tk_Width(canvasPtr->tkwin),
	    canvasPtr->yOrigin + Tk_Height(canvasPtr->tkwin));
}

/*
 *--------------------------------------------------------------
 *
 * DisplayCanvas --
 *
 *	This procedure redraws the contents of a canvas window.
 *	It is invoked as a do-when-idle handler, so it only runs
 *	when there's nothing else for the application to do.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Information appears on the screen.
 *
 *--------------------------------------------------------------
 */

static void
DisplayCanvas(clientData)
    ClientData clientData;	/* Information about widget. */
{
    XtalCanvas *canvasPtr = (XtalCanvas *) clientData;
    Tk_Window tkwin = canvasPtr->tkwin;
    int screenX1, screenX2, screenY1, screenY2, width, height;

    if (canvasPtr->tkwin == NULL) {
	return;
    }
    if (!Tk_IsMapped(tkwin)) {
	goto done;
    }


    /*
     * Compute the intersection between the area that needs redrawing
     * and the area that's visible on the screen.
     */

    if ((canvasPtr->redrawX1 < canvasPtr->redrawX2)
	    && (canvasPtr->redrawY1 < canvasPtr->redrawY2)) {
	screenX1 = canvasPtr->xOrigin + canvasPtr->inset;
	screenY1 = canvasPtr->yOrigin + canvasPtr->inset;
	screenX2 = canvasPtr->xOrigin + Tk_Width(tkwin) - canvasPtr->inset;
	screenY2 = canvasPtr->yOrigin + Tk_Height(tkwin) - canvasPtr->inset;
	if (canvasPtr->redrawX1 > screenX1) {
	    screenX1 = canvasPtr->redrawX1;
	}
	if (canvasPtr->redrawY1 > screenY1) {
	    screenY1 = canvasPtr->redrawY1;
	}
	if (canvasPtr->redrawX2 < screenX2) {
	    screenX2 = canvasPtr->redrawX2;
	}
	if (canvasPtr->redrawY2 < screenY2) {
	    screenY2 = canvasPtr->redrawY2;
	}
	if ((screenX1 >= screenX2) || (screenY1 >= screenY2)) {
	    goto borders;
	}
    
    
	canvasPtr->drawableXOrigin = screenX1;
	canvasPtr->drawableYOrigin = screenY1;

        XSetForeground(display,canvasPtr->pixmapGC,
                       BlackPixel(display, screen_num));

        if (canvasPtr->doubleBuffered && canvasPtr->pixmap) {

            XFillRectangle(display,canvasPtr->pixmap,
              canvasPtr->pixmapGC,0,0,
             Tk_Width(canvasPtr->tkwin),Tk_Height(canvasPtr->tkwin));

            canvasPtr->canvas = canvasPtr->pixmap ;
            if (canvasPtr->segmentList !=NULL)
               xExecuteCommands(canvasPtr,&(canvasPtr->segmentList));

	    XCopyArea(Tk_Display(tkwin), canvasPtr->pixmap, Tk_WindowId(tkwin),
		canvasPtr->pixmapGC,
		screenX1 - canvasPtr->xOrigin,         /* source X origin */
		screenY1 - canvasPtr->yOrigin,         /* source Y origin */
		(unsigned) (screenX2 - screenX1),      /* width */
		(unsigned) (screenY2 - screenY1),      /* height */
		screenX1 - canvasPtr->xOrigin,         /* dest X origin */
                screenY1 - canvasPtr->yOrigin);        /* dest Y origin */
        }    
        else {

            canvasPtr->canvas = Tk_WindowId(canvasPtr->tkwin);
	    width = screenX2 - screenX1;
	    height = screenY2 - screenY1;

	   /*
            * Clear the area to be redrawn.
            */

	    XFillRectangle(Tk_Display(tkwin), canvasPtr->canvas, 
                canvasPtr->pixmapGC, screenX1 - canvasPtr->xOrigin,
		screenY1 - canvasPtr->yOrigin, (unsigned int) width,
		(unsigned int) height);
/*            XClearWindow(display, canvasPtr->canvas );  */

	   /*
	    * Scan through the item list, redrawing those items that need it.
	    * An item must be redraw if either (a) it intersects the smaller
	    * on-screen area or (b) it intersects the full canvas area and its
	    * type requests that it be redrawn always (e.g. so subwindows can
	    * be unmapped when they move off-screen).
	    */

            if (canvasPtr->segmentList !=NULL)
                xExecuteCommands(canvasPtr,&(canvasPtr->segmentList));
    
        } 
        
        if (canvasPtr->auxrefcom) (canvasPtr->auxrefcom)() ;

    }

    /*
     * Draw the window borders, if needed.
     */

    borders:
    if (canvasPtr->flags & REDRAW_BORDERS) {
	canvasPtr->flags &= ~REDRAW_BORDERS;
	if (canvasPtr->borderWidth > 0) {
	    Tk_Draw3DRectangle(tkwin, Tk_WindowId(tkwin),
		    canvasPtr->bgBorder, canvasPtr->highlightWidth,
		    canvasPtr->highlightWidth,
		    Tk_Width(tkwin) - 2*canvasPtr->highlightWidth,
		    Tk_Height(tkwin) - 2*canvasPtr->highlightWidth,
		    canvasPtr->borderWidth, canvasPtr->relief);
	}
	if (canvasPtr->highlightWidth != 0) {
	    GC bgGC;

	    bgGC = Tk_GCForColor(canvasPtr->highlightBgColorPtr,
		    Tk_WindowId(tkwin));

/*	    TkpDrawHighlightBorder(tkwin, bgGC, bgGC,
  			canvasPtr->highlightWidth, Tk_WindowId(tkwin));
 */	    
	}
    }

    done:
    canvasPtr->flags &= ~REDRAW_PENDING;
    canvasPtr->redrawX1 = canvasPtr->redrawX2 = 0;
    canvasPtr->redrawY1 = canvasPtr->redrawY2 = 0;
    if (canvasPtr->flags & UPDATE_SCROLLBARS) {
	CanvasUpdateScrollbars(canvasPtr);
    }
}

/*
 *--------------------------------------------------------------
 *
 * CanvasEventProc --
 *
 *	This procedure is invoked by the Tk dispatcher for various
 *	events on canvases.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	When the window gets deleted, internal structures get
 *	cleaned up.  When it gets exposed, it is redisplayed.
 *
 *--------------------------------------------------------------
 */

static void
CanvasEventProc(clientData, eventPtr)
    ClientData clientData;	/* Information about window. */
    XEvent *eventPtr;		/* Information about event. */
{
    XtalCanvas *canvasPtr = (XtalCanvas *) clientData;
    

    if (eventPtr->type == Expose) {
	int x, y;

	x = eventPtr->xexpose.x + canvasPtr->xOrigin;
	y = eventPtr->xexpose.y + canvasPtr->yOrigin;
	TkCanvasEventuallyRedraw(canvasPtr, x, y,
		x + eventPtr->xexpose.width,
		y + eventPtr->xexpose.height);
	if ((eventPtr->xexpose.x < canvasPtr->inset)
		|| (eventPtr->xexpose.y < canvasPtr->inset)
		|| ((eventPtr->xexpose.x + eventPtr->xexpose.width)
		    > (Tk_Width(canvasPtr->tkwin) - canvasPtr->inset))
		|| ((eventPtr->xexpose.y + eventPtr->xexpose.height)
		    > (Tk_Height(canvasPtr->tkwin) - canvasPtr->inset))) {
	    canvasPtr->flags |= REDRAW_BORDERS;
	}


    } else if (eventPtr->type == DestroyNotify) {

	if (canvasPtr->tkwin != NULL) {
	    canvasPtr->tkwin = NULL;
            Tcl_DeleteCommandFromToken(canvasPtr->interp,
		    canvasPtr->widgetCmd);
	}
	if (canvasPtr->flags & REDRAW_PENDING) {
	    Tcl_CancelIdleCall(DisplayCanvas, (ClientData) canvasPtr);
	}

	Tcl_EventuallyFree((ClientData) canvasPtr, DestroyCanvas);

    } else if (eventPtr->type == ConfigureNotify) {
	canvasPtr->flags |= UPDATE_SCROLLBARS;

        xResize(canvasPtr, Tk_Width(canvasPtr->tkwin), Tk_Height(canvasPtr->tkwin));

	/*
	 * The call below is needed in order to recenter the canvas if
	 * it's confined and its scroll region is smaller than the window.
	 */

	CanvasSetOrigin(canvasPtr, canvasPtr->xOrigin, canvasPtr->yOrigin);
	TkCanvasEventuallyRedraw(canvasPtr, canvasPtr->xOrigin,
		canvasPtr->yOrigin,
		canvasPtr->xOrigin + Tk_Width(canvasPtr->tkwin),
		canvasPtr->yOrigin + Tk_Height(canvasPtr->tkwin));
	canvasPtr->flags |= REDRAW_BORDERS;

    } else if (eventPtr->type == FocusIn) {
	if (eventPtr->xfocus.detail != NotifyInferior) {
	    CanvasFocusProc(canvasPtr, 1);
	}
    } else if (eventPtr->type == FocusOut) {
	if (eventPtr->xfocus.detail != NotifyInferior) {
	    CanvasFocusProc(canvasPtr, 0);
	}
    } else if (eventPtr->type == UnmapNotify) {

    }
}


/*
 *----------------------------------------------------------------------
 *
 * xResize --
 *
 *      Handles resize events by noting what resizeFactor, xOffset,
 *      and yOffset will be necessary to retain an aspect ratio of 1/1.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Generates a new backbuff pixmap if required
 *	
 *
 *----------------------------------------------------------------------
 */
void
xResize(XtalCanvas *canvasPtr, int width, int height)
{
   FortAuxWinConfig  functn;   /* function to be called */
   float             aspect;   /* new drawable aspect ratio */

   canvasPtr->width = width;
   canvasPtr->height= height;
   aspect = (float) width / (float) height;
   if ( canvasPtr->saspect) 
      *(canvasPtr->saspect) =aspect ;

   if (canvasPtr->doubleBuffered) { 
                          
      if (canvasPtr->pixmap)    /* delete the old pixmap */
           Tk_FreePixmap(Tk_Display(canvasPtr->tkwin), canvasPtr->pixmap);

                           /* create new pixmap */
      canvasPtr->pixmap = Tk_GetPixmap(Tk_Display(canvasPtr->tkwin),
                           Tk_WindowId(canvasPtr->tkwin), width,height,
                          Tk_Depth(canvasPtr->tkwin));

      if (canvasPtr->pixmap ) {
            XSetForeground(display,canvasPtr->pixmapGC,
                       BlackPixel(display, screen_num));
            XFillRectangle(display,canvasPtr->pixmap,
              canvasPtr->pixmapGC,0,0,width,height);
      }
      else {
         printf("\n --- Tk library  Can't create new resized pixmap.\n");
         fflush(stdout);
         canvasPtr->doubleBuffered = 0 ; /* cancell this mode */
      }
   }

   if (canvasPtr->canvas != Tk_WindowId(canvasPtr->tkwin) &&
             canvasPtr->pixmap) 
       canvasPtr->canvas = canvasPtr->pixmap;  /* default */

   if ((functn = canvasPtr->auxwinconf) ) {
       functn(&width,&height) ;
   }
  
}

/*
 *----------------------------------------------------------------------
 *
 * CanvasCmdDeletedProc --
 *
 *	This procedure is invoked when a widget command is deleted.  If
 *	the widget isn't already in the process of being destroyed,
 *	this command destroys it.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The widget is destroyed.
 *
 *----------------------------------------------------------------------
 */

static void
CanvasCmdDeletedProc(clientData)
    ClientData clientData;	/* Pointer to widget record for widget. */
{
    XtalCanvas *canvasPtr = (XtalCanvas *) clientData;
    Tk_Window tkwin = canvasPtr->tkwin;


    /*
     * This procedure could be invoked either because the window was
     * destroyed and the command was then deleted (in which case tkwin
     * is NULL) or because the command was deleted, and then this procedure
     * destroys the widget.
     */

    if (tkwin != NULL) {
	canvasPtr->tkwin = NULL;
	Tk_DestroyWindow(tkwin);
printf("Unusual occurence: The Xtal command was deleted %x\n",tkwin);fflush(stdout);
    }
}

/*
 *--------------------------------------------------------------
 *
 * TkCanvasEventuallyRedraw --
 *
 *	Arrange for part or all of a canvas widget to redrawn at
 *	some convenient time in the future.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The screen will eventually be refreshed.
 *
 *--------------------------------------------------------------
 */

void
TkCanvasEventuallyRedraw(canvas, x1, y1, x2, y2)
    XtalCanvas *canvas;		/* Information about widget. */
    int x1, y1;			/* Upper left corner of area to redraw.
				 * Pixels on edge are redrawn. */
    int x2, y2;			/* Lower right corner of area to redraw.
				 * Pixels on edge are not redrawn. */
{
    XtalCanvas *canvasPtr = canvas;
    if ((x1 == x2) || (y1 == y2)) {
	return;
    }
    if (canvasPtr->flags & REDRAW_PENDING) {
	if (x1 <= canvasPtr->redrawX1) {
	    canvasPtr->redrawX1 = x1;
	}
	if (y1 <= canvasPtr->redrawY1) {
	    canvasPtr->redrawY1 = y1;
	}
	if (x2 >= canvasPtr->redrawX2) {
	    canvasPtr->redrawX2 = x2;
	}
	if (y2 >= canvasPtr->redrawY2) {
	    canvasPtr->redrawY2 = y2;
	}
    } else {
	canvasPtr->redrawX1 = x1;
	canvasPtr->redrawY1 = y1;
	canvasPtr->redrawX2 = x2;
	canvasPtr->redrawY2 = y2;
	Tcl_DoWhenIdle(DisplayCanvas, (ClientData) canvasPtr);
	canvasPtr->flags |= REDRAW_PENDING;
    }
}


/*
 * xDrawSegmentBorder:  draws a border box 10 pixels outside the boundary
 *      of a segment
 */
static void
xDrawSegmentBorder(XtalCanvas *canvasPtr, xSegment seg)
{
   float   w,                      /* size of segment */
           h;
   XtalCanvas *temp = currentGraphic;
   unsigned long  fg,bg;           /* fg pixel for xor function */
   unsigned long  mask;            /* values mask */
   currentGraphic = canvasPtr;

   fg = WhitePixel(display, screen_num);
   bg = BlackPixel(display, screen_num);
/* fg = (fg | colors[0]) & !(fg & colors[0]);    fails on alpha stations */
   fg = (fg | bg);    /*                    An alternative */
   mask = (unsigned long)GCForeground ;
   XSetForeground(display,canvasPtr->pixmapGC,fg ); /* apply new foreground */
   XSetFillStyle(display,canvasPtr->pixmapGC,FillSolid); 
   XSetLineAttributes(display,canvasPtr->pixmapGC,1,
          LineSolid,CapButt,JoinMiter ); 

   w = seg->xmax - seg->xmin;
   h = seg->ymax - seg->ymin;
            
   XSetFunction(display,canvasPtr->pixmapGC, GXxor);
   XDrawRectangle(display, Tk_WindowId(canvasPtr->tkwin), canvasPtr->pixmapGC,
      (int)(seg->xmin * canvasPtr->resizeFactor.w  - 10 +canvasPtr->xOffset),
      (int)(canvasPtr->height - (seg->ymax * canvasPtr->resizeFactor.h +  10+canvasPtr->yOffset)),
      (unsigned int)(w * canvasPtr->resizeFactor.w + 20),
      (unsigned int)(h * canvasPtr->resizeFactor.h + 20));
   XSetFunction(display, canvasPtr->pixmapGC, GXcopy);
   currentGraphic = temp;

} /* xDrawSegmentBorder */



/*
 *--------------------------------------------------------------
 *
 * CanvasBindProc --
 *
 *	This procedure is invoked by the Tk dispatcher to handle
 *	events associated with bindings on items.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Depends on the command invoked as part of the binding
 *	(if there was any).
 *
 *--------------------------------------------------------------
 */
Bool isAMotionEvent(Display *disp, XEvent *event, char *arg)
{
     if(event->type == MotionNotify) return True;
     return False;
}

static void
CanvasBindProc(clientData, eventPtr)
    ClientData clientData;		/* Pointer to canvas structure. */
    XEvent *eventPtr;			/* Pointer to X event that just
					 * happened. */
{
   Tcl_HashEntry  *hPtr;
   FortMouseEventCB  functn;
   xSegment    seg;
static xSegment segSelected=NULL;
static xSegment curSeg=NULL;

   float x,y;
   int XCoord=0;
   int YCoord=0;
   unsigned long button=0;
   unsigned long type=0;
   unsigned long modstate=0;
   unsigned long time=0;
   int eventmask;
   XtalCanvas *canvasPtr = (XtalCanvas *) clientData;

   if (canvasPtr->tkwin == NULL)  return;

   Tcl_Preserve((ClientData) canvasPtr);
   parseEvent(eventPtr,&modstate,&button,&time,&XCoord,&YCoord,&type);
   x=(                   (float)XCoord - canvasPtr->xOffset) /
                                         canvasPtr->resizeFactor.w;
   y=(canvasPtr->height -(float)YCoord - canvasPtr->yOffset) /
                                         canvasPtr->resizeFactor.h;
   if (button)modstate &= 0x07F;
   eventmask=  (button <<18) | (type <<12) | modstate  ;
                                              /* erase prior button states */


   if      (xtalEventMode == XTAL_FRAME) {
       switch (type) {
          case ButtonRelease:
              xtalWidget   = (char *) canvasPtr ;
              xtalEventMode=XTAL_NOMODE;
              break;
          default: 
              break;
       }
   }

   else if (xtalEventMode == XTAL_GETSTRING) {
       char   inbuf[20];              /* input buffer */
       int    inlen,                  /* length of substring */
              i;                      /* inner loop control var. */
       int    status;

       switch (type) {

           /* if the user pressed a key... */
           case KeyPress:

               /* figure out what ASCII characters were generated */
#ifdef __WIN32__

               inlen = XmbLookupString(inputContext,
                       (XKeyPressedEvent*)&(eventPtr->xkey),
                           inbuf, 20, NULL, &status);
#else
               inlen = XLookupString((XKeyEvent*)&(eventPtr->xkey),
                           inbuf, 20, NULL, NULL);
#endif

               /* for each character... */
               for (i = 0; i < inlen; i++)

                   /* we're done if it was the return key */
                   if (inbuf[i] == '\015') {

                           xtalWidget   = (char *) canvasPtr ;
                           xtalEventMode= XTAL_NOMODE;
                           break;  /* break out of inner loop */

                   /* else, if it's a "normal" character, and
                    * there's room, add it to the output buffer */
                   } else if ((isprint(inbuf[i]) ||
                             inbuf[i] == '\t') && xtalStringNOut < STR_BUF_SIZE)
                        xtalStringBuf[xtalStringNOut++]= inbuf[i];

                   /* else, if the user hit a backspace, and if
                    * there's something in the buffer to be
                    * deleted, then delete a character from the
                    * buffer */
                   else if ((inbuf[i] == '\010' ||
                             inbuf[i] == '\177') && xtalStringNOut > 0)
                         xtalStringNOut--;
               

               break;
        
           default: 
               break;
       }
   }
   else if (xtalEventMode == XTAL_GETXY) {
       switch (type) {
          
          case ButtonPress:
              hPtr=Tcl_FindHashEntry(&(canvasPtr->eventtable),(char*)eventmask);
              if (hPtr == NULL)  goto done  ;      /* doing nothing important */
              functn  = (FortMouseEventCB) Tcl_GetHashValue(hPtr);
              if (functn) functn(&x,&y);
              break;

          case MotionNotify:
              x=(      (float)eventPtr->xmotion.x - canvasPtr->xOffset) /
                                                canvasPtr->resizeFactor.w;
              y=(canvasPtr->height -(float)eventPtr->xmotion.y - 
                          canvasPtr->yOffset) / canvasPtr->resizeFactor.h;
              hPtr=Tcl_FindHashEntry(&(canvasPtr->eventtable),(char*)eventmask);
              if (hPtr == NULL)  goto done  ;      /* doing nothing important */
              functn  = (FortMouseEventCB) Tcl_GetHashValue(hPtr);
              if (functn) functn(&x,&y);
              break;

          case ButtonRelease:
              xtalXCoord = XCoord ;
              xtalYCoord = YCoord ;
              xtalWidget   = (char *) canvasPtr ;
              xtalWidgetLevel=0;    
              xtalEventMode=XTAL_NOMODE;
              hPtr=Tcl_FindHashEntry(&(canvasPtr->eventtable),(char*)eventmask);
              if (hPtr == NULL)  goto done  ;      /* doing nothing important */
              functn  = (FortMouseEventCB) Tcl_GetHashValue(hPtr);
              if (functn) functn(&x,&y);
              break;
          default: 
              break;
       }
   }
   else if (xtalEventMode == XTAL_GETSEG) {
       switch (type) {

          case ButtonPress:
             for (seg = canvasPtr->segmentList; seg; seg = seg->next)
                 if (seg->detect == True && !(
                       x < seg->xmin ||
                       x > seg->xmax ||
                       y < seg->ymin ||
                       y > seg->ymax)) {
                    segSelected = curSeg = seg;
                    xDrawSegmentBorder(canvasPtr,curSeg);
                    break;
                 }
             if (segSelected) goto done ;         /* doing nothing important */
             hPtr=Tcl_FindHashEntry(&(canvasPtr->eventtable),(char*)eventmask);
             if (hPtr == NULL)  goto done  ;      /* doing nothing important */
             functn  = (FortMouseEventCB) Tcl_GetHashValue(hPtr);
             if (functn) functn(&x,&y);
             break;

          case MotionNotify:

               /* if a segment was currently selected... */
             if (segSelected)  {
               /* if the segment is no longer selected, * undraw its border */
                if (x<segSelected->xmin ||x>segSelected->xmax ||
                    y<segSelected->ymin ||y>segSelected->ymax) {
                   if (curSeg) {
                       xDrawSegmentBorder(canvasPtr,curSeg);
                       curSeg = NULL;
                   } 
                }
                else if (curSeg == NULL) {
                   curSeg = segSelected;
                   xDrawSegmentBorder(canvasPtr,curSeg);
                } /* otherwise, it's still selected. Loop around for more user input */
                break;
             }

             hPtr=Tcl_FindHashEntry(&(canvasPtr->eventtable),(char*)eventmask);
             if (hPtr == NULL)  goto done ;        /* doing nothing important */

             functn  = (FortMouseEventCB) Tcl_GetHashValue(hPtr);
             if (functn) functn(&x,&y);
             break;


          case ButtonRelease:

             if (segSelected){
                if (curSeg != NULL) {
                   xDrawSegmentBorder(canvasPtr,curSeg);
                   xtalSegment  = curSeg->id ;
                   xtalWidget   = (char *) canvasPtr ;
                   xtalWidgetLevel = 0;    
                   xtalEventMode= XTAL_NOMODE;  /* the job is done */
                   curSeg=NULL;
                } /* if */
                segSelected = NULL;     
                break;
             }

             hPtr=Tcl_FindHashEntry(&(canvasPtr->eventtable),(char*)eventmask);
             if (hPtr == NULL)  goto done ;       /* doing nothing important */
             functn  = (FortMouseEventCB) Tcl_GetHashValue(hPtr);
             if (functn) functn(&x,&y);
             break;


          default: ;
       }
   }

   done:
   Tcl_Release((ClientData) canvasPtr);
}



static void
parseEvent(XEvent *eventPtr,
      unsigned long *state,
      unsigned long *buttonNumber,
      unsigned long *time,
      int       *XCoord,
      int       *YCoord,
      unsigned long       *type)
{
   int flags = 0 ;

   if (eventPtr->type < TK_LASTEVENT)
               flags = flagArray[eventPtr->type];

   if (flags & MOTION) {
          *buttonNumber = 0;
   } else *buttonNumber = eventPtr->xbutton.button;

   if (flags & (KEY_BUTTON_MOTION_VIRTUAL)) {
         *state = eventPtr->xkey.state;
   } else if (flags & CROSSING) {
         *state = eventPtr->xcrossing.state;
   }

   if (flags & (KEY_BUTTON_MOTION_VIRTUAL)) {
         *time = (int) eventPtr->xkey.time;
   } else if (flags & CROSSING) {
         *time = (int) eventPtr->xcrossing.time;
   } else if (flags & PROP) {
         *time = (int) eventPtr->xproperty.time;
   }

   if (flags & (KEY_BUTTON_MOTION_VIRTUAL)) {
        *XCoord = eventPtr->xkey.x;
   } else if (flags & CROSSING) {
        *YCoord = eventPtr->xcrossing.x;
   }

   if (flags & (KEY_BUTTON_MOTION_VIRTUAL)) {
        *YCoord = eventPtr->xkey.y;
   } else if (flags & EXPOSE) {
        *YCoord = eventPtr->xexpose.y;
   }

   *type = eventPtr->type ;
}


/*
 *----------------------------------------------------------------------
 *
 * CanvasFocusProc --
 *
 *	This procedure is called whenever a canvas gets or loses the
 *	input focus.  It's also called whenever the window is
 *	reconfigured while it has the focus.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The cursor gets turned on or off.
 *
 *----------------------------------------------------------------------
 */

static void
CanvasFocusProc(canvasPtr, gotFocus)
    XtalCanvas *canvasPtr;	/* Canvas that just got or lost focus. */
    int gotFocus;		/* 1 means window is getting focus, 0 means
				 * it's losing it. */
{
    if (canvasPtr->highlightWidth > 0) {

	canvasPtr->flags |= REDRAW_BORDERS;
	if (!(canvasPtr->flags & REDRAW_PENDING)) {
	    Tcl_DoWhenIdle(DisplayCanvas, (ClientData) canvasPtr);
	    canvasPtr->flags |= REDRAW_PENDING;
	}
    }
}


/*
 *--------------------------------------------------------------
 *
 * GridAlign --
 *
 *	Given a coordinate and a grid spacing, this procedure
 *	computes the location of the nearest grid line to the
 *	coordinate.
 *
 * Results:
 *	The return value is the location of the grid line nearest
 *	to coord.
 *
 * Side effects:
 *	None.
 *
 *--------------------------------------------------------------
 */

static double
GridAlign(coord, spacing)
    double coord;		/* Coordinate to grid-align. */
    double spacing;		/* Spacing between grid lines.   If <= 0
				 * then no alignment is done. */
{
    if (spacing <= 0.0) {
	return coord;
    }
    if (coord < 0) {
	return -((int) ((-coord)/spacing + 0.5)) * spacing;
    }
    return ((int) (coord/spacing + 0.5)) * spacing;
}

/*
 *----------------------------------------------------------------------
 *
 * PrintScrollFractions --
 *
 *	Given the range that's visible in the window and the "100%
 *	range" for what's in the canvas, print a string containing
 *	the scroll fractions.  This procedure is used for both x
 *	and y scrolling.
 *
 * Results:
 *	The memory pointed to by string is modified to hold
 *	two real numbers containing the scroll fractions (between
 *	0 and 1) corresponding to the other arguments.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static void
PrintScrollFractions(screen1, screen2, object1, object2, string)
    int screen1;		/* Lowest coordinate visible in the window. */
    int screen2;		/* Highest coordinate visible in the window. */
    int object1;		/* Lowest coordinate in the object. */
    int object2;		/* Highest coordinate in the object. */
    char *string;		/* Two real numbers get printed here.  Must
				 * have enough storage for two %g
				 * conversions. */
{
    double range, f1, f2;

    range = object2 - object1;
    if (range <= 0) {
	f1 = 0;
	f2 = 1.0;
    } else {
	f1 = (screen1 - object1)/range;
	if (f1 < 0) {
	    f1 = 0.0;
	}
	f2 = (screen2 - object1)/range;
	if (f2 > 1.0) {
	    f2 = 1.0;
	}
	if (f2 < f1) {
	    f2 = f1;
	}
    }
    sprintf(string, "%g %g", f1, f2);
}

/*
 *--------------------------------------------------------------
 *
 * CanvasUpdateScrollbars --
 *
 *	This procedure is invoked whenever a canvas has changed in
 *	a way that requires scrollbars to be redisplayed (e.g. the
 *	view in the canvas has changed).
 *      Only ever called by DisplayCanvas.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	If there are scrollbars associated with the canvas, then
 *	their scrolling commands are invoked to cause them to
 *	redisplay.  If errors occur, additional Tcl commands may
 *	be invoked to process the errors.
 *
 *--------------------------------------------------------------
 */

static void
CanvasUpdateScrollbars(canvasPtr)
    XtalCanvas *canvasPtr;		/* Information about canvas. */
{
    int result;
    char buffer[200];
    Tcl_Interp *interp;
    int xOrigin, yOrigin, inset, width, height, scrollX1, scrollX2,
        scrollY1, scrollY2;
    char *xScrollCmd, *yScrollCmd;

    /*
     * Save all the relevant values from the canvasPtr, because it might be
     * deleted as part of either of the two calls to Tcl_VarEval below.
     */
    
    interp = canvasPtr->interp;
    Tcl_Preserve((ClientData) interp);
    xScrollCmd = canvasPtr->xScrollCmd;
    if (xScrollCmd != (char *) NULL) {
        Tcl_Preserve((ClientData) xScrollCmd);
    }
    yScrollCmd = canvasPtr->yScrollCmd;
    if (yScrollCmd != (char *) NULL) {
        Tcl_Preserve((ClientData) yScrollCmd);
    }
    xOrigin = canvasPtr->xOrigin;
    yOrigin = canvasPtr->yOrigin;
    inset = canvasPtr->inset;
    width = Tk_Width(canvasPtr->tkwin);
    height = Tk_Height(canvasPtr->tkwin);
    scrollX1 = canvasPtr->scrollX1;
    scrollX2 = canvasPtr->scrollX2;
    scrollY1 = canvasPtr->scrollY1;
    scrollY2 = canvasPtr->scrollY2;
    canvasPtr->flags &= ~UPDATE_SCROLLBARS;
    if (canvasPtr->xScrollCmd != NULL) {
/* the xtal modification */
        if (canvasPtr->auxwinxscroll )  
            canvasPtr->auxwinxscroll(&(canvasPtr->xScale),&(canvasPtr->xCentre));  
/* end of modification */
 
	PrintScrollFractions(xOrigin + inset, xOrigin + width - inset,
                scrollX1, scrollX2, buffer);
	result = Tcl_VarEval(interp, xScrollCmd, " ", buffer, (char *) NULL);
	if (result != TCL_OK) {
	    Tcl_BackgroundError(interp);
	}
	Tcl_ResetResult(interp);
        Tcl_Release((ClientData) xScrollCmd);
    }

    if (yScrollCmd != NULL) {
/* the xtal modification */
        if (canvasPtr->auxwinyscroll )  
           canvasPtr->auxwinyscroll(&(canvasPtr->yScale),&(canvasPtr->yCentre));  
/* end of modification */
	PrintScrollFractions(yOrigin + inset, yOrigin + height - inset,
                scrollY1, scrollY2, buffer);
	result = Tcl_VarEval(interp, yScrollCmd, " ", buffer, (char *) NULL);
	if (result != TCL_OK) {
	    Tcl_BackgroundError(interp);
	}
	Tcl_ResetResult(interp);
        Tcl_Release((ClientData) yScrollCmd);
    }
    Tcl_Release((ClientData) interp);
}

/*
 *--------------------------------------------------------------
 *
 * CanvasSetOrigin --
 *
 *	This procedure is invoked to change the mapping between
 *	canvas coordinates and screen coordinates in the canvas
 *	window.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	The canvas will be redisplayed to reflect the change in
 *	view.  In addition, scrollbars will be updated if there
 *	are any.
 *
 *--------------------------------------------------------------
 */

static void
CanvasSetOrigin(canvasPtr, xOrigin, yOrigin)
    XtalCanvas *canvasPtr;	/* Information about canvas. */
    int xOrigin;		/* New X origin for canvas (canvas x-coord
				 * corresponding to left edge of canvas
				 * window). */
    int yOrigin;		/* New Y origin for canvas (canvas y-coord
				 * corresponding to top edge of canvas
				 * window). */
{
    int left, right, top, bottom, delta;
    int twidth,theight;
    int width,height;
    float  aspect,unith;  /* new drawable aspect ratio */

    /*
     * If scroll increments have been set, round the window origin
     * to the nearest multiple of the increments.  Remember, the
     * origin is the place just inside the borders,  not the upper
     * left corner.
     */

    if (canvasPtr->xScrollIncrement > 0) {
	if (xOrigin >= 0) {
	    xOrigin += canvasPtr->xScrollIncrement/2;
	    xOrigin -= (xOrigin + canvasPtr->inset)
		    % canvasPtr->xScrollIncrement;
	} else {
	    xOrigin = (-xOrigin) + canvasPtr->xScrollIncrement/2;
	    xOrigin = -(xOrigin - (xOrigin - canvasPtr->inset)
		    % canvasPtr->xScrollIncrement);
	}
    }
    if (canvasPtr->yScrollIncrement > 0) {
	if (yOrigin >= 0) {
	    yOrigin += canvasPtr->yScrollIncrement/2;
	    yOrigin -= (yOrigin + canvasPtr->inset)
		    % canvasPtr->yScrollIncrement;
	} else {
	    yOrigin = (-yOrigin) + canvasPtr->yScrollIncrement/2;
	    yOrigin = -(yOrigin - (yOrigin - canvasPtr->inset)
		    % canvasPtr->yScrollIncrement);
	}
    }

    /*
     * Adjust the origin if necessary to keep as much as possible of the
     * canvas in the view.  The variables left, right, etc. keep track of
     * how much extra space there is on each side of the view before it
     * will stick out past the scroll region.  If one side sticks out past
     * the edge of the scroll region, adjust the view to bring that side
     * back to the edge of the scrollregion (but don't move it so much that
     * the other side sticks out now).  If scroll increments are in effect,
     * be sure to adjust only by full increments.
     */


    if ((canvasPtr->confine) && (canvasPtr->regionString != NULL)) {
	left = xOrigin + canvasPtr->inset - canvasPtr->scrollX1;
	right = canvasPtr->scrollX2
		- (xOrigin + Tk_Width(canvasPtr->tkwin) - canvasPtr->inset);
	top = yOrigin + canvasPtr->inset - canvasPtr->scrollY1;
	bottom = canvasPtr->scrollY2
		- (yOrigin + Tk_Height(canvasPtr->tkwin) - canvasPtr->inset);

	if ((left < 0) && (right > 0)) {
	    delta = (right > -left) ? -left : right;
	    if (canvasPtr->xScrollIncrement > 0) {
		delta -= delta % canvasPtr->xScrollIncrement;
	    }
	    xOrigin += delta;
	} else if ((right < 0) && (left > 0)) {
	    delta = (left > -right) ? -right : left;
	    if (canvasPtr->xScrollIncrement > 0) {
		delta -= delta % canvasPtr->xScrollIncrement;
	    }
	    xOrigin -= delta;
	}
	if ((top < 0) && (bottom > 0)) {
	    delta = (bottom > -top) ? -top : bottom;
	    if (canvasPtr->yScrollIncrement > 0) {
		delta -= delta % canvasPtr->yScrollIncrement;
	    }
	    yOrigin += delta;
	} else if ((bottom < 0) && (top > 0)) {
	    delta = (top > -bottom) ? -bottom : top;
	    if (canvasPtr->yScrollIncrement > 0) {
		delta -= delta % canvasPtr->yScrollIncrement;
	    }
	    yOrigin -= delta;
	}
    }

    /*
     * The Xtal Canvas Hack
     * needs some cleaning up 
     *
     */

   width   = Tk_Width(canvasPtr->tkwin);
   height  = Tk_Height(canvasPtr->tkwin);
   twidth  = canvasPtr->scrollX2 - canvasPtr->scrollX1 ;
   theight = canvasPtr->scrollY2 - canvasPtr->scrollY1 ;
   if (twidth <=0)  twidth  = width;
   if (theight <=0) theight = height;
   aspect = (float) width / (float) height;      /* changed */
   canvasPtr->xScale = (float)twidth /(float)(width - 2*canvasPtr->inset);
   canvasPtr->yScale = (float)theight/(float)(height - 2*canvasPtr->inset);

   if (aspect < canvasPtr->aspectRatio) 
         unith = (float)width / canvasPtr->aspectRatio;
   else  unith = (float)height;  /* one unit of height=1.0 */
   canvasPtr->resizeFactor.w = canvasPtr->xScale* unith ;
   canvasPtr->resizeFactor.h = canvasPtr->yScale* unith ;
          /* centres must lie in between 0. and 1. - whatever dimens*/
   canvasPtr->xCentre = ((float)xOrigin +0.5f *(float)width )/(float)twidth ;
   canvasPtr->yCentre = ((float)yOrigin +0.5f *(float)height )/(float)theight ;
       /* offsets are in pixels - in the scaled coordinate system */
   canvasPtr->xOffset = (int)((float)width /2. - canvasPtr->aspectRatio* 
                  canvasPtr->resizeFactor.w  * canvasPtr->xCentre);
   canvasPtr->yOffset = (int)((float)height /2. -
                  canvasPtr->resizeFactor.h  * canvasPtr->yCentre);
   if (canvasPtr->sxmag)  *(canvasPtr->sxmag)  = canvasPtr->xScale ;
   if (canvasPtr->symag)  *(canvasPtr->symag)  = canvasPtr->yScale ;
   if (canvasPtr->sxcent) *(canvasPtr->sxcent) = canvasPtr->xCentre   ;
   if (canvasPtr->sycent) *(canvasPtr->sycent) = canvasPtr->yCentre  ;

/*
printf("aspect %f %f \n",aspect ,canvasPtr->aspectRatio  ); fflush(stdout);
printf("virtual dimens %d %d \n", twidth ,theight ); fflush(stdout);
printf("real dimens %d %d \n", canvasPtr->width ,canvasPtr->height ); fflush(stdout);
printf("offsets :%d %d \n", canvasPtr->xOffset ,canvasPtr->yOffset ); fflush(stdout);
printf("resizing the window %f %f \n", 
           canvasPtr->resizeFactor.w,
           canvasPtr->resizeFactor.h); fflush(stdout);
 */

    /*
     * back to Tk ...
     * 
     */

    if ((xOrigin == canvasPtr->xOrigin) && (yOrigin == canvasPtr->yOrigin)) {
	return;
    }

    /*
     * Tricky point: must redisplay not only everything that's visible
     * in the window's final configuration, but also everything that was
     * visible in the initial configuration.  This is needed because some
     * item types, like windows, need to know when they move off-screen
     * so they can explicitly undisplay themselves.
     */

/*    TkCanvasEventuallyRedraw(canvasPtr,
	    canvasPtr->xOrigin, canvasPtr->yOrigin,
	    canvasPtr->xOrigin + Tk_Width(canvasPtr->tkwin),
	    canvasPtr->yOrigin + Tk_Height(canvasPtr->tkwin)); */

    canvasPtr->xOrigin = xOrigin;
    canvasPtr->yOrigin = yOrigin;
    canvasPtr->flags |= UPDATE_SCROLLBARS;
    TkCanvasEventuallyRedraw(canvasPtr,
	    canvasPtr->xOrigin, canvasPtr->yOrigin,
	    canvasPtr->xOrigin + Tk_Width(canvasPtr->tkwin),
	    canvasPtr->yOrigin + Tk_Height(canvasPtr->tkwin));
}



int
XtalCanvasGetCoord(interp, canvas, string, doublePtr)
    Tcl_Interp *interp;         /* Interpreter for error reporting. */
    XtalCanvas *canvas;           /* Canvas to which coordinate applies. */
    char *string;               /* Describes coordinate (any screen
                                 * coordinate form may be used here). */
    double *doublePtr;          /* Place to store converted coordinate. */
{
    XtalCanvas *canvasPtr = canvas;
    if (Tk_GetScreenMM(canvasPtr->interp, canvasPtr->tkwin, string,
            doublePtr) != TCL_OK) {
        return TCL_ERROR;
    }
    *doublePtr *= canvasPtr->pixelsPerMM;
    return TCL_OK;
}



/* 
 * END of builtin Canvas functionality commands
 *
 */

/*
 *##########################################################################
 *                 FACILITATING TCL/TK API FUNCTIONS
 *##########################################################################
 */


void xgetid_(XtlPtr *pt,char *path,int len)
{
  char lpath[200];  
  for( ;len>0 && path[len-1] == ' '; len--) ;
  strncpy(lpath,path,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 
  *pt = (XtlPtr) Tk_GetUid(lpath);    /* return with unique pointer (id) */
}

void __stdcall XGETID(XtlPtr *pt,char *path,int len)
{ xgetid_(pt,path,len) ; }
#ifndef VMS
void xgetid(XtlPtr *pt,char *path,int len)
{ xgetid_(pt,path,len) ; }
#endif

void xlinki_(int *ptr,char *varname,int len)
{
  char lpath[200];  
  for( ;len>0 && varname[len-1] == ' '; len--) ;
  strncpy(lpath,varname,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 

  if (!interp) {
       printf( "\n--- Tk interface:  xlinki_ called before xcont_\n");
      fflush(stdout);
  }
  else if (Tcl_LinkVar(interp,lpath,(char *)ptr,TCL_LINK_INT) ==TCL_ERROR) {
      printf( "\n--- Tk interface:  unable to link variable %s\n",lpath);
      fflush(stdout);
  }
}

void __stdcall XLINKI(int *ptr,char *varname,int len)
{ xlinki_(ptr,varname,len) ; }
#ifndef VMS
void xlinki(int *ptr,char *varname,int len)
{ xlinki_(ptr,varname,len) ; }
#endif



void xlinkd_(float *ptr,char *varname,int len)
{
  char lpath[200];  
  for( ;len>0 && varname[len-1] == ' '; len--) ;
  strncpy(lpath,varname,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 

  if (!interp) {
       printf( "\n--- Tk interface:  xlinki_ called before xcont_\n");
      fflush(stdout);
  }
  else if (Tcl_LinkVar(interp,lpath,(char *)ptr,TCL_LINK_DOUBLE) ==TCL_ERROR) {
      printf( "\n--- Tk interface:  unable to link variable %s\n",lpath);
      printf( "\n--- Tk interface:  %s\n",interp->result);
      fflush(stdout);
  }
}

void __stdcall XLINKD(float *ptr,char *varname,int len)
{ xlinkd_(ptr,varname,len) ; }
#ifndef VMS
void xlinkd(float *ptr,char *varname,int len)
{ xlinkd_(ptr,varname,len) ; }
#endif



void xlinks_(char *ptr,char *varname,int len)
{
  char lpath[200];  
  for( ;len>0 && varname[len-1] == ' '; len--) ;
  strncpy(lpath,varname,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 

  if (!interp) {
       printf( "\n--- Tk interface:  xlinki_ called before xcont_\n");
      fflush(stdout);
  }
  else if (Tcl_LinkVar(interp,lpath,(char *)ptr,TCL_LINK_STRING) ==TCL_ERROR) {
      printf( "\n--- Tk interface:  unable to link variable %s-%s\n",lpath,ptr);
      fflush(stdout);
  }
}
/* potential for problems here with _WIN32_ */
void __stdcall XLINKS(char *ptr,char *varname,int len)
{    xlinks_(ptr,varname,len) ;}
#ifndef VMS
void xlinks(char *ptr,char *varname,int len)
{    xlinks_(ptr,varname,len) ;}
#endif



void xulink_(char *varname,int len)
{
  char lpath[200];  
  for( ;len>0 && varname[len-1] == ' '; len--) ;
  strncpy(lpath,varname,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 
  Tcl_UnlinkVar(interp,lpath);     /* unlink this variable */
}

void __stdcall XULINK(char *varname,int len)
{ xulink_(varname,len);}
#ifndef VMS
void xulink(char *varname,int len)
{ xulink_(varname,len);}
#endif


void xgetiv_(int *ptr, char *vname, int len)
{
   int        iptr=0;
   char       lvar[200];  
   Tcl_Obj    *ret ;

   for( ;len>0 && vname[len-1] == ' '; len--) ;
   strncpy(lvar,vname,len);
   lvar[len] ='\0';                /* terminate the fortran string */ 

   if ((ret= Tcl_GetVar2Ex(interp,lvar,NULL,TCL_LEAVE_ERR_MSG ))== NULL) {
      printf( "\n--- Tk interface:  xgetiv_ undefined int variable %s\n",lvar);
      fflush(stdout);
      return; 
   }

   if (Tcl_GetIntFromObj(interp, ret, &iptr) == TCL_ERROR ) {
      printf("\n--- Tk interface :xgetiv: %s \n",interp->result); 
      fflush(stdout);  
      return; 
   }
   *ptr = iptr;
}

void __stdcall XGETIV(int *ptr, char *vname, int len)
{ xgetiv_(ptr,vname,len);}
#ifndef VMS
void xgetiv(int *ptr, char *vname, int len)
{ xgetiv_(ptr,vname,len);}
#endif

void xsetiv_(int *ptr, char *vname, int len)
{
  Tcl_Obj *objPtr ;
  char lvar[200];  

  for( ;len>0 && vname[len-1] == ' '; len--) ;
  strncpy(lvar,vname,len);
  lvar[len] ='\0';                /* terminate the fortran string */ 

  objPtr = Tcl_NewIntObj(*ptr) ;

  if (Tcl_SetVar2Ex(interp,lvar,NULL,objPtr, TCL_LEAVE_ERR_MSG )==NULL) {
     printf( "\n--- Tk interface: xsetiv_ error %s : %s\n",lvar,interp->result);
     fflush(stdout);
  }
}

void __stdcall XSETIV(int *ptr, char *vname, int len)
{ xsetiv_(ptr,vname,len);}
#ifndef VMS
void xsetiv(int *ptr, char *vname, int len)
{ xsetiv_(ptr,vname,len);}
#endif


void xgetfv_(float *ptr, char *vname, int len)
{
  Tcl_Obj     *ret ;
   double     dptr=0.;
   char       lvar[200];  

   for( ;len>0 && vname[len-1] == ' '; len--) ;
   strncpy(lvar,vname,len);
   lvar[len] ='\0';                /* terminate the fortran string */ 

   if ((ret= Tcl_GetVar2Ex(interp,lvar,NULL,TCL_LEAVE_ERR_MSG ))== NULL) {
      printf( "\n--- Tk interface: xgetfv_ undefined real variable %s : %s \n",
             lvar,interp->result);
      fflush(stdout);
      return; 
   }

   if (Tcl_GetDoubleFromObj(interp, ret, &dptr) == TCL_ERROR ) {
      printf("\n--- Tk interface :xgetfv: %s \n",interp->result); 
      fflush(stdout);  
      return; 
   }
   *ptr = (float) dptr;
}
void __stdcall XGETFV(float *ptr, char *vname, int len)
{xgetfv_(ptr,vname,len);}
#ifndef VMS
void xgetfv(float *ptr, char *vname, int len)
{xgetfv_(ptr,vname,len);}
#endif



void xsetfv_(float *ptr, char *vname, int len)
{
  Tcl_Obj *objPtr ;
  char lvar[200];  

  for( ;len>0 && vname[len-1] == ' '; len--) ;
  strncpy(lvar,vname,len);
  lvar[len] ='\0';                /* terminate the fortran string */ 

  objPtr = Tcl_NewDoubleObj((double)*ptr) ; /* ref count = 0 */

  if (Tcl_SetVar2Ex(interp,lvar,NULL,objPtr, TCL_LEAVE_ERR_MSG )==NULL) {
      printf( "\n--- Tk interface: xsetfv_ error %s : %s\n",lvar,interp->result);
      fflush(stdout);
  }
}
void __stdcall XSETFV(float *ptr, char *vname, int len)
{xsetfv_(ptr,vname,len);}
#ifndef VMS
void xsetfv(float *ptr, char *vname, int len)
{xsetfv_(ptr,vname,len);}
#endif


void xevali_(int *rval, char *command,int len)
{
  char lpath[200];  
  int  error;
  Tcl_Obj *retObj;

  for( ;len>0 && command[len-1] == ' '; len--) ;
  strncpy(lpath,command,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 
  error= Tcl_Eval(interp, lpath);  /* set the script name */
  if (Tcl_Eval(interp, lpath) !=TCL_ERROR ){
     retObj = Tcl_GetObjResult(interp);
     Tcl_GetIntFromObj(interp,retObj,rval) ;
  }
}
void __stdcall XEVALI(int *rval, char *command,int len)
{    xevali_(rval,command,len);}
#ifndef VMS
void xevali(int *rval, char *command,int len)
{    xevali_(rval,command,len);}
#endif



void xevalf_(float *rval, char *command,int len)
{
  char lpath[200];  
  double  dub;
  int  error;
  Tcl_Obj *retObj;

  for( ;len>0 && command[len-1] == ' '; len--) ;
  strncpy(lpath,command,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 
  error= Tcl_Eval(interp, lpath);  /* set the script name */
  if (Tcl_Eval(interp, lpath) !=TCL_ERROR ){
     retObj = Tcl_GetObjResult(interp);
     Tcl_GetDoubleFromObj(interp,retObj,&dub) ;
     *rval = (float) dub ;
  }
}
void __stdcall XEVALF(float *rval, char *command,int len)
{    xevalf_(rval,command,len);}
#ifndef VMS
void xevalf(float *rval, char *command,int len)
{    xevalf_(rval,command,len);}
#endif


void xtclev_(char *command,int len)
{
  char lpath[200];  
  int  error;

  for( ;len>0 && command[len-1] == ' '; len--) ;
  strncpy(lpath,command,len);
  lpath[len] ='\0';                /* terminate the fortran string */ 
  error= Tcl_Eval(interp, lpath);  /* set the script name */
  XTALTCLERROR;
}

void __stdcall XTCLEV(char *command,int len)
{    xtclev_(command,len);}
#ifndef VMS
void xtclev(char *command,int len)
{    xtclev_(command,len);}
#endif


void xgsval_(char *variable, char *stret,int *len, int vlen1,int vlen2)
{
  char lvar[200];  
  char *pt;

  for( ;vlen1>0 && variable[vlen1-1] == ' '; vlen1--) ;
  strncpy(lvar,variable,vlen1);
  lvar[vlen1] ='\0';                /* terminate the fortran string */ 

  pt = Tcl_GetVar(interp, lvar,TCL_GLOBAL_ONLY);  /* set the script name */
  *len =strlen(pt);
  *len = (*len < vlen2) ? *len : vlen2;
  strncpy(stret,pt,*len);
}

/* problems here with _WIN32_ */
#ifdef __WIN32__
void __stdcall XGSVAL(char *variable,int vlen1,char *stret,int vlen2, int *len)
#else
void XGSVAL(char *variable, char *stret,int *len, int vlen1,int vlen2)
#endif
{    xgsval_(variable,stret,len,vlen1,vlen2);}
#ifndef VMS
void xgsval(char *variable, char *stret,int *len, int vlen1,int vlen2)
{    xgsval_(variable,stret,len,vlen1,vlen2);}
#endif



/*
 * xtalBinding(toplevel level segnum) should be called by any
 * Tk widgets expecting Xtal to respond from within xgetsg_
 *
 */

int
xtalBinding(clientData, linterp, objc, objv)
    ClientData clientData;	/* Main window associated with
				 * interpreter. */
    Tcl_Interp *linterp;	/* Current interpreter. */
    int objc;			/* Number of arguments. */
    Tcl_Obj *CONST objv[];	/* Argument strings. */
{
    int level;
    int length;
    int segnum;
    int error;
    char  *wPtr;                /* pointer to the string window name*/

    /* only fire the bindings in segment mode */
    if (xtalEventMode != XTAL_GETSEG) { return TCL_OK;}

    if (objc < 3 || objc >4) {
	Tcl_AppendResult(linterp, "wrong # args: should be \"",
                Tcl_GetStringFromObj(objv[0],&length),
		" pathName ?options?\"", (char *) NULL);
	return TCL_ERROR;
    }
    wPtr = Tcl_GetStringFromObj(objv[1],&length);

    if (objc==4) {                    /* three argument method */
        error = Tcl_GetIntFromObj(linterp,objv[3],&segnum);
        if (error !=TCL_OK) { return error ; }

        error = Tcl_GetIntFromObj(linterp,objv[2],&level);
        if (error !=TCL_OK) { return error ; }
    }
    else {                           /* two argument method */
        error = Tcl_GetIntFromObj(linterp,objv[2],&segnum);
        if (error !=TCL_OK) { return error ; }
        level=0;                     /* level defaults to zero */
    }
    

    xtalWidget      = Tk_GetUid(wPtr);
    xtalWidgetLevel = level;
    xtalSegment     = segnum;
    xtalEventMode   = XTAL_NOMODE;   /* mission accomplished */
    return TCL_OK;
}


/*
 * getIndexedColour queries the xtal canvas Tk colourmap
 * Tk widgets expecting Xtal to respond from within xgetsg_
 *
 */

int
getIndexedColour(clientData, linterp, objc, objv)
    ClientData clientData;	/* Main window associated with
				 * interpreter. */
    Tcl_Interp *linterp;	/* Current interpreter. */
    int objc;			/* Number of arguments. */
    Tcl_Obj *CONST objv[];	/* Argument strings. */
{
    char *color=NULL;
    int length=0;
    int error;
    int colID=0;


    if (objc < 2 || objc >2) {
	Tcl_AppendResult(linterp, "wrong # args: should be \"",
                Tcl_GetStringFromObj(objv[0],&length),
		" index", (char *) NULL);
	return TCL_ERROR;
    }
    error = Tcl_GetIntFromObj(linterp,objv[1],&colID);
    if (error != TCL_OK) { return error ; }


    if (colID > maxColUsed || gcCol[colID]==NULL ) {
       color = Tcl_Alloc(5); /* alloc 5 bytes for the string and null term*/
       if (color) strncpy(color,"none",5); /* no color value */
    }
    else {
       color = Tcl_Alloc(1+strlen(cstring[colID])) ;
       if (color) strcpy(color,cstring[colID]);  
    }

    if (color)  {
       Tcl_SetResult(linterp,color,TCL_DYNAMIC) ; /* no color value */
       return TCL_OK;
    } 
    else  {
       Tcl_SetResult(linterp,
          "\n --- Tk: Couldn't alloc mem for color string.",TCL_STATIC) ; 
       return TCL_ERROR;
    }
}


/*
 *##########################################################################
 *                 FACILITATING TCL/TK API FUNCTIONS
 *##########################################################################
 */

/*
 * The following command invoked from Fortran returns with a pointer
 * To the Canvas Ptr of an xtal canvas. 
 *
 */

void xstart_(int *dbuf, XtlPtr *pt, char *path, int len) 
{
   char lpath[256];
   Tk_Window   tkwin;
   XtalCanvas   *canvasPtr=xtalWindowList;


   if (!canvasPtr) {
      printf( 
"\n--- Tk interface:  No xtal canvas is available for attachment in start_\n");
      fflush(stdout);
      *pt = (XtlPtr) 0;
      return;
   }

   for( ;len>0 && path[len-1] == ' '; len--) ;
   strncpy(lpath,path,len);
   lpath[len] ='\0';                /* terminate the fortran string */ 

             /* return pointer as a long word */
   if ( ! ( tkwin = Tk_NameToWindow(interp,lpath,
                    Tk_MainWindow(interp)) )  ) {
      printf( "\n--- Tk interface:  xstart_ found no record of %s \n",lpath);
      fflush(stdout);
      *pt = (XtlPtr) 0;
      return;
   }

           /* scan Xtal window list for a match  */
   
   while(canvasPtr->tkwin !=tkwin && canvasPtr->next!=NULL)  {
          canvasPtr=canvasPtr->next ;
   }

   if (canvasPtr->tkwin != tkwin) {
      printf( "\n--- Tk interface:  xstart_ found no record of %s \n",lpath);
      fflush(stdout);
      *pt = (XtlPtr) 0;
      return;
   }

   Tk_MakeWindowExist(tkwin);   /* existance of this window is essential */

   if (*dbuf) {                   /* double buffering requested */
      if ((canvasPtr->pixmap=Tk_GetPixmap(Tk_Display(tkwin),Tk_WindowId(tkwin),
                 Tk_Width(tkwin),Tk_Height(tkwin), Tk_Depth(tkwin)) ) != (Pixmap)0 ){
        canvasPtr->doubleBuffered = 1; 
      }
      else { 
         canvasPtr->doubleBuffered = 0;
         *dbuf = 0;
         printf(
  "\n--- Tk interface:  xstart_ couldn't create pixmap buffer for window %s\n",
                 lpath);
         fflush(stdout);
      }
   }
     

   *pt = (XtlPtr) canvasPtr ;     /* return with pointer to record */

}
void __stdcall XSTART(int *dbuf, XtlPtr *pt, char *path, int len) 
{ xstart_(dbuf,pt,path,len) ;}
#ifndef VMS
void xstart(int *dbuf, XtlPtr *pt, char *path, int len) 
{ xstart_(dbuf,pt,path,len) ;}
#endif


/*
 * xadarc_ adds an auxilliary fortran subroutine to the list of commands 
 *       which should be executed during Expose events
 */
void xadarc_(XtlPtr *pt, FortAuxRefSub sub)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   if (*pt) {
      canvasPtr->auxrefcom = sub ;
   }
}
void __stdcall XADARC(XtlPtr *pt, FortAuxRefSub sub)
{ xadarc_(pt, sub);}
#ifndef VMS
void xadarc(XtlPtr *pt, FortAuxRefSub sub)
{ xadarc_(pt, sub);}
#endif



void
xadarz_(XtlPtr *pt, FortAuxWinConfig sub)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   if (*pt) {
      canvasPtr->auxwinconf = sub ;
   }
}
void __stdcall XADARZ(XtlPtr *pt, FortAuxWinConfig sub)
{ xadarz_(pt, sub);}
#ifndef VMS
void xadarz(XtlPtr *pt, FortAuxWinConfig sub)
{ xadarz_(pt, sub);}
#endif


void
xadasc_(XtlPtr *pt, FortAuxWinScroll sub,int *type)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   if (*pt) {
      if     (*type == 1) canvasPtr->auxwinxscroll = sub ;
      else if(*type ==-1) canvasPtr->auxwinxscroll = NULL ;
      else if(*type == 2) canvasPtr->auxwinyscroll = sub ;
      else if(*type ==-2) canvasPtr->auxwinyscroll = NULL ;
   }
}

void __stdcall XADASC(XtlPtr *pt, FortAuxWinScroll sub,int *type)
{ xadasc_(pt, sub,type);}
#ifndef VMS
void xadasc(XtlPtr *pt, FortAuxWinScroll sub,int *type)
{ xadasc_(pt, sub,type);}
#endif



void
xscrol_(XtlPtr *pt, float *xmag,float *ymag,float *xcent,float *ycent)
{
   char command[200];
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   XtalCanvas *next;
   float frac;
   int twidth,theight;
   int error;

   for(next=xtalWindowList; next && next!=canvasPtr; next=next->next) ;

   if (next) {
      /* disable old pointers to fortran global commons 
         this is important because the call to 'configure'
         resets the shadow variables ! */
      canvasPtr->sxmag  = NULL ;  
      canvasPtr->symag  = NULL ; 
      canvasPtr->sxcent = NULL ; 
      canvasPtr->sycent = NULL ;
      
      twidth =(int)(*xmag * (float) canvasPtr->width);
      theight=(int)(*ymag * (float) canvasPtr->height);
      sprintf(command,"%s configure -scrollregion [list 0 0 %d %d ]\n",
                   Tk_PathName(canvasPtr->tkwin),twidth,theight);
      error= Tcl_Eval(interp, command);  /* set the script name */
      XTALTCLERROR

      frac = *xcent - 0.5f *(float)canvasPtr->width /(float)twidth ;
      frac = (frac <0.f ) ? 0.f : frac;
      frac = (frac >1.f ) ? 1.f : frac;
      sprintf(command,"%s xview moveto %f\n",
                   Tk_PathName(canvasPtr->tkwin),frac );
      error= Tcl_Eval(interp, command);  /* set the script name */
      XTALTCLERROR;

      frac = *ycent - 0.5f *(float)canvasPtr->height /(float)theight ;
      frac = (frac <0.f ) ? 0.f : frac;
      frac = (frac >1.f ) ? 1.f : frac;
      sprintf(command,"%s yview moveto  %f \n",
                   Tk_PathName(canvasPtr->tkwin),frac );
      error= Tcl_Eval(interp, command);  /* set the script name */
      XTALTCLERROR;

      canvasPtr->sxmag  = xmag ;  /* set pointer to shadow variable X magnif */
      canvasPtr->symag  = ymag ;  /* set pointer to shadow variable Y magnif */
      canvasPtr->sxcent = xcent ; /* set pointer to shadow variable X Centre */
      canvasPtr->sycent = ycent ; /* set pointer to shadow variable Y Centre */
   }

}

void __stdcall XSCROL(XtlPtr *pt, float *xmag,float *ymag,float *xcent,float *ycent)
{ xscrol_(pt,xmag,ymag,xcent,ycent);}
#ifndef VMS
void xscrol(XtlPtr *pt, float *xmag,float *ymag,float *xcent,float *ycent)
{ xscrol_(pt,xmag,ymag,xcent,ycent);}
#endif





/*
 * xaspct_  adds shadow variables to the xtal canvas record
 *          The variables pointed to will be updated whenever the 
 *          canvas size changes.
 *
 */
void
xsaspc_(XtlPtr *pt, float *aspect)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   XtalCanvas *next;

   for(next=xtalWindowList; next && next!=canvasPtr; next=next->next) ;
   if (next) {
      next->aspectRatio = *aspect;/* log requested aspect ratio */
   }
   else {
      *aspect=0.;               /* no match on window */
       printf(
  "\n--- Tk interface:  xsaspt_ couldn't couldn't locate window %x\n",*pt);
       fflush(stdout);
   }
}
void __stdcall XSASPC(XtlPtr *pt, float *aspect)
{    xsaspc_(pt,aspect);}
#ifndef VMS
void xsaspc(XtlPtr *pt, float *aspect)
{    xsaspc_(pt,aspect);}
#endif


void
xview_(XtlPtr *pt)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   XtalCanvas *next;
   char command[200];
   int error;

   for(next=xtalWindowList; next && next!=canvasPtr; next=next->next) ;
   if (next) {
     Tk_MapWindow(canvasPtr->tkwin); 
     sprintf(command,"tkwait visibility %s\n", Tk_PathName(canvasPtr->tkwin));
     error= Tcl_Eval(interp, command);  /* set the script name */
     XTALTCLERROR
   }
   else {
       printf(
  "\n--- Tk interface:  xview_ couldn't couldn't locate window %x\n",*pt);
       fflush(stdout);

   }
}
void __stdcall XVIEW(XtlPtr *pt)
{    xview_(pt);}
#ifndef VMS
void xview(XtlPtr *pt)
{    xview_(pt);}
#endif


/*
 * xlaspc_ link the aspect variable for shadowing
 */
void
xlaspc_(XtlPtr *pt, float *aspect)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   XtalCanvas *next;
   int twidth,theight;

   for(next=xtalWindowList; next && next!=canvasPtr; next=next->next) ;
   if (next) {
      next->saspect    = aspect;/* save pointer to shadow variable */
      twidth  = canvasPtr->scrollX2 - canvasPtr->scrollX1 ;
      theight = canvasPtr->scrollY2 - canvasPtr->scrollY1 ;
      if (theight) *(next->saspect) = (float)twidth /(float)theight ;
      else *(next->saspect) =1.0;
   }
   else {
      *aspect=0.;               /* no match on window */
       printf(
  "\n--- Tk interface:  xlaspt_ couldn't couldn't locate window %x\n",*pt);
       fflush(stdout);
   }
}
void __stdcall XLASPC(XtlPtr *pt, float *aspect)
{    xlaspc_(pt,aspect);}
#ifndef VMS
void xlaspc(XtlPtr *pt, float *aspect)
{    xlaspc_(pt,aspect);}
#endif


void
xprobe_(XtlPtr *pt)
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   XtalCanvas *next;

   for(next=xtalWindowList; next && next!=canvasPtr; next=next->next) ;
   if (next) {
       printf("   Attributes for xtal canvas labelled: %s\n",
                        Tk_PathName(canvasPtr->tkwin));
       printf("   real width  %d\n",canvasPtr->width);
       printf("   real height %d\n",canvasPtr->height);
       printf("   virt width  %d\n",canvasPtr->scrollX2-canvasPtr->scrollX1);
       printf("   virt height %d\n",canvasPtr->scrollY2-canvasPtr->scrollY1);
       printf("   xCentre     %f\n",canvasPtr->xCentre);
       printf("   yCentre     %f\n",canvasPtr->yCentre);
       printf("   xOffset     %d\n",canvasPtr->xOffset);
       printf("   yOffset     %d\n",canvasPtr->xOffset);
       printf("   xresizeFact %f\n",canvasPtr->resizeFactor.w);
       printf("   yresizeFact %f\n",canvasPtr->resizeFactor.h);
       printf("reqaspectRatio %f\n",canvasPtr->aspectRatio);
       printf("actaspectRatio %f\n",
              (float)canvasPtr->height / (float)canvasPtr->width);
       fflush(stdout);
   }
}
void __stdcall XPROBE(XtlPtr *pt)
{    xprobe_(pt);}
#ifndef VMS
void xprobe(XtlPtr *pt)
{    xprobe_(pt);}
#endif



/*
 * xbndme_ adds event bindings to the XtalCanvas *record
 *         should be executed during Expose events
 */
void
xbndme_(XtlPtr *pt, FortMouseEventCB sub, char *string, int len)
{
   char command[256];
   Tcl_HashEntry  *entryPtr;
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   char          *p;
   int count ;
   unsigned long  eventMask;
   int newEntry;

   if (!*pt)  return ;
   if (!interp) return ;

   strncpy(command,string,len);
   command[len] ='\0';                /* terminate the fortran string */ 
   count =0;
   p=command ;
   while (isspace(UCHAR(*p))) { p++; }  /* strip leading blanks */
   if (*p != '\0') 
       count = ParseEventDescription(interp, &p, &eventMask);
   if (count == 0) return ;

                                     /* check if already assigned */
   entryPtr = Tcl_FindHashEntry(&(canvasPtr->eventtable),(char *) eventMask);


   if (entryPtr == NULL)            /* doesn't exist yet  so create key */
     entryPtr =  Tcl_CreateHashEntry(&(canvasPtr->eventtable),
                                  (char *)eventMask,&newEntry);
   Tcl_SetHashValue(entryPtr, sub); /* create pointer to the data */

/*
 * printf("command: %s eventMask %x\n",command,eventMask);fflush(stdout);
 */
}
void __stdcall XBNDME(XtlPtr *pt, FortMouseEventCB sub, char *string, int len)
{    xbndme_(pt,sub,string,len);}
#ifndef VMS
void xbndme(XtlPtr *pt, FortMouseEventCB sub, char *string, int len)
{    xbndme_(pt,sub,string,len);}
#endif



/*
 *##########################################################################
 *     THE FOLLOWING IS A COMPLETE HACK ON tkBind.c FROM THE Tk8.1 PACKAGE
 *##########################################################################
 *
 *---------------------------------------------------------------------------
 *
 * ParseEventDescription --
 *
 *	Fill Pattern buffer with information about event from
 *	event string.
 *
 * Results:
 *	Leaves error message in interp and returns 0 if there was an
 *	error due to a badly formed event string.  Returns 1 if proper
 *	event was specified, 2 if Double modifier was used in event
 *	string, or 3 if Triple was used.
 *
 * Side effects:
 *	On exit, eventStringPtr points to rest of event string (after the
 *	closing '>', so that this procedure can be called repeatedly to
 *	parse all the events in the entire sequence.
 *
 *---------------------------------------------------------------------------
 */

int
ParseEventDescription(interp, eventStringPtr, eventMask)
    Tcl_Interp *interp;		/* For error messages. */
    char **eventStringPtr;	/* On input, holds a pointer to start of
				 * event string.  On exit, gets pointer to
				 * rest of string after parsed event. */
    unsigned long *eventMask;   /* Filled with event mask of matched event. */
{

    char *p;
    int eventFlags;
#define FIELD_SIZE 48
    char field[FIELD_SIZE];
    Tcl_HashEntry *hPtr;
    int event=0;
    int count=1;
    int modMask=0;
    int eventType;

    p = *eventStringPtr;

    count = 1;
    
    /*
     * Handle simple ASCII characters.
     */

    if (*p != '<') {
	char string[2];

	string[0] = *p;
	string[1] = 0;
	event   = XStringToKeysym(string);
        modMask = 0;
        eventType = KeyPress;

	if (event == NoSymbol) {
	    if (isprint(UCHAR(*p))) {
		event = *p;
	    } else {
		sprintf(interp->result,
			"bad ASCII character 0x%x", (unsigned char) *p);
		return 0;
	    }
	}
	p++;
	goto end;
    }

    /*
     * A fancier event description.  This can be either a virtual event
     * or a physical event.
     *
     * A virtual event description consists of:
     *
     * 1. double open angle brackets.
     * 2. virtual event name.
     * 3. double close angle brackets.
     *
     * A physical event description consists of:
     *
     * 1. open angle bracket.
     * 2. any number of modifiers, each followed by spaces
     *    or dashes.
     * 3. an optional event name.
     * 4. an option button or keysym name.  Either this or
     *    item 3 *must* be present;  if both are present
     *    then they are separated by spaces or dashes.
     * 5. a close angle bracket.
     */

    p++;
    if (*p == '<') {
	/*
	 * This is a virtual event: soak up all the characters up to
	 * the next '>'.
	 */
        interp->result = "virtual events \"<<>>\" aren\'t supported ";
	return 0;

    }

    /* look for modifiers */
    while (1) {
	ModInfo *modPtr;
	p = GetField(p, field, FIELD_SIZE);
	if (*p == '>') {
	    /*
	     * This solves the problem of, e.g., <Control-M> being
	     * misinterpreted as Control + Meta + missing keysym
	     * instead of Control + KeyPress + M.
	     */
	     break;
	}
	hPtr = Tcl_FindHashEntry(&modTable, field);
	if (hPtr == NULL) {
	    break;
	}
	modPtr = (ModInfo *) Tcl_GetHashValue(hPtr);
	modMask |= modPtr->mask;
	if (modPtr->flags & (DOUBLE|TRIPLE)) {
	    if (modPtr->flags & DOUBLE) {
		count = 2;
	    } else {
		count = 3;
	    }
	}
	while ((*p == '-') || isspace(UCHAR(*p))) {
	    p++;
	}
    }

     /* look for event types */
    eventFlags = 0;
    hPtr = Tcl_FindHashEntry(&eventTable, field);
    if (hPtr != NULL) {
	EventInfo *eiPtr;
	eiPtr = (EventInfo *) Tcl_GetHashValue(hPtr);

	eventType  = eiPtr->type;
	eventFlags = flagArray[eiPtr->type];
	while ((*p == '-') || isspace(UCHAR(*p))) {
	    p++;
	}
	p = GetField(p, field, FIELD_SIZE);
    }

     /* look for event details*/
    if (*field != '\0') {
	if ((*field >= '1') && (*field <= '5') && (field[1] == '\0')) {
	    if (eventFlags == 0) {
		eventType = ButtonPress;
	    } else if (eventFlags & KEY) {
		goto getKeysym;
	    } else if ((eventFlags & BUTTON) == 0) {
		Tcl_AppendResult(interp, "specified button \"", field,
			"\" for non-button event", (char *) NULL);
		return 0;
	    }
            event = *field - '0'  ;

	} else {
	    getKeysym:
	    event     = XStringToKeysym(field);

	    if (event == NoSymbol) {
		Tcl_AppendResult(interp, "bad event type or keysym \"",
			field, "\"", (char *) NULL);
		return 0;
	    }
	    if (eventFlags == 0) {
		eventType = KeyPress;
	    } else if ((eventFlags & KEY) == 0) {
		Tcl_AppendResult(interp, "specified keysym \"", field,
			"\" for non-key event", (char *) NULL);
		return 0;
	    }
	}
    } else if (eventFlags == 0) {
	interp->result = "no event type or button # or keysym";
	return 0;
    }

    while ((*p == '-') || isspace(UCHAR(*p))) {
	p++;
    }
    if (*p != '>') {
	while (*p != '\0') {
	    p++;
	    if (*p == '>') {
		interp->result = "extra characters after detail in binding";
		return 0;
	    }
	}
	interp->result = "missing \">\" in binding";
	return 0;
    }
    p++;

end:

/*
 *printf("modmask %8.8x %8.8x %8.8x\n",modMask,eventType,event);fflush(stdout);
 *
 */

    *eventStringPtr = p;
    *eventMask      = (event <<18) | (eventType <<12) | modMask ;
    return count;
}


/*
 *----------------------------------------------------------------------
 *
 * GetField --
 *
 *	Used to parse pattern descriptions.  Copies up to
 *	size characters from p to copy, stopping at end of
 *	string, space, "-", ">", or whenever size is
 *	exceeded.
 *
 * Results:
 *	The return value is a pointer to the character just
 *	after the last one copied (usually "-" or space or
 *	">", but could be anything if size was exceeded).
 *	Also places NULL-terminated string (up to size
 *	character, including NULL), at copy.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

char *
GetField(p, copy, size)
    char *p;		/* Pointer to part of pattern. */
    char *copy;	/* Place to copy field. */
    int size;			/* Maximum number of characters to
				 * copy. */
{
    while ((*p != '\0') && !isspace(UCHAR(*p)) && (*p != '>')
	    && (*p != '-') && (size > 1)) {
	*copy = *p;
	p++;
	copy++;
	size--;
    }
    *copy = '\0';
    return p;
}


/*
 *##########################################################################
 *     THE FOLLOWING PROVIDE SIMPLE CONVENIENCE ROUTINES FOR PREEXISTNG
 *##########################################################################
 */
/*
 * Convenience button press release callback functions 
 */

void  xb1pcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<ButtonPress-1>",15);
     else      xbndme_(pt, newCallback,          "<ButtonPress-1>",15);
}
void  xb2pcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback) 
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<ButtonPress-2>",15);
     else      xbndme_(pt, newCallback,          "<ButtonPress-2>",15);
}
void  xb3pcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<ButtonPress-3>",15);
     else      xbndme_(pt, newCallback,          "<ButtonPress-3>",15);
}

void __stdcall XB1PCB(XtlPtr *pt,int *n,FortMouseEventCB cb){xb1pcb_(pt,n,cb);} 
void __stdcall XB2PCB(XtlPtr *pt,int *n,FortMouseEventCB cb){xb2pcb_(pt,n,cb);} 
void __stdcall XB3PCB(XtlPtr *pt,int *n,FortMouseEventCB cb){xb3pcb_(pt,n,cb);} 
#ifndef VMS
void xb1pcb(XtlPtr *pt, int *n, FortMouseEventCB cb) { xb1pcb_(pt,n,cb); } 
void xb2pcb(XtlPtr *pt, int *n, FortMouseEventCB cb) { xb2pcb_(pt,n,cb); } 
void xb3pcb(XtlPtr *pt, int *n, FortMouseEventCB cb) { xb3pcb_(pt,n,cb); } 
#endif


void  xb1mcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<B1-Motion>",11);
     else      xbndme_(pt, newCallback,          "<B1-Motion>",11);
}
void  xb2mcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<B2-Motion>",11);
     else      xbndme_(pt, newCallback,          "<B2-Motion>",11);
}
void  xb3mcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<B3-Motion>",11);
     else      xbndme_(pt, newCallback,          "<B3-Motion>",11);
}
void __stdcall XB1MCB(XtlPtr *pt,int *n,FortMouseEventCB cb) {xb1mcb_(pt,n,cb);}
void __stdcall XB2MCB(XtlPtr *pt,int *n,FortMouseEventCB cb) {xb2mcb_(pt,n,cb);}
void __stdcall XB3MCB(XtlPtr *pt,int *n,FortMouseEventCB cb) {xb3mcb_(pt,n,cb);}
#ifndef VMS
void xb1mcb(XtlPtr *pt, int *n, FortMouseEventCB cb ) { xb1mcb_(pt,n,cb); } 
void xb2mcb(XtlPtr *pt, int *n, FortMouseEventCB cb ) { xb2mcb_(pt,n,cb); } 
void xb3mcb(XtlPtr *pt, int *n, FortMouseEventCB cb ) { xb3mcb_(pt,n,cb); } 
#endif


void  xb1rcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<ButtonRelease-1>",17);
     else      xbndme_(pt, newCallback,          "<ButtonRelease-1>",17);
}
void  xb2rcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<ButtonRelease-2>",17);
     else      xbndme_(pt, newCallback,          "<ButtonRelease-2>",17);
}
void  xb3rcb_(XtlPtr *pt, int *n, FortMouseEventCB newCallback)
{
     if (!*n)  xbndme_(pt,(FortMouseEventCB)NULL,"<ButtonRelease-3>",17);
     else      xbndme_(pt, newCallback,          "<ButtonRelease-3>",17);
}
void __stdcall XB1RCB(XtlPtr *pt,int *n,FortMouseEventCB cb) {xb1rcb_(pt,n,cb);}
void __stdcall XB2RCB(XtlPtr *pt,int *n,FortMouseEventCB cb) {xb2rcb_(pt,n,cb);}
void __stdcall XB3RCB(XtlPtr *pt,int *n,FortMouseEventCB cb) {xb3rcb_(pt,n,cb);}
#ifndef VMS
void xb1rcb(XtlPtr *pt, int *n, FortMouseEventCB cb ) { xb1rcb_(pt,n,cb); } 
void xb2rcb(XtlPtr *pt, int *n, FortMouseEventCB cb ) { xb2rcb_(pt,n,cb); } 
void xb3rcb(XtlPtr *pt, int *n, FortMouseEventCB cb ) { xb3rcb_(pt,n,cb); } 
#endif




/*
 *##########################################################################
 *                 MORE FORTRAN EVENT DRIVEN API FUNCTIONS
 *##########################################################################
 */

/*
 *  close_:   Destroy a specific XtalCanvas item from a Tk Window
 *            Also destroy all its segment records and buffers.
 *
 */

void xclose_(XtlPtr *pt) 
{
   XtalCanvas *canvasPtr= (XtalCanvas *) *pt;
   Tk_DestroyWindow(canvasPtr->tkwin);      /* mark for killing by tcl/tk */
   *pt=0 ;
}
void __stdcall XCLOSE(XtlPtr *pt) 
{    xclose_(pt);} 
#ifndef VMS
void xclose(XtlPtr *pt) 
{    xclose_(pt);} 
#endif

/*
 *  xfinis_:   Destroy all memory of the current Tcl/Tk Interface
 *            - forever.
 *
 */

void xfinis_() 
{
   XtlPtr  tmp ;
   XtalCanvas *next, *this = xtalWindowList; 
   register int i,j;
   GC  ptr;   /* pointer to graphics context */

   while(this) {
       next=this->next ;
       tmp = (XtlPtr) this ;         /* return with pointer to record */
       xclose_( &tmp) ;
       this = next;
   }

   /* free the graphic context list */
   for (i=0; i<NUM_COLORS; i++){
      ptr=gcCol[i] ;
      j=i+1;
      while (ptr && j<NUM_COLORS) { /* loop for common GC pointers */
         if (gcCol[j]==ptr) gcCol[j]= (GC) NULL;/* set pointer to NULL*/
         j++;
      }
      
      if(ptr) {
        /*  have to call Tk_FreeColor also !!! , 
            but its probably freed when all the windows shutdown
         */
        XFreeGC(display, ptr); /* free graphics context memory*/
      }
      gcCol[i] = (GC) NULL;
   }
   for (i=0; i<NUM_COLORS; i++){
      if (cstring[i]) free(cstring[i]) ;
      cstring[i] = NULL ;
   }
   maxColUsed = -1 ;


   if (interp) { 
     Tcl_DeleteHashTable(&modTable); 
     Tcl_DeleteHashTable(&eventTable); 
     Tcl_DeleteInterp(interp); 
     interp = NULL; 
   } 

}
void __stdcall XFINIS() 
{    xfinis_();}
#ifndef VMS
void xfinis() 
{    xfinis_();}
#endif


/*
 * xsetpg_:  Set the canvas to which all subsequent Tk renderings should be 
 *           sent.  It could equally well be the backbuffer, if type ==1
 * 
 */
void xsetpg_(XtlPtr *pt, int *type) 
{
   XtalCanvas *page = (XtalCanvas *) *pt;
   XtalCanvas *next ;

   if (!interp) {
       printf("\n--- Tk interface:  xsetpg_ called before xcont_\n");
       fflush(stdout);
       return;
   } /* if */


   /* search through possible canvas list */
   for(next=xtalWindowList; next && next!=page; next=next->next) ;
   
   if (next) {     /* got a page match or NULL! */
      currentGraphic = page;
      if  (*type==0) currentGraphic->canvas = Tk_WindowId(page->tkwin) ;
      else           currentGraphic->canvas = page->pixmap ;
   } 
   else {
       printf(
          "\n--- Tk interface:  xsetpg_: xtal canvas %x couldn't be located \n",
           page);
       fflush(stdout);
       return;
   }
}
void __stdcall XSETPG(XtlPtr *pt, int *type) 
{    xsetpg_(pt,type) ;}
#ifndef VMS
void xsetpg(XtlPtr *pt, int *type) 
{    xsetpg_(pt,type) ;}
#endif


/*
 * xclear_:  Set the canvas to which all subsequent Tk renderings should be 
 *           sent.  It could equally well be the backbuffer, if type ==1
 * 
 */
void xclear_(XtlPtr *pt, int *CONST flag) 
{
   XtalCanvas *canvasPtr = (XtalCanvas *) *pt;
   XtalCanvas *next ;
   
          /* search through possible canvas list */
   for(next=xtalWindowList; next && next!=canvasPtr; next=next->next) ;

   if (! next) {     /* got a page match or NULL! */
       printf(
          "\n--- Tk interface:  xclear_: xtal canvas %x couldn't be located \n",
           canvasPtr);
       fflush(stdout);
       return;
   }

   if ( *flag == 1 && canvasPtr->segmentList!=NULL)  {
      xFreeSegments(&(canvasPtr->segmentList));
      canvasPtr->openSegment=canvasPtr->lastSegment=canvasPtr->segmentList=NULL;
   }

   if (canvasPtr->doubleBuffered && canvasPtr->pixmap ) {
        XFillRectangle(display,canvasPtr->pixmap,
           canvasPtr->pixmapGC,0,0,canvasPtr->width,canvasPtr->height);

        canvasPtr->canvas = canvasPtr->pixmap ;

        if (canvasPtr->segmentList !=NULL)
             xExecuteCommands(canvasPtr,&(canvasPtr->segmentList));
         
        XCopyArea(Tk_Display(canvasPtr->tkwin), canvasPtr->pixmap, 
                  Tk_WindowId(canvasPtr->tkwin), canvasPtr->pixmapGC,0,0,
                   (unsigned)canvasPtr->width,
                   (unsigned)canvasPtr->height,0,0);
   }

   else {
        XClearWindow(display,canvasPtr->canvas);
        canvasPtr->canvas = Tk_WindowId(canvasPtr->tkwin) ;
        if (canvasPtr->segmentList !=NULL)
             xExecuteCommands(canvasPtr,&(canvasPtr->segmentList));
   }

   if ( *flag != 1 && canvasPtr->auxrefcom) /* nothing to refresh!*/ 
              (canvasPtr->auxrefcom)() ;

}
void __stdcall XCLEAR(XtlPtr *pt, int *CONST flag) 
{    xclear_(pt, flag) ;}
#ifndef VMS
void xclear(XtlPtr *pt, int *CONST flag) 
{    xclear_(pt, flag) ;}
#endif



/*
 * xgetxy:  Configure the event BindingProc to return
 *          with coordinates after a mouse click.
 * 
 */

void xgetxy_(XtlPtr *pt, float *x, float *y)
{
   XtalCanvas *page= (XtalCanvas *) *pt;
   XtalCanvas   *canvasPtr=xtalWindowList;
   xtalXCoord = xtalYCoord = 0;
   xtalEventMode=XTAL_GETXY;       

   while(canvasPtr)  {
      if (canvasPtr !=page) Tk_DefineCursor(canvasPtr->tkwin,arrowCursor);
      else                  Tk_DefineCursor(canvasPtr->tkwin,crossCursor);
      canvasPtr=canvasPtr->next ;
   }
   while(xtalEventMode && Tk_GetNumMainWindows() > 0) { 
      Tcl_DoOneEvent(0); 
   }
   if (Tk_GetNumMainWindows()<=0) {
      xtalWidget = NULL ;
      *x = 0. ;
      *y = 0. ;
   }
   else {
      canvasPtr=xtalWindowList;
      while(canvasPtr)  {
          Tk_DefineCursor(canvasPtr->tkwin,watchCursor);
          canvasPtr=canvasPtr->next ;
      }
   *x = ((float)xtalXCoord - page->xOffset)/page->resizeFactor.w;
   *y = (page->height - (float)xtalYCoord - page->yOffset)/page->resizeFactor.h;
   }

   page = (XtalCanvas *) xtalWidget ;
   *pt =(XtlPtr ) page;           /* return with pointer to record */
}
void __stdcall XGETXY(XtlPtr *pt, float *x, float *y)
{    xgetxy_(pt,x,y); }
#ifndef VMS
void xgetxy(XtlPtr *pt, float *x, float *y)
{    xgetxy_(pt,x,y); }
#endif




/*
 * xgetsg:  Configure the event BindingProc to return
 *          with an xtal segment on a mouse release .
 *          but in the mean time handle any extra 
 *          callbacks thrown at it.
 *          This is the primary event loop.
 *          It can also return with activated widget ID numbers
 *          as specified with  (X)xtalBinding ...
 */

void xgetsg_(XtlPtr *pt,int *level,int *seg)
{
   XtalCanvas *page= (XtalCanvas *) *pt;
   XtalCanvas   *canvasPtr=xtalWindowList;

   if (Tk_GetNumMainWindows() <= 0 || !interp) {
         printf("\n--- TkLib:  xgetsg_ called with no windows!\n"); 
         fflush(stdout);
         *seg   =-1;           /* universal no go flag */
         *pt   = (XtlPtr) NULL ;     /* return with pointer to record */
         *level =-1;           /* universal no go flag */
         return ; 
   }
   if (! *pt ) {
         printf("\n--- TkLib:  Passed in a null canvas \n"); 
         fflush(stdout);
         *seg   =-1;           /* universal no go flag */
         *pt   = (XtlPtr) NULL ;     /* return with pointer to record */
         *level =-1;           /* universal no go flag */
         return ; 
   }

   while(canvasPtr)  {
      Tk_DefineCursor(canvasPtr->tkwin,arrowCursor);
      canvasPtr=canvasPtr->next ;
   }
      
   xtalSegment=0;
   xtalEventMode=XTAL_GETSEG; 
   xtalWidget=NULL ;
   xtalWidgetLevel=0;    

   while(xtalEventMode && Tk_GetNumMainWindows() > 0) { 
      Tcl_DoOneEvent(0); 
   }

   if (Tk_GetNumMainWindows() > 0 && watchCursor && 
          page && page->tkwin) {
      canvasPtr=xtalWindowList;
      while(canvasPtr)  {
          Tk_DefineCursor(canvasPtr->tkwin,watchCursor);
          canvasPtr=canvasPtr->next ;
      }
   }

/* note that if all windows have been destroyed then xtalSegment is -1 */
   *pt    =(XtlPtr ) xtalWidget; /* return with pointer to widget item */
   *level = xtalWidgetLevel;     /* return with sublevel of item */
   *seg   = xtalSegment ;

}
void __stdcall XGETSG(XtlPtr *pt,int *level,int *seg)
{    xgetsg_(pt,level,seg); }
#ifndef VMS
void xgetsg(XtlPtr *pt,int *level,int *seg)
{    xgetsg_(pt,level,seg); }
#endif


/*
 * xframe:  Destroy all segment records, refresh the window
 *          but do not destroy it. Optionally wait for a 
 *          users mouse event before erasing
 */

void xframe_(XtlPtr *pt, int *wait)
{
   XtalCanvas *page= (XtalCanvas *) *pt;
   XtalCanvas   *canvasPtr=xtalWindowList;

   while(canvasPtr)  {
      Tk_DefineCursor(canvasPtr->tkwin,crossCursor);
      canvasPtr=canvasPtr->next ;
   }

   xtalEventMode=XTAL_FRAME; /* ensure mouse bindings are active! */
   if (*wait) while(xtalEventMode && Tk_GetNumMainWindows() > 0) { 
      Tcl_DoOneEvent(0); 
   }

   if (Tk_GetNumMainWindows() <= 0) {
     *pt = 0;
   }
   else { /* clear out the old segment list */
      XClearWindow(display, Tk_WindowId(page->tkwin));

      canvasPtr=xtalWindowList;
      while(canvasPtr)  {
          Tk_DefineCursor(canvasPtr->tkwin,watchCursor);
          canvasPtr=canvasPtr->next ;
      }
      xFreeSegments(&(page->segmentList));
      page->openSegment = page->lastSegment = page->segmentList = NULL;
   }
}
void __stdcall XFRAME(XtlPtr *pt, int *wait)
{    xframe_(pt,wait); }
#ifndef VMS
void xframe(XtlPtr *pt, int *wait)
{    xframe_(pt,wait); }
#endif



/*
 * xgetst_:  lets the user type in a string, and returns that string.
 *           does NOT null-terminate the string.
 */

void xgetst_(XtlPtr *pt, int *n, char *str,int junk)
{
   int error;
   XtalCanvas *page= (XtalCanvas *) *pt;
   XtalCanvas   *canvasPtr=xtalWindowList;

   /* let the user know they should type in text by changing the
    * cursor from a watch to the xterm I-beam */
   while(canvasPtr)  {
      if (canvasPtr !=page) Tk_DefineCursor(canvasPtr->tkwin,arrowCursor);
      else                  Tk_DefineCursor(canvasPtr->tkwin,textCursor);
      canvasPtr=canvasPtr->next ;
   }

   Tcl_Eval(interp,"set XtalOldFocusWindow [focus]");
   Tcl_VarEval(interp,"focus ",Tk_PathName(page->tkwin), NULL);
   error = Tk_Grab(interp,page->tkwin,0);
   if (error ) {
           printf("\n--- Tk interface: %s \n",interp->result); 
            fflush(stdout);  
   }
#ifdef __WIN32__
   inputContext= XCreateIC();
#endif
   xtalStringNOut=0;
   xtalEventMode=XTAL_GETSTRING; /* ensure mouse bindings are active! */
   while(xtalEventMode) { Tcl_DoOneEvent(0); }

   if (Tk_GetNumMainWindows() <= 0) {
     *pt = 0;
      *n = 0;
   }
   else {
#ifdef __WIN32__
      XDestroyIC(inputContext);
#endif
 
      Tk_Ungrab(page->tkwin);
      Tcl_Eval(interp,"focus $XtalOldFocusWindow ");
      canvasPtr=xtalWindowList;
      while(canvasPtr)  {
         Tk_DefineCursor(canvasPtr->tkwin,watchCursor);
         canvasPtr=canvasPtr->next ;
      }


        /* return the user-specified string and its size */
      if (xtalStringBuf[0] != '\0')
          strncpy(str, xtalStringBuf, xtalStringNOut);
      *n = xtalStringNOut;
  }
}

void __stdcall XGETST(XtlPtr *pt, int *n, char *str,int junk)
{ xgetst_(pt, n, str,junk);}
#ifndef VMS
void xgetst(XtlPtr *pt, int *n, char *str,int junk)
{ xgetst_(pt, n, str,junk);}
#endif




/*
 *##########################################################################
 *                 FORTRAN TIME DRIVEN API FUNCTIONS
 *##########################################################################
 */

/*
 * XtalTimerDispatch is called by the Tcl_DoOneEvent event loop 
 * when timers expire. We then delete the expired timers from our own
 * timerList and execute the timer callback (which may add another timer 
 * to the list, so clean up first before executing the callback ).
 */

void 
XtalTimerDispatch(ClientData clientData)
{
   FTimerCb        timerCallback;
   timerRec  *tpt;
   timerRec  *timer = (timerRec *)clientData;  /* pointer to a timer record */


   if (timer == timerList) 
        timerList=timerList->next; /* unlink timer */
   else {
      for(tpt=timerList; tpt->next && tpt->next!=timer; tpt=tpt->next) ;
      if (tpt->next != timer ) {
         printf("\n--- TkLib:  can't find the callback timer \n"); 
         fflush(stdout);
         return ; /* obviously not here */
      }
      tpt->next=timer->next ;              /* unlink this timer */
   }

   timerCallback = timer->timerCallback; /* save callBack  */
   free(timer);                          /* free expired timer */
   if(*(timerCallback))timerCallback();  /* execute the callback */
}

/*
 * xFreeTimers is invoked to cancell all timer callbacks
 * 
 */

void  xFreeTimers()
{
        timerRec *tpt1,*tpt2;     /* pointer to a timer record */

        tpt1=timerList;                  /* first entry in timer list */
        while((tpt2=tpt1) ){
             tpt1=tpt1->next;            /* get next timer in list */
             Tcl_DeleteTimerHandler(tpt2->token);
             free(tpt2);                 /* free current timer */
             }
        timerList=tpt1;                  /* should be NULL */
}

void xfretm_() {xFreeTimers();}
void __stdcall XFRETM()  {xFreeTimers();}
#ifndef VMS
void xfretm()  {xFreeTimers();}
#endif

/*
 * xsettm_:  Set a timer
 *
 */
void
xsettm_(int *n,int *id, FTimerCb newCallBack )
{
   timerRec       *newtimer;      /* pointer to a timer record */
   timerRec       *tpt;           /* pointer to a timer record */
   Tcl_TimerToken token;


   /* get memory for callback routine */
   if((newtimer=(timerRec *)malloc((unsigned)(sizeof(timerRec))))== NULL){
        *n = 0 ;
        return ;
   }

   tpt=timerList;                        /* first entry in timer list */
   while(tpt && tpt->next) tpt=tpt->next;/* get end of  timer list */
   if (tpt!= NULL)tpt->next = newtimer;  /* add new timer to end of list */
   else           timerList = newtimer;  /* or create new list */
   
   token = Tcl_CreateTimerHandler(*n,XtalTimerDispatch,(ClientData)newtimer);
   newtimer->id = *id;                   /* save an id number  */
   newtimer->token = token;              /* save an id number  */
   newtimer->timerCallback = newCallBack;/*store callback routine */
   newtimer->next = NULL;                /* make next entry null */

}

void __stdcall XSETTM(int *n,int *id, FTimerCb cb ) { xsettm_(n,id,cb); }
#ifndef VMS
void xsettm(int *n,int *id, FTimerCb cb ) { xsettm_(n,id,cb); }
#endif


/*
 * xdestm_:  delete one specific timer callback
 *
 */
void
xdestm_(int *id)
{
        timerRec *tpt1,*tpt2;            /* pointer to a timer record */

        if (! timerList) return;         /* no timer to delete */

        tpt1=timerList;                  /* first entry in timer list */
        if (tpt1->id == *id ) {          /* if first entry in list */
             timerList = tpt1->next;    /* set to next in list or nil if none*/
             Tcl_DeleteTimerHandler(tpt1->token);
             free(tpt1);                 /* free current timer */
             }
        else while((tpt2=tpt1) ){
             tpt1=tpt1->next;            /* get next timer in list */
             if (tpt1 && tpt1->id == *id) {
                  tpt2->next = tpt1->next; /* unlink timer tpt1 */
                  Tcl_DeleteTimerHandler(tpt1->token);
                  free(tpt1);            /* free next timer tpt1 */
                  break;                 /* deleted, modified so break */
                  }
             }
}
void __stdcall XDESTM(int *id) { xdestm_(id); }
#ifndef VMS
void xdestm(int *id) { xdestm_(id); }
#endif






/*
 *##########################################################################
 *                 FORTRAN bufferswapping API
 *##########################################################################
 */
void
xBufferSwap(XtalCanvas *canvasPtr)
{

   /* bail out if xstart_ hasn't yet been called */
   if (! interp) {
       printf("\n--- Tk interface:  xbfcpy_ called before xstart_\n");
       fflush(stdout);
       return;
   } /* if */

   if (!canvasPtr) {
       printf("\n--- Tk interface:  xbfcpy_ called without a valid canvas\n");
       fflush(stdout);
       return;
   }

   if (!canvasPtr->doubleBuffered){
       printf("\n--- Tk interface:  xbfcpy_ called in single buffer mode\n");
       fflush(stdout);
       return;
   } /* if */

   
   if (canvasPtr->pixmap)
       XCopyArea(display,canvasPtr->pixmap, Tk_WindowId(canvasPtr->tkwin), 
          canvasPtr->pixmapGC,0,0,Tk_Width(canvasPtr->tkwin),
         Tk_Height(canvasPtr->tkwin),0,0 );
   else {
       printf("\n--- Tk interface:  xbfcpy_ missing the pixmap! \n");
       fflush(stdout);
  }
         
}

/*
 * xbfcpy_:  copy pixmap back buffer to window
 *
 */
void xbfcpy_(XtlPtr *pt) { xBufferSwap((XtalCanvas *)*pt); }
void __stdcall  XBFCPY(XtlPtr *pt) { xBufferSwap((XtalCanvas *)*pt); }
#ifndef VMS
void  xbfcpy(XtlPtr *pt) { xBufferSwap((XtalCanvas *)*pt); }
#endif




/*----------------------------------------------------------------------*
 * Functions needed to set and remember X Graphics Context values.
 *----------------------------------------------------------------------*/

void xcolmp_(int *val)
{

    if (*val) ownColMap=True;       /* use own private colormap */
    else      ownColMap=False;      /* use    a shared colormap */
}
void __stdcall XCOLMP(int *val) {xcolmp_(val); }
#ifndef VMS
void xcolmp(int *val) {xcolmp_(val); }
#endif



void xgcxor_(int *val)
{
    if (*val == 1) { 
	gcFunc = GXxor;			/* has xstart_ been called? */
        currentGraphic->canvas = Tk_WindowId(currentGraphic->tkwin ) ;
    }
    else { 
	gcFunc = GXcopy;	/* has xstart_ been called? */
	if (currentGraphic->doubleBuffered) 
             currentGraphic->canvas = currentGraphic->pixmap;
	else currentGraphic->canvas = Tk_WindowId(currentGraphic->tkwin);
    }
}

void __stdcall XGCXOR(int *val) {xgcxor_(val); }
#ifndef VMS
void xgcxor(int *val) {xgcxor_(val); }
#endif

void flushx_() { 
#ifdef __WIN32__
/*    GdiFlush();  */
#else
    XFlush(display);
#endif
}

void __stdcall FLUSHX()  { flushx_(); }
#ifndef VMS
void flushx()  { flushx_(); }
#endif


/*
 * xSetFunction: swap graphics mode if change needed
 */
static void
xSetFunction(int index)
{
   XGCValues       values;                 
		/* current graphics function  */
   static int	currentGCFunc[NUM_COLORS] = {GXcopy};

	/* change dash style, if necessary */
   if (index != currentGCFunc[colGC]) {
       if (index == GXcopy ) {
           values.fill_style    = FillSolid;
           currentGCFunc[colGC] = values.function = index ;
           XChangeGC(display, gcCol[colGC], GCFillStyle | GCFunction, &values);
       }
       else  {   
           values.fill_style    = FillOpaqueStippled;
           currentGCFunc[colGC] = values.function = index ;
           XChangeGC(display, gcCol[colGC], GCFillStyle | GCFunction  ,&values);
       }
   }
}



/*
 * xSetDashes:  given a valid dash style index, sets the current dash
 *	style, if necessary.  prints an error message if the dash index
 *	is invalid.  
 */
static void
xSetDashes(int index)
{
      /* current device state */
   static int   currentDashIndex[NUM_COLORS] = {DefaultDashIndex};

      /* dash style patterns */
   static char   dashedStyle[] = { 4, 4 };
   static char   dottedStyle[] = { 1, 4 };
   static char   dashDotStyle[] = { 4, 4, 1, 4 };
   static char   doubleStyle[] = { 4, 2, 1, 2, 1, 2, 1, 2 };

   /* change dash style, if necessary */
   if (index != currentDashIndex[colGC]) {
#ifdef __WIN32__

#else
      switch(currentDashIndex[colGC] = index) {
      case 2:
         XSetDashes(display, gcCol[colGC], 0, dashedStyle, 2);
         break;
      case 3:
         XSetDashes(display, gcCol[colGC], 0, dottedStyle, 2);
         break;
      case 4:
         XSetDashes(display, gcCol[colGC], 0, dashDotStyle, 4);
         break;
      case 5:
         XSetDashes(display, gcCol[colGC], 0, doubleStyle, 8);
         break;
      default:
         printf("\n--- Tk interface:  illegal dash style %d\n",
            index);
         fflush(stdout);
      } /* switch */
#endif

   } /* if */
} /* xSetDashes */

/*
 * xSetFillStyle:  changes the current fill style, if necessary.
 */
static void
xSetFillStyle(int fillStyle)
{
    static int	currentFillStyle[NUM_COLORS] = {DefaultFillStyle};

    if (fillStyle != currentFillStyle[colGC])
	XSetFillStyle(display, gcCol[colGC], currentFillStyle[colGC]=fillStyle);
} /* xSetFillStyle */



/*
 * xSetLineAttributes:  changes the line width and/or the line style, if
 *	necessary.  note that valid values for "line style" are LineSolid
 *	or LineOnOffDash in the X11 sense, not 1 through 5 in the PX/PV/PG
 *	sense.
 *		a calling function may change the current line width
 *	without changing the current line style by specifying a line style
 *	of IgnoreLineStyle.  similarly, the line width parameter will be
 * 	ignored if it is set to IgnoreLineWidth.
 */
static void
xSetLineAttributes(unsigned int lineWidth, int lineStyle)
{
    static unsigned int	currentLineWidth[NUM_COLORS] = {DefaultLineWidth};
    static int		currentLineStyle[NUM_COLORS] = {DefaultLineStyle};

    Bool			changed = False;

	/* if requested and if necessary, change current line width */
    if (lineWidth != IgnoreLineWidth && lineWidth != currentLineWidth[colGC]) {
		currentLineWidth[colGC] = lineWidth;
		changed = True;
		} /* if */

	/* if requested and if necessary, change current line style */
    if (lineStyle != IgnoreLineStyle && lineStyle != currentLineStyle[colGC]) {
		currentLineStyle[colGC] = lineStyle;
		changed = True;
		} /* if */

	/* if something changed, let X know */
	if (changed == True)
	    XSetLineAttributes(display, gcCol[colGC], currentLineWidth[colGC], 
	    currentLineStyle[colGC], CapButt, JoinMiter);
} /* xSetLineAttributes */

/*
 * xSetStipple:  given a valid pattern index, sets the current stipple
 *	pixmap, if necessary.  prints an error message if the pattern
 *	index is invalid.  re-initializes its static variables if index 
 *	is set to PatternIndexFlag.
 */
static void
xSetStipple(int index)
{
		/* current device state */
	static int	currentPatternIndex[NUM_COLORS] = {DefaultPatternIndex};

		/* widths of fill patterns */
	static int	patternWidth[] = {
				0,
				pattern1_width,
				pattern2_width,
				pattern3_width,
				pattern4_width,
				pattern5_width,
				pattern6_width,
				pattern7_width,
				pattern8_width,
				pattern9_width
				};

		/* heights of fill patterns */
	static int	patternHeight[] = {
				0,
				pattern1_height,
				pattern2_height,
				pattern3_height,
				pattern4_height,
				pattern5_height,
				pattern6_height,
				pattern7_height,
				pattern8_height,
				pattern9_height
				};

		/* initial fill pattern descriptions */
	static char	*patterns[] = {
				"",
				pattern1_bits,
				pattern2_bits,
				pattern3_bits,
				pattern4_bits,
				pattern5_bits,
				pattern6_bits,
				pattern7_bits,
				pattern8_bits,
				pattern9_bits
				};

		/* bitmaps generated from pattern descriptions */
	static Pixmap	patternBitmaps[] = {
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0,
				(Pixmap) 0
				};

	/* re-initialize, if requested */
	if (index == PatternIndexFlag) {
		int	i;		/* loop control variable */

		for (i = 1; i <= NUM_PATTERNS; i++)
			patternBitmaps[i] = (Pixmap) 0;
		return;
		} /* if */

	/* make sure we have a valid index */
	if (index < 1 || index > NUM_PATTERNS) {
		printf(
		   "\n--- Tk interface:  illegal shade pattern index #%d\n",
		   index);
		fflush(stdout);
		return;
		} /* if */

	/* if we need to change the current stipple pixmap... */
	if (index != currentPatternIndex[colGC]) {

		/* create bitmap, if necessary */
		if (patternBitmaps[index] == (Pixmap) 0) {
			Pixmap		bitmap;

			if ((bitmap = XCreateBitmapFromData(display, 
                                Tk_WindowId(toplevel) ,
			   patterns[index], patternWidth[index],
			   patternHeight[index])) == (Pixmap) 0) {
				printf(
			   "\n--- Tk interface:  unable to create bitmap\n");
				fflush(stdout);
				return;
				} /* if */
			patternBitmaps[index] = bitmap;
			} /* if */

		/* set the stipple pattern.  note that
		 * xSetFillStyle(FillStipple) will have to be called
		 * for this change to have effect. */
		XSetStipple(display, gcCol[colGC],
		   patternBitmaps[currentPatternIndex[colGC] = index]);

		} /* if */
} /* xSetStipple */

/*
 * xSetLineStyleIndex:  given a line style index from 1 to 5, calls 
 *	xSetLineAttributes and xSetDashes appropriately.  acts an interface
 *	between the PX/PV/PG concept of a "line style" (solid, dashed,
 *	dotted, etc) and the Tk concepts of current "line style"
 *	(LineSolid or LineOnOffDash) and current dash pattern.  prints
 *	an error message if the given index is invalid.
 */
static void
xSetLineStyleIndex(int index)
{
	/* for solid lines, we need only call xSetLineAttributes */
	if (index == 1)
		xSetLineAttributes((unsigned)IgnoreLineWidth, LineSolid);

	/* for dashed lines, call both xSetLineAttributes and xSetDashes */
	else if (index >= 2 && index <= 5) {
		xSetLineAttributes((unsigned)IgnoreLineWidth, LineOnOffDash);
		xSetDashes(index);

	/* all other indices are invalid */
	} else {
		printf("\n--- Tk interface:  invalid line style index %d\n",
		   index);
		fflush(stdout);
		} /* else */
} /* xSetLineStyleIndex */



/*----------------------------------------------------------------------*
 * Functions needed to set and remember X Graphics Context values.
 *----------------------------------------------------------------------*/

/*
 * xgetmc_:  returns maximum number of available color cells.  calculates
 *      this amount by asking the server for NUM_COLORS cells.  if the
 *      request is denied, the amount is dropped by one until a request
 *      is honored.  NOTE:  the cells are then freed; use xdefc_ to allocate
 *      color cells one by one.
 */
void
xgetmc_(int *minCells,int *maxCells)
{
   int     cells=0;
   Tcl_Obj *resObj ;  

   /* bail out if xstart_ hasn't yet been called */
   if (!interp) {
       printf("\n--- Tk interface:  xgetmc_ called before xcont_\n");
       fflush(stdout);
       return;
   } /* if */

   *minCells = maxColUsed;  /* colours used so far */ 
   *maxCells = 230 ;        /* assuming ... */

   if (Tcl_EvalEx(interp,"winfo cells .",-1,TCL_EVAL_GLOBAL)==TCL_ERROR) {
       printf("\n--- Tk interface:  xgetmc_ error \n %s",interp->result);
       fflush(stdout);
       return;
   } /* if */
   if (( resObj = Tcl_GetObjResult(interp)) ==NULL) {
       printf("\n--- Tk interface:  getmc_ can't get result from obj\n");
       fflush(stdout);
       return;
   } /* if */
   
   if (Tcl_GetIntFromObj(interp,resObj,&cells)==TCL_ERROR) {
       printf("\n--- Tk interface:  xgetmc_ error \n %s",interp->result);
       fflush(stdout);
       return;
   } /* if */

   *maxCells = cells ;
}
void __stdcall XGETMC(int *minCells,int *maxCells)
{    xgetmc_(minCells, maxCells); }
#ifndef VMS
void xgetmc(int *minCells,int *maxCells)
{    xgetmc_(minCells, maxCells); }
#endif


/*
 * xstdc_:  gets pixel values for 7 standard, read-only colors, placing
 *      these values in 'colors[1]' through 'colors[7]'.  note that 
 *      'colors[0]' is always defined as BlackPixel.
 */
void
xstdc_()
{
   XGCValues       values;                 /* used in GC creation */
   XColor          *pixel;
   int             i;                      /* loop control variable */
   int             saveGC;
   static char     *name[] = {             /* names of "standard" colors */
                            "Black",
                            "Red",
                            "Green",
                            "Blue",
                            "Yellow",
                            "Cyan",
                            "Magenta",
                            "White",
                            NULL
                            };

   /* bail out if xstart_ hasn't yet been called */
   if (!interp ) {
       printf("\n--- Tk interface:  xstdc_ called before xstart_\n");
       fflush(stdout);
       return;
   } /* if */
 
   /* for each "standard" color name... */
   saveGC=colGC;
   values.function   = GXcopy;
   values.plane_mask = AllPlanes;
   values.background = BlackPixel(display, screen_num);
   values.line_width = DefaultLineWidth;
   values.line_style = DefaultLineStyle;
   values.cap_style  = CapButt;
   values.join_style = JoinMiter;
   values.fill_style = DefaultFillStyle;
   values.fill_rule  = EvenOddRule;
   values.arc_mode   = ArcPieSlice ;
   values.tile       = None;
   values.stipple    = None;
   values.ts_x_origin = 0;
   values.ts_y_origin = 0;
   values.subwindow_mode = ClipByChildren;
   values.font = None;
   values.graphics_exposures=False;
   values.clip_x_origin = 0;
   values.clip_y_origin = 0;
   values.clip_mask     = None;
   values.dash_offset   = 0;
   values.dashes        = 4;

/*   values.fill_style = FillStippled;
 * GCStipple   | GCTile       |    GCClipMask  
 */

   for (i = 0; name[i] != NULL; i++) {
      pixel = Tk_GetColor(interp,toplevel,name[i]); /* add color to colormap*/

      values.foreground = pixel->pixel;
      colGC = i ;
      if (gcCol[colGC] ) {
          free(cstring[colGC]) ; /* should free any existing entry here */
          cstring[colGC] = NULL;
      }
      gcCol[colGC]= XCreateGC(display, RootWindow(display, screen_num) ,
       GCFunction   | GCPlaneMask  |GCForeground  |GCBackground | GCLineWidth | 
       GCLineStyle  | GCCapStyle   |GCJoinStyle   |GCFillStyle  | GCFillRule  |
       GCTileStipXOrigin | GCTileStipYOrigin |
       GCSubwindowMode| GCGraphicsExposures | GCClipXOrigin | GCClipYOrigin | 
       GCDashOffset        | GCDashList    | GCArcMode   
                , &values);

      if (!gcCol[colGC]) {
           gcCol[colGC]=gcCol[0];                 /* use black */
           printf("\n--- Tk interface:  unable to create color context %d \n",colGC);
           fflush(stdout);
           continue;
      }
      cstring[colGC] = malloc(1+strlen(name[i])); /* get mem for the name */
                                                  /* duplicate the name */ 
      if (cstring[colGC]) strcpy(cstring[colGC],name[i]); 

      maxColUsed = i ;
      xSetDashes(DefaultDashIndex);
      xSetFunction(gcFunc=GXcopy);
      xSetFillStyle(DefaultFillStyle);
      xSetLineAttributes((unsigned)DefaultLineWidth,DefaultLineStyle);
/*      xSetStipple(DefaultPatternIndex); */
   } /* for */
   colGC=saveGC;/* reset default color */

}
void __stdcall XSTDC() { xstdc_(); }
#ifndef VMS
void xstdc() { xstdc_(); }
#endif


/*
 * xdefc_:  given an index and a set of RGB values, allocates a colorcell,
 *      stores the RGB values into the colorcell, and stores that colorcell's
 *      pixel value into the 'colors' array, at the specified index.  note
 *      that 'colors[0]' is always defined as BlackPixel.
 */
void
xdefc_(int *index, float *red, float *green, float *blue)
{
   XGCValues       values;                /* used in GC creation */
   XColor          *pixel;
   int             i;
   char            color[] = "#RRRRGGGGBBBB";

        /* bail out if xstart_ hasn't yet been called */
   if (!interp) {
      printf("\n--- Tk interface:  xdefc_ called before xstart_\n");
      fflush(stdout);
      return;
   } /* if */


   /* bail out if an invalid index was specified */
   if (*index < 0 || *index >= NUM_COLORS) {
        printf(
        "\n--- Tk interface:  invalid color index %d given to xdefc_\n",
         *index);
      fflush(stdout);
      return;
   } /* if */

   sprintf(color,"#%4.4X%4.4X%4.4X\0",(unsigned int)(*red  * MAX_COLOR_COMP),
                                    (unsigned int)(*green* MAX_COLOR_COMP),
                                    (unsigned int)(*blue * MAX_COLOR_COMP));
   if (cstring[*index])  {
      free(cstring[*index]);  /* should free any existing entry here */
      cstring[*index] = NULL;
   }

   /* allocate the colour in the colormap */
   if (!(pixel = Tk_GetColor(interp,toplevel,color) ) ) {
      printf("\n--- Tk interface:  %s \n",interp->result);
      fflush(stdout);
      return;
   }

   values.function   = GXcopy;
   values.plane_mask = AllPlanes;
   values.background = BlackPixel(display, screen_num);
   values.foreground = pixel->pixel;
   values.line_width = DefaultLineWidth;
   values.line_style = DefaultLineStyle;
   values.cap_style  = CapButt;
   values.join_style = JoinMiter;
   values.fill_style = DefaultFillStyle;
   values.fill_rule  = EvenOddRule;
   values.arc_mode   = ArcPieSlice ;
   values.tile       = None;
   values.stipple    = None;
   values.ts_x_origin = 0;
   values.ts_y_origin = 0;
   values.subwindow_mode = ClipByChildren;
   values.font = None;
   values.graphics_exposures=False;
   values.clip_x_origin = 0;
   values.clip_y_origin = 0;
   values.clip_mask     = None;
   values.dash_offset   = 0;
   values.dashes        = 4;
/*  GCStipple   | GCTile       |    GCClipMask     | */

   colGC=*index;
   if(colGC!=0) i=0;
   else         i=1;   /* test for pointer to identical color entries */
   /* break on a match */
   while(i<NUM_COLORS && gcCol[colGC]!=gcCol[i]) {
       if(i==colGC-1)i+=2; /* skip obvious match */
       else ++i;
   }
   maxColUsed = maxColUsed < colGC ? colGC : maxColUsed ;

        
   /* got a match or existing is null so create new */
   if (i<NUM_COLORS || ! gcCol[colGC]) {
       if (! (gcCol[colGC] = XCreateGC(display, RootWindow(display, screen_num),
       GCFunction   | GCPlaneMask  |GCForeground  |GCBackground | GCLineWidth | 
       GCLineStyle  | GCCapStyle   |GCJoinStyle   |GCFillStyle  | GCFillRule  |
       GCTileStipXOrigin | GCTileStipYOrigin |
       GCSubwindowMode| GCGraphicsExposures | GCClipXOrigin | GCClipYOrigin | 
       GCDashOffset        | GCDashList    | GCArcMode   
                , &values) ) ) {
          printf("\n--- Tk interface:  unable to create color context %d \n",
                         *index);
          fflush(stdout);
       }
   }


   /*  Modify context for our purposes ... */
   cstring[colGC] = malloc(1+strlen(color)); /* get mem for the name */
   if (cstring[colGC]) strcpy(cstring[colGC],color); /* duplicate the name */ 
   xSetDashes(DefaultDashIndex);
   xSetFunction(gcFunc=GXcopy);
   xSetFillStyle(DefaultFillStyle);
   xSetLineAttributes((unsigned)DefaultLineWidth, DefaultLineStyle);
/*   xSetStipple(DefaultPatternIndex); */
} /* xdefc_ */

void __stdcall XDEFC(int *index, float *red, float *green, float *blue)
{    xdefc_(index, red, green, blue); }
#ifndef VMS
void xdefc(int *index, float *red, float *green, float *blue)
{    xdefc_(index, red, green, blue); }
#endif



/*##########################################################################
 * Functions for storing and executing commands.
 *##########################################################################
 */

/*
 * xCopyCoords:  returns a malloced copy of a floating-point array of
 *	coordinate values.  also, if 'min' and 'max' are not NULL, they
 *	are set to the min and max of the coordinate array.  on error,
 *	a message is printed and NULL is returned.  'coords' and 'n' are
 *	assumed to be valid.
 */
 float *
xCopyCoords(float *coords, int n, float *min, float *max)
{
	float		*copy,		/* ptr to newly allocated space */
			low,		/* lowest value found so far */
			high;		/* highest value found so far */
	register int	i;		/* loop control variable */

	/* allocate space for copy */
	if ((copy = (float *)malloc((unsigned)(sizeof(float) * n))) == NULL) {
		printf(
   "\n--- Tk interface:  unable to allocate space for coordinate array\n");
		fflush(stdout);
		return(NULL);
		} /* if */

	/* copy the values over, keeping track of the low and high values */
	low = coords[0];
	high = coords[0];
	for (i = 0; i < n; i++) {
		if (low > coords[i])
			low = coords[i];
		else if (high < coords[i])
			high = coords[i];
		copy[i] = coords[i];
		} /* for */

	/* if requested, return low and high values along with array */
	if (min != NULL)
		*min = low;
	if (max != NULL)
		*max = high;
	return(copy);
} /* xCopyCoords */

/*
 * xStoreCommand:  allocates space for and returns an xCommand struct.
 *	enters the command into the currently open segment.  ASSumes that
 *	'openSegment' is non-NULL.  the calling function is responsible
 *	for filling in the 'any.type' field and the fields specific to
 *	the command's type.
 */
xCommand
xStoreCommand(xSegment seg)
{
	xCommand	cmd;		/* new command */

	/* allocate space for new xCommandRec */
	if (seg == NULL || 
	    (cmd = (xCommand)malloc((unsigned)(sizeof(xCommandRec)))) == NULL) {
		printf(
   "\n--- Tk interface:  unable to allocate space for command struct\n");
		fflush(stdout);
		return(NULL);
		} /* if */
	cmd->any.next = NULL;

	/* make this command the first in the list, if necessary */
	if (seg->cmdList == NULL) seg->cmdList = seg->lastCmd = cmd;
	else {				/* else, attach to end of list */
		seg->lastCmd->any.next = cmd;
		seg->lastCmd = cmd;
	} /* else */

	return(cmd);
} /* xStoreCommand */

/*
 * xExecuteCommands:  executes the list of retained graphics commands
 *	contained within each segment.
 */
void
xExecuteCommands(XtalCanvas *graphic,xSegment *segptr)
{
	XtalCanvas	*saveCurGraphic;
	xSegment	seg;
	xCommand	cmd;

	saveCurGraphic =currentGraphic;
	currentGraphic = graphic;
	seg = graphic->segmentList;
	/* while more segments... */
	while (seg != NULL) {

		/* while more commands to be executed... */
		cmd = seg->cmdList;
		while (cmd != NULL) {

			/* call the appropriate basic graphics routine */
			switch(cmd->any.type) {
				case xLinesCommandType:
					xLines(graphic,&(cmd->lines));
					break;
				case xFillCommandType:
					xFill(graphic,&(cmd->fill));
					break;
				case xMarkCommandType:
					xMark(graphic,&(cmd->mark));
					break;
				case xTextCommandType:
					xText(graphic,&(cmd->text));
					break;
				default:
					printf(
   "\n--- Tk interface:  undefined retained command type encountered\n");
					fflush(stdout);
				} /* switch */

			/* move to the next command in the list */
			cmd = cmd->any.next;
			} /* while */

		/* move to the next segment in the list */
		seg = seg->next;
		} /* while */
	currentGraphic = saveCurGraphic;
} /* xExecuteCommands */

/*
 * xFreeCommands:  frees the given list of retained graphics commands.
 */
void
xFreeCommands(xCommand cmds)
{
	xCommand	tmp;

	/* while more commands to be freed... */
	while (cmds != NULL) {

		/* remember the next command in the list */
		tmp = cmds->any.next;

		/* free the current command, depending upon its type */
		switch(cmds->any.type) {
			case xLinesCommandType:
				free(cmds->lines.x);
				free(cmds->lines.y);
				free(cmds);
				break;
			case xFillCommandType:
				free(cmds->fill.x);
				free(cmds->fill.y);
				free(cmds);
				break;
			case xMarkCommandType:
				free(cmds->mark.x);
				free(cmds->mark.y);
				free(cmds);
				break;
			case xTextCommandType:
				free(cmds->text.string);
				free(cmds);
				break;
			default:
				printf(
   "\n--- Tk interface:  undefined retained command type encountered\n");
				fflush(stdout);
			} /* switch */

		/* move to the next command in the list */
		cmds = tmp;
		} /* if */
} /* xFreeCommands */


/*----------------------------------------------------------------------*
 * Functions for dealing with segments.
 *----------------------------------------------------------------------*/

/*
 * initiate and initialise a new segment
 */

xSegment
xInitSegment(int *n)
{
	xSegment	seg ;
	/* allocate and fill space for new xSegmentRec */
	if ((seg = (xSegment)malloc((unsigned)(sizeof(xSegmentRec)))) == NULL) 
				return  (NULL) ;
	seg->next = NULL;
	seg->id = *n;
	seg->detect = False;
	seg->cmdList = NULL;
	seg->lastCmd = NULL;
	seg->xmin = 9999.9f;
	seg->xmax = -9999.9f;
	seg->ymin = 9999.9f;
	seg->ymax = -9999.9f;
	return (seg);
}

/*
 * xsmake_:  creates a new segment, named with the given id number.  this
 *	new segment becomes the currently open segment.  an error message
 *	is produced if the given id number is already in use.
 */
void
xsmake_(int *n)
{
	xSegment	seg;			/* new segment */

	/* make sure segment number isn't already in use */
	if (xFindSegment(&(currentGraphic->segmentList),*n) != NULL) {
		printf("\n--- Tk interface:  segment #%d already exists\n",*n);
		fflush(stdout);
		return;
		} /* if */

	/* allocate and fill space for new xSegmentRec */
	if ((seg = xInitSegment(n) ) ==NULL){
		printf(
	   "\n--- Tk interface:  unable to allocate space for new segment\n");
		fflush(stdout);
		return;
		} /* if */

	/* make this segment the first in the list, if necessary */
	if (currentGraphic->segmentList == NULL)
		currentGraphic->segmentList = currentGraphic->lastSegment = seg;

	/* else, attach it to the end of the list */
	else {
		currentGraphic->lastSegment->next = seg;
		currentGraphic->lastSegment = seg;
		} /* else */

	/* make this segment the currently open segment */
	currentGraphic->openSegment = seg;
} /* xsmake_ */
void __stdcall XSMAKE(int *n) { xsmake_(n); } /* XSMAKE */
#ifndef VMS
void xsmake(int *n) { xsmake_(n);} /* xsmake */
#endif



/*
 * xsclos_:  closes the currently open segment, if any.  does NOT produce
 *	an error message if no segment is open.
 */
void xsclos_() { currentGraphic->openSegment = NULL; } /* xsclos_ */

void __stdcall XSCLOS() { xsclos_(); } /* XSCLOS */
#ifndef VMS
void xsclos() { xsclos_(); } /* xsclos */
#endif


/*
 * xFreeSegments:  frees the current list of segments.
 */
void
xFreeSegments(xSegment *segptr)
{
	xSegment	segList,tmp;

	/* while more segments to be freed... */
	segList = *segptr  ;
	while (segList != NULL) {

		/* remember the next segment in the list */
		tmp = segList->next;

		/* free the current segment */
		xFreeCommands(segList->cmdList);
		free(segList);

		/* move to the next segment in the list */
		segList = tmp;
	} /* if */

	/* be sure to mark the end of the list as empty, and no segment
	 * as open, as well */

	*segptr = NULL ;

} /* xFreeSegments */

/*
 * xFindSegment:  given a segment id number, returns the corresponding
 *	segment or NULL.
 */
xSegment
xFindSegment(xSegment *source, int id)
{
	xSegment	tmp;
	tmp = *source;
	while (tmp && tmp->id != id)
		tmp = tmp->next;
	return(tmp);
} /* xFindSegment */

/*
 * xssetd_:  sets the detectability of the given segment.  produces an
 *	error message if the segment does not exist.
 */
void
xssetd_(int *n, int *detect)
{
	xSegment	seg = xFindSegment(&(currentGraphic->segmentList),*n);

	/* print a message if the segment couldn't be found */
	if (seg == NULL) {
		printf("\n--- Tk interface:  unable to find segment #%d\n",
		   *n);
		fflush(stdout);
		return;
		} /* if */

	/* set the detectability of the segment */
	seg->detect = (Bool)(*detect);
} /* xssetd_ */
void __stdcall XSSETD(int *n, int *detect) { xssetd_(n, detect); } /* XSSETD */
#ifndef VMS
void xssetd(int *n, int *detect) { xssetd_(n, detect); } /* xssetd */
#endif



/*
 * xsdel_:  deletes the given segment.  produces an error message if the
 *	segment does not exist.
 */
void
xsdel_(int *n)
{
	xSegment	seg,			/* segment to be deleted */
			ptr;			/* pointer into segmentList */

	/* find the given segment's position within the list */
	seg = ptr = currentGraphic->segmentList;
	while (seg && seg->id != *n) {
		ptr = seg;
		seg = seg->next;
		} /* while */

	/* print a message if the segment couldn't be found */
	if (seg == NULL) {
		printf("\n--- Tk interface:  unable to find segment #%d\n",
		   *n);
		fflush(stdout);
		return;
		} /* if */

	/* repair the hole it will make in the linked list */
	if (seg == currentGraphic->lastSegment) {
		if (seg == currentGraphic->segmentList) /* first and last seg */
			currentGraphic->lastSegment = NULL; /* reset as null */
		else
			currentGraphic->lastSegment = ptr;
		} /* if */

	if (seg == currentGraphic->segmentList)         /* first in list */
		currentGraphic->segmentList = seg->next;
	else
		ptr->next = seg->next;

	/* free the segment */
	xFreeCommands(seg->cmdList);
	free(seg);

	/* if the given segment was open, make note that no segment is
	 * now open */
	if (currentGraphic->openSegment == seg)
		currentGraphic->openSegment = NULL;
} /* xsdel_ */

void __stdcall XSDEL(int *n) { xsdel_(n); } /* XSDEL */
#ifndef VMS
void xsdel(int *n) { xsdel_(n); } /* xsdel */
#endif




/*
 *##########################################################################
 * These routines adjust the current pen col/styles etc
 *##########################################################################
 */

/*
 * xsetfc_:  sets fill color, for use in future xfill_ calls
 */
void
xsetfc_(int *fc)
{
	if (*fc < 0 || *fc >= NUM_COLORS) {
		printf(
	   "\n--- Tk interface:  invalid color index %d given to xsetfc_\n",
		   *fc);
		fflush(stdout);
		return;
		} /* if */

	currentGraphic->fillColor = *fc;
} /* xsetfc_ */

void __stdcall XSETFC(int *fc) { xsetfc_(fc); } /* XSETFC */
#ifndef VMS
void xsetfc(int *fc) { xsetfc_(fc); } /* xsetfc */
#endif


/*
 * xsetfs_:  sets fill style, for use in future xfill_ calls
 */
void
xsetfs_(int *fs)
{
    if (*fs < 1 || *fs > NUM_PATTERNS) {
	printf(
   "\n--- Tk interface:  invalid pattern index %d given to xsetfs_\n", *fs);
	fflush(stdout);
	return;
    } /* if */

    if(currentGraphic) currentGraphic->fillStyle = *fs;
    else {
	printf(
   "\n--- Tk interface:  no valid graphic in xsetfs_ %d\n", *fs);
	fflush(stdout);
	return;
   }
} /* xsetfs_ */

void __stdcall XSETFS(int *fs) { xsetfs_(fs); } /* XSETFS */
#ifndef VMS
void xsetfs(int *fs) { xsetfs_(fs); } /* xsetfs */
#endif


/*
 * xsetlc_:  sets line color, for use in future xline_ calls
 */
void
xsetlc_(int *lc)
{
	if (*lc < 0 || *lc >= NUM_COLORS) {
		printf(
	   "\n--- Tk interface:  invalid color index %d given to xsetlc_\n",
		   *lc);
		fflush(stdout);
		return;
		} /* if */

	currentGraphic->lineColor = *lc;
} /* xsetlc_ */

void __stdcall XSETLC(int *lc) { xsetlc_(lc); } /* XSETLC */

#ifndef VMS
void xsetlc(int *lc) { xsetlc_(lc); } /* xsetlc */
#endif


/*
 * xsetls_:  sets line style, for use in future xline_ calls
 */
void
xsetls_(int *ls)
{
	if (*ls < 1 || *ls > 5) {
		printf(
   "\n--- Tk interface:  invalid line style index %d given to xsetls_\n",
		   *ls);
		fflush(stdout);
		return;
		} /* if */

	currentGraphic->lineIndex = *ls;
} /* xsetls_ */

void __stdcall XSETLS(int *ls) { xsetls_(ls); } /* XSETLS */

#ifndef VMS
void xsetls(int *ls) { xsetls_(ls); } /* xsetls */
#endif


/*
 * xsetlw_:  sets line width, for use in future xline_ calls
 */
void
xsetlw_(float *lw)
{
	if (*lw < 0.0) {
		printf(
	   "\n--- Tk interface:  invalid line width %f given to xsetlw_\n",
		   *lw);
		fflush(stdout);
		return;
		} /* if */

	/* substitute a line width of 0 for specified line widths of 1, since
	 * X draws lines of "0" width faster */
	if (*lw <= 1.0)
		currentGraphic->lineWidth = 0;
	else
		currentGraphic->lineWidth = (unsigned int)*lw;
} /* xsetlw_ */

void __stdcall XSETLW(float *lw) { xsetlw_(lw); } /* XSETLW */
#ifndef VMS
void xsetlw(float *lw) { xsetlw_(lw); } /* xsetlw */
#endif



/*
 * xsetmc_:  sets marker color, for use in future xmark_ calls
 */
void
xsetmc_(int *mc)
{
	if (*mc < 0 || *mc >= NUM_COLORS) {
		printf(
	   "\n--- Tk interface:  invalid color index %d given to xsetmc_\n",
		   *mc);
		fflush(stdout);
		return;
		} /* if */

	currentGraphic->markerColor = *mc;
} /* xsetmc_ */

void __stdcall XSETMC(int *mc) { xsetmc_(mc); } /* XSETMC */
#ifndef VMS
void xsetmc(int *mc) { xsetmc_(mc); } /* xsetmc */
#endif



/*
 * xsetms_:  sets marker size, for use in future xmark_ calls
 */
void
xsetms_(float *ms)
{
	if (*ms <= 0.0) {
		printf(
	   "\n--- Tk interface:  invalid marker size %f given to xsetms_\n",
		   *ms);
		fflush(stdout);
		return;
		} /* if */

	currentGraphic->markerSize = *ms;
} /* xsetms_ */

void __stdcall XSETMS(float *ms) { xsetms_(ms); } /* XSETMS */
#ifndef VMS
void xsetms(float *ms) { xsetms_(ms); } /* xsetms */
#endif



/*
 *##########################################################################
 *     The Main Xtal Segment rendering routines
 *##########################################################################
 */

/*
 * xLines:  if linesCmd is non-NULL, takes a set of points and draws
 *	a polyline; otherwise, re-initializes static variables.
 *		an 'up' value of 2 means that the pen should be left
 *	down at the end of the draw, letting this draw connect with the
 *	next one; an 'up' value of 3 means that the pen should be
 *	raised at the end of the draw, disconnecting it from the next
 *	draw.
 */
void
xLines(XtalCanvas *graphic, xLinesCommand linesCmd)
{
	XPoint		points[CMD_BUF_SIZE+1];	/* array of coordinates */
	int		npoints;		/* number of points */
	float		*x,			/* x-coords */
			*y;			/* y-coords */
	register int	i,			/* loop control variables */
			j;

	/* re-initialize, if necessary */
	if (linesCmd == NULL) {
		graphic->penUp = True;
		return;
		} /* if */

	/* pull values out of the structure */
	npoints = linesCmd->n;
	x = linesCmd->x,
	y = linesCmd->y;

	/* if the pen is down at the beginning of this draw command, then
	 * we have to remember to include the use of the last drawn point */
	if (graphic->penUp == True)
		i = 0;
	else {
		points[0].x = graphic->pen.x;
		points[0].y = graphic->pen.y;
		i = 1;
		npoints++;
		} /* else */

	/* fill 'points' array */
	for (j = 0; i < npoints; i++, j++) {
		points[i].x = (short)(x[j] * graphic->resizeFactor.w +
					     graphic->xOffset);
		points[i].y = (short)(graphic->height -
		 (y[j]*graphic->resizeFactor.h+graphic->yOffset));
		} /* for */

	/* make sure the gc is set properly */
        colGC=linesCmd->fg;
        xSetFunction(gcFunc);
        xSetFillStyle(FillSolid);
        xSetLineAttributes((unsigned)linesCmd->lw, IgnoreLineStyle);
        xSetLineStyleIndex(linesCmd->ls);


	/* draw polyline */
        XDrawLines(display, graphic->canvas, gcCol[colGC], points, npoints, 
                            CoordModeOrigin);

	/* if necessary, remember last point drawn */
	if (linesCmd->up == 3)
		graphic->penUp = True;
	else {
		graphic->pen.x = points[--npoints].x;
		graphic->pen.y = points[npoints].y;
		graphic->penUp = False;
		} /* else */
} /* xLines */

/*
 * x initiate Fill command
 */
xCommand
xInitLinesCommand(XtalCanvas *page,xSegment seg, int *n,float x[],float y[],
									int *up)
{
	xCommand		cmd;		/* stored command */
	xLinesCommand		linesCmd;	/* cmd to be executed */
/*	xLinesCommandRec	tmp;		 temporary storage space */
	float			min,max;		/* coordinate extrema */
	
	/* if no segment is currently open, store the command in temporary
	 * space */
	if (seg == NULL)  cmd = &tmpCmd ; 

	/* otherwise, try to store this command permanently */
	else if ((cmd = xStoreCommand(seg)) != NULL)
					cmd->any.type = xLinesCommandType;
	else  cmd = &tmpCmd ; 
	linesCmd = &(cmd->lines);

	/* store basic information into the xFillCommand struct */
	linesCmd->fg = page->lineColor;
	linesCmd->ls = page->lineIndex;
	linesCmd->lw = page->lineWidth;
	linesCmd->n = *n;
	linesCmd->up = *up;

	/* if storing permanently, make copies of coordinate information,
	 * updating the openSegment's extrema information at the same time */
	if (cmd != &tmpCmd) {
	      if ((linesCmd->x = xCopyCoords(x, linesCmd->n, &min, &max))==NULL)
							linesCmd->n = 0;
		else {
			if (seg->xmin > min) seg->xmin = min;
			if (seg->xmax < max) seg->xmax = max;
		} /* else */

	       if((linesCmd->y = xCopyCoords(y, linesCmd->n, &min, &max))==NULL)
							linesCmd->n = 0;
		else {
			if (seg->ymin > min) seg->ymin = min;
			if (seg->ymax < max) seg->ymax = max;
		} /* else */
	}
	/* otherwise, just copy pointers */
	else {
		linesCmd->x = x;
		linesCmd->y = y;
	} /* else */
	return (cmd);
}

/*
 * xlines_:  if a segment is currently open, xlines_ calls xStoreCommand
 *	to save this command into that segment (for possible future use 
 *	after an expose or resize event), then calls xLines to actually
 *	do the requested draw.  in contrast, if no segment is currently 
 *	open, the command is not stored, but is simply executed directly.
 * 		commands to draw a single-point line are ignored.
 */
void
xlines_(
	int			*n,		/* number of points */
	float			x[],		/* x-coords */
	float			y[],		/* y-coords */
	int			*up)		/* 2 for pen-down, 3 for -up */
{
	XtalCanvas	*graphic;	/* drawable graphic surface */
	xCommand		cmd;		/* stored command */

	/* bail out if xstart_ hasn't yet been called */
	if (!interp) {
		printf("\n--- Tk interface:  xlines_ called before xstart_\n");
		fflush(stdout);
		return;
		} /* if */

	graphic =currentGraphic;
	if (graphic == NULL) {
     printf("\n--- Tk interface:  no drawing surface specified for xlines_\n");
		fflush(stdout);
		return;
		} /* if */
		

	/* silently ignore bogus commands */
	if (*n < 2)
		return;

	cmd = xInitLinesCommand(graphic,graphic->openSegment,n,x,y,up);
                  /* execute the command */
        if(graphic->tkwin && Tk_WindowId(graphic->tkwin)) 
                 xLines(graphic,&(cmd->lines)); 


} /* xlines_ */

void __stdcall XLINES(int *n, float x[], float y[], int *up)
{ xlines_(n, x, y, up); } /* XLINES */
#ifndef VMS
void xlines(int *n, float x[], float y[], int *up)
{ xlines_(n, x, y, up); } /* xlines */
#endif


/*
 * xFill:  given a set of points, fills a polygon.
 */
void
xFill(XtalCanvas *graphic, xFillCommand fillCmd)
{
	XPoint		points[CMD_BUF_SIZE];	/* array of coordinates */
	int		npoints;		/* number of points */
	float		*x,			/* x-coords */
			*y;			/* y-coords */
	register int	i;			/* loop control variable */

	/* pull values out of the structure */
	npoints = fillCmd->n;
	x = fillCmd->x,
	y = fillCmd->y;

	/* fill 'points' array */
	for (i = 0; i < npoints; i++) {
		points[i].x = (short)(x[i] * graphic->resizeFactor.w + 
					     graphic->xOffset);
		points[i].y = (short)(graphic->height - (y[i] * 
		       graphic->resizeFactor.h+graphic->yOffset));
	} /* for */

	/* if we're displaying in color, or drawing with the background
	 * color... */
	if (colorDisplay == True || fillCmd->fg == 0) {

		/* set fill style, color, and stipple as requested */
                colGC=fillCmd->fg; /* first set color context */
                xSetFunction(gcFunc);
                xSetFillStyle(FillStippled);
/*                xSetStipple(fillCmd->fs); */


                if(npoints<=5)
                XFillPolygon(display, graphic->canvas, gcCol[colGC], points, 
                                npoints, Convex, CoordModeOrigin);
		/* fill in a polygon */
                else
                XFillPolygon(display, graphic->canvas, gcCol[colGC], points, 
                                npoints, Nonconvex, CoordModeOrigin);

	/* else, if we're drawing white on black... */
	} else {

		/* set drawing style to that of a plain line */
                colGC=fillCmd->fg; /* first set color context */
                xSetFunction(gcFunc);
                xSetFillStyle(FillSolid);
                xSetLineAttributes((unsigned)DefaultLineWidth,
                   DefaultLineStyle);

		/* draw the outline of a polygon instead of filling it in */
                XDrawLines(display, graphic->canvas, gcCol[colGC], points, 
                                        npoints, CoordModeOrigin);

	} /* else */
} /* xFill */


void xerase_(float *x,float *y)
{
        XtalCanvas *graphic = currentGraphic; 

	int x1 = (short)(x[0] * graphic->resizeFactor.w + 
					     graphic->xOffset);
        int y1 = (short)(graphic->height - (y[0] * 
		       graphic->resizeFactor.h+graphic->yOffset));
	int x2 = (short)(x[1] * graphic->resizeFactor.w + 
					     graphic->xOffset);
        int y2 = (short)(graphic->height - (y[1] * 
		       graphic->resizeFactor.h+graphic->yOffset));

        colGC = graphic->fillColor;
        xSetFunction(gcFunc);
        xSetFillStyle(FillSolid);
        XFillRectangle(display,graphic->canvas,gcCol[colGC],
                       x1,y1,x2-x1,y2-y1);
}

void __stdcall XERASE(float *x,float *y)
{    xerase_(x,y); }
#ifndef VMS
void xerase(float *x,float *y)
{    xerase_(x,y); }
#endif


/*
 * x initiate Fill command
 */
xCommand
xInitFillCommand(XtalCanvas *page,xSegment seg, int *n,float x[],float y[])
{
	xCommand		cmd;		/* stored command */
	xFillCommand		fillCmd;	/* stored command */
	float			min,max;		/* coordinate extrema */
	
	/* if no segment is currently open, store the command in temporary
	 * space */
	if (seg == NULL)  cmd = &tmpCmd ; 

	/* otherwise, try to store this command permanently */
	else if ((cmd = xStoreCommand(seg)) != NULL)
					cmd->any.type = xFillCommandType;
	else  cmd = &tmpCmd ; 
	fillCmd = &(cmd->fill);

	/* store basic information into the xFillCommand struct */
	fillCmd->fg = page->fillColor;
	fillCmd->fs = page->fillStyle;
	fillCmd->n = *n;

	/* if storing permanently, make copies of coordinate information,
	 * updating the openSegment's extrema information at the same time */
	if (cmd != &tmpCmd ) {
		if ((fillCmd->x = xCopyCoords(x, fillCmd->n, &min, &max))==NULL)
							fillCmd->n = 0;
		else {
			if (seg->xmin > min) seg->xmin = min;
			if (seg->xmax < max) seg->xmax = max;
		} /* else */

		if((fillCmd->y = xCopyCoords(y, fillCmd->n, &min, &max))==NULL)
							fillCmd->n = 0;
		else {
			if (seg->ymin > min) seg->ymin = min;
			if (seg->ymax < max) seg->ymax = max;
		} /* else */
	}
	/* otherwise, just copy pointers */
	else {
		fillCmd->x = x;
		fillCmd->y = y;
	} /* else */
	return (cmd);
}


/*
 * xfill_:  if a segment is currently open, xfill_ calls xStoreCommand
 *	to save this command into that segment (for possible future use 
 *	after an expose or resize event), then calls xFill to actually
 *	do the requested fill.  in contrast, if no segment is currently 
 *	open, the command is not stored, but is simply executed directly.
 * 		commands to draw a two-point filled area are ignored.
 */
void
xfill_(
	int			*n,		/* number of points */
	float			x[],		/* x-coords */
	float			y[])		/* y-coords */
{
	XtalCanvas	*graphic;	/* drawable graphic surface */
	xCommand		cmd;		/* stored command */

	/* bail out if xstart_ hasn't yet been called */
	if (!interp) {
		printf("\n--- Tk interface:  xfill_ called before xstart_\n");
		fflush(stdout);
		return;
		} /* if */

	graphic = currentGraphic;
	if (graphic == NULL) {
     printf("\n--- Tk interface:  no drawing surface specified for xfill_\n");
		fflush(stdout);
		return;
		} /* if */

	/* silently ignore bogus commands */
	if (*n < 3)
		return;


	cmd = xInitFillCommand(graphic,graphic->openSegment,n,x,y);  
	/* execute the command */
        if(graphic->tkwin && Tk_WindowId(graphic->tkwin)) 
              xFill(graphic, &(cmd->fill));

} /* xfill_ */

void __stdcall XFILL(int *n, float x[], float y[])
{    xfill_(n, x, y); } 
#ifndef VMS
void xfill(int *n, float x[], float y[])
{    xfill_(n, x, y); } 
#endif



/*
 * xMark:  draws markers.  note that we only have one marker type (a
 *	circle).
 */

void	
xMark(XtalCanvas *graphic, xMarkCommand markCmd)
{
    static float	pixPerCM = 0.0;		/* pixels per centimeter */

    unsigned int	size;			/* marker diameter */
    float		halfSize;		/* marker radius */
    int		x,			/* marker position */
    		y;
    register int	i;			/* loop control variable */

    /* make sure the gc is set properly */
    colGC= markCmd->fg ; /* first set color */
    xSetFunction(gcFunc);
/*      xSetFillStyle(FillSolid);      */
    xSetLineAttributes((unsigned)0, LineSolid);

    /* find diameter and radius of marker, in pixels */
/*
	if (pixPerCM == 0.0)
		pixPerCM = DisplayHeight(display, screenNumber) /
		   (DisplayHeightMM(display, screenNumber) / 10.0);
	halfSize = (markCmd->size * pixPerCM) *
	   (graphic->resizeFactor.h/DisplayHeight(display,screenNumber));
 */
/*	size = (unsigned int)halfSize;   possible resizeFactor problem */
/*	halfSize /= 2.0; */


   /* find diameter and radius of marker, in pixels */
   size = (unsigned int) (markCmd->size * graphic->resizeFactor.h );
   halfSize = (float)size /2.0f ;

   if (size>2){

      /* for each point... */
      for (i = 0; i < markCmd->n; i++) {

         /* get a position */
      	 x = (int)(markCmd->x[i] * graphic->resizeFactor.w + 
      				  graphic->xOffset - halfSize);
      	 y = (int)(graphic->height - (markCmd->y[i]*
            graphic->resizeFactor.h+graphic->yOffset+halfSize));

      	 /* draw a circle */

      	 XDrawArc(display, graphic->canvas,  gcCol[colGC],
      				 x, y, size, size, 0, FULL_CIRCLE);
      } /* for */
   } else {
   
      for (i = 0; i < markCmd->n; i++) {

       	 /* get a position */
      	 x = (int)(markCmd->x[i] * graphic->resizeFactor.w + 
      				  graphic->xOffset - halfSize);
      	 y = (int)(graphic->height - (markCmd->y[i]*
             graphic->resizeFactor.h+graphic->yOffset+halfSize));

         /* draw a point */
#ifdef __WIN32__
         XDrawLine(display, graphic->canvas, gcCol[colGC], x, y,x,y);
#else
         XDrawPoint(display, graphic->canvas, gcCol[colGC], x, y);
#endif

        }
   }
} /* xMark */

/*
 * x initiate Mark command
 */

xCommand
xInitMarkCommand(XtalCanvas *page,xSegment seg, int *n,float *x,float *y)
{
	xCommand		cmd;		/* stored command */
	xMarkCommand		markCmd;	/* stored command */
	float			min,max;		/* coordinate extrema */
	
	/* if no segment is currently open, store the command in temporary
	 * space */
	if (seg == NULL)  cmd = &tmpCmd ; 

	/* otherwise, try to store this command permanently */
	else if ((cmd = xStoreCommand(seg)) != NULL)
					cmd->any.type = xMarkCommandType;
	else  cmd = &tmpCmd ; 
	markCmd = &(cmd->mark);

	/* store basic information into the xMarkCommand struct */
	markCmd->fg   = page->markerColor;
	markCmd->size = page->markerSize;
	markCmd->n    = *n;

	/* if storing permanently, make copies of coordinate information,
	 * updating the openSegment's extrema information at the same time */
	if (cmd != &tmpCmd) {
		if ((markCmd->x = xCopyCoords(x, markCmd->n, &min, &max))==NULL)
							markCmd->n = 0;
		else {
			if (seg->xmin > min) seg->xmin = min;
			if (seg->xmax < max) seg->xmax = max;
		} /* else */

		if((markCmd->y = xCopyCoords(y, markCmd->n, &min, &max))==NULL)
							markCmd->n = 0;
		else {
			if (seg->ymin > min) seg->ymin = min;
			if (seg->ymax < max) seg->ymax = max;
		} /* else */
	}
	/* otherwise, just copy pointers */
	else {
		markCmd->x = x;
		markCmd->y = y;
	} /* else */
	return (cmd);
}

/*
 * xmark_:  if a segment is currently open, xmark_ calls xStoreCommand
 *	to save this command into that segment (for possible use after an
 *	expose or resize event), then calls xMark to actually draw the
 *	markers.  in contrast, if no segment is currently open, the command
 *	is not stored, but is simply executed directly.
 */
void
xmark_(int *n, float *x, float *y)
{
	XtalCanvas	*graphic;	/* drawable graphic surface */
	xCommand	cmd;		/* stored command */

	/* bail out if xstart_ hasn't yet been called */
	if (!interp) {
		printf("\n--- Tk interface:  xmark_ called before xstart_\n");
		fflush(stdout);
		return;
		} /* if */

	graphic = currentGraphic ;
	if (graphic == NULL) {
     printf("\n--- Tk interface:  no drawing surface specified for xmark_\n");
		fflush(stdout);
		return;
		} /* if */

	cmd = xInitMarkCommand(graphic,graphic->openSegment,n,x,y);

        if(graphic->tkwin && Tk_WindowId(graphic->tkwin)) 
              xMark(graphic, &(cmd->mark)); /* execute the command */
} /* xmark_ */

void  __stdcall XMARK(int *n, float *x, float *y)
{     xmark_(n, x, y); } /* XMARK */
#ifndef VMS
void  xmark(int *n, float *x, float *y)
{     xmark_(n, x, y); }   /* xmark */
#endif




/*
 * xText:  draws text.
 *	
 */
void	
 xText(XtalCanvas *graphic, xTextCommand textCmd)
{

    int    key;
    char   *ipt ;
    float  xc, yc, dxc, dyc; 
    int    greek=0;
    int    italic=0;
    int    spflag=0;
    float  w=graphic->resizeFactor.w;
    float  h=graphic->resizeFactor.h;
    float  delx = textCmd->delx  ;
    float  dely = textCmd->dely ;
    float  dx1=delx/7.f;  /* vector graphics rescale */
    float  dy1=dely/7.f; /* vector graphics rescale */
    float  xsh =0.75f* delx* w ;
    float  ysh =0.75f* dely* h ;
    float  scrxsh =0.43f* dely* w ;
    float  scrysh =0.43f* delx* h ;

            /* Text string lower left x origin */
    dxc = xc   = textCmd->x * w + graphic->xOffset; 
            /* Text string lower left y origin */
    dyc = yc   =-(textCmd->y * h + graphic->yOffset)+ 
             (float)graphic->height ;


    /* make sure the gc is set properly */
    colGC= textCmd->fg ;  /* first set color */
    xSetFunction(gcFunc);
#ifdef SCREENSHOT
    xSetLineAttributes((unsigned)2, LineSolid); /* set local variables */
    XSetLineAttributes(display, gcCol[colGC], (unsigned)2 , LineSolid, 
#else
    xSetLineAttributes((unsigned)0, LineSolid); /* set local variables */
    XSetLineAttributes(display, gcCol[colGC], (unsigned)1 , LineSolid, 
#endif
           CapProjecting, JoinBevel); /* but use different line caps & joins*/


/*    if(gcFunc == GXxor) xQueryGC() ; */

/*    if(gcFunc == GXxor) xSetFillStyle(FillStippled);  
         FillSolid runs incredibly slow under XOR ! */

    for(ipt= textCmd->string ; *ipt; ipt++) {
        /* scan string for components */
        switch (*ipt){
           case '|':                  /* italic character */
               italic = 1-italic ;
               continue;
               break;
           case '%':                  /* greek character */
	       if (greek != 1) {
                    greek = 1;
                    continue;
               }                      /* else %% so draw % sign */
               break;
           case '^':                  /* superscript */
               if(!spflag) {
                   spflag=+1;
                   dxc +=  -scrxsh;/* shift by (0,-3/7 )and angle rotate*/
                   dyc +=  -scrysh;/* shift by (0,-3/7 )and angle rotate*/
                   continue;
               }                      /* else ^^ so draw ^ */
               break;
           case '~':                  /* subscript */
               if(!spflag) {
                   spflag=-1; 
                   dxc +=   scrxsh;/* shift by (0,-3/7 )and angle rotate*/
                   dyc +=   scrysh;/* shift by (0,-3/7 )and angle rotate*/
                   continue;
               }                      /* else ~~ so draw ~ */
               break;
           case ' ':                  /* space bar character */
               break;
        }
                    
        if     (isalpha((int) *ipt)) {
            if (isupper((int) *ipt)) key = 1; /* uppercase */
            else                     key = 2; /* lowercase */
        }
        else if(isdigit((int) *ipt)) key = 3; /* a digit */
        else                         key = 4; /* special */
        if  (greek)                  key +=4; /* upper = 5  lower = 6 */


	vectorChar(graphic, key, *ipt, dxc, dyc, dx1, dy1, italic);
        spflag = 0;
	greek  = 0;
        dxc = xc += xsh  ;    /* shift by 3/4 of the 7*7   */
        dyc = yc -= ysh  ;    /* shift by 3/4 of the 7*7   */
    }

/*    XSetLineAttributes(display, gcCol[colGC], (unsigned)1 , LineSolid, 
           CapButt, JoinMiter);    reset original */

} 



void vectorChar(XtalCanvas *graphic, int key, char ch, 
     float x, float y, float delx, float dely, int italic)
{
    register int i;
    float    dx, dy;
    int      ix, iy;
    float    xv, yv;
    int      npts;
    unsigned char  ivec;
    char     *ptr;
    XPoint   points[20];
static   char     upcase[26][21] =  {
 {'A',   7, 65, 73, 75, 69, 27,144, 48,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},   
 {'B',  11, 72, 75, 69, 62, 54, 51,154, 48, 34, 26, 23,  0,0,0,0,0,0,0,0},
 {'C',   8,169, 75, 73, 65, 30, 24, 26, 34,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'D',   6, 72, 75, 69, 34, 26, 23,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'E',   6,127, 23, 72, 76,151, 55,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'F',   4, 72, 76,151, 55,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'G',  12,169, 75, 73, 65, 30, 24, 26, 34, 41, 47,146, 48,0,0,0,0,0,0,0},
 {'H',   5, 72,176, 27,151, 55,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'I',   6,124, 26,125, 74,173, 75,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'J',   7,130, 24, 25, 33, 75,172, 76,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'K',   5, 72,144, 76,152, 27,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'L',   3,172, 23, 27,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'M',   4, 72, 53, 76, 27,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'N',   3, 72, 27, 76,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'O',   9,130, 65, 73, 75, 69, 34, 26, 24, 30,  0,  0,  0,0,0,0,0,0,0,0},
 {'P',   6, 72, 75, 69, 62, 54, 51,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'Q',  11,130, 65, 73, 75, 69, 34, 26, 24, 30,139, 27,  0,0,0,0,0,0,0,0},
 {'R',   8, 72, 75, 69, 62, 54, 51,153, 27,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'S',  12,169, 75, 73, 65, 58, 52, 47, 41, 34, 26, 24, 30,0,0,0,0,0,0,0},
 {'T',   4,172, 76,174, 25,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'U',   6,172, 30, 24, 26, 34, 76,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'V',   3,172, 25, 76,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'W',   5,172, 24, 46, 26, 76,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'X',   4,172, 27,176, 23,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'Y',   5,172, 53, 76,153, 25,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'Z',   4,172, 76, 23, 27,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0}};

static  char  lowcase[26][21] =  {
 {'a',  11,133, 25, 24, 30, 44, 52, 53, 47,154, 26, 27,  0,0,0,0,0,0,0,0},
 {'b',   9, 72,144, 52, 53, 47, 33, 25, 24, 30,  0,  0,  0,0,0,0,0,0,0,0},
 {'c',   8,147, 53, 52, 44, 30, 24, 25, 33,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'d',  10,133, 25, 24, 30, 44, 52, 53, 47,175, 26,  0,  0,0,0,0,0,0,0,0},
 {'e',  10,137, 40, 47, 53, 52, 44, 30, 24, 25, 33,  0,  0,0,0,0,0,0,0,0},
 {'f',   7,169, 75, 74, 66, 24,151, 53,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'g',  13,133, 25, 24, 30, 44, 52, 53, 47,154, 12,  4,  3,9,0,0,0,0,0,0},
 {'h',   8, 72,144, 52, 53, 47, 40, 33, 26,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'i',   7,123, 25,124, 52, 51,159, 59,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'j',   8,161, 61,153, 54, 12,  4,  3,  9,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'k',   6, 72,137, 38, 26,138, 54,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'l',   5,172, 73, 24,123, 25,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'m',   9, 51,144, 52, 46, 25,146, 54, 48, 27,  0,  0,  0,0,0,0,0,0,0,0},
 {'n',   6, 51,144, 52, 53, 47, 26,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'o',   9,130, 44, 52, 53, 47, 33, 25, 24, 30,  0,  0,  0,0,0,0,0,0,0,0},
 {'p',  10,151,  2,144, 52, 53, 47, 33, 25, 24, 30,  0,  0,0,0,0,0,0,0,0},
 {'q',  11,130, 44, 52, 53, 47, 33, 25, 24, 30,154,  5,  0,0,0,0,0,0,0,0},
 {'r',   5, 51,144, 52, 53, 47,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'s',  10,130, 24, 26, 34, 40, 38, 44, 52, 54, 48,  0,  0,0,0,0,0,0,0,0},
 {'t',   6,173, 31, 25, 26,151, 54,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'u',   8,151, 30, 24, 25, 33,154, 26, 27,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'v',   6,151, 37, 24, 25, 40, 54,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'w',   9,151, 30, 24, 32, 46,132, 26, 34, 55,  0,  0,  0,0,0,0,0,0,0,0},
 {'x',   4,151, 26,154, 23,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'y',  11,151, 30, 24, 25, 33, 54,133, 12,  4,  3,  9,  0,0,0,0,0,0,0,0},
 {'z',   4,151, 54, 23, 26,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0}};

static   char  digitch[10][21] =  {
 {'0',  11,130, 65, 73, 75, 69, 34, 26, 24, 30,123, 76,  0, 0,0,0,0,0,0,0},
 {'1',   5,166, 74, 25,124, 26,  0,  0,  0,  0,  0,  0,  0, 0,0,0,0,0,0,0},
 {'2',   8,165, 73, 75, 69, 62, 30, 23, 27,  0,  0,  0,  0, 0,0,0,0,0,0,0},
 {'3',  13,165, 73, 75, 69, 62, 54, 52,154, 48, 34, 26, 24,30,0,0,0,0,0,0},
 {'4',   4,126, 75, 37, 41,  0,  0,  0,  0,  0,  0,  0,  0, 0,0,0,0,0,0,0},
 {'5',   9,130, 24, 26, 34, 48, 54, 51, 72, 76,  0,  0,  0, 0,0,0,0,0,0,0},
 {'6',  12,169, 75, 73, 65, 30, 24, 26, 34, 48, 54, 52, 44, 0,0,0,0,0,0,0},
 {'7',   5,165, 72, 76, 69, 24,  0,  0,  0,  0,  0,  0,  0, 0,0,0,0,0,0,0},
 {'8',  16,152, 44, 30, 24, 26, 34, 48, 54, 52, 58, 65,73,75,69,62,54,0,0,0},
 {'9',  12,130, 24, 26, 34, 69, 75, 73, 65, 51, 45, 47, 55, 0,0,0,0,0,0,0}};
 
static   char  punctch[19][21] =  {
 {'+',   4,160, 32,144, 48,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'-',   2,144, 48,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0}, 
 {'*',   8,160, 32,144, 48,158, 34,130, 62,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'/',   1, 76,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'(',   4,174, 66, 31, 25,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {')',   4,173, 67, 32, 24,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'=',   4,144, 48,130, 34,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {',',   6, 30, 31, 24, 23,124, 16,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'.',   4, 30, 31, 24, 23,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'!',   9,124, 26, 33, 31, 24,139, 75, 73, 39,  0,  0,  0,0,0,0,0,0,0,0},
 {'%',  12,165, 72, 73, 66, 65,126, 33, 34, 27, 26,123, 76,0,0,0,0,0,0,0},
{'\'',   4,172, 58, 73, 72,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'^',   3,158, 74, 62,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'~',   4,144, 52, 46, 54,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'[',   4,174, 73, 24, 25,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {']',   4,173, 74, 25, 24,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'<',   4,125, 44, 67,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'>',   4,123, 46, 65,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'?',   9,124, 26, 33, 31, 24,139, 53, 62, 69, 75, 73, 64,58,0,0,0,0,0,0}};

static   char  greekUpch[26][21] =  {
 {'A',   7, 65, 73, 75, 69, 27,144, 48,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'B',  11, 72, 75, 69, 62, 54, 51,154, 48, 34, 26, 23,  0,0,0,0,0,0,0,0},
 {'C',   4,172, 27,176, 23,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'D',   3, 74, 27, 23,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'E',   6,127, 23, 72, 76,151, 55,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'F',  15,131, 37, 58, 66, 68, 62, 41, 33, 31,124,26,125,74,173,75,0,0,0,0},
 {'G',   5,172, 76, 69,173, 24,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'H',   5, 72,176, 27,151, 55,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'I',   6,124, 26,125, 74,173, 75,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
/* script J */
{ 'J',   7,130, 24, 25, 33, 75,172, 76,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'K',   5, 72,144, 76,152, 27,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'L',   2, 74, 27,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'M',   4, 72, 53, 76, 27,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'N',   3, 72, 27, 76,  0,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'O',   9,130, 65, 73, 75, 69, 34, 26, 24, 30,  0,  0,  0,0,0,0,0,0,0,0},
 {'P',   8,124, 73,165, 72, 76, 69,175, 26,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'Q',  15,124, 30, 65, 73, 75, 69, 34, 26, 24,159,45,152,54,161,47,0,0,0,0},
 {'R',   6, 72, 75, 69, 62, 54, 51,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'S',   7,134, 27, 23, 53, 72, 76, 69,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'T',   4,172, 76,174, 25,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'U',   5,172, 53, 76,153, 25,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
/* Aring */
 {'V',  11, 44, 74, 48, 27,148, 44,174, 80, 88, 82, 74,  0,0,0,0,0,0,0,0},
 {'W',  12,130, 23, 24, 37, 58, 73, 75, 62, 41, 26, 27, 34,0,0,0,0,0,0,0},
 {'X',  10,130, 23, 27, 34,152, 54,165, 72, 76, 69,  0,  0,0,0,0,0,0,0,0},
 {'Y',  12,172, 51, 45, 47, 55, 76,124, 26,125, 74,173, 75,0,0,0,0,0,0,0},
 {'Z',   4,172, 76, 23, 27,  0,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0}};

static   char  greekLowch[26][21] =  {
 {'a',  10,155, 33, 25, 24, 30, 44, 52, 53, 47, 27,  0,  0,0,0,0,0,0,0,0},
 {'b',  13,109, 58, 66, 67, 61, 53, 52,153, 47, 33, 25,24,30,0,0,0,0,0,0},
 {'c',   6,158, 66, 26, 27,123, 69,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'d',  13,153, 52, 44, 30, 24, 25, 33, 47, 53, 58, 65,67,61,0,0,0,0,0,0},
 {'e',  11,133, 25, 24, 30, 38, 39,138, 44, 52, 53, 47,  0,0,0,0,0,0,0,0},
 {'f',  11,124, 30, 44, 52, 54, 48, 34, 26, 24,167, 11,  0,0,0,0,0,0,0,0},
 {'g',   8,151, 44, 26, 19, 11, 25, 41, 55,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'h',   6, 51,144, 52, 53, 47, 12,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'i',   5,124, 26,125, 53, 52,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
/* script j */
 {'j',   8,161, 61,153, 54, 12,  4,  3,  9,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'k',   6, 51,147, 53, 37, 25, 26,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'l',   6, 53,165, 66, 53, 33, 27,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'m',  10,109, 51,130, 24, 25, 33, 54,133, 26, 27,  0,  0,0,0,0,0,0,0,0},
 {'n',   5,151, 24, 33, 48, 54,  0,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'o',   9,124, 30, 44, 52, 53, 47, 33, 25, 24,  0,  0,  0,0,0,0,0,0,0,0},
 {'p',   8,124, 52,144, 51, 55, 48,154, 26,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'q',  11,124, 37, 51, 66, 68, 55, 41, 26, 24,144, 48,  0,0,0,0,0,0,0,0},
 {'r',   9,109, 44, 52, 53, 47, 33, 25, 24, 30,  0,  0,  0,0,0,0,0,0,0,0},
 {'s',   9,153, 47, 33, 25, 24, 30, 44, 52, 55,  0,  0,  0,0,0,0,0,0,0,0},
 {'t',   8,144, 51, 55, 48,153, 25, 26, 34,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'u',   6,151, 30, 24, 25, 33, 54,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
/* script v */
 {'v',   6,151, 37, 24, 25, 40, 54,  0,  0,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'w',  11,152, 44, 30, 24, 32, 39,132, 26, 34, 48, 54,  0,0,0,0,0,0,0,0},
 {'x',  15,158, 60, 68, 67, 51, 45, 46,145, 37, 30, 24,25,19,11,10,0,0,0,0},
 {'y',   8,151, 30, 24, 26, 34, 55,167, 11,  0,  0,  0,  0,0,0,0,0,0,0,0},
 {'z',  12,158, 60, 68, 67, 59, 37, 30, 24, 25, 19, 11, 10,0,0,0,0,0,0,0}};

    

    switch(key) {
      case 1: 
            ptr = &upcase[ch-'A'][0] ;  /* address of  required char */
            break;
      case 2: 
            ptr = &lowcase[ch-'a'][0] ; 
            break;
      case 3: 
      case 7: 
            ptr = &digitch[ch-'0'][0] ;
            break;
      case 5: 
            ptr = &greekUpch[ch-'A'][0] ;
            break;
      case 6: 
            ptr = &greekLowch[ch-'a'][0] ;
            break;
      case 4: 
      case 8: 
      default:
            if(ch == ' ') return;       /* ignore spaces */
            for(i=0; i<19 ;i++) if (punctch[i][0]==ch) break;
            if( i>=19) ptr = &punctch[1][0] ;  /* use minus sign */
            else       ptr = &punctch[i][0] ;  /* found required */
            break;
    }

    ptr += 2;        /*  step to first vertex */
    if ((unsigned int )*ptr<100){   /*  char has default origin  */
	points[0].x = (short)(x) ;
	points[0].y = (short)(y) ;
        npts = 1;    /* got one point */
    }
    else npts = 0;   /* no points yet */


    while ((ivec =(unsigned char) *ptr)>0) {  /* break on first zero point */
	if (ivec > 100) {        /* pen up  - move to */
            if (npts>0) XDrawLines(display, graphic->canvas, gcCol[colGC],
                         points, npts, CoordModeOrigin);
            npts= 0;
	    ivec = ivec - 100;
        }
      
	iy = (int)ivec / 7;                    /* font is 7x10 */
	ix = (int)ivec - iy *7;              
	xv = 0.90f*(float) (ix - 2);       /* origin is at (2,3) */
	yv = (float) (iy - 3);
	if (italic) xv +=  (float)(2*iy ) / 7.f ;   /* slanted text */
	dx = (xv * delx - yv * dely) * graphic->resizeFactor.w ;
	dy = (xv * dely + yv * delx) * graphic->resizeFactor.h ;
	points[npts].x =  (short)(x+dx) ;
	points[npts].y =  (short)(y-dy) ;
        npts++ ;
        ptr++ ;
    }
    if (npts>0)
        XDrawLines(display, graphic->canvas, gcCol[colGC],
                         points, npts, CoordModeOrigin);


}


/*
 * x initiate Mark command
 */
xCommand
xInitTextCommand(XtalCanvas *page,xSegment seg, float *x, float *y, 
    float *angle, int *col, float *size, int nc, char *string)
{
	xTextCommand	textCmd;	/* cmd to be executed */
	xCommand	cmd;		/* stored command */
        float           radang,lang,ca,sa;
        float           x1,x2,x3,x4,y1,y2,y3,y4 ;
        double          ret;

	

	/* if no segment is currently open, store the command in temporary
	 * space */
	if (seg == NULL)  cmd = &tmpCmd ; 

	/* otherwise, try to store this command permanently */
	else if ((cmd = xStoreCommand(seg)) != NULL)
					cmd->any.type = xTextCommandType;
	else  cmd = &tmpCmd ; 
	textCmd = &(cmd->text);


	/* store basic information into the xTextCommand struct */
	textCmd->fg   = *col;
	textCmd->x    = *x;
	textCmd->y    = *y;
	textCmd->size = *size;
	
        lang = (float) (360. * modf(*angle/360.,&ret)) ;
        if       (lang>-0.5f && lang< 0.5f ) { /* horiz */
            ca = 1.0 ;            /* 0 +/- half a degree in radians */
            sa = 0.0 ;
        } else if (lang>89.5f && lang< 90.5f ) { /* vert */
            ca = 0.0 ;            /* 90 +/- half a degree in radians */
            sa = 1.0 ;
        } else {                        /* arbitrary angle*/
            radang=lang *  0.017453293f; /* in radians */
            ca = (float)cos(radang) ;
            sa = (float)sin(radang) ;
        }
        textCmd->delx = *size * ca;
        textCmd->dely = *size * sa;


	if (cmd == &tmpCmd) textCmd->string = string ;       /* just copy pointer */
                 /* if storing permanently, make copy of the string */
        else if ((textCmd->string=(char *)malloc(sizeof(char)*(nc+1)))==NULL){
            printf(
            "\n--- Tk interface:  unable to allocate space for text string\n");
            free( textCmd) ;
            return  (xCommand) NULL;              /* can't do much here */
        } else {
                strcpy(textCmd->string,string);     /* copy charactersup to null */
                x1 = *x  ;
                x2 = x1 + textCmd->delx * 0.75f* (float)nc ;
                x3 = x1 - textCmd->dely ;
                x4 = x2 - textCmd->dely ;
                y1 = *y  ;
                y2 = y1 + textCmd->dely * 0.75f* (float)nc ;
                y3 = y1 + textCmd->delx ;
                y4 = y2 + textCmd->delx ;
                if       (sa >=0.0 & ca >=0.0 ) {  /* quadrant 1  0 - 90 */
		    if (seg->ymin > y1) seg->ymin = y1;
		    if (seg->ymax < y4) seg->ymax = y4;
		    if (seg->xmin > x3) seg->xmin = x3;
		    if (seg->xmax < x2) seg->xmax = x2;
                } else if(sa >=0.0 & ca < 0.0 ) {  /* quadrant 2 90 - 180 */
		    if (seg->ymin > y3) seg->ymin = y3;
		    if (seg->ymax < y2) seg->ymax = y2;
		    if (seg->xmin > x4) seg->xmin = x4;
		    if (seg->xmax < x1) seg->xmax = x1;
                } else if(sa < 0.0 & ca < 0.0 ) {  /* quadrant 3 180 - 270 */
		    if (seg->ymin > y4) seg->ymin = y4;
		    if (seg->ymax < y1) seg->ymax = y1;
		    if (seg->xmin > x2) seg->xmin = x2;
		    if (seg->xmax < x3) seg->xmax = x3;
                } else  {                          /* quadrant 4  270 - 360 */
		    if (seg->ymin > y2) seg->ymin = y3;
		    if (seg->ymax < y3) seg->ymax = y2;
		    if (seg->xmin > x1) seg->xmin = x1;
		    if (seg->xmax < x4) seg->xmax = x4;
                }
	} 
	return (cmd);
}

/*
 * xtext_:  if a segment is currently open, xtext_ calls xStoreCommand
 *	to save this command into that segment (for possible use after an
 *	expose or resize event), then calls xText to actually draw the text
 *	markers.  in contrast, if no segment is currently open, the command
 *	is not stored, but is simply executed directly.
 */
void
xtext_(float *x, float *y, float *angle, int *col, float *size, int *nc, char *string, int slength)
{

	XtalCanvas	*graphic;	/* drawable graphic surface */
	xCommand	cmd;		/* stored command */
        int             cnt;            /* actual character count */
        char            dummy[256];


	/* bail out if xstart_ hasn't yet been called */
	if (!interp) {
		printf("\n--- Tk interface:  xtext_ called before xstart_\n");
		fflush(stdout);
		return;
		} /* if */

	graphic = currentGraphic ;
	if (graphic == NULL) {
     printf("\n--- Tk interface:  no drawing surface specified for xtext_\n");
		fflush(stdout);
		return;
		} /* if */


        cnt = (*nc < slength ? *nc : slength) ; 
        for( ;cnt>0 && string[cnt-1] == ' '; cnt--) ; /* strip trailing white space*/
        strncpy(dummy,string, (size_t) cnt);          /* copy characters */
        dummy[cnt]='\0';                              /* add trailing zero */


	cmd = xInitTextCommand(graphic,graphic->openSegment,
                   x, y, angle, col, size, cnt, dummy); 

        if(graphic->tkwin && Tk_WindowId(graphic->tkwin)) 
               xText(graphic, &(cmd->text)); /* execute the command */


} /* xtext_ */

void __stdcall XTEXT(float *x, float *y, float *angle, int *col, 
         float *size, int *nc, char *string, int slength)
{ xtext_(x, y, angle, col, size, nc, string,slength); } /* XTEXT */
#ifndef VMS
void xtext(float *x, float *y, float *angle, int *col,
         float *size, int *nc, char *string, int slength)
{ xtext_(x, y, angle, col, size, nc, string,slength); } /* xtext */
#endif





#ifndef __WIN32__
static void
xQueryGC()
{
    unsigned long valmask;
    XGCValues values ;
    valmask= GCFunction | GCPlaneMask | GCForeground | GCBackground | GCLineWidth |
            GCLineStyle |  GCCapStyle | GCJoinStyle | GCFillStyle | GCFillRule |
           GCTile | GCStipple | GCTileStipXOrigin | GCTileStipYOrigin | GCFont |
         GCSubwindowMode | GCGraphicsExposures | GCClipXOrigin | GCClipYOrigin |
          GCDashOffset | GCArcMode ;


    XGetGCValues(display, gcCol[colGC], valmask, &values);

    printf("\n GCFunction    %d\n",values.function);
      printf(" GCPlaneMask   %x\n",values.plane_mask);
      printf(" GCForeground  %x\n",values.foreground);
      printf(" GCBackground  %x\n",values.background);
      printf(" GCLineWidth   %d\n",values.line_width);
      printf(" GCLineStyle   %d\n",values.line_style);
      printf(" GCCapSTyle    %d\n",values.cap_style);
      printf(" GCJoinStyle   %d\n",values.join_style);
      printf(" GCFillStyle   %d\n",values.fill_style);
      printf(" GCFillRule    %d\n",values.fill_rule);
      printf(" GCTile        %d\n",values.tile);
      printf(" GCStipple     %d\n",values.stipple);
      printf(" GCTileStipXOrigin  %d\n",values.ts_x_origin);
      printf(" GCTileStipYOrigin  %d\n",values.ts_y_origin);
      printf(" GCFont        %d\n",values.font);
      printf(" GCSubwindowMode  %d\n",values.subwindow_mode);
      printf(" GCGraphicsExposures  %d\n",values.graphics_exposures);
      printf(" GCClipXOrigin %d\n",values.clip_x_origin);
      printf(" GCClipYOrigin %d\n",values.clip_y_origin);
      printf(" GCDashOffset  %d\n",values.dash_offset);
      printf(" GCArcMode     %d\n",values.arc_mode);
    fflush(stdout);

}
#endif

#ifdef __cplusplus
}
#endif
